@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -65,6 +65,10 @@ class t_const final : public t_named {
   const t_program* program() const { return program_; }
   const t_type_ref& type() const { return type_; }
   const t_const_value* value() const { return value_.get(); }
+  const t_const_value& get_value_from_structured_annotation(
+      const char* key) const;
+  const t_const_value* get_value_from_structured_annotation_or_null(
+      const char* key) const;
 
  private:
   t_program* const program_;
@@ -86,6 +90,11 @@ class t_const final : public t_named {
             std::move(name),
             std::move(value)) {}
 
+  std::unique_ptr<t_const> clone() const {
+    return std::make_unique<t_const>(
+        get_program(), get_type(), name(), value_->clone());
+  }
+
   t_program* get_program() const { return program_; }
   const t_type* get_type() const { return type_.get_type(); }
   t_const_value* get_value() const { return value_.get(); }
