@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -19,8 +19,10 @@
 #include <memory>
 #include <sstream>
 #include <string>
+#include <boost/optional.hpp>
 
 #include <thrift/compiler/ast/node_list.h>
+#include <thrift/compiler/ast/t_const.h>
 #include <thrift/compiler/ast/t_const_value.h>
 #include <thrift/compiler/ast/t_named.h>
 #include <thrift/compiler/ast/t_type.h>
@@ -32,9 +34,10 @@ namespace compiler {
 class t_struct;
 
 enum class t_field_qualifier {
-  unspecified = 0,
+  none,
   required,
   optional,
+  terse,
 };
 
 using t_field_id = int16_t;
@@ -48,11 +51,8 @@ using t_field_id = int16_t;
  */
 class t_field final : public t_named {
  public:
-  // Field IDs starting with `0b10` as the most significant binary digits are
-  // reserved. e.g.
-  // * 0b1011'1111'1111'1111 (-16385) leads with 0b10, it's largest reserved id
-  // * 0b1100'0000'0000'0000 (-16384) leads with 0b11, it's smallest allowed id
-  static constexpr auto min_id = -static_cast<t_field_id>(1 << 14);
+  // Field ids < -32 are for internal use
+  static constexpr auto min_id = -32;
 
   /**
    * Constructor for t_field
@@ -62,21 +62,24 @@ class t_field final : public t_named {
    * @param id  - The numeric identifier of the field
    */
   t_field(
-      t_type_ref type,
-      std::string name,
-      t_field_id id,
-      bool has_explicit_id = true)
+      t_type_ref type, std::string name, boost::optional<t_field_id> id = {})
       : t_named(std::move(name)),
         type_(std::move(type)),
-        id_(id),
-        has_explicit_id_(has_explicit_id) {}
+        id_(id.value_or(0)),
+        explicit_id_(id) {}
 
   t_field(t_field&&) = delete;
   t_field& operator=(t_field&&) = delete;
 
   const t_type_ref& type() const { return type_; }
-  int32_t id() const { return id_; }
-  bool has_explicit_id() const { return has_explicit_id_; }
+  t_field_id id() const { return id_; }
+  boost::optional<t_field_id> explicit_id() const { return explicit_id_; }
+  bool is_injected() const { return injected_; }
+  void set_injected_id(t_field_id id) {
+    id_ = id;
+    explicit_id_ = id;
+    injected_ = true;
+  }
 
   const t_const_value* default_value() const { return value_.get(); }
   void set_default_value(std::unique_ptr<t_const_value> value) {
@@ -94,10 +97,18 @@ class t_field final : public t_named {
   std::unique_ptr<t_field> clone_DO_NOT_USE() const {
     auto clone = std::make_unique<t_field>(type_, name_, id_);
 
-    if (!!value_) {
+    if (value_) {
       clone->value_ = value_->clone();
     }
 
+    // unstructured annotations
+    clone->reset_annotations(annotations());
+
+    // structued annotations
+    for (auto annot : structured_annotations()) {
+      clone->add_structured_annotation(annot->clone());
+    }
+
     clone->qual_ = qual_;
 
     return clone;
@@ -105,12 +116,16 @@ class t_field final : public t_named {
 
  private:
   t_type_ref type_;
-  const t_field_id id_;
-  bool has_explicit_id_;
+  // TODO(afuller): Make this const, once we remove support for auto assigned
+  // ids.
+  t_field_id id_;
+  boost::optional<t_field_id> explicit_id_;
 
   t_field_qualifier qual_ = {};
   std::unique_ptr<t_const_value> value_;
 
+  bool injected_{};
+
   // TODO(afuller): Delete everything below here. It is only provided for
   // backwards compatibility.
  public:
@@ -118,10 +133,9 @@ class t_field final : public t_named {
     required = 0,
     optional = 1,
     opt_in_req_out = 2,
+    terse = 3,
   };
 
-  t_field(t_type_ref type, std::string name)
-      : t_field(type, std::move(name), 0, false) {}
   t_field(const t_type* type, std::string name, t_field_id key = 0)
       : t_field(t_type_ref::from_req_ptr(type), std::move(name), key) {}
 
@@ -137,7 +151,7 @@ class t_field final : public t_named {
   void set_req(e_req req) {
     switch (req) {
       case e_req::opt_in_req_out:
-        set_qualifier(t_field_qualifier::unspecified);
+        set_qualifier(t_field_qualifier::none);
         break;
       case e_req::required:
         set_qualifier(t_field_qualifier::required);
@@ -145,22 +159,28 @@ class t_field final : public t_named {
       case e_req::optional:
         set_qualifier(t_field_qualifier::optional);
         break;
+      case e_req::terse:
+        set_qualifier(t_field_qualifier::terse);
+        break;
     }
   }
   e_req get_req() const {
     switch (qual_) {
-      case t_field_qualifier::unspecified:
+      case t_field_qualifier::none:
         return e_req::opt_in_req_out;
       case t_field_qualifier::required:
         return e_req::required;
       case t_field_qualifier::optional:
         return e_req::optional;
+      case t_field_qualifier::terse:
+        return e_req::terse;
     }
     return {};
   }
 
   const t_type* get_type() const { return type().get_type(); }
   int32_t get_key() const { return id(); }
+  void set_implicit_id(t_field_id id) { id_ = id; }
 };
 
 using t_field_list = node_list<t_field>;
