@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,8 +14,10 @@
  * limitations under the License.
  */
 
+#include <boost/algorithm/string.hpp>
 #include <thrift/compiler/ast/t_program.h>
 
+#include <cctype>
 #include <map>
 #include <memory>
 #include <string>
@@ -26,15 +28,67 @@ namespace apache {
 namespace thrift {
 namespace compiler {
 
+void t_program::add_definition(std::unique_ptr<t_named> definition) {
+  assert(definition != nullptr);
+
+  // Resolve Thrift URI if need be.
+  if (definition->uri().empty()) { // Inherit from package.
+    definition->set_uri(package_.get_uri(definition->name()));
+    if (auto* uri = definition->find_annotation_or_null("thrift.uri")) {
+      definition->set_uri(*uri); // Manual override.
+    }
+  }
+
+  // Index the node.
+  if (auto* node = dynamic_cast<t_exception*>(definition.get())) {
+    objects_.push_back(node);
+    exceptions_.push_back(node);
+  } else if (auto* node = dynamic_cast<t_struct*>(definition.get())) {
+    objects_.push_back(node);
+    structs_.push_back(node);
+  } else if (auto* node = dynamic_cast<t_interaction*>(definition.get())) {
+    interactions_.push_back(node);
+  } else if (auto* node = dynamic_cast<t_service*>(definition.get())) {
+    services_.push_back(node);
+  } else if (auto* node = dynamic_cast<t_enum*>(definition.get())) {
+    enums_.push_back(node);
+  } else if (auto* node = dynamic_cast<t_typedef*>(definition.get())) {
+    typedefs_.push_back(node);
+  } else if (auto* node = dynamic_cast<t_const*>(definition.get())) {
+    consts_.push_back(node);
+  }
+
+  // Transfer ownership of the definition.
+  definitions_.push_back(std::move(definition));
+}
+
 const std::string& t_program::get_namespace(const std::string& language) const {
   auto pos = namespaces_.find(language);
   static const auto& kEmpty = *new std::string();
   return (pos != namespaces_.end() ? pos->second : kEmpty);
 }
 
+std::vector<std::string> t_program::gen_namespace_or_default(
+    const std::string& language,
+    std::function<std::vector<std::string>()> gen) const {
+  auto pos = namespaces_.find(language);
+  if (pos != namespaces_.end()) {
+    std::vector<std::string> ret;
+    if (!pos->second.empty()) {
+      split(ret, pos->second, boost::algorithm::is_any_of("."));
+    }
+    return ret;
+  }
+
+  return gen();
+}
+
 std::unique_ptr<t_program> t_program::add_include(
-    std::string path, std::string include_site, int lineno) {
-  auto program = std::make_unique<t_program>(path);
+    std::string path,
+    std::string include_site,
+    int lineno,
+    const source_range& range) {
+  auto program = std::unique_ptr<t_program>(new t_program(path, scope_));
 
   std::string include_prefix;
   const auto last_slash = include_site.find_last_of("/\\");
@@ -46,6 +100,7 @@ std::unique_ptr<t_program> t_program::add_include(
 
   auto include = std::make_unique<t_include>(program.get());
   include->set_lineno(lineno);
+  include->set_src_range(range);
 
   add_include(std::move(include));
 
@@ -73,17 +128,6 @@ std::string t_program::compute_name_from_file_path(std::string path) {
   return path;
 }
 
-size_t t_program::get_byte_offset(
-    size_t line, size_t line_offset) const noexcept {
-  if (line == 0) {
-    return noffset; // Not specified.
-  }
-  if (line > line_to_offset_.size()) {
-    return noffset; // No offset data provided.
-  }
-  return line_to_offset_[line - 1] + line_offset;
-}
-
 } // namespace compiler
 } // namespace thrift
 } // namespace apache
