@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -33,7 +33,8 @@
 #include <thrift/compiler/ast/t_interaction.h>
 #include <thrift/compiler/ast/t_list.h>
 #include <thrift/compiler/ast/t_map.h>
-#include <thrift/compiler/ast/t_node.h>
+#include <thrift/compiler/ast/t_named.h>
+#include <thrift/compiler/ast/t_package.h>
 #include <thrift/compiler/ast/t_scope.h>
 #include <thrift/compiler/ast/t_service.h>
 #include <thrift/compiler/ast/t_set.h>
@@ -46,22 +47,23 @@ namespace apache {
 namespace thrift {
 namespace compiler {
 
+enum class t_statement_type {
+  standard_header,
+  program_header,
+  definition,
+};
+
+enum class t_header_type {
+  standard = int(t_statement_type::standard_header),
+  // All annotations on headers of these types are
+  // attributed to the program.
+  program = int(t_statement_type::program_header),
+};
+
 /**
- * class t_program
- *
- * Top level class representing an entire thrift program. A program consists
- * fundamentally of the following:
- *
- *   Typedefs
- *   Enumerations
- *   Constants
- *   Structs
- *   Exceptions
- *   Services
- *
- * The program module also contains the definitions of the base types.
+ * Top level class representing an entire thrift program.
  */
-class t_program : public t_node {
+class t_program : public t_named {
  public:
   // The value used when an offset is not specified/unknown.
   static constexpr auto noffset = static_cast<size_t>(-1);
@@ -71,56 +73,22 @@ class t_program : public t_node {
    *
    * @param path - A *.thrift file path.
    */
-  explicit t_program(std::string path) : path_(std::move(path)) {}
+  explicit t_program(std::string path)
+      : t_program(std::move(path), std::make_shared<t_scope>()) {}
 
-  /**
-   * Set program elements
-   */
-  void add_typedef(std::unique_ptr<t_typedef> td) {
-    assert(td != nullptr);
-    typedefs_.push_back(td.get());
-    nodes_.push_back(std::move(td));
-  }
-  void add_enum(std::unique_ptr<t_enum> te) {
-    assert(te != nullptr);
-    enums_.push_back(te.get());
-    nodes_.push_back(std::move(te));
-  }
-  void add_const(std::unique_ptr<t_const> tc) {
-    assert(tc != nullptr);
-    consts_.push_back(tc.get());
-    nodes_.push_back(std::move(tc));
-  }
-  void add_struct(std::unique_ptr<t_struct> ts) {
-    assert(ts != nullptr);
-    objects_.push_back(ts.get());
-    structs_.push_back(ts.get());
-    nodes_.push_back(std::move(ts));
-  }
-  void add_exception(std::unique_ptr<t_exception> tx) {
-    assert(tx != nullptr);
-    objects_.push_back(tx.get());
-    exceptions_.push_back(tx.get());
-    nodes_.push_back(std::move(tx));
-  }
-  void add_service(std::unique_ptr<t_service> ts) {
-    assert(ts != nullptr);
-    services_.push_back(ts.get());
-    nodes_.push_back(std::move(ts));
-  }
-  void add_interaction(std::unique_ptr<t_interaction> ti) {
-    assert(ti != nullptr);
-    interactions_.push_back(ti.get());
-    nodes_.push_back(std::move(ti));
-  }
+  void set_package(t_package package) { package_ = std::move(package); }
+  const t_package package() const { return package_; }
 
-  void add_placeholder_typedef(std::unique_ptr<t_placeholder_typedef> ptd) {
-    assert(ptd != nullptr);
-    placeholder_typedefs_.push_back(ptd.get());
-    nodes_.push_back(std::move(ptd));
-  }
+  // Defintions, in the order they were added.
+  node_list_view<t_named> definitions() { return definitions_; }
+  node_list_view<const t_named> definitions() const { return definitions_; }
+  void add_definition(std::unique_ptr<t_named> definition);
 
-  // Adds a concrete instantiation of a templated type.
+  // Concrete instantiation of a templated type.
+  node_list_view<t_templated_type> type_instantiations() { return type_insts_; }
+  node_list_view<const t_templated_type> type_instantiations() const {
+    return type_insts_;
+  }
   void add_type_instantiation(std::unique_ptr<t_templated_type> type_inst) {
     assert(type_inst != nullptr);
     type_insts_.emplace_back(std::move(type_inst));
@@ -143,7 +111,7 @@ class t_program : public t_node {
   }
 
   /**
-   * Get program elements
+   * Get program elements by kind.
    */
   const std::vector<t_typedef*>& typedefs() const { return typedefs_; }
   const std::vector<t_enum*>& enums() const { return enums_; }
@@ -152,20 +120,10 @@ class t_program : public t_node {
   const std::vector<t_exception*>& exceptions() const { return exceptions_; }
   const std::vector<t_struct*>& objects() const { return objects_; }
   const std::vector<t_service*>& services() const { return services_; }
-  const std::vector<t_placeholder_typedef*>& placeholder_typedefs() const {
-    return placeholder_typedefs_;
-  }
   const std::vector<t_interaction*>& interactions() const {
     return interactions_;
   }
-  node_list_view<t_templated_type> type_instantiations() { return type_insts_; }
-  node_list_view<const t_templated_type> type_instantiations() const {
-    return type_insts_;
-  }
 
-  /**
-   * t_program setters
-   */
   void add_cpp_include(std::string path) {
     cpp_includes_.push_back(std::move(path));
   }
@@ -185,8 +143,6 @@ class t_program : public t_node {
    */
   const std::string& path() const { return path_; }
 
-  const std::string& name() const { return name_; }
-
   const std::string& include_prefix() const { return include_prefix_; }
 
   /**
@@ -225,6 +181,10 @@ class t_program : public t_node {
    */
   const std::string& get_namespace(const std::string& language) const;
 
+  std::vector<std::string> gen_namespace_or_default(
+      const std::string& language,
+      std::function<std::vector<std::string>()> gen) const;
+
   /**
    * This creates a new program for every thrift file in an
    * include statement and sets their include_prefix by parsing
@@ -235,7 +195,10 @@ class t_program : public t_node {
    * @param lineno       - The line number of the include statement
    */
   std::unique_ptr<t_program> add_include(
-      std::string path, std::string include_site, int lineno);
+      std::string path,
+      std::string include_site,
+      int lineno,
+      const source_range& range);
 
   void add_include(std::unique_ptr<t_include> include) {
     includes_.push_back(include.get());
@@ -257,28 +220,23 @@ class t_program : public t_node {
    */
   std::string compute_name_from_file_path(std::string path);
 
-  // Add the byte offset, from the beginning of the file, for the next new line
-  // ('\n').
-  void add_line_offset(size_t offset) {
-    assert(line_to_offset_.empty() || offset > line_to_offset_.back());
-    line_to_offset_.push_back(offset);
+  // Helpers for constrcuting program scoped names.
+  std::string scope_name(const std::string& defname) const {
+    return name() + "." + defname;
+  }
+  std::string scope_name(const t_named& owner, const t_named& node) const {
+    return name() + "." + owner.name() + "." + node.name();
+  }
+  std::string scope_name(const t_named& node) const {
+    return scope_name(node.name());
   }
-  /**
-   * Computes the bytes offset for the given line, based on the offsets provied
-   * via `add_line_offset`
-   *
-   * @param line The 1-based line number.
-   * @param line_offset Optional additional 0-based byte offset to add to the
-   * result, iff line offset could be determined.
-   *
-   * @returns The byte offset, or `noffset` if line==0 or no offset data was
-   * provided for the given for line via `add_line_offset`.
-   */
-  size_t get_byte_offset(size_t line, size_t line_offset = 0) const noexcept;
 
  private:
+  t_package package_;
+
   // All the elements owned by this program.
   node_list<t_node> nodes_;
+  node_list<t_named> definitions_;
   node_list<t_templated_type> type_insts_;
 
   /**
@@ -292,24 +250,59 @@ class t_program : public t_node {
   std::vector<t_service*> services_;
   std::vector<t_include*> includes_;
   std::vector<t_interaction*> interactions_;
-  std::vector<t_placeholder_typedef*> placeholder_typedefs_;
   std::vector<t_struct*> objects_; // structs_ + exceptions_
 
   std::string path_; // initialized in ctor init-list
-  std::string name_{compute_name_from_file_path(path_)};
   std::string include_prefix_;
   std::map<std::string, std::string> namespaces_;
   std::vector<std::string> cpp_includes_;
-  std::unique_ptr<t_scope> scope_{new t_scope{}};
-  std::vector<size_t> line_to_offset_{0};
+  std::shared_ptr<t_scope> scope_;
+
+  t_program(std::string path, std::shared_ptr<t_scope> scope)
+      : path_(std::move(path)), scope_(std::move(scope)) {
+    set_name(compute_name_from_file_path(path_));
+  }
 
   // TODO(afuller): Remove everything below this comment. It is only provided
   // for backwards compatibility.
  public:
+  void add_typedef(std::unique_ptr<t_typedef> node) {
+    add_definition(std::move(node));
+  }
+  void add_enum(std::unique_ptr<t_enum> node) {
+    add_definition(std::move(node));
+  }
+  void add_const(std::unique_ptr<t_const> node) {
+    add_definition(std::move(node));
+  }
+  void add_struct(std::unique_ptr<t_struct> node) {
+    add_definition(std::move(node));
+  }
+  void add_exception(std::unique_ptr<t_exception> node) {
+    add_definition(std::move(node));
+  }
+  void add_service(std::unique_ptr<t_service> node) {
+    add_definition(std::move(node));
+  }
+  void add_interaction(std::unique_ptr<t_interaction> node) {
+    add_definition(std::move(node));
+  }
   void add_xception(std::unique_ptr<t_exception> tx) {
     add_exception(std::move(tx));
   }
   const std::vector<t_exception*>& xceptions() const { return exceptions(); }
+
+  // Looks for an annotation on the given node, then if not found, and the node
+  // is not generated, looks for the same annotation on the program.
+  const t_const* inherit_annotation_or_null(
+      const t_named& node, const char* uri) const {
+    if (const t_const* annot = node.find_structured_annotation_or_null(uri)) {
+      return annot;
+    } else if (node.generated()) { // Generated nodes do not inherit.
+      return nullptr;
+    }
+    return find_structured_annotation_or_null(uri);
+  }
 };
 
 } // namespace compiler
