@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -21,6 +21,10 @@
 #include <unordered_set>
 #include <utility>
 
+#include <thrift/compiler/ast/node_list.h>
+#include <thrift/compiler/ast/t_type.h>
+#include <thrift/compiler/ast/t_typedef.h>
+
 namespace apache {
 namespace thrift {
 namespace compiler {
@@ -28,7 +32,6 @@ namespace compiler {
 class t_const;
 class t_interaction;
 class t_service;
-class t_type;
 
 /**
  * This represents a scope used for looking up types, services and other AST
@@ -38,6 +41,10 @@ class t_type;
  */
 class t_scope {
  public:
+  // Get a (poteinally unresolve) reference to given type, declared in the given
+  // program.
+  t_type_ref ref_type(const t_program& program, const std::string& name);
+
   void add_type(std::string name, const t_type* type) {
     types_[std::move(name)] = type;
   }
@@ -78,7 +85,23 @@ class t_scope {
   // Dumps the content of type map to stdout.
   void dump() const;
 
+  t_type_ref add_placeholder_typedef(
+      std::unique_ptr<t_placeholder_typedef> ptd) {
+    assert(ptd != nullptr);
+    auto result = t_type_ref::for_placeholder(*ptd);
+    placeholder_typedefs_.emplace_back(std::move(ptd));
+    return result;
+  }
+  node_list_view<t_placeholder_typedef> placeholder_typedefs() {
+    return placeholder_typedefs_;
+  }
+  node_list_view<const t_placeholder_typedef> placeholder_typedefs() const {
+    return placeholder_typedefs_;
+  }
+
  private:
+  node_list<t_placeholder_typedef> placeholder_typedefs_;
+
   template <typename T>
   static const T* find_or_null(
       const std::unordered_map<std::string, const T*>& map,
