@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -31,14 +31,25 @@ const std::string* t_typedef::get_first_annotation_or_null(
   return result;
 }
 
+const t_const* t_typedef::get_first_structured_annotation_or_null(
+    const t_type* type, const char* uri) {
+  const t_const* result = nullptr;
+  find_type_if(type, [&result, uri](const t_type* type) {
+    return (result = type->find_structured_annotation_or_null(uri)) != nullptr;
+  });
+  return result;
+}
+
 bool t_typedef::is_defined() const {
   return dynamic_cast<const t_placeholder_typedef*>(this) == nullptr;
 }
 
 bool t_placeholder_typedef::resolve() {
-  assert(type_.get_type() == nullptr);
-  type_.set_type(scope_->find_type(program()->name() + "." + name()));
-  return type_.get_type() != nullptr;
+  if (type_.empty()) {
+    type_ = t_type_ref::from_ptr(
+        program()->scope()->find_type(program()->name() + "." + name()));
+  }
+  return !type_.empty();
 }
 
 } // namespace compiler
