@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -20,7 +20,7 @@
 #include <string>
 
 #include <thrift/compiler/ast/alias_span.h>
-#include <thrift/compiler/ast/t_scope.h>
+#include <thrift/compiler/ast/t_const.h>
 #include <thrift/compiler/ast/t_type.h>
 
 namespace apache {
@@ -68,6 +68,11 @@ class t_typedef : public t_type {
         std::forward<D>(default_value));
   }
 
+  // Finds the first matching structured annoation in the typedef's hierarchy.
+  // Return null if not found.
+  static const t_const* get_first_structured_annotation_or_null(
+      const t_type* type, const char* uri);
+
   std::string get_full_name() const override {
     // TODO(afuller): Just return name() as, unlike a t_placeholder_typedef, the
     // full name for a typedef is just it's name, not the name of the
@@ -81,7 +86,7 @@ class t_typedef : public t_type {
   // TODO(afuller): Remove everything below here, as it is just provided for
   // backwards compatibility.
  public:
-  t_typedef(t_program* program, const t_type* type, std::string name, t_scope*)
+  t_typedef(t_program* program, const t_type* type, std::string name, void*)
       : t_typedef(program, std::move(name), t_type_ref::from_req_ptr(type)) {}
 
   const t_type* get_type() const { return type_.get_type(); }
@@ -106,9 +111,8 @@ class t_typedef : public t_type {
 // the order of IDL declarations.
 class t_placeholder_typedef final : public t_typedef {
  public:
-  t_placeholder_typedef(
-      t_program* program, const std::string& name, t_scope* scope)
-      : t_typedef(program, std::move(name), {}), scope_(scope) {}
+  t_placeholder_typedef(t_program* program, std::string name)
+      : t_typedef(program, std::move(name), {}) {}
 
   /**
    * Resolve and find the actual type that the symbolic name refers to.
@@ -117,9 +121,6 @@ class t_placeholder_typedef final : public t_typedef {
   bool resolve();
 
   std::string get_full_name() const override { return type_->get_full_name(); }
-
- private:
-  t_scope* scope_;
 };
 
 } // namespace compiler
