@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -32,14 +32,16 @@
 #include <unistd.h>
 #endif
 #include <ctime>
+#include <fstream>
+#include <iostream>
 
 #include <boost/algorithm/string/join.hpp>
 #include <boost/algorithm/string/split.hpp>
 #include <boost/filesystem.hpp>
 
-#include <thrift/compiler/ast/diagnostic.h>
 #include <thrift/compiler/ast/diagnostic_context.h>
-#include <thrift/compiler/common.h>
+#include <thrift/compiler/ast/t_program_bundle.h>
+#include <thrift/compiler/diagnostic.h>
 #include <thrift/compiler/generate/t_generator.h>
 #include <thrift/compiler/mutator/mutator.h>
 #include <thrift/compiler/parse/parsing_driver.h>
@@ -82,7 +84,9 @@ void usage() {
       "  -I dir      Add a directory to the list of directories\n"
       "              searched for include directives\n");
   fprintf(stderr, "  -nowarn     Suppress all compiler warnings (BAD!)\n");
-  fprintf(stderr, "  -strict     Strict compiler warnings on\n");
+  fprintf(
+      stderr,
+      "  -legacy-strict     Strict compiler warnings on (DEPRECATED)\n");
   fprintf(stderr, "  -v[erbose]  Verbose mode\n");
   fprintf(stderr, "  -r[ecurse]  Also generate included files\n");
   fprintf(stderr, "  -debug      Parse debug trace to stdout\n");
@@ -90,7 +94,9 @@ void usage() {
       stderr,
       "  --allow-neg-keys  Allow negative field keys (Used to preserve protocol\n"
       "                    compatibility with older .thrift files)\n");
-  fprintf(stderr, "  --allow-neg-enum-vals Allow negative enum vals\n");
+  fprintf(
+      stderr,
+      "  --allow-neg-enum-vals Allow negative enum vals (DEPRECATED)\n");
   fprintf(
       stderr,
       "  --allow-64bit-consts  Do not print warnings about using 64-bit constants\n");
@@ -124,6 +130,14 @@ void usage() {
   }
 }
 
+bool isPathSeparator(const char& c) {
+#ifdef _WIN32
+  return c == ';';
+#else
+  return c == ':';
+#endif
+}
+
 bool isComma(const char& c) {
   return c == ',';
 }
@@ -161,7 +175,8 @@ std::string parseArgs(
   };
 
   // Hacky parameter handling... I didn't feel like using a library sorry!
-  bool nowarn = false; // Guard so --nowarn and --strict are order agnostic.
+  bool nowarn =
+      false; // Guard so --nowarn and --legacy-strict are order agnostic.
   for (; arg_i < arguments.size() - 1;
        ++arg_i) { // Last argument is the src file.
     // Parse flag.
@@ -186,24 +201,22 @@ std::string parseArgs(
           pparams.allow_experimental_features, *arg, isComma);
     } else if (flag == "debug") {
       dparams.debug = true;
-      g_debug = 1;
     } else if (flag == "nowarn") {
-      dparams.warn_level = g_warn = 0;
+      dparams.warn_level = 0;
       nowarn = true;
-    } else if (flag == "strict") {
+    } else if (flag == "legacy-strict") {
       pparams.strict = 255;
       if (!nowarn) { // Don't override nowarn.
-        dparams.warn_level = g_warn = 2;
+        dparams.warn_level = 2;
       }
     } else if (flag == "v" || flag == "verbose") {
       dparams.info = true;
-      g_verbose = 1;
     } else if (flag == "r" || flag == "recurse") {
       gparams.gen_recurse = true;
     } else if (flag == "allow-neg-keys") {
       pparams.allow_neg_field_keys = true;
     } else if (flag == "allow-neg-enum-vals") {
-      dparams.allow_neg_enum_vals = true;
+      // no-op
     } else if (flag == "allow-64bit-consts") {
       pparams.allow_64bit_consts = true;
     } else if (flag == "record-genfiles") {
@@ -218,8 +231,6 @@ std::string parseArgs(
         return {};
       }
       gparams.targets.push_back(*arg);
-    } else if (flag == "cpp_use_include_prefix") {
-      g_cpp_use_include_prefix = true;
     } else if (flag == "I") {
       auto* arg = consume_arg("include directory");
       if (arg == nullptr) {
@@ -273,6 +284,13 @@ std::string parseArgs(
     }
   }
 
+  if (const char* env_p = std::getenv("THRIFT_INCLUDE_PATH")) {
+    std::vector<std::string> components;
+    boost::algorithm::split(components, env_p, isPathSeparator);
+    pparams.incl_searchpath.insert(
+        pparams.incl_searchpath.end(), components.begin(), components.end());
+  }
+
   // You gotta generate something!
   if (gparams.targets.empty()) {
     fprintf(stderr, "!!! No output language(s) specified\n\n");
@@ -285,20 +303,19 @@ std::string parseArgs(
   return arguments[arg_i];
 }
 
-/**
- * Generate code
- */
+// Generate code.
 bool generate(
     const gen_params& params,
-    t_program* program,
-    std::set<std::string>& already_generated) {
+    t_program& program,
+    std::set<std::string>& already_generated,
+    diagnostics_engine& diags) {
   // Oooohh, recursive code generation, hot!!
   if (params.gen_recurse) {
     // Add the path we are about to generate.
-    already_generated.emplace(program->path());
-    for (const auto& include : program->get_included_programs()) {
+    already_generated.emplace(program.path());
+    for (const auto& include : program.get_included_programs()) {
       if (!already_generated.count(include->path()) &&
-          !generate(params, include, already_generated)) {
+          !generate(params, *include, already_generated, diags)) {
         return false;
       }
     }
@@ -306,11 +323,11 @@ bool generate(
 
   // Generate code!
   try {
-    pverbose("Program: %s\n", program->path().c_str());
-
-    if (dump_docs) {
-      dump_docstrings(program);
-    }
+    diags.report(
+        source_location(),
+        diagnostic_level::info,
+        "program: {}",
+        program.path());
 
     bool success = true;
     std::ofstream genfile;
@@ -320,8 +337,9 @@ bool generate(
     for (auto target : params.targets) {
       auto pos = target.find(':');
       std::string lang = target.substr(0, pos);
-      auto generator = std::unique_ptr<t_generator>{
-          t_generator_registry::get_generator(program, params.context, target)};
+      auto generator =
+          std::unique_ptr<t_generator>{t_generator_registry::get_generator(
+              &program, params.context, target)};
       if (generator == nullptr) {
         continue;
       }
@@ -329,7 +347,7 @@ bool generate(
       validator::diagnostics_t diagnostics;
       validator_list validators(diagnostics);
       generator->fill_validator_list(validators);
-      validators.traverse(program);
+      validators.traverse(&program);
 
       bool has_failure = false;
       for (const auto& d : diagnostics) {
@@ -341,7 +359,12 @@ bool generate(
         continue;
       }
 
-      pverbose("Generating \"%s\"\n", target.c_str());
+      diags.report(
+          source_location(),
+          diagnostic_level::info,
+          "generating \"{}\"",
+          target);
+
       generator->generate_program();
       if (genfile.is_open()) {
         for (const std::string& s : generator->get_genfiles()) {
@@ -359,9 +382,10 @@ bool generate(
   }
 }
 
-bool generate(const gen_params& params, t_program* program) {
+bool generate(
+    const gen_params& params, t_program& program, diagnostics_engine& diags) {
   std::set<std::string> already_generated;
-  return generate(params, program, already_generated);
+  return generate(params, program, already_generated, diags);
 }
 
 std::string get_include_path(
@@ -400,83 +424,117 @@ std::string get_include_path(
 
 } // namespace
 
-// TODO(urielrivas): Reuse somehow this function in compile(...).
-std::unique_ptr<t_program_bundle> parse_and_get_program(
-    const std::vector<std::string>& arguments) {
-  // Parse arguments.
-  parsing_params pparams{};
-  gen_params gparams{};
-  diagnostic_params dparams{};
-  std::string input_filename = parseArgs(arguments, pparams, gparams, dparams);
+std::unique_ptr<t_program_bundle> parse_and_mutate_program(
+    source_manager& sm,
+    diagnostic_context& ctx,
+    const std::string& filename,
+    parsing_params params) {
+  parsing_driver driver(sm, ctx, filename, std::move(params));
+  auto program = driver.parse();
+  if (program != nullptr) {
+    auto result = standard_mutators()(ctx, *program);
+    if (result.unresolvable_typeref) {
+      // Stop processing if there is unresolvable typeref
+      program = nullptr;
+    }
+  }
+  return program;
+}
 
-  if (input_filename.empty()) {
+std::pair<std::unique_ptr<t_program_bundle>, diagnostic_results>
+parse_and_mutate_program(
+    source_manager& sm,
+    const std::string& filename,
+    parsing_params params,
+    diagnostic_params dparams) {
+  diagnostic_results results;
+  diagnostic_context ctx(sm, results, std::move(dparams));
+  return {
+      parse_and_mutate_program(sm, ctx, filename, std::move(params)), results};
+}
+
+std::unique_ptr<t_program_bundle> parse_and_dump_diagnostics(
+    const std::string& filename,
+    parsing_params pparams,
+    diagnostic_params dparams) {
+  source_manager source_mgr;
+  diagnostic_results results;
+  diagnostic_context ctx(source_mgr, results, std::move(dparams));
+  auto program =
+      parse_and_mutate_program(source_mgr, ctx, filename, std::move(pparams));
+  for (const auto& diag : results.diagnostics()) {
+    std::cerr << diag << "\n";
+  }
+  return program;
+}
+
+std::unique_ptr<t_program_bundle> parse_and_get_program(
+    source_manager& sm, const std::vector<std::string>& arguments) {
+  // Parse arguments.
+  parsing_params pparams;
+  gen_params gparams;
+  diagnostic_params dparams;
+  std::string filename = parseArgs(arguments, pparams, gparams, dparams);
+
+  if (filename.empty()) {
     return {};
   }
-
-  // Parse and mutate it!
-  auto ctx = diagnostic_context::ignore_all();
-  parsing_driver driver{ctx, input_filename, std::move(pparams)};
-  auto program = driver.parse();
-  if (program) {
-    mutator::mutate(program->root_program());
-  }
-
-  return program;
+  auto ctx = diagnostic_context::ignore_all(sm);
+  return parse_and_mutate_program(sm, ctx, filename, std::move(pparams));
 }
 
 compile_result compile(const std::vector<std::string>& arguments) {
   compile_result result;
 
   // Parse arguments.
-  g_stage = "arguments";
-  parsing_params pparams{};
-  gen_params gparams{};
-  diagnostic_params dparams{};
+  parsing_params pparams;
+  gen_params gparams;
+  diagnostic_params dparams;
   std::string input_filename = parseArgs(arguments, pparams, gparams, dparams);
   if (input_filename.empty()) {
     return result;
   }
-  diagnostic_context ctx{result.detail, std::move(dparams)};
+  source_manager source_mgr;
+  gparams.context.set_source_manager(source_mgr);
+  diagnostic_context ctx(source_mgr, result.detail, std::move(dparams));
 
   // Parse it!
-  g_stage = "parse";
-  parsing_driver driver{ctx, input_filename, std::move(pparams)};
-  auto program = driver.parse();
+  auto program = parse_and_mutate_program(
+      source_mgr, ctx, input_filename, std::move(pparams));
   if (!program) {
     return result;
   }
 
-  // Mutate it!
+  // TODO(afuller): Migrate to ast_mutator.
   try {
     mutator::mutate(program->root_program());
-  } catch (MutatorException& e) {
+  } catch (mutator_exception& e) {
     ctx.report(std::move(e.message));
     return result;
   }
-  standard_mutator().mutate(ctx, *program->root_program());
-  if (result.detail.has_failure()) {
-    return result;
-  }
 
   program->root_program()->set_include_prefix(
       get_include_path(gparams.targets, input_filename));
 
   // Validate it!
-  ctx.report_all(validator::validate(program->root_program()));
+  for (auto&& diag : validator::validate(program->root_program())) {
+    ctx.report(std::forward<decltype(diag)>(diag));
+  }
   standard_validator()(ctx, *program->root_program());
   if (result.detail.has_failure()) {
     return result;
   }
 
   // Generate it!
-  g_stage = "generation";
+  ctx.begin_visit(*program->root_program());
   try {
-    if (generate(gparams, program->root_program())) {
+    if (generate(gparams, *program->root_program(), ctx)) {
       result.retcode = compile_retcode::success;
     }
   } catch (const std::exception& e) {
-    ctx.failure(*program->root_program(), e.what());
+    ctx.failure(*program->root_program(), [&](auto& o) { o << e.what(); });
   }
+  ctx.end_visit(*program->root_program());
   return result;
 }
 
