@@ -18,6 +18,7 @@
 
 #include <stdexcept>
 
+#include <fmt/core.h>
 #include <thrift/compiler/ast/t_list.h>
 #include <thrift/compiler/ast/t_map.h>
 #include <thrift/compiler/ast/t_node.h>
@@ -30,54 +31,78 @@ namespace thrift {
 namespace compiler {
 namespace gen {
 namespace cpp {
-namespace {
 
-const std::string* find_structured_adapter_annotation(const t_named* node) {
-  if (const t_const* annotation = node->find_structured_annotation_or_null(
-          "facebook.com/thrift/annotation/cpp/Adapter")) {
-    for (const auto& item : annotation->value()->get_map()) {
-      if (item.first->get_string() == "name") {
-        return &item.second->get_string();
-      }
-    }
-  }
-  return nullptr;
-}
+const std::string& type_resolver::get_type_name(
+    const t_field& field, const t_structured& parent) {
+  const t_type& type = *field.type();
 
-} // namespace
-
-const std::string& type_resolver::get_storage_type_name(const t_field* node) {
-  if (const std::string* adapter = find_structured_adapter_annotation(node)) {
-    const t_type* type = &*node->type();
-    return detail::get_or_gen(adapter_storage_type_cache_, node, [&]() {
-      return gen_type(type, adapter, node->id());
+  // TODO (dokwon): Support composing adapters.
+  if (const std::string* adapter_on_field =
+          find_structured_adapter_annotation(field)) {
+    return detail::get_or_gen(field_type_cache_, &field, [&]() {
+      return gen_field_type(field.id(), type, parent, adapter_on_field);
     });
   }
 
-  auto ref_type = find_ref_type(*node);
-  if (ref_type == reference_type::none) {
-    // The storage type is just the type name.
-    return get_type_name(node->get_type());
+  // If @cpp.Adapter is used on typedef of the field, use the typedef name.
+  if (const auto* typedf = dynamic_cast<const t_typedef*>(&type)) {
+    if (find_structured_adapter_annotation(*typedf)) {
+      return namespaces_.get_namespaced_name(*typedf);
+    }
   }
 
-  auto key = std::make_pair(node->get_type(), ref_type);
-  return detail::get_or_gen(
-      storage_type_cache_, key, [&]() { return gen_storage_type(key); });
+  return get_type_name(type);
 }
 
-const std::string* type_resolver::find_first_adapter(const t_field* field) {
+const std::string& type_resolver::get_type_name(const t_typedef& node) {
+  // When `t_placeholder_typedef` is used, `t_type_ref::deref` will
+  // automatically dereference `t_placeholder_typedef` as well. Since
+  // unstructured annotations are stored in `t_placeholder_typedef`, we can't
+  // use `t_type_ref::deref`.
+  const t_type* type = node.type().get_type();
+  if (type == nullptr) {
+    throw std::runtime_error("t_type_ref has no type.");
+  }
+  if (const std::string* adapter = find_structured_adapter_annotation(node)) {
+    return detail::get_or_gen(
+        type_cache_, &node, [&]() { return gen_type(*type, adapter); });
+  }
+  return get_type_name(*type);
+}
+
+const std::string& type_resolver::get_storage_type_name(
+    const t_field& field, const t_structured& parent) {
+  auto ref_type = find_ref_type(field);
+  if (ref_type == reference_type::none) {
+    // The storage type is just the type name.
+    return get_type_name(field, parent);
+  }
+  return detail::get_or_gen(storage_type_cache_, {&field, ref_type}, [&]() {
+    return gen_storage_type(ref_type, field, parent);
+  });
+}
+
+const std::string* type_resolver::find_first_adapter(const t_type& node) {
+  if (auto annotation = t_typedef::get_first_structured_annotation_or_null(
+          &node, "facebook.com/thrift/annotation/cpp/Adapter")) {
+    return &annotation->get_value_from_structured_annotation("name")
+                .get_string();
+  }
+  return t_typedef::get_first_annotation_or_null(&node, {"cpp.adapter"});
+}
+
+const std::string* type_resolver::find_first_adapter(const t_field& field) {
   if (const std::string* adapter = find_structured_adapter_annotation(field)) {
     return adapter;
   }
-  if (const std::string* adapter =
-          gen::cpp::type_resolver::find_first_adapter(&*field->type())) {
+  if (const std::string* adapter = find_first_adapter(*field.type())) {
     return adapter;
   }
   return nullptr;
 }
 
-bool type_resolver::can_resolve_to_scalar(const t_type* node) {
-  return node->get_true_type()->is_scalar() || find_first_adapter(node) ||
+bool type_resolver::can_resolve_to_scalar(const t_type& node) {
+  return node.get_true_type()->is_scalar() || find_first_adapter(node) ||
       find_first_type(node);
 }
 
@@ -144,31 +169,22 @@ const std::string& type_resolver::default_type(t_base_type::type btype) {
       "unknown base type: " + std::to_string(static_cast<int>(btype)));
 }
 
-std::string type_resolver::gen_type(
-    const t_type* node,
-    const std::string* adapter,
-    boost::optional<int16_t> field_id) {
-  std::string name;
-  // Find the base name.
+std::string type_resolver::gen_type(const t_type& node) {
   if (const auto* type = find_type(node)) {
     // Use the override.
-    name = *type;
-  } else {
-    // Use the unmodified name.
-    name = gen_raw_type_name(node, &type_resolver::get_type_name);
+    return *type;
   }
-
-  // Adapt the base name.
-  return adapter ? gen_adapted_type(*adapter, std::move(name), field_id) : name;
+  // Use the unmodified name.
+  return gen_raw_type_name(node, &type_resolver::get_type_name);
 }
 
-std::string type_resolver::gen_standard_type(const t_type* node) {
+std::string type_resolver::gen_standard_type(const t_type& node) {
   if (const auto* type = find_type(node)) {
     // Return the override.
     return *type;
   }
 
-  if (const auto* ttypedef = dynamic_cast<const t_typedef*>(node)) {
+  if (const auto* ttypedef = dynamic_cast<const t_typedef*>(&node)) {
     // Traverse the typedef.
     // TODO(afuller): Always traverse the adapter. There are some cpp.type and
     // cpp.template annotations that rely on the namespacing of the typedef to
@@ -177,7 +193,7 @@ std::string type_resolver::gen_standard_type(const t_type* node) {
     // adapter requires we do so. However, we should update all annotations to
     // using fully qualified names, then always traverse here.
     if (find_first_adapter(node) != nullptr) {
-      return get_standard_type_name(ttypedef->get_type());
+      return get_standard_type_name(*ttypedef->get_type());
     }
   }
 
@@ -185,11 +201,19 @@ std::string type_resolver::gen_standard_type(const t_type* node) {
 }
 
 std::string type_resolver::gen_storage_type(
-    const std::pair<const t_type*, reference_type>& ref_type) {
-  const std::string& type_name = get_type_name(ref_type.first);
-  switch (ref_type.second) {
+    reference_type& ref_type,
+    const t_field& field,
+    const t_structured& parent) {
+  const std::string& type_name = get_type_name(field, parent);
+  const std::string* cpp_template =
+      field.find_annotation_or_null("cpp.template");
+  switch (ref_type) {
     case reference_type::unique:
-      return detail::gen_template_type("::std::unique_ptr", {type_name});
+      if (cpp_template != nullptr) {
+        return detail::gen_template_type(*cpp_template, {type_name});
+      } else {
+        return detail::gen_template_type("::std::unique_ptr", {type_name});
+      }
     case reference_type::shared_mutable:
       return detail::gen_template_type("::std::shared_ptr", {type_name});
     case reference_type::shared_const:
@@ -204,105 +228,141 @@ std::string type_resolver::gen_storage_type(
 }
 
 std::string type_resolver::gen_raw_type_name(
-    const t_type* node, type_resolve_fn resolve_fn) {
+    const t_type& node, type_resolve_fn resolve_fn) {
   // Base types have fixed type mappings.
-  if (const auto* tbase_type = dynamic_cast<const t_base_type*>(node)) {
+  if (const auto* tbase_type = dynamic_cast<const t_base_type*>(&node)) {
     return default_type(tbase_type->base_type());
   }
 
   // Containers have fixed template mappings.
-  if (const auto* tcontainer = dynamic_cast<const t_container*>(node)) {
-    return gen_container_type(tcontainer, resolve_fn);
+  if (const auto* tcontainer = dynamic_cast<const t_container*>(&node)) {
+    return gen_container_type(*tcontainer, resolve_fn);
   }
 
   // Streaming types have special handling.
-  if (const auto* tstream_res = dynamic_cast<const t_stream_response*>(node)) {
-    return gen_stream_resp_type(tstream_res, resolve_fn);
+  if (const auto* tstream_res = dynamic_cast<const t_stream_response*>(&node)) {
+    return gen_stream_resp_type(*tstream_res, resolve_fn);
   }
-  if (const auto* tsink = dynamic_cast<const t_sink*>((node))) {
-    return gen_sink_type(tsink, resolve_fn);
+  if (const auto* tsink = dynamic_cast<const t_sink*>(&node)) {
+    return gen_sink_type(*tsink, resolve_fn);
   }
 
   // For everything else, just use namespaced name.
-  return namespaces_.gen_namespaced_name(node);
+  return namespaces_.get_namespaced_name(node);
 }
 
 std::string type_resolver::gen_container_type(
-    const t_container* node, type_resolve_fn resolve_fn) {
+    const t_container& node, type_resolve_fn resolve_fn) {
   const auto* val = find_template(node);
   const auto& template_name =
-      val ? *val : default_template(node->container_type());
+      val ? *val : default_template(node.container_type());
 
-  switch (node->container_type()) {
+  switch (node.container_type()) {
     case t_container::type::t_list:
       return detail::gen_template_type(
           template_name,
           {resolve(
-              resolve_fn, static_cast<const t_list*>(node)->get_elem_type())});
+              resolve_fn, *static_cast<const t_list&>(node).get_elem_type())});
     case t_container::type::t_set:
       return detail::gen_template_type(
           template_name,
           {resolve(
-              resolve_fn, static_cast<const t_set*>(node)->get_elem_type())});
+              resolve_fn, *static_cast<const t_set&>(node).get_elem_type())});
     case t_container::type::t_map: {
-      const auto* tmap = static_cast<const t_map*>(node);
+      const auto& tmap = static_cast<const t_map&>(node);
       return detail::gen_template_type(
           template_name,
-          {resolve(resolve_fn, tmap->get_key_type()),
-           resolve(resolve_fn, tmap->get_val_type())});
+          {resolve(resolve_fn, *tmap.get_key_type()),
+           resolve(resolve_fn, *tmap.get_val_type())});
     }
   }
   throw std::runtime_error(
       "unknown container type: " +
-      std::to_string(static_cast<int>(node->container_type())));
+      std::to_string(static_cast<int>(node.container_type())));
 }
 
 std::string type_resolver::gen_stream_resp_type(
-    const t_stream_response* node, type_resolve_fn resolve_fn) {
-  if (node->first_response_type() != boost::none) {
+    const t_stream_response& node, type_resolve_fn resolve_fn) {
+  if (!node.first_response_type().empty()) {
     return detail::gen_template_type(
         "::apache::thrift::ResponseAndServerStream",
-        {resolve(resolve_fn, node->get_first_response_type()),
-         resolve(resolve_fn, node->get_elem_type())});
+        {resolve(resolve_fn, *node.get_first_response_type()),
+         resolve(resolve_fn, *node.get_elem_type())});
   }
   return detail::gen_template_type(
       "::apache::thrift::ServerStream",
-      {resolve(resolve_fn, node->get_elem_type())});
+      {resolve(resolve_fn, *node.get_elem_type())});
 }
 
 std::string type_resolver::gen_sink_type(
-    const t_sink* node, type_resolve_fn resolve_fn) {
-  if (node->first_response_type() != boost::none) {
+    const t_sink& node, type_resolve_fn resolve_fn) {
+  if (!node.first_response_type().empty()) {
     return detail::gen_template_type(
         "::apache::thrift::ResponseAndSinkConsumer",
-        {resolve(resolve_fn, node->get_first_response_type()),
-         resolve(resolve_fn, node->get_sink_type()),
-         resolve(resolve_fn, node->get_final_response_type())});
+        {resolve(resolve_fn, *node.get_first_response_type()),
+         resolve(resolve_fn, *node.get_sink_type()),
+         resolve(resolve_fn, *node.get_final_response_type())});
   }
   return detail::gen_template_type(
       "::apache::thrift::SinkConsumer",
-      {resolve(resolve_fn, node->get_sink_type()),
-       resolve(resolve_fn, node->get_final_response_type())});
+      {resolve(resolve_fn, *node.get_sink_type()),
+       resolve(resolve_fn, *node.get_final_response_type())});
 }
 
 std::string type_resolver::gen_adapted_type(
-    const std::string& adapter,
+    const std::string* adapter, const std::string& standard_type) {
+  return adapter == nullptr ? standard_type
+                            : detail::gen_template_type(
+                                  "::apache::thrift::adapt_detail::adapted_t",
+                                  {*adapter, standard_type});
+}
+std::string type_resolver::gen_adapted_type(
+    const std::string* adapter,
+    int16_t field_id,
     const std::string& standard_type,
-    boost::optional<int16_t> field_id) {
-  if (field_id == boost::none) {
-    return detail::gen_template_type(
-        "::apache::thrift::adapt_detail::adapted_t", {adapter, standard_type});
-  }
-  return detail::gen_template_type(
-      "::apache::thrift::adapt_detail::adapted_field_t",
-      {adapter,
-       std::to_string(*field_id),
-       standard_type,
-       "__fbthrift_cpp2_type"});
+    const t_structured& parent) {
+  return adapter == nullptr
+      ? standard_type
+      : detail::gen_template_type(
+            "::apache::thrift::adapt_detail::adapted_field_t",
+            {
+                *adapter,
+                std::to_string(field_id),
+                standard_type,
+                namespace_resolver::get_cpp_name(parent),
+            });
 }
 
-// TODO(ytj): Support type::adapted, type::cpp_type, and cpp.template
-std::string type_resolver::gen_type_tag(t_type const& original_type) {
+// TODO(ytj): Support type::adapted, and cpp.template
+std::string type_resolver::gen_type_tag(const t_type& type) {
+  auto tag = gen_thrift_type_tag(type);
+  if (const auto* cpp_type = find_first_type(type)) {
+    tag =
+        fmt::format("::apache::thrift::type::cpp_type<{}, {}>", *cpp_type, tag);
+  }
+  if (const auto* cpp_adapter = find_first_adapter(type)) {
+    tag = fmt::format(
+        "::apache::thrift::type::adapted<{}, {}>", *cpp_adapter, tag);
+  }
+  return tag;
+}
+
+std::string type_resolver::gen_type_tag(const t_field& field) {
+  std::string type_tag = gen_type_tag(*field.type());
+  if (const std::string* adapter = find_structured_adapter_annotation(field)) {
+    return fmt::format(
+        "::apache::thrift::type::field_t<::apache::thrift::FieldId{{{}}}, ::apache::thrift::type::adapted<{}, {}>>",
+        field.id(),
+        *adapter,
+        type_tag);
+  }
+  return fmt::format(
+      "::apache::thrift::type::field_t<::apache::thrift::FieldId{{{}}}, {}>",
+      field.id(),
+      type_tag);
+}
+
+std::string type_resolver::gen_thrift_type_tag(const t_type& original_type) {
   static const std::string ns = "::apache::thrift::type::";
   auto const& type = *original_type.get_true_type();
   if (type.is_void()) {
@@ -322,7 +382,7 @@ std::string type_resolver::gen_type_tag(t_type const& original_type) {
   } else if (type.is_double()) {
     return ns + "double_t";
   } else if (type.is_enum()) {
-    return ns + "enum_t<" + get_type_name(&type) + ">";
+    return ns + "enum_t<" + get_standard_type_name(type) + ">";
   } else if (type.is_string()) {
     return ns + "string_t";
   } else if (type.is_binary()) {
@@ -345,11 +405,11 @@ std::string type_resolver::gen_type_tag(t_type const& original_type) {
     auto val_tag = gen_type_tag(val);
     return ns + "map<" + key_tag + ", " + val_tag + ">";
   } else if (type.is_union()) {
-    return ns + "union_t<" + get_type_name(&type) + ">";
+    return ns + "union_t<" + get_standard_type_name(type) + ">";
   } else if (type.is_struct()) {
-    return ns + "struct_t<" + get_type_name(&type) + ">";
+    return ns + "struct_t<" + get_standard_type_name(type) + ">";
   } else if (type.is_exception()) {
-    return ns + "exception_t<" + get_type_name(&type) + ">";
+    return ns + "exception_t<" + get_standard_type_name(type) + ">";
   } else {
     throw std::runtime_error("unknown type for: " + type.get_full_name());
   }
