@@ -15,6 +15,7 @@
  */
 
 #include <thrift/compiler/gen/cpp/type_resolver.h>
+#include <thrift/compiler/gen/testing.h>
 
 #include <memory>
 #include <stdexcept>
@@ -32,33 +33,40 @@ namespace {
 
 class TypeResolverTest : public ::testing::Test {
  public:
-  TypeResolverTest() noexcept : program_("path/to/program.thrift") {
+  TypeResolverTest() noexcept
+      : program_("path/to/program.thrift"), struct_(&program_, "ThriftStruct") {
     program_.set_namespace("cpp2", "path.to");
   }
 
   bool can_resolve_to_scalar(const t_type& node) {
-    return resolver_.can_resolve_to_scalar(&node);
+    return resolver_.can_resolve_to_scalar(node);
   }
 
   const std::string& get_type_name(const t_type& node) {
-    return resolver_.get_type_name(&node);
+    return resolver_.get_type_name(node);
+  }
+
+  const std::string& get_type_name(const t_field& node) {
+    return resolver_.get_type_name(node, struct_);
   }
 
   const std::string& get_standard_type_name(const t_type& node) {
-    return resolver_.get_standard_type_name(&node);
+    return resolver_.get_standard_type_name(node);
   }
 
   const std::string& get_storage_type_name(const t_field& node) {
-    return resolver_.get_storage_type_name(&node);
+    return resolver_.get_storage_type_name(node, struct_);
   }
 
-  std::string gen_type_tag(const t_type& type) {
-    return resolver_.gen_type_tag(type);
+  std::string get_type_tag(const t_type& type) {
+    return resolver_.get_type_tag(type);
   }
 
+  type_resolver resolver_;
+
  protected:
-  type_resolver resolver_;
   t_program program_;
+  t_struct struct_;
 };
 
 TEST_F(TypeResolverTest, BaseTypes) {
@@ -85,25 +93,25 @@ TEST_F(TypeResolverTest, BaseTypes) {
   EXPECT_FALSE(can_resolve_to_scalar(t_base_type::t_binary()));
 
   EXPECT_EQ(
-      gen_type_tag(t_base_type::t_void()), "::apache::thrift::type::void_t");
+      get_type_tag(t_base_type::t_void()), "::apache::thrift::type::void_t");
   EXPECT_EQ(
-      gen_type_tag(t_base_type::t_byte()), "::apache::thrift::type::byte_t");
+      get_type_tag(t_base_type::t_byte()), "::apache::thrift::type::byte_t");
   EXPECT_EQ(
-      gen_type_tag(t_base_type::t_string()),
+      get_type_tag(t_base_type::t_string()),
       "::apache::thrift::type::string_t");
   EXPECT_EQ(
-      gen_type_tag(t_base_type::t_binary()),
+      get_type_tag(t_base_type::t_binary()),
       "::apache::thrift::type::binary_t");
   EXPECT_EQ(
-      gen_type_tag(t_base_type::t_i16()), "::apache::thrift::type::i16_t");
+      get_type_tag(t_base_type::t_i16()), "::apache::thrift::type::i16_t");
   EXPECT_EQ(
-      gen_type_tag(t_base_type::t_i32()), "::apache::thrift::type::i32_t");
+      get_type_tag(t_base_type::t_i32()), "::apache::thrift::type::i32_t");
   EXPECT_EQ(
-      gen_type_tag(t_base_type::t_i64()), "::apache::thrift::type::i64_t");
+      get_type_tag(t_base_type::t_i64()), "::apache::thrift::type::i64_t");
   EXPECT_EQ(
-      gen_type_tag(t_base_type::t_float()), "::apache::thrift::type::float_t");
+      get_type_tag(t_base_type::t_float()), "::apache::thrift::type::float_t");
   EXPECT_EQ(
-      gen_type_tag(t_base_type::t_double()),
+      get_type_tag(t_base_type::t_double()),
       "::apache::thrift::type::double_t");
 }
 
@@ -271,6 +279,8 @@ TEST_F(TypeResolverTest, TypeDefs_Adapter) {
       get_type_name(*ttypedef1.type()),
       "::apache::thrift::adapt_detail::adapted_t<HashAdapter, uint64_t>");
   EXPECT_TRUE(can_resolve_to_scalar(ttypedef1));
+  ASSERT_TRUE(resolver_.find_first_adapter(ttypedef1));
+  EXPECT_EQ(*resolver_.find_first_adapter(ttypedef1), "HashAdapter");
 
   // Type defs can also be adapted.
   t_typedef ttypedef2(ttypedef1);
@@ -280,6 +290,17 @@ TEST_F(TypeResolverTest, TypeDefs_Adapter) {
       get_type_name(ttypedef2),
       "::apache::thrift::adapt_detail::adapted_t<TypeDefAdapter, ::path::to::MyHash>");
   EXPECT_TRUE(can_resolve_to_scalar(ttypedef2));
+  ASSERT_TRUE(resolver_.find_first_adapter(ttypedef2));
+  EXPECT_EQ(*resolver_.find_first_adapter(ttypedef2), "TypeDefAdapter");
+
+  // Structured annotation
+  t_base_type booll(t_base_type::t_bool());
+  t_typedef typedef1(&program_, "MyBool", booll);
+  typedef1.add_structured_annotation(
+      adapter_builder(program_, "cpp").make("MyAdapter"));
+  t_typedef typedef2(&program_, "DoubleBool", typedef1);
+  ASSERT_TRUE(resolver_.find_first_adapter(typedef2));
+  EXPECT_EQ(*resolver_.find_first_adapter(typedef2), "MyAdapter");
 }
 
 TEST_F(TypeResolverTest, CustomType) {
@@ -379,7 +400,6 @@ TEST_F(TypeResolverTest, StorageType) {
   t_base_type ui64(t_base_type::t_i64());
   ui64.set_annotation("cpp.type", "uint64_t");
   ui64.set_annotation("cpp.adapter", "HashAdapter");
-
   {
     t_field ui64_field(ui64, "hash", 1);
     EXPECT_EQ(
@@ -434,6 +454,23 @@ TEST_F(TypeResolverTest, StorageType) {
         "::std::shared_ptr<const ::apache::thrift::adapt_detail::adapted_t<HashAdapter, uint64_t>>");
   }
 
+  {
+    t_field ui64_field(ui64, "hash", 1);
+    ui64_field.set_annotation("thrift.box", "");
+    EXPECT_EQ(
+        get_storage_type_name(ui64_field),
+        "::apache::thrift::detail::boxed_value_ptr<::apache::thrift::adapt_detail::adapted_t<HashAdapter, uint64_t>>");
+  }
+
+  {
+    t_field ui64_field(ui64, "hash", 1);
+    ui64_field.add_structured_annotation(
+        thrift_annotation_builder::box(program_).make());
+    EXPECT_EQ(
+        get_storage_type_name(ui64_field),
+        "::apache::thrift::detail::boxed_value_ptr<::apache::thrift::adapt_detail::adapted_t<HashAdapter, uint64_t>>");
+  }
+
   { // Unrecognized throws an exception.
     t_field ui64_field(ui64, "hash", 1);
     ui64_field.set_annotation("cpp.ref_type", "blah");
@@ -559,50 +596,129 @@ TEST_F(TypeResolverTest, Typedef_cpptype_and_adapter) {
   EXPECT_TRUE(can_resolve_to_scalar(*field2.type()));
 }
 
-std::unique_ptr<t_const_value> make_string(const char* value) {
-  auto name = std::make_unique<t_const_value>();
-  name->set_string(value);
-  return name;
-}
-
-struct adapter_builder {
-  t_program* program;
-  t_struct type;
-
-  explicit adapter_builder(t_program* p) : program(p), type(p, "Adapter") {
-    type.set_annotation(
-        "thrift.uri", "facebook.com/thrift/annotation/cpp/Adapter");
-  }
-
-  std::unique_ptr<t_const> make() & {
-    auto map = std::make_unique<t_const_value>();
-    map->set_map();
-    map->add_map(make_string("name"), make_string("MyAdapter"));
-    map->set_ttype(t_type_ref::from_ptr(&type));
-    return std::make_unique<t_const>(program, &type, "", std::move(map));
-  }
-};
-
 TEST_F(TypeResolverTest, AdaptedFieldType) {
   auto i64 = t_base_type::t_i64();
   auto field = t_field(i64, "n", 42);
-  auto adapter = adapter_builder(&program_);
-  field.add_structured_annotation(adapter.make());
+  field.add_structured_annotation(
+      adapter_builder(program_, "cpp").make("MyAdapter"));
+  EXPECT_EQ(
+      get_type_name(field),
+      "::apache::thrift::adapt_detail::adapted_field_t<"
+      "MyAdapter, 42, ::std::int64_t, ThriftStruct>");
   EXPECT_EQ(
       get_storage_type_name(field),
       "::apache::thrift::adapt_detail::adapted_field_t<"
-      "MyAdapter, 42, ::std::int64_t, __fbthrift_cpp2_type>");
+      "MyAdapter, 42, ::std::int64_t, ThriftStruct>");
+}
+
+TEST_F(TypeResolverTest, AdaptedFieldStorageType) {
+  auto i64 = t_base_type::t_i64();
+  auto adapter = adapter_builder(program_, "cpp");
+  {
+    auto field = t_field(i64, "n", 42);
+    field.add_structured_annotation(adapter.make("MyAdapter"));
+    EXPECT_EQ(
+        get_storage_type_name(field),
+        "::apache::thrift::adapt_detail::adapted_field_t<"
+        "MyAdapter, 42, ::std::int64_t, ThriftStruct>");
+  }
+
+  {
+    auto field = t_field(i64, "n", 42);
+    field.add_structured_annotation(adapter.make("MyAdapter"));
+    field.set_annotation("cpp.ref", "");
+    EXPECT_EQ(
+        get_storage_type_name(field),
+        "::std::unique_ptr<::apache::thrift::adapt_detail::adapted_field_t<"
+        "MyAdapter, 42, ::std::int64_t, ThriftStruct>>");
+  }
+  {
+    auto field = t_field(i64, "n", 42);
+    field.add_structured_annotation(adapter.make("MyAdapter"));
+    field.set_annotation("cpp2.ref", ""); // Works with cpp2.
+    EXPECT_EQ(
+        get_storage_type_name(field),
+        "::std::unique_ptr<::apache::thrift::adapt_detail::adapted_field_t<"
+        "MyAdapter, 42, ::std::int64_t, ThriftStruct>>");
+  }
+
+  {
+    auto field = t_field(i64, "n", 42);
+    field.add_structured_annotation(adapter.make("MyAdapter"));
+    field.set_annotation("cpp.ref_type", "unique");
+    EXPECT_EQ(
+        get_storage_type_name(field),
+        "::std::unique_ptr<::apache::thrift::adapt_detail::adapted_field_t<"
+        "MyAdapter, 42, ::std::int64_t, ThriftStruct>>");
+  }
+
+  {
+    auto field = t_field(i64, "n", 42);
+    field.add_structured_annotation(adapter.make("MyAdapter"));
+    field.set_annotation("cpp.ref_type", "shared");
+    EXPECT_EQ(
+        get_storage_type_name(field),
+        "::std::shared_ptr<::apache::thrift::adapt_detail::adapted_field_t<"
+        "MyAdapter, 42, ::std::int64_t, ThriftStruct>>");
+  }
+
+  {
+    auto field = t_field(i64, "n", 42);
+    field.add_structured_annotation(adapter.make("MyAdapter"));
+    field.set_annotation("cpp.ref_type", "shared_mutable");
+    EXPECT_EQ(
+        get_storage_type_name(field),
+        "::std::shared_ptr<::apache::thrift::adapt_detail::adapted_field_t<"
+        "MyAdapter, 42, ::std::int64_t, ThriftStruct>>");
+  }
+
+  {
+    auto field = t_field(i64, "n", 42);
+    field.add_structured_annotation(adapter.make("MyAdapter"));
+    field.set_annotation("cpp.ref_type", "shared_const");
+    EXPECT_EQ(
+        get_storage_type_name(field),
+        "::std::shared_ptr<const ::apache::thrift::adapt_detail::adapted_field_t<"
+        "MyAdapter, 42, ::std::int64_t, ThriftStruct>>");
+  }
+
+  {
+    auto field = t_field(i64, "n", 42);
+    field.add_structured_annotation(adapter.make("MyAdapter"));
+    field.set_annotation("thrift.box", "");
+    EXPECT_EQ(
+        get_storage_type_name(field),
+        "::apache::thrift::detail::boxed_value_ptr<::apache::thrift::adapt_detail::adapted_field_t<"
+        "MyAdapter, 42, ::std::int64_t, ThriftStruct>>");
+  }
+
+  {
+    auto field = t_field(i64, "n", 42);
+    field.add_structured_annotation(adapter.make("MyAdapter"));
+    field.add_structured_annotation(
+        thrift_annotation_builder::box(program_).make());
+    EXPECT_EQ(
+        get_storage_type_name(field),
+        "::apache::thrift::detail::boxed_value_ptr<::apache::thrift::adapt_detail::adapted_field_t<"
+        "MyAdapter, 42, ::std::int64_t, ThriftStruct>>");
+  }
+
+  { // Unrecognized throws an exception.
+    auto field = t_field(i64, "n", 42);
+    field.add_structured_annotation(adapter.make("MyAdapter"));
+    field.set_annotation("cpp.ref_type", "blah");
+    EXPECT_THROW(get_storage_type_name(field), std::runtime_error);
+  }
 }
 
 TEST_F(TypeResolverTest, TransitivelyAdaptedFieldType) {
   auto annotation = t_struct(nullptr, "MyAnnotation");
 
-  auto adapter = adapter_builder(&program_);
-  annotation.add_structured_annotation(adapter.make());
+  annotation.add_structured_annotation(
+      adapter_builder(program_, "cpp").make("MyAdapter"));
 
   auto transitive = t_struct(nullptr, "Transitive");
-  transitive.set_annotation(
-      "thrift.uri", "facebook.com/thrift/annotation/Transitive");
+  transitive.set_uri("facebook.com/thrift/annotation/Transitive");
   annotation.add_structured_annotation(
       std::make_unique<t_const>(&program_, &transitive, "", nullptr));
 
@@ -613,7 +729,7 @@ TEST_F(TypeResolverTest, TransitivelyAdaptedFieldType) {
   EXPECT_EQ(
       get_storage_type_name(field1),
       "::apache::thrift::adapt_detail::adapted_field_t<"
-      "MyAdapter, 1, ::std::int64_t, __fbthrift_cpp2_type>");
+      "MyAdapter, 1, ::std::int64_t, ThriftStruct>");
 
   auto field2 = t_field(i64, "field2", 42);
   field2.add_structured_annotation(
@@ -621,7 +737,7 @@ TEST_F(TypeResolverTest, TransitivelyAdaptedFieldType) {
   EXPECT_EQ(
       get_storage_type_name(field2),
       "::apache::thrift::adapt_detail::adapted_field_t<"
-      "MyAdapter, 42, ::std::int64_t, __fbthrift_cpp2_type>");
+      "MyAdapter, 42, ::std::int64_t, ThriftStruct>");
 }
 
 TEST_F(TypeResolverTest, GenTypeTagContainer) {
@@ -631,13 +747,13 @@ TEST_F(TypeResolverTest, GenTypeTagContainer) {
   t_set i32_set(i32);
   t_map i32_i16_map(i32, i16);
   EXPECT_EQ(
-      gen_type_tag(i32_list),
+      get_type_tag(i32_list),
       "::apache::thrift::type::list<::apache::thrift::type::i32_t>");
   EXPECT_EQ(
-      gen_type_tag(i32_set),
+      get_type_tag(i32_set),
       "::apache::thrift::type::set<::apache::thrift::type::i32_t>");
   EXPECT_EQ(
-      gen_type_tag(i32_i16_map),
+      get_type_tag(i32_i16_map),
       "::apache::thrift::type::map<::apache::thrift::type::i32_t, ::apache::thrift::type::i16_t>");
 }
 
@@ -647,11 +763,11 @@ TEST_F(TypeResolverTest, GenTypeTagStruct) {
   t_union u(&p, "union_name");
   t_exception e(&p, "exception_name");
   EXPECT_EQ(
-      gen_type_tag(s), "::apache::thrift::type::struct_t<::cpp2::struct_name>");
+      get_type_tag(s), "::apache::thrift::type::struct_t<::cpp2::struct_name>");
   EXPECT_EQ(
-      gen_type_tag(u), "::apache::thrift::type::union_t<::cpp2::union_name>");
+      get_type_tag(u), "::apache::thrift::type::union_t<::cpp2::union_name>");
   EXPECT_EQ(
-      gen_type_tag(e),
+      get_type_tag(e),
       "::apache::thrift::type::exception_t<::cpp2::exception_name>");
 }
 
