@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -101,6 +101,24 @@ std::unordered_set<const t_type*> collect_types(const t_struct* strct) {
   return types;
 }
 
+const char* kNoLegacyUri = "facebook.com/thrift/annotation/NoLegacy";
+
+bool generate_legacy_api(const t_program& p) {
+  return p.find_structured_annotation_or_null(kNoLegacyUri) == nullptr;
+}
+
+bool generate_legacy_api(const t_struct& s) {
+  return s.program()->inherit_annotation_or_null(s, kNoLegacyUri) == nullptr;
+}
+
+bool generate_legacy_api(const t_enum& e) {
+  return e.program()->inherit_annotation_or_null(e, kNoLegacyUri) == nullptr;
+}
+
+bool generate_legacy_api(const t_service& s) {
+  return s.program()->inherit_annotation_or_null(s, kNoLegacyUri) == nullptr;
+}
+
 } // namespace compiler
 } // namespace thrift
 } // namespace apache
