@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -179,6 +179,11 @@ class t_concat_generator : public t_generator {
    */
   int get_indent() const { return indent_; }
 
+  /**
+   * Set current indentation level.
+   */
+  void set_indent(int indent) { indent_ = indent; }
+
   /**
    * Indentation level modifiers
    */
