@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,24 +16,29 @@
 
 #include <thrift/compiler/generate/t_generator.h>
 
+#include <stdexcept>
 #include <utility>
 
 #include <boost/algorithm/string/split.hpp>
-#include <thrift/compiler/common.h>
+#include <boost/filesystem.hpp>
+#include <fmt/core.h>
 
 namespace apache {
 namespace thrift {
 namespace compiler {
 
 t_generation_context::t_generation_context(
-    std::string out_path, bool is_out_path_absolute)
+    std::string out_path, bool is_out_path_absolute, source_manager* sm)
     : out_path_(std::move(out_path)),
-      is_out_path_absolute_(is_out_path_absolute) {
+      is_out_path_absolute_(is_out_path_absolute),
+      source_mgr_(sm) {
+  boost::filesystem::path path = {out_path_};
   if (!out_path_.empty()) {
     if (!(out_path_.back() == '/' || out_path_.back() == '\\')) {
-      out_path_.push_back('/');
+      path += boost::filesystem::path::preferred_separator;
     }
   }
+  out_path_ = path.make_preferred().string();
 }
 
 t_generator::t_generator(t_program* program, t_generation_context context)
@@ -42,13 +47,11 @@ t_generator::t_generator(t_program* program, t_generation_context context)
 }
 
 void t_generator_registry::register_generator(t_generator_factory* factory) {
-  gen_map_t& the_map = get_generator_map();
-  if (the_map.find(factory->get_short_name()) != the_map.end()) {
-    failure(
-        "Duplicate generators for language \"%s\"!\n",
-        factory->get_short_name().c_str());
+  std::string name = factory->get_short_name();
+  if (!get_generator_map().insert(std::make_pair(name, factory)).second) {
+    throw std::logic_error(
+        fmt::format("duplicate generator for language \"{}\"", name));
   }
-  the_map[factory->get_short_name()] = factory;
 }
 
 t_generator* t_generator_registry::get_generator(
