@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -25,7 +25,6 @@
 
 #include <thrift/compiler/detail/mustache/mstch.h>
 
-#include <thrift/compiler/common.h>
 #include <thrift/compiler/filesystem.h>
 #include <thrift/compiler/generate/t_generator.h>
 #include <thrift/compiler/generate/templates.h>
@@ -132,6 +131,7 @@ mstch::map t_mstch_generator::dump(const t_field& field, int32_t index) {
       {"required?", req == t_field::e_req::required},
       {"optional?", req == t_field::e_req::optional},
       {"opt_in_req_out?", req == t_field::e_req::opt_in_req_out},
+      {"terse?", req == t_field::e_req::terse},
       {"annotations", dump_elems(field.annotations())},
   };
 
