@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -39,6 +39,10 @@ void mstch_base::register_has_option(std::string key, std::string option) {
       });
 }
 
+mstch::node mstch_base::is_struct() {
+  return dynamic_cast<mstch_struct*>(this) != nullptr;
+}
+
 std::shared_ptr<mstch_base> enum_value_generator::generate(
     t_enum_value const* enum_value,
     std::shared_ptr<mstch_generators const> generators,
@@ -296,11 +300,11 @@ mstch::node mstch_type::get_sink_final_reponse_type() {
 
 mstch::node mstch_type::get_sink_first_response_type() {
   if (type_->is_sink()) {
-    return generators_->type_generator_->generate(
-        dynamic_cast<const t_sink*>(type_)->get_first_response_type(),
-        generators_,
-        cache_,
-        pos_);
+    if (const auto sinkresponse =
+            dynamic_cast<const t_sink*>(type_)->get_first_response_type()) {
+      return generators_->type_generator_->generate(
+          sinkresponse, generators_, cache_, pos_);
+    }
   }
   return mstch::node();
 }
@@ -318,12 +322,12 @@ mstch::node mstch_type::get_stream_elem_type() {
 
 mstch::node mstch_type::get_stream_first_response_type() {
   if (resolved_type_->is_streamresponse()) {
-    return generators_->type_generator_->generate(
-        dynamic_cast<const t_stream_response*>(resolved_type_)
-            ->get_first_response_type(),
-        generators_,
-        cache_,
-        pos_);
+    if (const auto streamresponse =
+            dynamic_cast<const t_stream_response*>(resolved_type_)
+                ->get_first_response_type()) {
+      return generators_->type_generator_->generate(
+          streamresponse, generators_, cache_, pos_);
+    }
   }
   return mstch::node();
 }
@@ -470,7 +474,7 @@ mstch::node mstch_const_value::is_const_struct() {
   if (!const_value_->ttype()) {
     return false;
   }
-  const auto* type = const_value_->ttype()->deref().get_true_type();
+  const auto* type = const_value_->ttype()->get_true_type();
   return type->is_struct() || type->is_xception();
 }
 
@@ -479,7 +483,7 @@ mstch::node mstch_const_value::const_struct_type() {
     return {};
   }
 
-  const auto* type = const_value_->ttype()->deref().get_true_type();
+  const auto* type = const_value_->ttype()->get_true_type();
   if (type->is_struct() || type->is_xception()) {
     return generators_->type_generator_->generate(type, generators_, cache_);
   }
@@ -492,7 +496,7 @@ mstch::node mstch_const_value::const_struct() {
   std::vector<const t_field*> fields;
   mstch::array a;
 
-  const auto* type = const_value_->ttype()->deref().get_true_type();
+  const auto* type = const_value_->ttype()->get_true_type();
   if (type->is_struct() || type->is_xception()) {
     auto const* strct = dynamic_cast<t_struct const*>(type);
     for (auto member : const_value_->get_map()) {
@@ -536,10 +540,6 @@ mstch::node mstch_struct::fields() {
   return generate_fields(strct_->get_members());
 }
 
-mstch::node mstch_struct::thrift_uri() {
-  return strct_->get_annotation("thrift.uri");
-}
-
 mstch::node mstch_struct::exception_safety() {
   if (!strct_->is_xception()) {
     return std::string("");
@@ -594,6 +594,24 @@ mstch::node mstch_struct::exception_kind() {
   }
 }
 
+const std::vector<const t_field*>& mstch_struct::get_members_in_key_order() {
+  if (strct_->fields().size() == fields_in_key_order_.size()) {
+    // Already reordered.
+    return fields_in_key_order_;
+  }
+
+  fields_in_key_order_ = strct_->fields().copy();
+  // Sort by increasing key.
+  std::sort(
+      fields_in_key_order_.begin(),
+      fields_in_key_order_.end(),
+      [](const auto* lhs, const auto* rhs) {
+        return lhs->get_key() < rhs->get_key();
+      });
+
+  return fields_in_key_order_;
+}
+
 mstch::node mstch_function::return_type() {
   return generators_->type_generator_->generate(
       function_->get_returntype(), generators_, cache_, pos_);
@@ -673,7 +691,7 @@ mstch::node mstch_const::program() {
 
 mstch::node mstch_program::has_thrift_uris() {
   for (const auto& strct : program_->structs()) {
-    if (strct->has_annotation("thrift.uri")) {
+    if (!strct->uri().empty()) {
       return true;
     }
   }
