@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -19,6 +19,7 @@
 #include <iomanip>
 #include <map>
 #include <memory>
+#include <set>
 #include <sstream>
 #include <string>
 #include <type_traits>
@@ -117,8 +118,8 @@ class type_generator {
 
 struct field_generator_context {
   const t_struct* strct = nullptr;
-  const t_field* prev = nullptr;
-  const t_field* next = nullptr;
+  const t_field* serialization_prev = nullptr;
+  const t_field* serialization_next = nullptr;
   int isset_index = -1;
 };
 
@@ -211,6 +212,19 @@ class service_generator {
   }
 };
 
+class interaction_generator {
+ public:
+  interaction_generator() = default;
+  virtual ~interaction_generator() = default;
+  virtual std::shared_ptr<mstch_base> generate(
+      t_interaction const* interaction,
+      std::shared_ptr<mstch_generators const> generators,
+      std::shared_ptr<mstch_cache> cache,
+      ELEMENT_POSITION pos,
+      int32_t index,
+      t_service const* containing_service) const = 0;
+};
+
 class typedef_generator {
  public:
   typedef_generator() = default;
@@ -320,6 +334,10 @@ class mstch_generators {
     service_generator_ = std::move(g);
   }
 
+  void set_interaction_generator(std::unique_ptr<interaction_generator> g) {
+    interaction_generator_ = std::move(g);
+  }
+
   void set_typedef_generator(std::unique_ptr<typedef_generator> g) {
     typedef_generator_ = std::move(g);
   }
@@ -343,6 +361,7 @@ class mstch_generators {
   std::unique_ptr<struct_generator> struct_generator_;
   std::unique_ptr<function_generator> function_generator_;
   std::unique_ptr<service_generator> service_generator_;
+  std::unique_ptr<interaction_generator> interaction_generator_;
   std::unique_ptr<typedef_generator> typedef_generator_;
   std::unique_ptr<const_generator> const_generator_;
   std::unique_ptr<program_generator> program_generator_;
@@ -381,6 +400,7 @@ class mstch_base : public mstch::object {
         {
             {"first?", &mstch_base::first},
             {"last?", &mstch_base::last},
+            {"is_struct?", &mstch_base::is_struct},
         });
   }
   virtual ~mstch_base() = default;
@@ -393,6 +413,7 @@ class mstch_base : public mstch::object {
     return pos_ == ELEMENT_POSITION::LAST ||
         pos_ == ELEMENT_POSITION::FIRST_AND_LAST;
   }
+  mstch::node is_struct();
 
   mstch::node annotations(t_named const* annotated) {
     return generate_annotations(annotated->annotations());
@@ -440,6 +461,22 @@ class mstch_base : public mstch::object {
         container, generators_->service_generator_.get(), args...);
   }
 
+  template <typename C, typename... Args>
+  mstch::array generate_interactions(
+      C const& container,
+      t_service const* containing_service,
+      Args const&... args) {
+    if (generators_->interaction_generator_) {
+      return generate_elements(
+          container,
+          generators_->interaction_generator_.get(),
+          containing_service,
+          args...);
+    }
+    return generate_elements(
+        container, generators_->service_generator_.get(), args...);
+  }
+
   template <typename C, typename... Args>
   mstch::array generate_annotations(C const& container, Args const&... args) {
     return generate_elements(
@@ -824,8 +861,9 @@ class mstch_type : public mstch_base {
   }
   mstch::node stream_has_first_response() {
     return resolved_type_->is_streamresponse() &&
-        dynamic_cast<const t_stream_response*>(resolved_type_)
-            ->first_response_type() != boost::none;
+        !dynamic_cast<const t_stream_response*>(resolved_type_)
+             ->first_response_type()
+             .empty();
   }
   mstch::node is_service() { return resolved_type_->is_service(); }
   mstch::node is_base() { return resolved_type_->is_base_type(); }
@@ -877,6 +915,7 @@ class mstch_field : public mstch_base {
             {"field:type", &mstch_field::type},
             {"field:index", &mstch_field::index},
             {"field:required?", &mstch_field::is_required},
+            {"field:terse?", &mstch_field::is_terse},
             {"field:optional?", &mstch_field::is_optional},
             {"field:opt_in_req_out?", &mstch_field::is_optInReqOut},
             {"field:annotations", &mstch_field::annotations},
@@ -889,6 +928,9 @@ class mstch_field : public mstch_base {
   mstch::node value();
   mstch::node type();
   mstch::node index() { return std::to_string(index_); }
+  mstch::node is_terse() {
+    return field_->qualifier() == t_field_qualifier::terse;
+  }
   mstch::node is_required() {
     return field_->get_req() == t_field::e_req::required;
   }
@@ -988,6 +1030,8 @@ class mstch_struct : public mstch_base {
             {"struct:name", &mstch_struct::name},
             {"struct:fields?", &mstch_struct::has_fields},
             {"struct:fields", &mstch_struct::fields},
+            {"struct:fields_in_serialization_order",
+             &mstch_struct::fields_in_serialization_order},
             {"struct:exception?", &mstch_struct::is_exception},
             {"struct:union?", &mstch_struct::is_union},
             {"struct:plain?", &mstch_struct::is_plain},
@@ -1003,15 +1047,20 @@ class mstch_struct : public mstch_base {
     // Populate field_context_generator for each field.
     auto ctx = field_generator_context{};
     ctx.strct = strct_;
-    auto fields = strct->fields();
-    for (auto it = fields.begin(); it != fields.end(); it++) {
-      const auto* field = &*it;
-      if (cpp2::field_has_isset(field)) {
+    for (const t_field& field : strct->fields()) {
+      if (cpp2::field_has_isset(&field)) {
         ctx.isset_index++;
       }
-      ctx.next = (it + 1) != fields.end() ? &*(it + 1) : nullptr;
-      context_map[field] = ctx;
-      ctx.prev = field;
+      context_map[&field] = ctx;
+    }
+
+    const t_field* prev = nullptr;
+    for (const t_field* curr : get_members_in_serialization_order()) {
+      if (prev) {
+        context_map[prev].serialization_next = curr;
+        context_map[curr].serialization_prev = prev;
+      }
+      prev = curr;
     }
   }
   mstch::node name() { return strct_->get_name(); }
@@ -1023,10 +1072,10 @@ class mstch_struct : public mstch_base {
     return !strct_->is_xception() && !strct_->is_union();
   }
   mstch::node annotations() { return mstch_base::annotations(strct_); }
-  mstch::node thrift_uri();
   mstch::node structured_annotations() {
     return mstch_base::structured_annotations(strct_);
   }
+  mstch::node thrift_uri() { return strct_->uri(); }
 
   mstch::node exception_safety();
 
@@ -1046,6 +1095,22 @@ class mstch_struct : public mstch_base {
     return a;
   }
 
+  // Returns the struct members ordered by the key.
+  const std::vector<const t_field*>& get_members_in_key_order();
+
+  field_range get_members_in_serialization_order() {
+    if (strct_->find_structured_annotation_or_null(
+            "facebook.com/thrift/annotation/SerializeInFieldIdOrder")) {
+      return get_members_in_key_order();
+    }
+
+    return strct_->get_members();
+  }
+
+  mstch::node fields_in_serialization_order() {
+    return generate_fields(get_members_in_serialization_order());
+  }
+
  protected:
   t_struct const* strct_;
   // Although mstch_fields can be generated from different orders than the IDL
@@ -1056,6 +1121,8 @@ class mstch_struct : public mstch_base {
   // std::vector<t_field*> to find the exact t_field to compute
   // field_generator_context.
   std::unordered_map<t_field const*, field_generator_context> context_map;
+
+  std::vector<const t_field*> fields_in_key_order_;
 };
 
 class mstch_function : public mstch_base {
@@ -1097,6 +1164,11 @@ class mstch_function : public mstch_base {
             {"function:structured_annotations",
              &mstch_function::structured_annotations},
             {"function:qualifier", &mstch_function::qualifier},
+            {"function:creates_interaction?",
+             &mstch_function::creates_interaction},
+            {"function:in_or_creates_interaction?",
+             &mstch_function::in_or_creates_interaction},
+            {"function:void?", &mstch_function::is_void},
         });
   }
 
@@ -1119,7 +1191,7 @@ class mstch_function : public mstch_base {
   mstch::node stream_has_first_response() {
     const auto& rettype = *function_->return_type();
     auto stream = dynamic_cast<const t_stream_response*>(&rettype);
-    return stream && stream->first_response_type() != boost::none;
+    return stream && !stream->first_response_type().empty();
   }
   mstch::node has_args() {
     if (function_->get_paramlist()->has_fields()) {
@@ -1162,6 +1234,18 @@ class mstch_function : public mstch_base {
     }
   }
 
+  mstch::node creates_interaction() {
+    return !function_->returned_interaction().empty();
+  }
+  mstch::node in_or_creates_interaction() {
+    return !function_->returned_interaction().empty() ||
+        function_->is_interaction_member();
+  }
+  mstch::node is_void() {
+    return function_->return_type().deref().is_void() &&
+        !function_->returned_interaction();
+  }
+
  protected:
   t_function const* function_;
 };
@@ -1185,6 +1269,7 @@ class mstch_service : public mstch_base {
             {"service:streams?", &mstch_service::has_streams},
             {"service:sinks?", &mstch_service::has_sinks},
             {"service:annotations", &mstch_service::annotations},
+            {"service:thrift_uri", &mstch_service::thrift_uri},
             {"service:parent", &mstch_service::parent},
             {"service:interaction?", &mstch_service::is_interaction},
             {"service:interactions", &mstch_service::interactions},
@@ -1192,7 +1277,20 @@ class mstch_service : public mstch_base {
             {"service:structured_annotations",
              &mstch_service::structured_annotations},
             {"interaction:serial?", &mstch_service::is_serial_interaction},
+            {"interaction:eb?", &mstch_service::is_event_base_interaction},
         });
+
+    // Collect performed interactions and cache them
+    for (auto const* function : get_functions()) {
+      if (function->get_returntype()->is_service()) {
+        interactions_.insert(
+            dynamic_cast<t_interaction const*>(function->get_returntype()));
+      } else if (auto interaction = function->returned_interaction()) {
+        interactions_.insert(
+            dynamic_cast<t_interaction const*>(interaction->get_true_type()));
+      }
+    }
+    assert(interactions_.count(nullptr) == 0);
   }
 
   virtual std::string get_service_namespace(t_program const*) { return {}; }
@@ -1203,9 +1301,10 @@ class mstch_service : public mstch_base {
   mstch::node functions();
   mstch::node extends();
   mstch::node annotations() { return mstch_base::annotations(service_); }
+  mstch::node thrift_uri() { return service_->uri(); }
 
   mstch::node parent() {
-    return cache_->parsed_options_["parent_service_name"];
+    return cache_->parsed_options_.at("parent_service_name");
   }
 
   mstch::node has_streams() {
@@ -1222,24 +1321,15 @@ class mstch_service : public mstch_base {
     });
   }
 
-  mstch::node has_interactions() {
-    auto& funcs = get_functions();
-    return std::any_of(funcs.cbegin(), funcs.cend(), [](auto const& func) {
-      return func->get_returntype()->is_service();
-    });
-  }
+  mstch::node has_interactions() { return !interactions_.empty(); }
   mstch::node interactions() {
     if (!service_->is_interaction()) {
+      // for Python interactions
       cache_->parsed_options_["parent_service_name"] = service_->get_name();
+      cache_->parsed_options_["parent_service_cpp_name"] =
+          cpp2::get_name(service_);
     }
-    std::vector<t_service const*> interactions;
-    for (auto const* function : get_functions()) {
-      if (function->get_returntype()->is_service()) {
-        interactions.push_back(
-            dynamic_cast<t_service const*>(function->get_returntype()));
-      }
-    }
-    return generate_services(interactions);
+    return generate_interactions(interactions_, service_);
   }
   mstch::node structured_annotations() {
     return mstch_base::structured_annotations(service_);
@@ -1248,11 +1338,15 @@ class mstch_service : public mstch_base {
   mstch::node is_serial_interaction() {
     return service_->is_serial_interaction();
   }
+  mstch::node is_event_base_interaction() {
+    return service_->has_annotation("process_in_event_base");
+  }
 
   virtual ~mstch_service() = default;
 
  protected:
   t_service const* service_;
+  std::set<t_interaction const*> interactions_;
 
   mstch::node generate_cached_extended_service(const t_service* service);
   virtual const std::vector<t_function*>& get_functions() const {
@@ -1361,7 +1455,6 @@ class mstch_program : public mstch_base {
         });
     register_has_option("program:frozen?", "frozen");
     register_has_option("program:json?", "json");
-    register_has_option("program:nimble?", "nimble");
     register_has_option("program:any?", "any");
     register_has_option(
         "program:unstructured_annotations_in_metadata?",
