@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -33,6 +33,13 @@ namespace compiler {
 
 namespace {
 
+std::vector<t_function*> lifecycleFunctions() {
+  static t_function onStartServing_{
+      nullptr, t_base_type::t_void(), "onStartServing"},
+      onStopRequested_{nullptr, t_base_type::t_void(), "onStopRequested"};
+  return {&onStartServing_, &onStopRequested_};
+}
+
 mstch::array createStringArray(const std::vector<std::string>& values) {
   mstch::array a;
   for (auto it = values.begin(); it != values.end(); ++it) {
@@ -45,10 +52,6 @@ mstch::array createStringArray(const std::vector<std::string>& values) {
   return a;
 }
 
-std::vector<std::string> get_py3_namespace(const t_program* prog) {
-  return split_namespace(prog->get_namespace("py3"));
-}
-
 std::vector<std::string> get_py3_namespace_with_name(const t_program* prog) {
   auto ns = get_py3_namespace(prog);
   ns.push_back(prog->name());
@@ -366,8 +369,6 @@ class mstch_py3_program : public mstch_program {
              &mstch_py3_program::unique_functions_by_return_type},
             {"program:cppNamespaces", &mstch_py3_program::getCpp2Namespace},
             {"program:py3Namespaces", &mstch_py3_program::getPy3Namespace},
-            {"program:hasServiceFunctions?",
-             &mstch_py3_program::hasServiceFunctions},
             {"program:includeNamespaces",
              &mstch_py3_program::includeNamespaces},
             {"program:cppIncludes", &mstch_py3_program::getCppIncludes},
@@ -390,6 +391,10 @@ class mstch_py3_program : public mstch_program {
     visit_types_for_constants();
     visit_types_for_typedefs();
     visit_types_for_mixin_fields();
+
+    for (auto func : lifecycleFunctions()) {
+      addFunctionByUniqueReturnType(func);
+    }
   }
 
   mstch::node getCppGenPath() {
@@ -475,19 +480,6 @@ class mstch_py3_program : public mstch_program {
     return createStringArray(get_py3_namespace(program_));
   }
 
-  mstch::node hasServiceFunctions() {
-    const auto& services = program_->services();
-    return std::any_of(services.begin(), services.end(), [](const auto& s) {
-      // TODO(ffrancet): This is here because service interaction functions
-      // aren't supported yet and won't be generated, so this shouldn't include
-      // them
-      const auto& functions = s->get_functions();
-      return std::any_of(functions.begin(), functions.end(), [](const auto& f) {
-        return !f->get_returntype()->is_service();
-      });
-    });
-  }
-
   mstch::node hasStream() {
     return !has_option("no_stream") && !streamTypes_.empty();
   }
@@ -534,6 +526,13 @@ class mstch_py3_program : public mstch_program {
     }
   }
 
+  void addFunctionByUniqueReturnType(const t_function* function) {
+    const t_type* return_type = function->get_returntype();
+    auto sa = cpp2::is_stack_arguments(cache_->parsed_options_, *function);
+    uniqueFunctionsByReturnType_.insert(
+        {{visit_type(return_type), sa}, function});
+  }
+
   void visit_type_single_service(const t_service* service) {
     for (const auto& function : service->functions()) {
       for (const auto& field : function.get_paramlist()->fields()) {
@@ -543,10 +542,7 @@ class mstch_py3_program : public mstch_program {
         const t_type* exType = field.get_type();
         streamExceptions_.emplace(visit_type(field.get_type()), exType);
       }
-      const t_type* return_type = function.get_returntype();
-      auto sa = cpp2::is_stack_arguments(cache_->parsed_options_, function);
-      uniqueFunctionsByReturnType_.insert(
-          {{visit_type(return_type), sa}, &function});
+      addFunctionByUniqueReturnType(&function);
     }
   }
 
@@ -728,10 +724,18 @@ class mstch_py3_service : public mstch_service {
             {"service:py3Namespaces", &mstch_py3_service::py3Namespaces},
             {"service:programName", &mstch_py3_service::programName},
             {"service:includePrefix", &mstch_py3_service::includePrefix},
+            {"service:cpp_name", &mstch_py3_service::cpp_name},
             {"service:parent_service_name",
              &mstch_py3_service::parent_service_name},
+            {"service:parent_service_cpp_name",
+             &mstch_py3_service::parent_service_cpp_name},
+            {"service:qualified_name", &mstch_py3_service::qualified_name},
             {"service:supportedFunctions",
              &mstch_py3_service::get_supported_functions},
+            {"service:lifecycleFunctions",
+             &mstch_py3_service::get_lifecycle_functions},
+            {"service:supportedFunctionsWithLifecycle",
+             &mstch_py3_service::get_supported_functions_with_lifecycle},
         });
   }
 
@@ -743,19 +747,29 @@ class mstch_py3_service : public mstch_service {
   }
 
   mstch::node py3Namespaces() {
-    return createStringArray(
-        split_namespace(service_->program()->get_namespace("py3")));
+    return createStringArray(get_py3_namespace(service_->program()));
   }
 
   mstch::node programName() { return service_->program()->name(); }
 
   mstch::node includePrefix() { return service_->program()->include_prefix(); }
 
+  mstch::node cpp_name() { return cpp2::get_name(service_); }
+
+  mstch::node qualified_name() {
+    return cpp2::get_gen_namespace(*service_->program()) +
+        "::" + cpp2::get_name(service_);
+  }
+
   mstch::node parent_service_name() {
     return cache_->parsed_options_.at("parent_service_name");
   }
 
-  mstch::node get_supported_functions() {
+  mstch::node parent_service_cpp_name() {
+    return cache_->parsed_options_.at("parent_service_cpp_name");
+  }
+
+  std::vector<t_function*> supportedFunctions() {
     std::vector<t_function*> funcs;
     bool no_stream = has_option("no_stream");
     for (auto func : service_->get_functions()) {
@@ -763,6 +777,22 @@ class mstch_py3_service : public mstch_service {
         funcs.push_back(func);
       }
     }
+    return funcs;
+  }
+
+  mstch::node get_lifecycle_functions() {
+    return generate_functions(lifecycleFunctions());
+  }
+
+  mstch::node get_supported_functions() {
+    return generate_functions(supportedFunctions());
+  }
+
+  mstch::node get_supported_functions_with_lifecycle() {
+    auto funcs = supportedFunctions();
+    for (auto func : lifecycleFunctions()) {
+      funcs.push_back(func);
+    }
     return generate_functions(funcs);
   }
 
@@ -1215,21 +1245,16 @@ class no_reserved_key_in_namespace_validator : virtual public validator {
 
  private:
   void validate(t_program* const prog) {
-    static const std::unordered_set<std::string> py3_reserved_keys = {
-        "include",
-    };
-
-    const auto& py3_namespace = prog->get_namespace("py3");
-    if (py3_namespace.empty()) {
+    auto namespace_tokens = get_py3_namespace(prog);
+    if (namespace_tokens.empty()) {
       return;
     }
-
-    std::vector<std::string> namespace_tokens = split_namespace(py3_namespace);
     for (const auto& field_name : namespace_tokens) {
-      if (py3_reserved_keys.find(field_name) != py3_reserved_keys.end()) {
+      if (get_python_reserved_names().find(field_name) !=
+          get_python_reserved_names().end()) {
         std::ostringstream ss;
-        ss << "Namespace '" << py3_namespace << "' contains reserved keyword '"
-           << field_name << "'";
+        ss << "Namespace '" << boost::algorithm::join(namespace_tokens, ".")
+           << "' contains reserved keyword '" << field_name << "'";
         add_error(boost::none, ss.str());
       }
     }
@@ -1238,10 +1263,10 @@ class no_reserved_key_in_namespace_validator : virtual public validator {
     std::vector<std::string> fields;
     boost::split(fields, prog->path(), boost::is_any_of(filepath_delimiters));
     for (const auto& field : fields) {
-      if (py3_reserved_keys.find(field) != py3_reserved_keys.end()) {
+      if (field == "include") {
         std::ostringstream ss;
-        ss << "Path '" << prog->path() << "' contains reserved keyword '"
-           << field << "'";
+        ss << "Path '" << prog->path()
+           << "' contains reserved keyword 'include'";
         add_error(boost::none, ss.str());
       }
     }
@@ -1385,8 +1410,12 @@ void t_mstch_py3_generator::generate_init_files() {
 }
 
 boost::filesystem::path t_mstch_py3_generator::package_to_path() {
-  auto package = get_program()->get_namespace("py3");
-  return boost::algorithm::replace_all_copy(package, ".", "/");
+  auto package = get_py3_namespace(get_program());
+  boost::filesystem::path path;
+  for (const auto& path_part : package) {
+    path /= path_part;
+  }
+  return path;
 }
 
 void t_mstch_py3_generator::generate_file(
