@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -137,6 +137,7 @@ mstch::map t_mstch_pyi_generator::extend_program(const t_program& program) {
       {"pyNamespaces", pyNamespaces},
       {"importModules", importModules},
       {"asyncio?", has_option("asyncio")},
+      {"cpp_transport?", has_option("cpp_transport")},
       {"json?", has_option("json")},
   };
   add_container_types(program, result);
@@ -260,14 +261,11 @@ void t_mstch_pyi_generator::generate_services(const t_program& program) {
 boost::filesystem::path t_mstch_pyi_generator::package_to_path(
     const t_program& program) {
   const auto ns = get_py_namespace_raw(program);
-  std::stringstream ss;
+  boost::filesystem::path path;
   for (auto itr = ns.begin(); itr != ns.end(); ++itr) {
-    ss << *itr;
-    if (itr != ns.end()) {
-      ss << "/";
-    }
+    path /= *itr;
   }
-  return boost::filesystem::path{ss.str()};
+  return path += boost::filesystem::path::preferred_separator;
 }
 
 mstch::array t_mstch_pyi_generator::get_return_types(const t_program& program) {
