@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -17,6 +17,7 @@
 #include <cassert>
 #include <cctype>
 #include <set>
+#include <stdexcept>
 #include <string>
 #include <unordered_set>
 
@@ -237,6 +238,27 @@ struct rust_codegen_options {
   std::string current_crate;
 };
 
+static bool validate_rust_serde(const t_node& node) {
+  const std::string* ann = node.find_annotation_or_null("rust.serde");
+
+  return ann == nullptr || *ann == "true" || *ann == "false";
+}
+
+static bool rust_serde_enabled(
+    const rust_codegen_options& options, const t_node& node) {
+  const std::string* ann = node.find_annotation_or_null("rust.serde");
+
+  if (ann == nullptr) {
+    return options.serde;
+  } else if (*ann == "true") {
+    return true;
+  } else if (*ann == "false") {
+    return false;
+  } else {
+    throw std::runtime_error("rust.serde should be `true` or `false`");
+  }
+}
+
 std::string get_import_name(
     const t_program* program, const rust_codegen_options& options) {
   if (program == options.current_program) {
@@ -253,11 +275,21 @@ std::string get_import_name(
 
 enum class FieldKind { Box, Arc, Inline };
 
+bool node_is_boxed(const t_named& node) {
+  return node.has_annotation("rust.box") || node.has_annotation("thrift.box") ||
+      node.find_structured_annotation_or_null(
+          "facebook.com/thrift/annotation/Box");
+}
+
+bool node_is_arced(const t_named& node) {
+  return node.has_annotation("rust.arc");
+}
+
 FieldKind field_kind(const t_named& node) {
-  if (node.has_annotation("rust.arc")) {
+  if (node_is_arced(node)) {
     return FieldKind::Arc;
   }
-  if (node.has_annotation("rust.box")) {
+  if (node_is_boxed(node)) {
     return FieldKind::Box;
   }
   return FieldKind::Inline;
@@ -510,6 +542,7 @@ class mstch_rust_struct : public mstch_struct {
             {"struct:is_exception_message_optional?",
              &mstch_rust_struct::is_exception_message_optional},
             {"struct:exception_message", &mstch_rust_struct::exception_message},
+            {"struct:serde?", &mstch_rust_struct::rust_serde},
         });
   }
   mstch::node rust_name() {
@@ -562,6 +595,7 @@ class mstch_rust_struct : public mstch_struct {
                ->get_req() == t_field::e_req::optional;
   }
   mstch::node exception_message() { return strct_->get_annotation("message"); }
+  mstch::node rust_serde() { return rust_serde_enabled(options_, *strct_); }
 
  private:
   const rust_codegen_options& options_;
@@ -659,7 +693,6 @@ class mstch_rust_function : public mstch_function {
             {"function:rust_name", &mstch_rust_function::rust_name},
             {"function:upcamel", &mstch_rust_function::rust_upcamel},
             {"function:index", &mstch_rust_function::rust_index},
-            {"function:void?", &mstch_rust_function::rust_void},
             {"function:uniqueExceptions",
              &mstch_rust_function::rust_unique_exceptions},
             {"function:uniqueStreamExceptions",
@@ -669,6 +702,8 @@ class mstch_rust_function : public mstch_function {
              &mstch_rust_function::rust_returns_by_name},
             {"function:docs?", &mstch_rust_function::rust_has_doc},
             {"function:docs", &mstch_rust_function::rust_doc},
+            {"function:interaction_name",
+             &mstch_rust_function::rust_interaction_name},
         });
   }
   mstch::node rust_name() {
@@ -689,7 +724,6 @@ class mstch_rust_function : public mstch_function {
     return upcamel_name;
   }
   mstch::node rust_index() { return index_; }
-  mstch::node rust_void() { return function_->get_returntype()->is_void(); }
   mstch::node rust_unique_exceptions() {
     return rust_make_unique_exceptions(function_->get_xceptions());
   }
@@ -733,6 +767,12 @@ class mstch_rust_function : public mstch_function {
   }
   mstch::node rust_has_doc() { return function_->has_doc(); }
   mstch::node rust_doc() { return quoted_rust_doc(function_); }
+  mstch::node rust_interaction_name() {
+    if (!function_->returned_interaction().empty()) {
+      return function_->returned_interaction()->get_name();
+    }
+    return function_->get_returntype()->get_name();
+  }
 
  private:
   int32_t index_;
@@ -784,6 +824,7 @@ class mstch_rust_enum : public mstch_enum {
             {"enum:variants_by_number", &mstch_rust_enum::variants_by_number},
             {"enum:docs?", &mstch_rust_enum::rust_has_doc},
             {"enum:docs", &mstch_rust_enum::rust_doc},
+            {"enum:serde?", &mstch_rust_enum::rust_serde},
         });
   }
   mstch::node rust_name() {
@@ -811,6 +852,7 @@ class mstch_rust_enum : public mstch_enum {
   }
   mstch::node rust_has_doc() { return enm_->has_doc(); }
   mstch::node rust_doc() { return quoted_rust_doc(enm_); }
+  mstch::node rust_serde() { return rust_serde_enabled(options_, *enm_); }
 
  private:
   const rust_codegen_options& options_;
@@ -1395,8 +1437,9 @@ class mstch_rust_typedef : public mstch_typedef {
       const t_typedef* typedf,
       std::shared_ptr<mstch_generators const> generators,
       std::shared_ptr<mstch_cache> cache,
-      ELEMENT_POSITION pos)
-      : mstch_typedef(typedf, generators, cache, pos) {
+      ELEMENT_POSITION pos,
+      const rust_codegen_options& options)
+      : mstch_typedef(typedf, generators, cache, pos), options_(options) {
     register_methods(
         this,
         {
@@ -1408,6 +1451,7 @@ class mstch_rust_typedef : public mstch_typedef {
             {"typedef:nonstandard?", &mstch_rust_typedef::rust_nonstandard},
             {"typedef:docs?", &mstch_rust_typedef::rust_has_docs},
             {"typedef:docs", &mstch_rust_typedef::rust_docs},
+            {"typedef:serde?", &mstch_rust_typedef::rust_serde},
         });
   }
   mstch::node rust_name() {
@@ -1445,6 +1489,10 @@ class mstch_rust_typedef : public mstch_typedef {
   }
   mstch::node rust_has_docs() { return typedf_->has_doc(); }
   mstch::node rust_docs() { return quoted_rust_doc(typedf_); }
+  mstch::node rust_serde() { return rust_serde_enabled(options_, *typedf_); }
+
+ private:
+  const rust_codegen_options& options_;
 };
 
 class mstch_rust_annotation : public mstch_annotation {
@@ -1628,8 +1676,15 @@ class type_rust_generator : public type_generator {
   const rust_codegen_options& options_;
 };
 
+struct name_less {
+  bool operator()(const t_type* lhs, const t_type* rhs) const {
+    return lhs->get_scoped_name() < rhs->get_scoped_name();
+  }
+};
+
 mstch::node mstch_rust_service::rust_all_exceptions() {
-  std::map<const t_type*, std::vector<const t_function*>> function_map;
+  std::map<const t_type*, std::vector<const t_function*>, name_less>
+      function_map;
   std::map<const t_type*, std::vector<const t_field*>> field_map;
   for (const auto& fun : service_->functions()) {
     for (const auto& fld : fun.get_xceptions()->fields()) {
@@ -1698,7 +1753,8 @@ class const_rust_generator : public const_generator {
 
 class typedef_rust_generator : public typedef_generator {
  public:
-  typedef_rust_generator() = default;
+  explicit typedef_rust_generator(const rust_codegen_options& options)
+      : options_(options) {}
   ~typedef_rust_generator() override = default;
   std::shared_ptr<mstch_base> generate(
       const t_typedef* typedf,
@@ -1706,8 +1762,12 @@ class typedef_rust_generator : public typedef_generator {
       std::shared_ptr<mstch_cache> cache,
       ELEMENT_POSITION pos,
       int32_t /*index*/) const override {
-    return std::make_shared<mstch_rust_typedef>(typedf, generators, cache, pos);
+    return std::make_shared<mstch_rust_typedef>(
+        typedf, generators, cache, pos, options_);
   }
+
+ private:
+  const rust_codegen_options& options_;
 };
 
 class annotation_rust_generator : public annotation_generator {
@@ -1750,7 +1810,7 @@ void t_mstch_rust_generator::set_mstch_generators() {
   generators_->set_const_generator(
       std::make_unique<const_rust_generator>(options_));
   generators_->set_typedef_generator(
-      std::make_unique<typedef_rust_generator>());
+      std::make_unique<typedef_rust_generator>(options_));
   generators_->set_annotation_generator(
       std::make_unique<annotation_rust_generator>());
 }
@@ -1807,13 +1867,19 @@ namespace {
 class annotation_validator : public validator {
  public:
   using validator::visit;
-  bool visit(t_struct* s) override;
+  bool visit(t_struct*) override;
+  bool visit(t_enum*) override;
 };
 
 bool annotation_validator::visit(t_struct* s) {
+  if (!validate_rust_serde(*s)) {
+    add_error(s->lineno(), "`rust.serde` must be `true` or `false`");
+  }
+
   for (auto& field : s->fields()) {
-    bool box = field.has_annotation("rust.box");
-    bool arc = field.has_annotation("rust.arc");
+    FieldKind kind = field_kind(field);
+    bool box = node_is_boxed(field) || kind == FieldKind::Box;
+    bool arc = node_is_arced(field) || kind == FieldKind::Arc;
     if (box && arc) {
       add_error(
           field.lineno(),
@@ -1822,6 +1888,15 @@ bool annotation_validator::visit(t_struct* s) {
   }
   return true;
 }
+
+bool annotation_validator::visit(t_enum* e) {
+  if (!validate_rust_serde(*e)) {
+    add_error(e->lineno(), "`rust.serde` must be `true` or `false`");
+  }
+
+  return true;
+}
+
 } // namespace
 
 void t_mstch_rust_generator::fill_validator_list(validator_list& l) const {
