@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,11 +16,14 @@
 
 #include <thrift/compiler/lib/cpp2/util.h>
 
+#include <algorithm>
 #include <memory>
+#include <random>
 #include <stdexcept>
 #include <string>
 #include <vector>
 
+#include <folly/Utility.h>
 #include <folly/portability/GMock.h>
 #include <folly/portability/GTest.h>
 
@@ -34,6 +37,8 @@
 #include <thrift/compiler/ast/t_type.h>
 #include <thrift/compiler/ast/t_typedef.h>
 #include <thrift/compiler/ast/t_union.h>
+#include <thrift/compiler/gen/testing.h>
+#include <thrift/compiler/util.h>
 
 namespace apache::thrift::compiler {
 namespace {
@@ -243,5 +248,150 @@ TEST_F(UtilTest, get_gen_type_class) {
       "::apache::thrift::type_class::string",
       cpp2::get_gen_type_class(t_base_type::t_string()));
 }
+
+TEST_F(UtilTest, is_custom_type) {
+  t_program p("path/to/program.thrift");
+
+  {
+    auto cppType = t_base_type::t_string();
+    auto typeDef = t_typedef(&p, "Type", cppType);
+    EXPECT_FALSE(cpp2::is_custom_type(cppType));
+    EXPECT_FALSE(cpp2::is_custom_type(typeDef));
+    cppType.set_annotation("cpp.type", "folly::fbstring");
+    EXPECT_TRUE(cpp2::is_custom_type(cppType));
+    EXPECT_TRUE(cpp2::is_custom_type(typeDef));
+  }
+
+  {
+    auto cpp2Type = t_base_type::t_string();
+    auto typeDef = t_typedef(&p, "Type", cpp2Type);
+    EXPECT_FALSE(cpp2::is_custom_type(cpp2Type));
+    EXPECT_FALSE(cpp2::is_custom_type(typeDef));
+    cpp2Type.set_annotation("cpp2.type", "folly::fbstring");
+    EXPECT_TRUE(cpp2::is_custom_type(cpp2Type));
+    EXPECT_TRUE(cpp2::is_custom_type(typeDef));
+  }
+
+  const auto i32 = t_base_type::t_i32();
+  {
+    auto cppTemplate = t_list(&i32);
+    auto typeDef = t_typedef(&p, "Type", cppTemplate);
+    EXPECT_FALSE(cpp2::is_custom_type(cppTemplate));
+    EXPECT_FALSE(cpp2::is_custom_type(typeDef));
+    cppTemplate.set_annotation("cpp.template", "std::deque");
+    EXPECT_TRUE(cpp2::is_custom_type(cppTemplate));
+    EXPECT_TRUE(cpp2::is_custom_type(typeDef));
+  }
+
+  {
+    auto cpp2Template = t_list(&i32);
+    auto typeDef = t_typedef(&p, "Type", cpp2Template);
+    EXPECT_FALSE(cpp2::is_custom_type(cpp2Template));
+    EXPECT_FALSE(cpp2::is_custom_type(typeDef));
+    cpp2Template.set_annotation("cpp2.template", "std::deque");
+    EXPECT_TRUE(cpp2::is_custom_type(cpp2Template));
+    EXPECT_TRUE(cpp2::is_custom_type(typeDef));
+  }
+
+  {
+    auto cppAdapter = i32;
+    auto typeDef = t_typedef(&p, "Type", cppAdapter);
+    EXPECT_FALSE(cpp2::is_custom_type(cppAdapter));
+    EXPECT_FALSE(cpp2::is_custom_type(typeDef));
+    cppAdapter.set_annotation("cpp.adapter", "Adapter");
+    EXPECT_TRUE(cpp2::is_custom_type(cppAdapter));
+    EXPECT_TRUE(cpp2::is_custom_type(typeDef));
+  }
+}
+
+TEST_F(UtilTest, lpt_slit) {
+  const std::vector<size_t> in{1, 4, 8, 7, 2, 2, 9, 5, 6, 10, 20, 1, 8};
+
+  // split 1
+  auto res1 = cpp2::lpt_split(in, 1, folly::identity);
+  std::vector<std::vector<size_t>> res1_expected{
+      {20, 10, 9, 8, 8, 7, 6, 5, 4, 2, 2, 1, 1}};
+  EXPECT_EQ(res1, res1_expected);
+
+  // split 2
+  auto res2 = cpp2::lpt_split(in, 2, folly::identity);
+  std::vector<std::vector<size_t>> res2_expected = {
+      {10, 9, 8, 7, 5, 2, 1}, {20, 8, 6, 4, 2, 1}};
+  EXPECT_EQ(res2, res2_expected);
+
+  // split 3
+  auto res3 = cpp2::lpt_split(in, 3, folly::identity);
+  std::vector<std::vector<size_t>> res3_expected = {
+      {9, 8, 7, 4}, {10, 8, 6, 2, 1}, {20, 5, 2, 1}};
+  EXPECT_EQ(res3, res3_expected);
+
+  // split 4
+  auto res4 = cpp2::lpt_split(in, 4, folly::identity);
+  std::vector<std::vector<size_t>> res4_expected = {
+      {8, 8, 5}, {9, 7, 4, 1}, {10, 6, 2, 2}, {20, 1}};
+  EXPECT_EQ(res4, res4_expected);
+}
+
+TEST_F(UtilTest, get_internal_injected_field_id) {
+  EXPECT_THROW(cpp2::get_internal_injected_field_id(-1), std::runtime_error);
+  EXPECT_EQ(cpp2::get_internal_injected_field_id(0), -1000);
+  EXPECT_EQ(cpp2::get_internal_injected_field_id(999), -1999);
+  EXPECT_THROW(cpp2::get_internal_injected_field_id(1000), std::runtime_error);
+}
+
+TEST_F(UtilTest, gen_adapter_dependency_graph) {
+  t_program p("path/to/program.thrift");
+  std::mt19937 gen;
+  auto test = [&](std::string name, std::vector<const t_type*> expected) {
+    constexpr int kIters = 42;
+    for (int iter = 0; iter < kIters; iter++) {
+      std::vector<const t_type*> objects = expected;
+      std::vector<t_typedef*> typedefs;
+      for (auto type : objects) {
+        if (auto typedf = dynamic_cast<t_typedef const*>(type)) {
+          typedefs.push_back(const_cast<t_typedef*>(typedf));
+        }
+      }
+      std::shuffle(objects.begin(), objects.end(), gen);
+      auto deps = cpp2::gen_adapter_dependency_graph(&p, objects, typedefs);
+      auto sorted_objects =
+          topological_sort<const t_type*>(objects.begin(), objects.end(), deps);
+      ASSERT_EQ(sorted_objects.size(), expected.size()) << name;
+      for (size_t i = 0; i < expected.size(); ++i) {
+        EXPECT_EQ(sorted_objects[i], expected[i])
+            << name << ": got " << sorted_objects[i]->name() << " instead of "
+            << expected[i]->name();
+      }
+    }
+  };
+
+  auto adapter = gen::adapter_builder(p, "cpp");
+  t_struct s1(&p, "struct");
+  t_typedef t1(&p, "typedef_with_adapter", s1);
+  t1.add_structured_annotation(adapter.make("MyAdapter"));
+  test("adapted typedef after struct", {&s1, &t1});
+
+  t_struct f2(nullptr, "foreign_struct");
+  t_typedef t2(&p, "typedef_with_adapter", f2);
+  t2.add_structured_annotation(adapter.make("MyAdapter"));
+  t_struct s2(&p, "dependent_struct");
+  s2.append_field(std::make_unique<t_field>(t2, "field"));
+  test("dependent struct after typedef", {&t2, &s2});
+
+  t_union u3(&p, "union");
+  t_typedef t3(&p, "typedef_with_adapter", u3);
+  t3.add_structured_annotation(adapter.make("MyAdapter"));
+  test("adapted typedef after union", {&u3, &t3});
+
+  t_typedef t4(&p, "typedef_with_adapter", f2);
+  t4.add_structured_annotation(adapter.make("MyAdapter"));
+  t_exception e4(&p, "exception");
+  e4.append_field(std::make_unique<t_field>(t4, "field"));
+  test("dependent exception after typedef", {&t4, &e4});
+
+  t_typedef t5(&p, "typedef_of_typedef", t2);
+  test("dependent typedef after typedef", {&t2, &t5});
+}
+
 } // namespace
 } // namespace apache::thrift::compiler
