@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -18,6 +18,7 @@
 
 #include <cassert>
 #include <cctype>
+#include <boost/algorithm/string.hpp>
 
 namespace apache {
 namespace thrift {
@@ -139,6 +140,21 @@ std::string package_to_path(std::string package) {
   boost::algorithm::replace_all(package, ".", "/");
   return package;
 }
+
+std::vector<std::string> gen_namespace_from_package(const t_package& package) {
+  if (package.empty()) {
+    return {};
+  }
+
+  const auto& domain = package.domain();
+  const auto& path = package.path();
+  std::vector<std::string> ret;
+  ret.reserve(domain.size() + path.size());
+  ret.insert(ret.end(), domain.rbegin(), domain.rend());
+  ret.insert(ret.end(), path.begin(), path.end());
+  return ret;
+}
+
 } // namespace java
 } // namespace compiler
 } // namespace thrift
