@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -21,6 +21,8 @@
 #include <boost/algorithm/string/predicate.hpp>
 #include <boost/algorithm/string/replace.hpp>
 
+#include <thrift/compiler/ast/t_program.h>
+
 namespace apache {
 namespace thrift {
 namespace compiler {
@@ -58,6 +60,12 @@ std::string quote_java_string(const std::string& unescaped);
  */
 std::string package_to_path(std::string package);
 
+/**
+ * Generate java namespace from package
+ * Example: "meta.com/foo/bar" -> ["com", "meta", "foo", "bar"]
+ */
+std::vector<std::string> gen_namespace_from_package(const t_package& package);
+
 } // namespace java
 } // namespace compiler
 } // namespace thrift
