@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -22,8 +22,8 @@ namespace apache {
 namespace thrift {
 namespace compiler {
 
-// The standard mutator for Thrift.
-ast_mutator standard_mutator();
+// Standard mutators for Thrift.
+ast_mutators standard_mutators();
 
 } // namespace compiler
 } // namespace thrift
