@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,6 +16,7 @@
 
 #include <thrift/compiler/sema/standard_validator.h>
 
+#include <algorithm>
 #include <string>
 #include <unordered_map>
 
@@ -32,6 +33,7 @@
 #include <thrift/compiler/ast/t_structured.h>
 #include <thrift/compiler/ast/t_throws.h>
 #include <thrift/compiler/ast/t_type.h>
+#include <thrift/compiler/ast/t_typedef.h>
 #include <thrift/compiler/ast/t_union.h>
 #include <thrift/compiler/gen/cpp/reference_type.h>
 #include <thrift/compiler/lib/cpp2/util.h>
@@ -44,6 +46,12 @@ namespace compiler {
 
 namespace {
 
+constexpr auto kCppRefUri = "facebook.com/thrift/annotation/cpp/Ref";
+constexpr auto kCppAdapterUri = "facebook.com/thrift/annotation/cpp/Adapter";
+constexpr auto kHackAdapterUri = "facebook.com/thrift/annotation/hack/Adapter";
+constexpr auto kCppUnstructuredAdapter = "cpp.adapter";
+constexpr auto kHackUnstructuredAdapter = "hack.adapter";
+
 const t_structured* get_mixin_type(const t_field& field) {
   if (cpp2::is_mixin(field)) {
     return dynamic_cast<const t_structured*>(field.type()->get_true_type());
@@ -85,7 +93,7 @@ void report_redef_failure(
     const t_node& child,
     const t_node& existing) {
   report_redef_failure(
-      ctx, kind, name, ctx.program()->path(), parent, child, existing);
+      ctx, kind, name, ctx.program().path(), parent, child, existing);
 }
 
 // Helper for checking for the redefinition of a name in the context of a node.
@@ -147,6 +155,101 @@ class redef_checker {
   name_index<t_named> seen_;
 };
 
+// Helper for validating the adapters
+class adapter_checker {
+ public:
+  explicit adapter_checker(diagnostic_context& ctx) : ctx_(ctx) {}
+
+  // Checks if adapter name is provided
+  // Do not allow composing two structured annotations on typedef
+  void check(
+      const t_named& node,
+      const char* structured_adapter_annotation,
+      const char* structured_adapter_annotation_error_name) {
+    const t_const* adapter_annotation =
+        node.find_structured_annotation_or_null(structured_adapter_annotation);
+    if (!adapter_annotation) {
+      return;
+    }
+
+    try {
+      adapter_annotation->get_value_from_structured_annotation("name");
+    } catch (const std::exception& e) {
+      ctx_.failure([&](auto& o) { o << e.what(); });
+      return;
+    }
+
+    // TODO (dokwon): Do not allow composing unstructured adapter annotation on
+    // typedef as well.
+    if (const auto* typedf = dynamic_cast<const t_typedef*>(&node)) {
+      if (t_typedef::get_first_structured_annotation_or_null(
+              &*typedf->type(), structured_adapter_annotation)) {
+        ctx_.failure([&](auto& o) {
+          o << "The `" << structured_adapter_annotation_error_name
+            << "` annotation cannot be annotated more than once in all typedef levels in `"
+            << node.name() << "`.";
+        });
+      }
+    }
+  }
+
+  void check(
+      const t_named&& node,
+      const char* structured_adapter_annotation,
+      const char* structured_adapter_annotation_error_name) = delete;
+
+  // Do not allow composing structured annotation on field/typedef
+  // and unstructured annotation on typedef/type
+  void check(
+      const t_field& field,
+      const char* structured_adapter_annotation,
+      const char* unstructured_adapter_annotation,
+      const char* structured_adapter_annotation_error_name,
+      bool disallow_structured_annotations_on_both_field_and_typedef) {
+    if (!field.type().resolved()) {
+      return;
+    }
+    auto type = field.type().get_type();
+
+    bool structured_annotation_on_field =
+        field.find_structured_annotation_or_null(structured_adapter_annotation);
+
+    bool structured_annotation_on_typedef =
+        t_typedef::get_first_structured_annotation_or_null(
+            type, structured_adapter_annotation) != nullptr;
+
+    if (disallow_structured_annotations_on_both_field_and_typedef &&
+        structured_annotation_on_field && structured_annotation_on_typedef) {
+      ctx_.failure([&](auto& o) {
+        o << "`" << structured_adapter_annotation_error_name
+          << "` cannot be applied on both field and typedef in `"
+          << field.name() << "`.";
+      });
+    }
+
+    if (structured_annotation_on_typedef || structured_annotation_on_field) {
+      if (t_typedef::get_first_annotation_or_null(
+              type, {unstructured_adapter_annotation})) {
+        ctx_.failure([&](auto& o) {
+          o << "`" << structured_adapter_annotation_error_name
+            << "` cannot be combined with `" << unstructured_adapter_annotation
+            << "` in `" << field.name() << "`.";
+        });
+      }
+    }
+  }
+
+  void check(
+      const t_field&& field,
+      const char* structured_adapter_annotation,
+      const char* unstructured_adapter_annotation,
+      const char* structured_adapter_annotation_error_name,
+      bool disallow_structured_annotations_on_both_field_and_typedef) = delete;
+
+ private:
+  diagnostic_context& ctx_;
+};
+
 struct service_metadata {
   name_index<t_service> function_name_to_service;
 
@@ -209,11 +312,11 @@ void validate_extends_service_function_name_uniqueness(
 void validate_throws_exceptions(diagnostic_context& ctx, const t_throws& node) {
   for (const auto& except : node.fields()) {
     auto except_type = except.type()->get_true_type();
-    if (dynamic_cast<const t_exception*>(except_type) == nullptr) {
-      ctx.failure(except, [&](auto& o) {
-        o << "Non-exception type, `" << except_type->name() << "`, in throws.";
-      });
-    }
+    ctx.check(
+        dynamic_cast<const t_exception*>(except_type),
+        except,
+        "Non-exception type, `{}`, in throws.",
+        except_type->name());
   }
 }
 
@@ -241,7 +344,7 @@ void validate_field_names_uniqueness(
 void validate_union_field_attributes(
     diagnostic_context& ctx, const t_union& node) {
   for (const auto& field : node.fields()) {
-    if (field.qualifier() != t_field_qualifier::unspecified) {
+    if (field.qualifier() != t_field_qualifier::none) {
       auto qual = field.qualifier() == t_field_qualifier::required ? "required"
                                                                    : "optional";
       ctx.failure(field, [&](auto& o) {
@@ -258,25 +361,23 @@ void validate_boxed_field_attributes(
     return;
   }
 
-  if (!dynamic_cast<const t_union*>(ctx.parent()) &&
-      node.qualifier() != t_field_qualifier::optional) {
-    ctx.failure([&](auto& o) {
-      o << "The `cpp.box` annotation can only be used with optional fields. Make sure `"
-        << node.name() << "` is optional.";
-    });
-  }
+  ctx.check(
+      dynamic_cast<const t_union*>(ctx.parent()) ||
+          node.qualifier() == t_field_qualifier::optional,
+      "The `cpp.box` annotation can only be used with optional fields. Make sure `{}` is optional.",
+      node.name());
 
-  if (node.has_annotation({
+  ctx.failure_if(
+      node.has_annotation({
           "cpp.ref",
           "cpp2.ref",
           "cpp.ref_type",
           "cpp2.ref_type",
-      })) {
-    ctx.failure([&](auto& o) {
-      o << "The `cpp.box` annotation cannot be combined with the `cpp.ref` or `cpp.ref_type` annotations. Remove one of the annotations from `"
-        << node.name() << "`.";
-    });
-  }
+      }),
+      [&](auto& o) {
+        o << "The `cpp.box` annotation cannot be combined with the `cpp.ref` or `cpp.ref_type` annotations. Remove one of the annotations from `"
+          << node.name() << "`.";
+      });
 }
 
 // Checks the attributes of a mixin field.
@@ -287,13 +388,11 @@ void validate_mixin_field_attributes(
   }
 
   auto* ttype = node.type()->get_true_type();
-  if (typeid(*ttype) != typeid(t_struct) && typeid(*ttype) != typeid(t_union)) {
-    ctx.failure([&](auto& o) {
-      o << "Mixin field `" << node.name()
-        << "` type must be a struct or union. Found `" << ttype->get_name()
-        << "`.";
-    });
-  }
+  ctx.check(
+      typeid(*ttype) == typeid(t_struct) || typeid(*ttype) == typeid(t_union),
+      "Mixin field `{}` type must be a struct or union. Found `{}`.",
+      node.name(),
+      ttype->get_name());
 
   if (const auto* parent = dynamic_cast<const t_union*>(ctx.parent())) {
     ctx.failure([&](auto& o) {
@@ -323,10 +422,9 @@ void validate_ref_field_attributes(
 
   if (node.qualifier() != t_field_qualifier::optional &&
       dynamic_cast<const t_union*>(ctx.parent()) == nullptr) {
-    ctx.warning([&](auto& o) {
-      o << "`cpp.ref` field `" << node.name()
-        << "` must be optional if it is recursive.";
-    });
+    ctx.warning(
+        "`cpp.ref` field `{}` must be optional if it is recursive.",
+        node.name());
   }
 }
 
@@ -340,13 +438,14 @@ void validate_enum_value_uniqueness(
   std::unordered_map<int32_t, const t_enum_value*> values;
   for (const auto& value : node.values()) {
     auto prev = values.emplace(value.get_value(), &value);
-    if (!prev.second) {
-      ctx.failure(value, [&](auto& o) {
-        o << "Duplicate value `" << value.name() << "=" << value.get_value()
-          << "` with value `" << prev.first->second->name() << "` in enum `"
-          << node.name() << "`.";
-      });
-    }
+    ctx.check(
+        prev.second,
+        value,
+        "Duplicate value `{}={}` with value `{}` in enum `{}`.",
+        value.name(),
+        value.get_value(),
+        prev.first->second->name(),
+        node.name());
   }
 }
 
@@ -356,10 +455,6 @@ void validate_enum_value(diagnostic_context& ctx, const t_enum_value& node) {
       o << "The enum value, `" << node.name()
         << "`, must have an explicitly assigned value.";
     });
-  } else if (node.get_value() < 0 && !ctx.params().allow_neg_enum_vals) {
-    ctx.warning([&](auto& o) {
-      o << "Negative value supplied for enum value `" << node.name() << "`.";
-    });
   }
 }
 
@@ -399,13 +494,13 @@ void validate_uri_uniqueness(diagnostic_context& ctx, const t_program& prog) {
   visit.add_definition_visitor(
       [&](const std::string& path, const t_named& node) {
         const auto& uri = node.uri();
-        if (uri.empty()) {
+        if (uri.empty() || uri == t_named::kTransitiveUri) {
           return;
         }
         auto result = uri_to_node.emplace(uri, &node);
         if (!result.second) {
           report_redef_failure(
-              ctx, "thrift.uri", uri, path, node, node, *result.first->second);
+              ctx, "Thrift URI", uri, path, node, node, *result.first->second);
         }
       });
   for (const auto* p : prog.get_included_programs()) {
@@ -415,37 +510,31 @@ void validate_uri_uniqueness(diagnostic_context& ctx, const t_program& prog) {
 }
 
 void validate_field_id(diagnostic_context& ctx, const t_field& node) {
-  if (!node.has_explicit_id()) {
-    ctx.warning([&](auto& o) {
-      o << "No field id specified for `" << node.name()
-        << "`, resulting protocol may"
-        << " have conflicts or not be backwards compatible!";
-    });
+  if (node.explicit_id() != node.id()) {
+    ctx.warning(
+        "No field id specified for `{}`, resulting protocol may have conflicts "
+        "or not be backwards compatible!",
+        node.name());
   }
 
-  if (node.id() == 0 &&
-      !node.has_annotation("cpp.deprecated_allow_zero_as_field_id")) {
-    ctx.failure([&](auto& o) {
-      o << "Zero value (0) not allowed as a field id for `" << node.get_name()
-        << "`";
-    });
-  }
+  ctx.failure_if(
+      node.id() == 0 &&
+          !node.has_annotation("cpp.deprecated_allow_zero_as_field_id"),
+      [&](auto& o) {
+        o << "Zero value (0) not allowed as a field id for `" << node.get_name()
+          << "`";
+      });
 
-  if (node.id() < t_field::min_id) {
-    ctx.failure([&](auto& o) {
-      o << "Reserved field id (" << node.id() << ") cannot be used for `"
-        << node.name() << "`.";
-    });
-  }
+  ctx.failure_if(
+      !node.is_injected() && node.id() < t_field::min_id, [&](auto& o) {
+        o << "Reserved field id (" << node.id() << ") cannot be used for `"
+          << node.name() << "`.";
+      });
 }
 
 void validate_compatibility_with_lazy_field(
     diagnostic_context& ctx, const t_structured& node) {
-  if (!has_lazy_field(node)) {
-    return;
-  }
-
-  if (node.has_annotation("cpp.methods")) {
+  if (has_lazy_field(node) && node.has_annotation("cpp.methods")) {
     ctx.failure([&](auto& o) {
       o << "cpp.methods is incompatible with lazy deserialization in struct `"
         << node.get_name() << "`";
@@ -454,8 +543,7 @@ void validate_compatibility_with_lazy_field(
 }
 
 void validate_ref_annotation(diagnostic_context& ctx, const t_field& node) {
-  if (node.find_structured_annotation_or_null(
-          "facebook.com/thrift/annotation/cpp/Ref") &&
+  if (node.find_structured_annotation_or_null(kCppRefUri) &&
       node.has_annotation(
           {"cpp.ref", "cpp2.ref", "cpp.ref_type", "cpp2.ref_type"})) {
     ctx.failure([&](auto& o) {
@@ -465,115 +553,223 @@ void validate_ref_annotation(diagnostic_context& ctx, const t_field& node) {
   }
 }
 
-void validate_adapter_annotation(diagnostic_context& ctx, const t_field& node) {
-  const t_const* adapter_annotation = nullptr;
-  for (const t_const* annotation : node.structured_annotations()) {
-    if (annotation->type()->uri() ==
-        "facebook.com/thrift/annotation/cpp/Adapter") {
-      adapter_annotation = annotation;
-      break;
-    }
-  }
-
-  if (adapter_annotation &&
-      t_typedef::get_first_annotation_or_null(&*node.type(), {"cpp.adapter"})) {
-    ctx.failure([&](auto& o) {
-      o << "`@cpp.Adapter` cannot be combined with `cpp_adapter` in `"
-        << node.name() << "`.";
-    });
-  }
+void validate_cpp_adapter_annotation(
+    diagnostic_context& ctx, const t_named& node) {
+  adapter_checker(ctx).check(node, kCppAdapterUri, "@cpp.Adapter");
 }
 
 void validate_hack_adapter_annotation(
-    diagnostic_context& ctx, const t_field& node) {
-  const t_const* field_adapter_annotation = nullptr;
-  for (const t_const* annotation : node.structured_annotations()) {
-    if (annotation->type()->uri() ==
-        "facebook.com/thrift/annotation/hack/ExperimentalAdapter") {
-      field_adapter_annotation = annotation;
-      break;
-    }
-  }
-
-  if (field_adapter_annotation &&
-      t_typedef::get_first_annotation_or_null(
-          &*node.type(), {"hack.adapter"})) {
-    ctx.failure([&](auto& o) {
-      o << "`@hack.ExperimentalAdapter` cannot be combined with "
-           "`hack_adapter` in `"
-        << node.name() << "`.";
-    });
-  }
+    diagnostic_context& ctx, const t_named& node) {
+  adapter_checker(ctx).check(node, kHackAdapterUri, "@hack.Adapter");
 }
 
-void validate_box_annotation(diagnostic_context& ctx, const t_field& node) {
-  if (node.has_annotation("cpp.box")) {
-    ctx.warning([&](auto& o) {
-      o << "Cpp.box is deprecated. Please use thrift.box annotation instead in `"
-        << node.name() << "`.";
-    });
+void validate_box_annotation(
+    diagnostic_context& ctx, const t_structured& node) {
+  if (node.generated()) {
+    return;
   }
-}
-void validate_box_annotation_in_struct(
-    diagnostic_context& ctx, const t_struct& node) {
-  if (node.is_struct()) {
-    for (const t_field& f : node.fields()) {
-      validate_box_annotation(ctx, f);
+
+  for (const auto& field : node.fields()) {
+    if (field.has_annotation({"cpp.box", "thrift.box"})) {
+      ctx.warning(
+          field,
+          "cpp.box and thrift.box are deprecated. Please use @thrift.Box "
+          "annotation instead in `{}`.",
+          field.name());
     }
   }
 }
 
 void validate_ref_unique_and_box_annotation(
     diagnostic_context& ctx, const t_field& node) {
+  const t_const* adapter_annotation =
+      node.find_structured_annotation_or_null(kCppAdapterUri);
+
   if (cpp2::is_unique_ref(&node)) {
     if (node.has_annotation({"cpp.ref", "cpp2.ref"})) {
-      ctx.warning([&](auto& o) {
-        o << "cpp.ref, cpp2.ref "
-          << "are deprecated. Please use thrift.box annotation instead in `"
-          << node.name() << "`.";
-      });
+      if (adapter_annotation) {
+        ctx.failure([&](auto& o) {
+          o << "cpp.ref, cpp2.ref "
+            << "are deprecated. Please use @thrift.Box annotation instead in `"
+            << node.name() << "` with @cpp.Adapter.";
+        });
+      } else {
+        ctx.warning(
+            "cpp.ref, cpp2.ref are deprecated. Please use @thrift.Box "
+            "annotation instead in `{}`.",
+            node.name());
+      }
     }
-    if (node.find_annotation_or_null({"cpp.ref_type", "cpp2.ref_type"})) {
-      ctx.warning([&](auto& o) {
-        o << "cpp.ref_type = `unique`, cpp2.ref_type = `unique` "
-          << "are deprecated. Please use thrift.box annotation instead in `"
-          << node.name() << "`.";
-      });
+    if (node.has_annotation({"cpp.ref_type", "cpp2.ref_type"})) {
+      if (adapter_annotation) {
+        ctx.failure([&](auto& o) {
+          o << "cpp.ref_type = `unique`, cpp2.ref_type = `unique` "
+            << "are deprecated. Please use @thrift.Box annotation instead in `"
+            << node.name() << "` with @cpp.Adapter.";
+        });
+      } else {
+        ctx.warning(
+            "cpp.ref_type = `unique`, cpp2.ref_type = `unique` "
+            "are deprecated. Please use @thrift.Box annotation instead in "
+            "`{}`.",
+            node.name());
+      }
     }
-    if (node.find_structured_annotation_or_null(
-            "facebook.com/thrift/annotation/cpp/Ref")) {
-      ctx.warning([&](auto& o) {
-        o << "@cpp.Ref{type = cpp.RefType.Unique} "
-          << "is deprecated. Please use thrift.box annotation instead in `"
-          << node.name() << "`.";
+    if (node.find_structured_annotation_or_null(kCppRefUri) != nullptr) {
+      if (adapter_annotation) {
+        ctx.failure([&](auto& o) {
+          o << "@cpp.Ref{type = cpp.RefType.Unique} "
+            << "is deprecated. Please use @thrift.Box annotation instead in `"
+            << node.name() << "` with @cpp.Adapter.";
+        });
+      } else {
+        ctx.warning(
+            "@cpp.Ref{{type = cpp.RefType.Unique}} is deprecated. Please use "
+            "@thrift.Box annotation instead in `{}`.",
+            node.name());
+      }
+    }
+  }
+}
+
+void validate_function_priority_annotation(
+    diagnostic_context& ctx, const t_node& node) {
+  if (auto* priority = node.find_annotation_or_null("priority")) {
+    std::string choices[] = {
+        "HIGH_IMPORTANT", "HIGH", "IMPORTANT", "NORMAL", "BEST_EFFORT"};
+    auto* end = choices + sizeof(choices) / sizeof(choices[0]);
+    ctx.failure_if(std::find(choices, end, *priority) == end, [&](auto& o) {
+      o << "Bad priority '" << *priority << "'. Choose one of '";
+      auto delim = "";
+      for (const auto& choice : choices) {
+        o << delim << choice;
+        delim = "','";
+      }
+      o << "'.";
+    });
+  }
+}
+
+void validate_exception_php_annotations(
+    diagnostic_context& ctx, const t_exception& node) {
+  constexpr const char* annotations[] = {"message", "code"};
+  for (const auto& annotation : annotations) {
+    if (node.get_field_by_name(annotation) != nullptr &&
+        strcmp(annotation, node.get_annotation(annotation).c_str()) != 0) {
+      ctx.warning(
+          "Some generators (e.g. PHP) will ignore annotation '{}' as it is "
+          "also used as field",
+          annotation);
+    }
+  }
+
+  // Check that value of "message" annotation is
+  // - a valid member of struct
+  // - of type STRING
+  if (node.has_annotation("message")) {
+    const std::string& v = node.get_annotation("message");
+    const auto* field = node.get_field_by_name(v);
+    if (field == nullptr) {
+      ctx.failure([&](auto& o) {
+        o << "member specified as exception 'message' should be a valid"
+          << " struct member, '" << v << "' in '" << node.name() << "' is not";
+      });
+    } else if (!field->get_type()->is_string_or_binary()) {
+      ctx.failure([&](auto& o) {
+        o << "member specified as exception 'message' should be of type "
+          << "STRING, '" << v << "' in '" << node.name() << "' is not";
       });
     }
   }
 }
+
+void validate_oneway_function(diagnostic_context& ctx, const t_function& node) {
+  if (!node.is_oneway()) {
+    return;
+  }
+
+  ctx.check(
+      node.return_type().get_type() != nullptr &&
+          node.return_type().get_type()->is_void() &&
+          !node.returned_interaction(),
+      "Oneway methods must have void return type: {}",
+      node.name());
+
+  ctx.check(
+      t_throws::is_null_or_empty(node.exceptions()),
+      "Oneway methods can't throw exceptions: {}",
+      node.name());
+}
+
+void validate_stream_exceptions_return_type(
+    diagnostic_context& ctx, const t_function& node) {
+  if (t_throws::is_null_or_empty(node.get_stream_xceptions())) {
+    return;
+  }
+
+  ctx.check(
+      dynamic_cast<const t_stream_response*>(node.return_type().get_type()),
+      "`stream throws` only valid on stream methods: {}",
+      node.name());
+}
+
+void validate_interaction_factories(
+    diagnostic_context& ctx, const t_function& node) {
+  if (node.is_interaction_member() && node.returned_interaction()) {
+    ctx.failure([&](auto& o) {
+      o << "Nested interactions are forbidden: " << node.name();
+    });
+  }
+}
+
+void validate_cpp_field_adapter_annotation(
+    diagnostic_context& ctx, const t_field& field) {
+  adapter_checker(ctx).check(
+      field,
+      kCppAdapterUri,
+      kCppUnstructuredAdapter,
+      "@cpp.Adapter",
+      false /* disallow_structured_annotations_on_both_field_and_typedef */);
+}
+
+void validate_hack_field_adapter_annotation(
+    diagnostic_context& ctx, const t_field& field) {
+  adapter_checker(ctx).check(
+      field,
+      kHackAdapterUri,
+      kHackUnstructuredAdapter,
+      "@hack.Adapter",
+      true /* disallow_structured_annotations_on_both_field_and_typedef */);
+}
 } // namespace
 
 ast_validator standard_validator() {
   ast_validator validator;
   validator.add_interface_visitor(&validate_interface_function_name_uniqueness);
+  validator.add_interface_visitor(&validate_function_priority_annotation);
   validator.add_service_visitor(
       &validate_extends_service_function_name_uniqueness);
   validator.add_throws_visitor(&validate_throws_exceptions);
+  validator.add_function_visitor(&validate_oneway_function);
+  validator.add_function_visitor(&validate_stream_exceptions_return_type);
+  validator.add_function_visitor(&validate_function_priority_annotation);
+  validator.add_function_visitor(&validate_interaction_factories);
 
   validator.add_structured_definition_visitor(&validate_field_names_uniqueness);
   validator.add_structured_definition_visitor(
       &validate_compatibility_with_lazy_field);
-  validator.add_structured_definition_visitor(
-      &validate_box_annotation_in_struct);
+  validator.add_structured_definition_visitor(&validate_box_annotation);
   validator.add_union_visitor(&validate_union_field_attributes);
+  validator.add_exception_visitor(&validate_exception_php_annotations);
   validator.add_field_visitor(&validate_field_id);
   validator.add_field_visitor(&validate_mixin_field_attributes);
   validator.add_field_visitor(&validate_boxed_field_attributes);
   validator.add_field_visitor(&validate_ref_field_attributes);
   validator.add_field_visitor(&validate_field_default_value);
   validator.add_field_visitor(&validate_ref_annotation);
-  validator.add_field_visitor(&validate_adapter_annotation);
-  validator.add_field_visitor(&validate_hack_adapter_annotation);
   validator.add_field_visitor(&validate_ref_unique_and_box_annotation);
+  validator.add_field_visitor(&validate_cpp_field_adapter_annotation);
+  validator.add_field_visitor(&validate_hack_field_adapter_annotation);
 
   validator.add_enum_visitor(&validate_enum_value_name_uniqueness);
   validator.add_enum_visitor(&validate_enum_value_uniqueness);
@@ -581,6 +777,8 @@ ast_validator standard_validator() {
 
   validator.add_definition_visitor(&validate_structured_annotation);
   validator.add_definition_visitor(&validate_annotation_scopes);
+  validator.add_definition_visitor(&validate_cpp_adapter_annotation);
+  validator.add_definition_visitor(&validate_hack_adapter_annotation);
 
   validator.add_const_visitor(&validate_const_type_and_value);
   validator.add_program_visitor(&validate_uri_uniqueness);
