@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -22,7 +22,6 @@
 
 #include <folly/portability/GMock.h>
 #include <folly/portability/GTest.h>
-#include <thrift/compiler/ast/diagnostic.h>
 #include <thrift/compiler/ast/diagnostic_context.h>
 #include <thrift/compiler/ast/t_base_type.h>
 #include <thrift/compiler/ast/t_enum.h>
@@ -36,6 +35,7 @@
 #include <thrift/compiler/ast/t_service.h>
 #include <thrift/compiler/ast/t_struct.h>
 #include <thrift/compiler/ast/t_typedef.h>
+#include <thrift/compiler/diagnostic.h>
 
 namespace apache::thrift::compiler {
 namespace {
@@ -43,12 +43,15 @@ namespace {
 using ::testing::UnorderedElementsAre;
 
 class StandardValidatorTest : public ::testing::Test {
+ public:
+  StandardValidatorTest()
+      : loc(source_mgr.add_string("/path/to/file.thrift", "").start) {}
+
  protected:
   std::vector<diagnostic> validate(
       diagnostic_params params = diagnostic_params::keep_all()) {
     diagnostic_results results;
-    diagnostic_context ctx{results, std::move(params)};
-    ctx.start_program(&program_);
+    diagnostic_context ctx(source_mgr, results, std::move(params));
     standard_validator()(ctx, program_);
     return std::move(results).diagnostics();
   }
@@ -66,11 +69,16 @@ class StandardValidatorTest : public ::testing::Test {
   diagnostic failure(int lineno, const std::string& msg) {
     return {diagnostic_level::failure, msg, "/path/to/file.thrift", lineno};
   }
+  diagnostic failure(const std::string& msg) {
+    return {diagnostic_level::failure, msg, "", 0};
+  }
 
   diagnostic warning(int lineno, const std::string& msg) {
     return {diagnostic_level::warning, msg, "/path/to/file.thrift", lineno};
   }
 
+  source_manager source_mgr;
+  source_location loc;
   t_program program_{"/path/to/file.thrift"};
 };
 
@@ -107,6 +115,53 @@ TEST_F(StandardValidatorTest, InterfaceNamesUniqueNoError) {
   EXPECT_THAT(validate(), ::testing::IsEmpty());
 }
 
+TEST_F(StandardValidatorTest, BadPriority) {
+  {
+    auto service = std::make_unique<t_service>(&program_, "Service");
+    service->set_lineno(1);
+    service->set_annotation("priority", "bad1");
+    auto fn = std::make_unique<t_function>(
+        &t_base_type::t_void(),
+        "foo",
+        std::make_unique<t_paramlist>(&program_));
+    fn->set_annotation("priority", "bad2");
+    fn->set_lineno(2);
+    service->add_function(std::move(fn));
+    program_.add_service(std::move(service));
+  }
+
+  {
+    auto interaction =
+        std::make_unique<t_interaction>(&program_, "Interaction");
+    interaction->set_lineno(3);
+    interaction->set_annotation("priority", "bad3");
+    auto fn = std::make_unique<t_function>(
+        &t_base_type::t_void(),
+        "foo",
+        std::make_unique<t_paramlist>(&program_));
+    fn->set_annotation("priority", "bad4");
+    fn->set_lineno(4);
+    interaction->add_function(std::move(fn));
+    program_.add_interaction(std::move(interaction));
+  }
+
+  EXPECT_THAT(
+      validate(),
+      ::testing::UnorderedElementsAre(
+          failure(
+              1,
+              "Bad priority 'bad1'. Choose one of 'HIGH_IMPORTANT','HIGH','IMPORTANT','NORMAL','BEST_EFFORT'."),
+          failure(
+              2,
+              "Bad priority 'bad2'. Choose one of 'HIGH_IMPORTANT','HIGH','IMPORTANT','NORMAL','BEST_EFFORT'."),
+          failure(
+              3,
+              "Bad priority 'bad3'. Choose one of 'HIGH_IMPORTANT','HIGH','IMPORTANT','NORMAL','BEST_EFFORT'."),
+          failure(
+              4,
+              "Bad priority 'bad4'. Choose one of 'HIGH_IMPORTANT','HIGH','IMPORTANT','NORMAL','BEST_EFFORT'.")));
+}
+
 TEST_F(StandardValidatorTest, ReapeatedNamesInService) {
   // Create interfaces with overlapping functions.
   {
@@ -203,14 +258,13 @@ TEST_F(StandardValidatorTest, DuplicatedEnumValues) {
 
   // Add enum_value with repeated value.
   auto enum_value_3 = std::make_unique<t_enum_value>("foo", 1);
-  enum_value_3->set_lineno(1);
   tenum_ptr->append(std::move(enum_value_3));
 
   // An error will be found.
   EXPECT_THAT(
       validate(),
-      UnorderedElementsAre(failure(
-          1, "Duplicate value `foo=1` with value `bar` in enum `foo`.")));
+      UnorderedElementsAre(
+          failure("Duplicate value `foo=1` with value `bar` in enum `foo`.")));
 }
 
 TEST_F(StandardValidatorTest, RepeatedNamesInEnumValues) {
@@ -315,8 +369,10 @@ TEST_F(StandardValidatorTest, FieldId) {
       std::make_unique<t_field>(t_base_type::t_i64(), "zero_id", 0));
   tstruct->append(
       std::make_unique<t_field>(t_base_type::t_i64(), "neg_id", -1));
-  tstruct->append(std::make_unique<t_field>(
-      t_base_type::t_i64(), "implicit_id", -2, false));
+  auto f = std::make_unique<t_field>(t_base_type::t_i64(), "implicit_id");
+  f->set_implicit_id(-2);
+  f->set_src_range({loc, loc});
+  tstruct->append(std::move(f));
 
   program_.add_struct(std::move(tstruct));
   EXPECT_THAT(
@@ -324,7 +380,7 @@ TEST_F(StandardValidatorTest, FieldId) {
       UnorderedElementsAre(
           failure(-1, "Zero value (0) not allowed as a field id for `zero_id`"),
           warning(
-              -1,
+              1,
               "No field id specified for `implicit_id`, resulting protocol may have conflicts or not be backwards compatible!")));
 }
 
@@ -343,21 +399,18 @@ TEST_F(StandardValidatorTest, MixinFieldType) {
   }
   {
     auto field = std::make_unique<t_field>(tunion.get(), "union_field", 2);
-    field->set_lineno(2);
     field->set_annotation("cpp.mixin");
     field->set_qualifier(t_field_qualifier::required);
     foo->append(std::move(field));
   }
   {
     auto field = std::make_unique<t_field>(texception.get(), "except_field", 3);
-    field->set_lineno(3);
     field->set_annotation("cpp.mixin");
     foo->append(std::move(field));
   }
   {
     auto field =
         std::make_unique<t_field>(&t_base_type::t_i32(), "other_field", 4);
-    field->set_lineno(4);
     field->set_annotation("cpp.mixin");
     foo->append(std::move(field));
   }
@@ -372,10 +425,8 @@ TEST_F(StandardValidatorTest, MixinFieldType) {
       UnorderedElementsAre(
           failure(1, "Mixin field `struct_field` cannot be optional."),
           failure(
-              3,
               "Mixin field `except_field` type must be a struct or union. Found `Exception`."),
           failure(
-              4,
               "Mixin field `other_field` type must be a struct or union. Found `i32`.")));
 }
 
@@ -406,8 +457,6 @@ TEST_F(StandardValidatorTest, RepeatedStructuredAnnotation) {
 }
 
 TEST_F(StandardValidatorTest, CustomDefaultValue) {
-  auto foo = std::make_unique<t_struct>(&program_, "Foo");
-
   auto custom_byte = std::make_unique<t_const_value>(
       static_cast<int64_t>(std::numeric_limits<int8_t>::max()) + 1);
   auto custom_short = std::make_unique<t_const_value>(
@@ -441,29 +490,34 @@ TEST_F(StandardValidatorTest, CustomDefaultValue) {
       "const_float_precision_loss",
       std::move(custom_float_precision_loss));
 
-  program_.add_const(std::move(const_byte));
-  program_.add_const(std::move(const_short));
-  program_.add_const(std::move(const_integer));
-  program_.add_const(std::move(const_float));
-  program_.add_const(std::move(const_float_precision_loss));
+  for (auto const_ptr :
+       {&const_byte,
+        &const_short,
+        &const_integer,
+        &const_float,
+        &const_float_precision_loss}) {
+    auto& c = *const_ptr;
+    c->set_src_range({loc, loc});
+    program_.add_const(std::move(c));
+  }
 
   EXPECT_THAT(
       validate(),
       UnorderedElementsAre(
           failure(
-              -1,
+              1,
               "value error: const `const_byte` has an invalid custom default value."),
           failure(
-              -1,
+              1,
               "value error: const `const_short` has an invalid custom default value."),
           failure(
-              -1,
+              1,
               "value error: const `const_integer` has an invalid custom default value."),
           failure(
-              -1,
+              1,
               "value error: const `const_float` has an invalid custom default value."),
           failure(
-              -1,
+              1,
               "value error: const `const_float_precision_loss` cannot be represented precisely as `float` or `double`.")));
 }
 
