@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -73,8 +73,8 @@ TEST_F(ValidatorTest, NestedInteractions) {
   t_program program("/path/to/file.thrift");
 
   auto interaction = std::make_unique<t_interaction>(&program, "foo");
-  auto func =
-      std::make_unique<t_function>(interaction.get(), "frobnicate", nullptr);
+  auto func = std::make_unique<t_function>(
+      interaction.get(), "frobnicate", std::make_unique<t_paramlist>(&program));
   func->set_lineno(42);
   interaction->add_function(std::move(func));
   program.add_interaction(std::move(interaction));
@@ -91,13 +91,14 @@ TEST_F(ValidatorTest, DuplicateInteractions) {
 
   auto interaction = std::make_unique<t_interaction>(&program, "Clyde");
   auto service = create_fake_service("foo");
-  auto func =
-      std::make_unique<t_function>(service.get(), "frobnicate", nullptr);
+  auto func = std::make_unique<t_function>(
+      service.get(), "frobnicate", std::make_unique<t_paramlist>(&program));
   func->set_lineno(1);
   func->set_is_interaction_constructor();
   service->add_function(std::move(func));
 
-  func = std::make_unique<t_function>(interaction.get(), "interact", nullptr);
+  func = std::make_unique<t_function>(
+      interaction.get(), "interact", std::make_unique<t_paramlist>(&program));
   func->set_lineno(2);
   func->set_is_interaction_constructor();
   service->add_function(std::move(func));
@@ -111,24 +112,16 @@ TEST_F(ValidatorTest, DuplicateInteractions) {
   func->set_is_interaction_constructor();
   service->add_function(std::move(func));
 
-  func = std::make_unique<t_function>(
-      interaction.get(), "functionInteract", nullptr);
-  func->set_lineno(4);
-  service->add_function(std::move(func));
-
   program.add_service(std::move(service));
 
   program.add_interaction(std::move(interaction));
 
   auto errors = run_validator<interactions_validator>(&program);
-  EXPECT_EQ(3, errors.size());
+  EXPECT_EQ(2, errors.size());
   EXPECT_EQ(
       "[FAILURE:/path/to/file.thrift:1] Only interactions can be performed.",
       errors.front().str());
   EXPECT_EQ(
       "[FAILURE:/path/to/file.thrift:3] Service `foo` has multiple methods for creating interaction `Clyde`.",
       errors.at(1).str());
-  EXPECT_EQ(
-      "[FAILURE:/path/to/file.thrift:4] Functions cannot return interactions.",
-      errors.at(2).str());
 }
