@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -121,9 +121,6 @@ bool interactions_validator::visit(t_service* s) {
     }
 
     if (!func->is_interaction_constructor()) {
-      if (ret->is_service()) {
-        add_error(func->get_lineno(), "Functions cannot return interactions.");
-      }
       continue;
     }
 
