@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -23,6 +23,7 @@
 #include <folly/lang/Exception.h>
 #include <thrift/conformance/cpp2/AnyRegistry.h>
 #include <thrift/conformance/cpp2/Object.h>
+#include <thrift/lib/cpp/util/EnumUtils.h>
 #include <thrift/lib/cpp2/op/Compare.h>
 #include <thrift/lib/cpp2/protocol/Serializer.h>
 
@@ -67,17 +68,31 @@ testing::TestInfo* RegisterTest(
 testing::AssertionResult RunRoundTripTest(
     ConformanceServiceAsyncClient& client, RoundTripTestCase roundTrip) {
   RoundTripResponse res;
-  client.sync_roundTrip(res, *roundTrip.request_ref());
+  try {
+    client.sync_roundTrip(res, *roundTrip.request());
+  } catch (const apache::thrift::TApplicationException&) {
+    return testing::AssertionFailure();
+  }
 
-  const Any& expectedAny = roundTrip.expectedResponse_ref()
-      ? *roundTrip.expectedResponse_ref().value_unchecked().value_ref()
-      : *roundTrip.request_ref()->value_ref();
+  const Any& expectedAny = roundTrip.expectedResponse()
+      ? *roundTrip.expectedResponse().value_unchecked().value()
+      : *roundTrip.request()->value();
 
-  // TODO(afuller): Make add asValueStruct support to AnyRegistry and use that
-  // instead of hard coding type.
-  auto actual = AnyRegistry::generated().load<Value>(*res.value_ref());
-  auto expected = AnyRegistry::generated().load<Value>(expectedAny);
-  if (!op::identical<type::struct_t<Value>>(actual, expected)) {
+  auto parseAny = [](const Any& a) {
+    switch (auto protocol = a.protocol().value_or(StandardProtocol::Compact)) {
+      case StandardProtocol::Compact:
+        return parseObject<apache::thrift::CompactProtocolReader>(*a.data());
+      case StandardProtocol::Binary:
+        return parseObject<apache::thrift::BinaryProtocolReader>(*a.data());
+      default:
+        throw std::invalid_argument(
+            "Unsupported protocol: " + util::enumNameSafe(protocol));
+    }
+  };
+
+  Object actual = parseAny(*res.value());
+  Object expected = parseAny(expectedAny);
+  if (!op::identical<type::struct_t<Object>>(actual, expected)) {
     // TODO(afuller): Report out the delta
     return testing::AssertionFailure();
   }
@@ -145,12 +160,12 @@ std::set<std::string> parseNonconforming(std::string_view data) {
 
 testing::AssertionResult RunTestCase(
     ConformanceServiceAsyncClient& client, const TestCase& testCase) {
-  switch (testCase.test_ref()->getType()) {
+  switch (testCase.test()->getType()) {
     case TestCaseUnion::roundTrip:
       return RunRoundTripTest(client, *testCase.roundTrip_ref());
     default:
       return testing::AssertionFailure()
-          << "Unsupported test case type: " << testCase.test_ref()->getType();
+          << "Unsupported test case type: " << testCase.test()->getType();
   }
 }
 
@@ -180,15 +195,15 @@ void RegisterTests(
     std::function<ConformanceServiceAsyncClient&()> clientFn,
     const char* file,
     int line) {
-  for (const auto& test : *suite->tests_ref()) {
-    for (const auto& testCase : *test.testCases_ref()) {
-      std::string suiteName = fmt::format(
-          "{}/{}/{}", category, *suite->name_ref(), *testCase.name_ref());
-      std::string fullName = fmt::format("{}.{}", suiteName, *test.name_ref());
+  for (const auto& test : *suite->tests()) {
+    for (const auto& testCase : *test.testCases()) {
+      std::string suiteName =
+          fmt::format("{}/{}/{}", category, *suite->name(), *testCase.name());
+      std::string fullName = fmt::format("{}.{}", suiteName, *test.name());
       bool conforming = nonconforming.find(fullName) == nonconforming.end();
       RegisterTest(
           suiteName.c_str(),
-          test.name_ref()->c_str(),
+          test.name()->c_str(),
           nullptr,
           conforming ? nullptr : "nonconforming",
           file,
