@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,20 +15,12 @@
  */
 
 #pragma once
-
-#include <thrift/conformance/cpp2/internal/Object.h>
+#include <thrift/conformance/if/gen-cpp2/object_types.h>
+#include <thrift/lib/cpp2/Object.h>
+// TODO: remove this file
 
 namespace apache::thrift::conformance {
-
-// Creates a Value struct for the given value.
-//
-// TT: The thrift type to use, for example
-// apache::thrift::conformance::type::binary_t.
-template <typename TT, typename T>
-Value asValueStruct(T&& value) {
-  Value result;
-  detail::ValueHelper<TT>::set(result, std::forward<T>(value));
-  return result;
-}
-
+using ::apache::thrift::type::asValueStruct;
+using ::apache::thrift::type::parseObject;
+using ::apache::thrift::type::serializeObject;
 } // namespace apache::thrift::conformance
