@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -19,10 +19,22 @@
 #include <initializer_list>
 #include <set>
 
+#include <boost/mp11.hpp>
+
 #include <thrift/conformance/cpp2/Protocol.h>
 
 namespace apache::thrift::conformance::data::detail {
 
+using PrimaryTypeTags = boost::mp11::mp_list<
+    type::bool_t,
+    type::byte_t,
+    type::i16_t,
+    type::i32_t,
+    type::float_t,
+    type::double_t,
+    type::string_t,
+    type::binary_t>;
+
 template <typename C>
 std::set<Protocol> toProtocols(const C& protocolCtorArgs) {
   std::set<Protocol> result;
@@ -33,8 +45,6 @@ std::set<Protocol> toProtocols(const C& protocolCtorArgs) {
 }
 
 constexpr std::initializer_list<StandardProtocol> kDefaultProtocols = {
-    StandardProtocol::Binary,
-    StandardProtocol::Compact,
-    StandardProtocol::SimpleJson};
+    StandardProtocol::Binary, StandardProtocol::Compact};
 
 } // namespace apache::thrift::conformance::data::detail
