@@ -1,11 +1,11 @@
 /*
- * Copyright 2017-present Facebook, Inc.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -24,8 +24,7 @@ namespace chatroom {
 class EchoHandler : virtual public EchoSvIf {
  public:
   void echo(
-      std::string& response,
-      std::unique_ptr<std::string> message) override;
+      std::string& response, std::unique_ptr<std::string> message) override;
 };
-} // chatroom
-} // example
+} // namespace chatroom
+} // namespace example
