@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -142,37 +142,6 @@ auto& getThreadManagerObserverSingleton() {
   return observer;
 }
 
-class ThreadIdCollector : public folly::WorkerProvider {
- public:
-  ThreadIdCollector() {}
-
-  IdsWithKeepAlive collectThreadIds() override final {
-    auto keepAlive = std::make_unique<WorkerKeepAlive>(
-        folly::SharedMutex::ReadHolder{&threadsExitMutex_});
-    auto locked = osThreadIds_.rlock();
-    return {std::move(keepAlive), {locked->begin(), locked->end()}};
-  }
-
-  folly::Synchronized<std::unordered_set<pid_t>> osThreadIds_;
-  folly::SharedMutex threadsExitMutex_;
-
- protected:
-  class WorkerKeepAlive : public folly::WorkerProvider::KeepAlive {
-   public:
-    explicit WorkerKeepAlive(folly::SharedMutex::ReadHolder idsLock)
-        : threadsExitLock_(std::move(idsLock)) {}
-    ~WorkerKeepAlive() override {}
-
-   private:
-    folly::SharedMutex::ReadHolder threadsExitLock_;
-  };
-};
-
-inline ThreadIdCollector* upcast(
-    std::unique_ptr<folly::WorkerProvider>& wpPtr) {
-  return static_cast<ThreadIdCollector*>(wpPtr.get());
-}
-
 } // namespace
 
 /**
@@ -355,6 +324,7 @@ class ThreadManager::Impl : public ThreadManager,
   }
 
   void enableCodel(bool) override;
+  bool codelEnabled() const override;
   folly::Codel* getCodel() override;
 
   // Methods to be invoked by workers
@@ -389,11 +359,10 @@ class ThreadManager::Impl : public ThreadManager,
   // returns a string to attach to namePrefix when recording
   // stats
   virtual std::string statContext(PRIORITY = PRIORITY::NORMAL) { return ""; }
-  // Creates a WorkerProvider instance which can be used to collect stack traces
+  // A WorkerProvider instance which can be used to collect stack traces
   // from threads consuming from lagging queues.
-  std::unique_ptr<folly::WorkerProvider> createWorkerProvider();
-  std::unique_ptr<folly::WorkerProvider> threadIdCollector_{
-      createWorkerProvider()};
+  std::unique_ptr<folly::ThreadIdWorkerProvider> threadIdCollector_{
+      std::make_unique<folly::ThreadIdWorkerProvider>()};
 
  private:
   void stopImpl(bool joinArg);
@@ -579,6 +548,9 @@ void SimpleThreadManager::clearPending() {
 void SimpleThreadManager::enableCodel(bool value) {
   return impl_->enableCodel(value);
 }
+bool SimpleThreadManager::codelEnabled() const {
+  return impl_->codelEnabled();
+}
 folly::Codel* SimpleThreadManager::getCodel() {
   return impl_->getCodel();
 }
@@ -623,16 +595,15 @@ class ThreadManager::Impl::Worker : public Runnable, public WorkerBaseHook {
     manager_->workerStarted(this);
 
     // Capture the current threads ID in the ThreadManager's tracking list.
-    auto collectorPtr = upcast(manager_->threadIdCollector_);
-    collectorPtr->osThreadIds_.wlock()->insert(folly::getOSThreadID());
+    auto collectorPtr = manager_->threadIdCollector_.get();
+    collectorPtr->addTid(folly::getOSThreadID());
     // On exit, we should remove the thread ID from the collector's tracking
     // list.e
     auto threadIdsGuard = folly::makeGuard([collectorPtr]() {
       // The observer could be capturing a stack trace from this thread
       // so it should block until the collection finishes to exit.
       if (collectorPtr) {
-        collectorPtr->osThreadIds_.wlock()->erase(folly::getOSThreadID());
-        folly::SharedMutex::WriteHolder w{collectorPtr->threadsExitMutex_};
+        collectorPtr->removeTid(folly::getOSThreadID());
       }
     });
 
@@ -1064,6 +1035,10 @@ void ThreadManager::Impl::enableCodel(bool enabled) {
   codelEnabled_ = enabled || FLAGS_codel_enabled;
 }
 
+bool ThreadManager::Impl::codelEnabled() const {
+  return codelEnabled_;
+}
+
 folly::Codel* ThreadManager::Impl::getCodel() {
   return &codel_;
 }
@@ -1080,11 +1055,6 @@ void ThreadManager::Impl::setupQueueObservers() {
   }
 }
 
-std::unique_ptr<folly::WorkerProvider>
-ThreadManager::Impl::createWorkerProvider() {
-  return std::make_unique<ThreadIdCollector>();
-}
-
 class PriorityThreadManager::PriorityImpl
     : public PriorityThreadManager,
       public folly::DefaultKeepAliveExecutor {
@@ -1327,6 +1297,15 @@ class PriorityThreadManager::PriorityImpl
     }
   }
 
+  bool codelEnabled() const override {
+    for (const auto& m : managers_) {
+      if (m->codelEnabled()) {
+        return true;
+      }
+    }
+    return false;
+  }
+
   folly::Codel* getCodel() override { return getCodel(NORMAL); }
 
   folly::Codel* getCodel(PRIORITY priority) override {
@@ -1647,8 +1626,18 @@ size_t ThreadManagerExecutorAdapter::workerCount() const {
   return aggregateForEachThreadManager(executors_, &ThreadManager::workerCount);
 }
 size_t ThreadManagerExecutorAdapter::pendingUpstreamTaskCount() const {
-  return aggregateForEachThreadManager(
-      executors_, &ThreadManager::pendingUpstreamTaskCount);
+  size_t count = 0;
+  for (size_t i = 0; i < N_PRIORITIES; i++) {
+    auto executorIdx = idxFromPriSrc(i, static_cast<int>(Source::UPSTREAM));
+    if (auto tm = dynamic_cast<ThreadManager*>(executors_[executorIdx])) {
+      count += tm->pendingUpstreamTaskCount();
+    } else if (
+        auto meteredExecutor =
+            dynamic_cast<folly::MeteredExecutor*>(executors_[executorIdx])) {
+      count += meteredExecutor->pendingTasks();
+    }
+  }
+  return count;
 }
 size_t ThreadManagerExecutorAdapter::pendingTaskCount() const {
   return aggregateForEachThreadManager(
