@@ -1,11 +1,11 @@
 /*
- * Copyright 2004-present Facebook, Inc.
+ * Copyright (c) Facebook, Inc. and its affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -19,16 +19,17 @@
 #include <boost/random.hpp>
 #include <folly/portability/GTest.h>
 
-#include <thrift/lib/cpp/async/TAsyncSocket.h>
-#include <thrift/lib/cpp/async/TAsyncTransport.h>
+#include <folly/io/async/AsyncSocket.h>
 #include <folly/io/async/AsyncTimeout.h>
-#include <thrift/lib/cpp/async/TBinaryAsyncChannel.h>
+#include <folly/io/async/AsyncTransport.h>
 #include <folly/io/async/EventBase.h>
+#include <folly/io/async/test/SocketPair.h>
+#include <folly/io/async/test/Util.h>
+#include <folly/net/NetworkSocket.h>
+#include <thrift/lib/cpp/async/TBinaryAsyncChannel.h>
 #include <thrift/lib/cpp/async/TFramedAsyncChannel.h>
 #include <thrift/lib/cpp/concurrency/Util.h>
 #include <thrift/lib/cpp/protocol/TBinaryProtocol.h>
-#include <folly/io/async/test/SocketPair.h>
-#include <folly/io/async/test/Util.h>
 
 using namespace boost;
 using namespace folly;
@@ -39,44 +40,46 @@ using std::string;
 using std::vector;
 
 using apache::thrift::async::TAsyncChannel;
-using apache::thrift::async::TAsyncTransport;
-using apache::thrift::async::TAsyncSocket;
-using folly::AsyncTimeout;
 using apache::thrift::async::TBinaryAsyncChannel;
-using folly::EventBase;
 using apache::thrift::async::TFramedAsyncChannel;
 using apache::thrift::protocol::TBinaryProtocolT;
 using apache::thrift::transport::TBufferBase;
 using apache::thrift::transport::TFramedTransport;
 using apache::thrift::transport::TMemoryBuffer;
 using apache::thrift::transport::TTransportException;
+using folly::AsyncSocket;
+using folly::AsyncTimeout;
+using folly::EventBase;
 
 ///////////////////////////////////////////////////////////////////////////
 // Utility code
 //////////////////////////////////////////////////////////////////////////
 
-
 class ChannelCallback {
  public:
   ChannelCallback()
-    : sendDone_(0)
-    , sendError_(0)
-    , recvDone_(0)
-    , recvError_(0)
-    , timestamp_(false) {}
+      : sendDone_(0),
+        sendError_(0),
+        recvDone_(0),
+        recvError_(0),
+        timestamp_(false) {}
 
-  void send(const std::shared_ptr<TAsyncChannel>& channel,
-            TMemoryBuffer* message) {
+  void send(
+      const std::shared_ptr<TAsyncChannel>& channel, TMemoryBuffer* message) {
     // helper function to get the correct callbacks
-    channel->sendMessage(std::bind(&ChannelCallback::sendDone, this),
-                         std::bind(&ChannelCallback::sendError, this), message);
+    channel->sendMessage(
+        std::bind(&ChannelCallback::sendDone, this),
+        std::bind(&ChannelCallback::sendError, this),
+        message);
   }
 
-  void recv(const std::shared_ptr<TAsyncChannel>& channel,
-            TMemoryBuffer* message) {
+  void recv(
+      const std::shared_ptr<TAsyncChannel>& channel, TMemoryBuffer* message) {
     // helper function to get the correct callbacks
-    channel->recvMessage(std::bind(&ChannelCallback::recvDone, this),
-                         std::bind(&ChannelCallback::recvError, this), message);
+    channel->recvMessage(
+        std::bind(&ChannelCallback::recvDone, this),
+        std::bind(&ChannelCallback::recvError, this),
+        message);
   }
 
   void sendDone() {
@@ -97,22 +100,12 @@ class ChannelCallback {
     timestamp_.reset();
   }
 
-  uint32_t getSendDone() const {
-    return sendDone_;
-  }
-  uint32_t getSendError() const {
-    return sendError_;
-  }
-  uint32_t getRecvDone() const {
-    return recvDone_;
-  }
-  uint32_t getRecvError() const {
-    return recvError_;
-  }
+  uint32_t getSendDone() const { return sendDone_; }
+  uint32_t getSendError() const { return sendError_; }
+  uint32_t getRecvDone() const { return recvDone_; }
+  uint32_t getRecvError() const { return recvError_; }
 
-  const TimePoint& getTimestamp() const {
-    return timestamp_;
-  }
+  const TimePoint& getTimestamp() const { return timestamp_; }
 
  private:
   uint32_t sendDone_;
@@ -125,8 +118,7 @@ class ChannelCallback {
 class Message {
  public:
   explicit Message(uint32_t len, bool addFrame = false)
-    : framed_(addFrame)
-    , buf_(new TMemoryBuffer(len + kPadding)) {
+      : framed_(addFrame), buf_(new TMemoryBuffer(len + kPadding)) {
     init(len);
   }
 
@@ -183,9 +175,7 @@ class Message {
     CHECK_EQ(memcmp(otherBuf, myBuf, length), 0);
   }
 
-  uint32_t getLength() const {
-    return buf_->available_read();
-  }
+  uint32_t getLength() const { return buf_->available_read(); }
 
   const uint8_t* getBuffer() const {
     uint32_t borrowLen = getLength();
@@ -215,11 +205,8 @@ class Message {
   std::shared_ptr<TMemoryBuffer> buf_;
 };
 
-
 struct ChunkInfo {
-  ChunkInfo(int b, milliseconds d)
-    : bytes(b)
-    , delayMS(d) {}
+  ChunkInfo(int b, milliseconds d) : bytes(b), delayMS(d) {}
 
   int bytes;
   milliseconds delayMS;
@@ -235,32 +222,28 @@ struct ChunkInfo {
  */
 using ChunkSchedule = vector<ChunkInfo>;
 
-class ChunkSender : private TAsyncTransport::WriteCallback,
+class ChunkSender : private AsyncTransport::WriteCallback,
                     private AsyncTimeout {
  public:
-  ChunkSender(EventBase* evb, TAsyncSocket* socket,
-              const Message* msg, const ChunkSchedule& schedule)
-    : AsyncTimeout(evb)
-    , bufOffset_(0)
-    , scheduleIndex_(0)
-    , currentChunkLen_(0)
-    , error_(false)
-    , socket_(socket)
-    , message_(msg)
-    , schedule_(schedule) {
-  }
+  ChunkSender(
+      EventBase* evb,
+      AsyncSocket* socket,
+      const Message* msg,
+      const ChunkSchedule& schedule)
+      : AsyncTimeout(evb),
+        bufOffset_(0),
+        scheduleIndex_(0),
+        currentChunkLen_(0),
+        error_(false),
+        socket_(socket),
+        message_(msg),
+        schedule_(schedule) {}
 
-  void start() {
-    scheduleNext();
-  }
+  void start() { scheduleNext(); }
 
-  const ChunkSchedule* getSchedule() const {
-    return &schedule_;
-  }
+  const ChunkSchedule* getSchedule() const { return &schedule_; }
 
-  bool error() const {
-    return error_;
-  }
+  bool error() const { return error_; }
 
  private:
   void scheduleNext() {
@@ -322,9 +305,8 @@ class ChunkSender : private TAsyncTransport::WriteCallback,
     }
   }
 
-  void writeError(
-      size_t /* bytesWritten */,
-      const TTransportException&) noexcept override {
+  void writeErr(size_t /* bytesWritten */, const AsyncSocketException&) noexcept
+      override {
     error_ = true;
   }
 
@@ -334,148 +316,138 @@ class ChunkSender : private TAsyncTransport::WriteCallback,
   uint32_t scheduleIndex_;
   uint32_t currentChunkLen_;
   bool error_;
-  TAsyncSocket* socket_;
+  AsyncSocket* socket_;
   const Message* message_;
   ChunkSchedule schedule_;
 };
 
 class MultiMessageSize : public vector<int> {
-  public:
-    MultiMessageSize(int len, ...) {
-      push_back(len);
+ public:
+  MultiMessageSize(int len, ...) {
+    push_back(len);
 
-      va_list ap;
-      va_start(ap, len);
+    va_list ap;
+    va_start(ap, len);
 
-      while (true) {
-        int b = va_arg(ap, int);
-        if (b <= 0) {
-          break;
-        } else {
-          push_back(b);
-        }
+    while (true) {
+      int b = va_arg(ap, int);
+      if (b <= 0) {
+        break;
+      } else {
+        push_back(b);
       }
-
-      va_end(ap);
     }
+
+    va_end(ap);
+  }
 };
 
-class MultiMessageSenderReceiver : private TAsyncTransport::WriteCallback,
+class MultiMessageSenderReceiver : private AsyncTransport::WriteCallback,
                                    private AsyncTimeout {
-  public:
-    MultiMessageSenderReceiver(EventBase* evb,
-                               TAsyncSocket* socket,
-                               const MultiMessageSize& multiMessage,
-                               bool framed,
-                               uint32_t writeTimes,
-                               bool queued = false,
-                               milliseconds delayMS = milliseconds(2))
-    : AsyncTimeout(evb)
-    , writeError_(false)
-    , readError_(false)
-    , socket_(socket)
-    , queued_(queued)
-    , delayMS_(delayMS) {
-      uint32_t totalSize = 0;
-      for (vector<int>::const_iterator it = multiMessage.begin();
-           it != multiMessage.end();
-           it++) {
-        Message message(*it, framed);
-        writeMessages_.push_back(message);
-        writeMemoryBuffer_.write(message.getBuffer(), message.getLength());
-        totalSize += message.getLength();
-      }
-
-      assert(writeMessages_.size() > 0);
-      writeSize_ = totalSize / writeTimes;
+ public:
+  MultiMessageSenderReceiver(
+      EventBase* evb,
+      AsyncSocket* socket,
+      const MultiMessageSize& multiMessage,
+      bool framed,
+      uint32_t writeTimes,
+      bool queued = false,
+      milliseconds delayMS = milliseconds(2))
+      : AsyncTimeout(evb),
+        writeError_(false),
+        readError_(false),
+        socket_(socket),
+        queued_(queued),
+        delayMS_(delayMS) {
+    uint32_t totalSize = 0;
+    for (vector<int>::const_iterator it = multiMessage.begin();
+         it != multiMessage.end();
+         it++) {
+      Message message(*it, framed);
+      writeMessages_.push_back(message);
+      writeMemoryBuffer_.write(message.getBuffer(), message.getLength());
+      totalSize += message.getLength();
     }
 
-    void initialize(const std::shared_ptr<TAsyncChannel>& channel) {
-      int n_msgs = writeMessages_.size();
-      uint32_t n_recvs = (queued_) ? n_msgs : 1;
-      for (uint32_t i = 0; i < n_recvs; i++) {
-        channel->recvMessage(
+    assert(writeMessages_.size() > 0);
+    writeSize_ = totalSize / writeTimes;
+  }
+
+  void initialize(const std::shared_ptr<TAsyncChannel>& channel) {
+    int n_msgs = writeMessages_.size();
+    uint32_t n_recvs = (queued_) ? n_msgs : 1;
+    for (uint32_t i = 0; i < n_recvs; i++) {
+      channel->recvMessage(
           std::bind(&MultiMessageSenderReceiver::recvDone, this),
           std::bind(&MultiMessageSenderReceiver::recvError, this),
           &readMemoryBuffer_);
-      }
+    }
+    scheduleNext();
+    recvChannel_ = channel;
+  }
+
+  bool getReadError() const { return readError_; }
+
+  bool getWriteError() const { return writeError_; }
+
+  vector<std::shared_ptr<TMemoryBuffer>>& getReadBuffers() {
+    return readBuffers_;
+  }
+
+  vector<Message>& getWriteMessages() { return writeMessages_; }
+
+  void recvDone() {
+    uint8_t* request;
+    uint32_t requestLen;
+    readMemoryBuffer_.extractReadBuffer(&request, &requestLen);
+    if (requestLen > 0) {
+      std::shared_ptr<TMemoryBuffer> recvBuffer(new TMemoryBuffer(
+          request, requestLen, TMemoryBuffer::TAKE_OWNERSHIP));
+      readBuffers_.push_back(recvBuffer);
+    } else if (request) {
+      delete request;
+    }
+
+    // Read another message if we haven't read all of the messages yet
+    if (!queued_ && readBuffers_.size() < writeMessages_.size()) {
+      recvChannel_->recvMessage(
+          std::bind(&MultiMessageSenderReceiver::recvDone, this),
+          std::bind(&MultiMessageSenderReceiver::recvError, this),
+          &readMemoryBuffer_);
+    }
+  }
+
+  void recvError() { readError_ = true; }
+
+  void writeSuccess() noexcept override {
+    uint32_t sentSize =
+        std::min(writeSize_, writeMemoryBuffer_.available_read());
+    writeMemoryBuffer_.consume(sentSize);
+    if (writeMemoryBuffer_.available_read() > 0) {
       scheduleNext();
-      recvChannel_ = channel;
     }
+  }
 
-    bool getReadError() const {
-      return readError_;
-    }
-
-    bool getWriteError() const {
-      return writeError_;
-    }
-
-    vector<std::shared_ptr<TMemoryBuffer> >& getReadBuffers() {
-      return readBuffers_;
-    }
-
-    vector<Message>& getWriteMessages() {
-      return writeMessages_;
-    }
-
-    void recvDone() {
-      uint8_t* request;
-      uint32_t requestLen;
-      readMemoryBuffer_.extractReadBuffer(&request, &requestLen);
-      if (requestLen > 0) {
-        std::shared_ptr<TMemoryBuffer> recvBuffer(new TMemoryBuffer(
-                                  request,
-                                  requestLen,
-                                  TMemoryBuffer::TAKE_OWNERSHIP));
-        readBuffers_.push_back(recvBuffer);
-      } else if (request) {
-        delete request;
-      }
-
-      // Read another message if we haven't read all of the messages yet
-      if (!queued_ && readBuffers_.size() < writeMessages_.size()) {
-        recvChannel_->recvMessage(
-            std::bind(&MultiMessageSenderReceiver::recvDone, this),
-            std::bind(&MultiMessageSenderReceiver::recvError, this),
-            &readMemoryBuffer_);
-      }
-    }
-
-    void recvError() {
-      readError_ = true;
-    }
-
-    void writeSuccess() noexcept override {
-      uint32_t sentSize = std::min(writeSize_,
-                                   writeMemoryBuffer_.available_read());
-      writeMemoryBuffer_.consume(sentSize);
-      if (writeMemoryBuffer_.available_read() > 0) {
-        scheduleNext();
-      }
-    }
-
-    void writeError(
-        size_t /* bytesWritten */,
-        const TTransportException&) noexcept override {
-      writeError_ = true;
-    }
+  void writeErr(size_t /* bytesWritten */, const AsyncSocketException&) noexcept
+      override {
+    writeError_ = true;
+  }
 
  private:
-   void scheduleNext() {
-     if (delayMS_.count() <= 0) {
-       send();
-     } else {
-       scheduleTimeout(delayMS_);
-     }
-   }
+  void scheduleNext() {
+    if (delayMS_.count() <= 0) {
+      send();
+    } else {
+      scheduleTimeout(delayMS_);
+    }
+  }
 
-   void timeoutExpired() noexcept override { send(); }
+  void timeoutExpired() noexcept override { send(); }
 
   void send() {
     uint32_t availableSize = writeMemoryBuffer_.available_read();
-    const uint8_t* sendBufPtr = writeMemoryBuffer_.borrow(nullptr,
-                                                          &availableSize);
+    const uint8_t* sendBufPtr =
+        writeMemoryBuffer_.borrow(nullptr, &availableSize);
     uint32_t sendSize = std::min(writeSize_, availableSize);
     if (sendSize > 0) {
       socket_->write(this, sendBufPtr, sendSize);
@@ -484,9 +456,9 @@ class MultiMessageSenderReceiver : private TAsyncTransport::WriteCallback,
 
   bool writeError_;
   bool readError_;
-  TAsyncSocket* socket_;
+  AsyncSocket* socket_;
   vector<Message> writeMessages_;
-  vector<std::shared_ptr<TMemoryBuffer> > readBuffers_;
+  vector<std::shared_ptr<TMemoryBuffer>> readBuffers_;
   TMemoryBuffer writeMemoryBuffer_;
   TMemoryBuffer readMemoryBuffer_;
   uint32_t writeSize_;
@@ -498,9 +470,8 @@ class MultiMessageSenderReceiver : private TAsyncTransport::WriteCallback,
 class EventBaseAborter : public AsyncTimeout {
  public:
   EventBaseAborter(EventBase* eventBase, uint32_t timeoutMS)
-    : AsyncTimeout(eventBase,
-                    AsyncTimeout::InternalEnum::INTERNAL)
-    , eventBase_(eventBase) {
+      : AsyncTimeout(eventBase, AsyncTimeout::InternalEnum::INTERNAL),
+        eventBase_(eventBase) {
     scheduleTimeout(timeoutMS);
   }
 
@@ -513,20 +484,20 @@ class EventBaseAborter : public AsyncTimeout {
   EventBase* eventBase_;
 };
 
-template<typename ChannelT>
+template <typename ChannelT>
 class SocketPairTest {
  public:
-  SocketPairTest()
-    : eventBase_()
-    , socketPair_() {
-    socket0_ = TAsyncSocket::newSocket(&eventBase_, socketPair_[0]);
-    socketPair_.extractFD0();
+  SocketPairTest() : eventBase_(), socketPair_() {
+    auto socket0 = AsyncSocket::newSocket(
+        &eventBase_, socketPair_.extractNetworkSocket0());
+    socket0_ = socket0.get();
 
-    socket1_ = TAsyncSocket::newSocket(&eventBase_, socketPair_[1]);
-    socketPair_.extractFD1();
+    auto socket1 = AsyncSocket::newSocket(
+        &eventBase_, socketPair_.extractNetworkSocket1());
+    socket1_ = socket1.get();
 
-    channel0_ = ChannelT::newChannel(socket0_);
-    channel1_ = ChannelT::newChannel(socket1_);
+    channel0_ = ChannelT::newChannel(std::move(socket0));
+    channel1_ = ChannelT::newChannel(std::move(socket1));
   }
   virtual ~SocketPairTest() {}
 
@@ -535,9 +506,7 @@ class SocketPairTest {
     eventBase_.loop();
   }
 
-  virtual void run() {
-    runWithTimeout(3000);
-  }
+  virtual void run() { runWithTimeout(3000); }
 
   virtual void runWithTimeout(uint32_t timeoutMS) {
     preLoop();
@@ -551,42 +520,35 @@ class SocketPairTest {
  protected:
   EventBase eventBase_;
   SocketPair socketPair_;
-  std::shared_ptr<TAsyncSocket> socket0_;
-  std::shared_ptr<TAsyncSocket> socket1_;
+  AsyncSocket* socket0_;
+  AsyncSocket* socket1_;
   std::shared_ptr<ChannelT> channel0_;
   std::shared_ptr<ChannelT> channel1_;
 };
 
-template<typename ChannelT>
-class NeedsFrame {
-};
+template <typename ChannelT>
+class NeedsFrame {};
 
-template<>
+template <>
 class NeedsFrame<TFramedAsyncChannel> {
  public:
-  static bool value() {
-    return true;
-  }
+  static bool value() { return true; }
 };
 
-template<>
+template <>
 class NeedsFrame<TBinaryAsyncChannel> {
  public:
-  static bool value() {
-    return false;
-  }
+  static bool value() { return false; }
 };
 
 ///////////////////////////////////////////////////////////////////////////
 // Test cases
 //////////////////////////////////////////////////////////////////////////
 
-template<typename ChannelT>
+template <typename ChannelT>
 class SendRecvTest : public SocketPairTest<ChannelT> {
  public:
-  explicit SendRecvTest(uint32_t msgLen)
-    : msg_(msgLen) {
-  }
+  explicit SendRecvTest(uint32_t msgLen) : msg_(msgLen) {}
 
   void preLoop() override {
     msg_.copyTo(&sendBuf_);
@@ -613,32 +575,33 @@ class SendRecvTest : public SocketPairTest<ChannelT> {
 TEST(TAsyncChannelTest, TestSendRecvFramed) {
   SendRecvTest<TFramedAsyncChannel>(1).run();
   SendRecvTest<TFramedAsyncChannel>(100).run();
-  SendRecvTest<TFramedAsyncChannel>(1024*1024).run();
+  SendRecvTest<TFramedAsyncChannel>(1024 * 1024).run();
 }
 
 TEST(TAsyncChannelTest, TestSendRecvBinary) {
   SendRecvTest<TBinaryAsyncChannel>(1).run();
   SendRecvTest<TBinaryAsyncChannel>(100).run();
-  SendRecvTest<TBinaryAsyncChannel>(1024*1024).run();
+  SendRecvTest<TBinaryAsyncChannel>(1024 * 1024).run();
 }
 
-template<typename ChannelT>
+template <typename ChannelT>
 class MultiSendRecvTest : public SocketPairTest<ChannelT> {
-  public:
-    MultiSendRecvTest(const MultiMessageSize& multiMessage,
-                      uint32_t writeTimes,
-                      bool queued = false,
-                      milliseconds delayMS = milliseconds(0))
-    : multiMessageSenderReceiver_(&this->eventBase_,
-                                  this->socket0_.get(),
-                                  multiMessage,
-                                  NeedsFrame<ChannelT>::value(),
-                                  writeTimes,
-                                  queued,
-                                  delayMS) {
-  }
+ public:
+  MultiSendRecvTest(
+      const MultiMessageSize& multiMessage,
+      uint32_t writeTimes,
+      bool queued = false,
+      milliseconds delayMS = milliseconds(0))
+      : multiMessageSenderReceiver_(
+            &this->eventBase_,
+            this->socket0_,
+            multiMessage,
+            NeedsFrame<ChannelT>::value(),
+            writeTimes,
+            queued,
+            delayMS) {}
 
-    void preLoop() override {
+  void preLoop() override {
     multiMessageSenderReceiver_.initialize(this->channel1_);
   }
 
@@ -646,24 +609,24 @@ class MultiSendRecvTest : public SocketPairTest<ChannelT> {
     CHECK_EQ(multiMessageSenderReceiver_.getReadError(), false);
     CHECK_EQ(multiMessageSenderReceiver_.getWriteError(), false);
 
-    vector<std::shared_ptr<TMemoryBuffer> >& readBuffers
-                      = multiMessageSenderReceiver_.getReadBuffers();
-    vector<Message>& writeMessages
-                      = multiMessageSenderReceiver_.getWriteMessages();
+    vector<std::shared_ptr<TMemoryBuffer>>& readBuffers =
+        multiMessageSenderReceiver_.getReadBuffers();
+    vector<Message>& writeMessages =
+        multiMessageSenderReceiver_.getWriteMessages();
     CHECK_EQ(readBuffers.size(), writeMessages.size());
     for (size_t i = 0; i < writeMessages.size(); i++) {
       writeMessages[i].checkEqual(readBuffers[i].get());
     }
   }
 
-  private:
-    MultiMessageSenderReceiver multiMessageSenderReceiver_;
+ private:
+  MultiMessageSenderReceiver multiMessageSenderReceiver_;
 };
 
 TEST(TAsyncChannelTest, TestMultiSendRecvBinary) {
   typedef MultiSendRecvTest<TBinaryAsyncChannel> MultiSendRecvBinaryTest;
 
-  //size below 1024 for each message
+  // size below 1024 for each message
   MultiMessageSize sizes(911, 911, 911, -1);
 
   // each time send one whole message below 1024
@@ -677,11 +640,8 @@ TEST(TAsyncChannelTest, TestMultiSendRecvBinary) {
   MultiSendRecvBinaryTest(sizes, 2).run();
   MultiSendRecvBinaryTest(sizes, 2, 2).run();
 
-  //size above 1024 for each message
-  MultiMessageSize bigSizes(1911 * 1911,
-                            1911 * 1911,
-                            1911 * 1911,
-                            -1);
+  // size above 1024 for each message
+  MultiMessageSize bigSizes(1911 * 1911, 1911 * 1911, 1911 * 1911, -1);
 
   // each time send one whole message above 1024
   MultiSendRecvBinaryTest(bigSizes, 3, 0).run();
@@ -698,7 +658,7 @@ TEST(TAsyncChannelTest, TestMultiSendRecvBinary) {
 TEST(TAsyncChannelTest, TestMultiSendRecvBinaryQueued) {
   typedef MultiSendRecvTest<TBinaryAsyncChannel> MultiSendRecvBinaryTest;
 
-  //size below 1024 for each message
+  // size below 1024 for each message
   MultiMessageSize sizes(911, 911, 911, -1);
 
   // each time send one whole message below 1024
@@ -712,11 +672,8 @@ TEST(TAsyncChannelTest, TestMultiSendRecvBinaryQueued) {
   MultiSendRecvBinaryTest(sizes, 2, true).run();
   MultiSendRecvBinaryTest(sizes, 2, true, milliseconds(2)).run();
 
-  //size above 1024 for each message
-  MultiMessageSize bigSizes(1911 * 1911,
-                            1911 * 1911,
-                            1911 * 1911,
-                            -1);
+  // size above 1024 for each message
+  MultiMessageSize bigSizes(1911 * 1911, 1911 * 1911, 1911 * 1911, -1);
 
   // each time send one whole message above 1024
   MultiSendRecvBinaryTest(bigSizes, 3, true, milliseconds(0)).run();
@@ -733,7 +690,7 @@ TEST(TAsyncChannelTest, TestMultiSendRecvBinaryQueued) {
 TEST(TAsyncChannelTest, TestMultiSendRecvFramed) {
   typedef MultiSendRecvTest<TFramedAsyncChannel> MultiSendRecvFramedTest;
 
-  //size below 1024 for each message
+  // size below 1024 for each message
   MultiMessageSize sizes(911, 911, 911, -1);
 
   // each time send one whole message below 1024
@@ -747,11 +704,8 @@ TEST(TAsyncChannelTest, TestMultiSendRecvFramed) {
   MultiSendRecvFramedTest(sizes, 2).run();
   MultiSendRecvFramedTest(sizes, 2, 2).run();
 
-  //size above 1024 for each message
-  MultiMessageSize bigSizes(1911 * 1911,
-                            1911 * 1911,
-                            1911 * 1911,
-                            -1);
+  // size above 1024 for each message
+  MultiMessageSize bigSizes(1911 * 1911, 1911 * 1911, 1911 * 1911, -1);
 
   // each time send one whole message above 1024
   MultiSendRecvFramedTest(bigSizes, 3, 0).run();
@@ -765,11 +719,10 @@ TEST(TAsyncChannelTest, TestMultiSendRecvFramed) {
   MultiSendRecvFramedTest(bigSizes, 2, 2).run();
 }
 
-
 TEST(TAsyncChannelTest, TestMultiSendRecvFramedQueued) {
   typedef MultiSendRecvTest<TFramedAsyncChannel> MultiSendRecvBinaryTest;
 
-  //size below 1024 for each message
+  // size below 1024 for each message
   MultiMessageSize sizes(911, 911, 911, -1);
 
   // each time send one whole message below 1024
@@ -783,11 +736,8 @@ TEST(TAsyncChannelTest, TestMultiSendRecvFramedQueued) {
   MultiSendRecvBinaryTest(sizes, 2, true).run();
   MultiSendRecvBinaryTest(sizes, 2, true, milliseconds(2)).run();
 
-  //size above 1024 for each message
-  MultiMessageSize bigSizes(1911 * 1911,
-                            1911 * 1911,
-                            1911 * 1911,
-                            -1);
+  // size above 1024 for each message
+  MultiMessageSize bigSizes(1911 * 1911, 1911 * 1911, 1911 * 1911, -1);
 
   // each time send one whole message above 1024
   MultiSendRecvBinaryTest(bigSizes, 3, true, milliseconds(0)).run();
@@ -804,31 +754,25 @@ TEST(TAsyncChannelTest, TestMultiSendRecvFramedQueued) {
 const int kRecvDelay = 200;
 const int kTimeout = 50;
 
-template<typename ChannelT>
+template <typename ChannelT>
 class TimeoutQueuedTest : public SocketPairTest<ChannelT> {
  public:
-
   explicit TimeoutQueuedTest(uint32_t n_msgs = 3)
-      : n_msgs_(n_msgs)
-      , start_(false)
-      , msg_(911) {
-  }
+      : n_msgs_(n_msgs), start_(false), msg_(911) {}
 
   void preLoop() override {
-
     this->channel1_->setRecvTimeout(kRecvDelay * n_msgs_ + kTimeout);
 
     for (size_t i = 0; i < n_msgs_; i++) {
       // queue some reads 200ms apart
       this->eventBase_.tryRunAfterDelay(
-        std::bind(&TimeoutQueuedTest<ChannelT>::recvMe, this),
-        kRecvDelay * i);
+          std::bind(&TimeoutQueuedTest<ChannelT>::recvMe, this),
+          kRecvDelay * i);
     }
 
     this->eventBase_.tryRunAfterDelay(
-      std::bind(&TimeoutQueuedTest<ChannelT>::sendMe, this),
-      kRecvDelay * n_msgs_);
-
+        std::bind(&TimeoutQueuedTest<ChannelT>::sendMe, this),
+        kRecvDelay * n_msgs_);
   }
 
   void sendMe() {
@@ -840,16 +784,16 @@ class TimeoutQueuedTest : public SocketPairTest<ChannelT> {
     start_.reset();
   }
 
-  void recvMe() {
-    recvCallback_.recv(this->channel1_, &readMemoryBuffer_);
-  }
+  void recvMe() { recvCallback_.recv(this->channel1_, &readMemoryBuffer_); }
 
   void postLoop() override {
     CHECK_EQ(recvCallback_.getRecvError(), 2);
     CHECK_EQ(recvCallback_.getRecvDone(), 1);
 
-    T_CHECK_TIMEOUT(start_, recvCallback_.getTimestamp(),
-                    milliseconds(n_msgs_ * kRecvDelay + kTimeout));
+    T_CHECK_TIMEOUT(
+        start_,
+        recvCallback_.getTimestamp(),
+        milliseconds(n_msgs_ * kRecvDelay + kTimeout));
   }
 
  private:
@@ -862,14 +806,12 @@ class TimeoutQueuedTest : public SocketPairTest<ChannelT> {
   ChannelCallback recvCallback_;
 };
 
-
 TEST(TAsyncChannelTest, TestTimeoutQueued) {
   TimeoutQueuedTest<TFramedAsyncChannel>().run();
   TimeoutQueuedTest<TBinaryAsyncChannel>().run();
 }
 
-
-template<typename ChannelT>
+template <typename ChannelT>
 class RecvChunksTest : public SocketPairTest<ChannelT> {
  public:
   explicit RecvChunksTest(
@@ -883,7 +825,7 @@ class RecvChunksTest : public SocketPairTest<ChannelT> {
         start_(false),
         timeout_(timeout),
         msg_(msgLen, NeedsFrame<ChannelT>::value()),
-        sender_(&this->eventBase_, this->socket0_.get(), &msg_, schedule) {}
+        sender_(&this->eventBase_, this->socket0_, &msg_, schedule) {}
 
   void preLoop() override {
     if (timeout_ > milliseconds(0)) {
@@ -902,8 +844,8 @@ class RecvChunksTest : public SocketPairTest<ChannelT> {
     for (ChunkSchedule::const_iterator it = sender_.getSchedule()->begin();
          it != sender_.getSchedule()->end();
          ++it) {
-      // Allow 2ms of processing overhead for every scheduled event.
-      tolerance += milliseconds(2);
+      // Allow 3ms of processing overhead for every scheduled event.
+      tolerance += milliseconds(3);
 
       if (milliseconds(0) < timeout_ && timeout_ < it->delayMS) {
         // We expect to time out waiting for this chunk of data
@@ -938,15 +880,15 @@ class RecvChunksTest : public SocketPairTest<ChannelT> {
       // We should time out after expectedMS
       LOG(INFO) << "RecvChunksTest: testing for timeout in " << file_ << ":"
                 << line_;
-      CHECK_EQ(sender_.error(), true);
-      CHECK_EQ(recvCallback_.getRecvError(), 1);
-      CHECK_EQ(recvCallback_.getRecvDone(), 0);
-      CHECK_EQ(this->channel1_->timedOut(), true);
-      CHECK_EQ(this->channel1_->error(), true);
-      CHECK_EQ(this->channel1_->good(), false);
+      EXPECT_EQ(sender_.error(), true);
+      EXPECT_EQ(recvCallback_.getRecvError(), 1);
+      EXPECT_EQ(recvCallback_.getRecvDone(), 0);
+      EXPECT_EQ(this->channel1_->timedOut(), true);
+      EXPECT_EQ(this->channel1_->error(), true);
+      EXPECT_EQ(this->channel1_->good(), false);
 
-      T_CHECK_TIMEOUT(start_, recvCallback_.getTimestamp(), expectedMS,
-                      tolerance);
+      T_CHECK_TIMEOUT(
+          start_, recvCallback_.getTimestamp(), expectedMS, tolerance);
     } else if (expectedBytes == 0) {
       // We should get EOF after expectedMS, before any data was ever sent
       //
@@ -955,43 +897,43 @@ class RecvChunksTest : public SocketPairTest<ChannelT> {
       // should consider changing this TAsyncChannel behavior?
       LOG(INFO) << "RecvChunksTest: testing for EOF with no data in " << file_
                 << ":" << line_;
-      CHECK_EQ(sender_.error(), false);
-      CHECK_EQ(recvCallback_.getRecvError(), 0);
-      CHECK_EQ(recvCallback_.getRecvDone(), 1);
-      CHECK_EQ(this->channel1_->timedOut(), false);
-      CHECK_EQ(this->channel1_->error(), false);
-      CHECK_EQ(this->channel1_->good(), false);
-      CHECK_EQ(recvBuf_.available_read(), 0);
+      EXPECT_EQ(sender_.error(), false);
+      EXPECT_EQ(recvCallback_.getRecvError(), 0);
+      EXPECT_EQ(recvCallback_.getRecvDone(), 1);
+      EXPECT_EQ(this->channel1_->timedOut(), false);
+      EXPECT_EQ(this->channel1_->error(), false);
+      EXPECT_EQ(this->channel1_->good(), false);
+      EXPECT_EQ(recvBuf_.available_read(), 0);
 
-      T_CHECK_TIMEOUT(start_, recvCallback_.getTimestamp(), expectedMS,
-                      tolerance);
+      T_CHECK_TIMEOUT(
+          start_, recvCallback_.getTimestamp(), expectedMS, tolerance);
     } else if (expectedBytes < msg_.getLength()) {
       // We should get EOF after expectedMS
       LOG(INFO) << "RecvChunksTest: testing for EOF in " << file_ << ":"
                 << line_;
-      CHECK_EQ(sender_.error(), false);
-      CHECK_EQ(recvCallback_.getRecvError(), 1);
-      CHECK_EQ(recvCallback_.getRecvDone(), 0);
-      CHECK_EQ(this->channel1_->timedOut(), false);
-      CHECK_EQ(this->channel1_->error(), false);
-      CHECK_EQ(this->channel1_->good(), false);
+      EXPECT_EQ(sender_.error(), false);
+      EXPECT_EQ(recvCallback_.getRecvError(), 1);
+      EXPECT_EQ(recvCallback_.getRecvDone(), 0);
+      EXPECT_EQ(this->channel1_->timedOut(), false);
+      EXPECT_EQ(this->channel1_->error(), false);
+      EXPECT_EQ(this->channel1_->good(), false);
 
-      T_CHECK_TIMEOUT(start_, recvCallback_.getTimestamp(), expectedMS,
-                      tolerance);
+      T_CHECK_TIMEOUT(
+          start_, recvCallback_.getTimestamp(), expectedMS, tolerance);
     } else {
       // We expect success after expectedMS
       LOG(INFO) << "RecvChunksTest: testing for success in " << file_ << ":"
                 << line_;
-      CHECK_EQ(sender_.error(), false);
-      CHECK_EQ(recvCallback_.getRecvError(), 0);
-      CHECK_EQ(recvCallback_.getRecvDone(), 1);
-      CHECK_EQ(this->channel1_->timedOut(), false);
-      CHECK_EQ(this->channel1_->error(), false);
-      CHECK_EQ(this->channel1_->good(), true);
+      EXPECT_EQ(sender_.error(), false);
+      EXPECT_EQ(recvCallback_.getRecvError(), 0);
+      EXPECT_EQ(recvCallback_.getRecvDone(), 1);
+      EXPECT_EQ(this->channel1_->timedOut(), false);
+      EXPECT_EQ(this->channel1_->error(), false);
+      EXPECT_EQ(this->channel1_->good(), true);
       msg_.checkEqual(&recvBuf_);
 
-      T_CHECK_TIMEOUT(start_, recvCallback_.getTimestamp(), expectedMS,
-                      tolerance);
+      T_CHECK_TIMEOUT(
+          start_, recvCallback_.getTimestamp(), expectedMS, tolerance);
     }
   }
 
@@ -1125,14 +1067,11 @@ TEST(TAsyncChannelTest, TestRecvBinaryChunks) {
       .run();
 }
 
-template<typename ChannelT>
+template <typename ChannelT>
 class SendTimeoutTest : public SocketPairTest<ChannelT> {
  public:
   explicit SendTimeoutTest(milliseconds timeout)
-    : timeout_(timeout)
-    , start_(false)
-    , msg_(1024*1024) {
-  }
+      : timeout_(timeout), start_(false), msg_(1024 * 1024) {}
 
   void preLoop() override {
     this->socket0_->setSendTimeout(timeout_.count());
@@ -1169,14 +1108,11 @@ TEST(TAsyncChannelTest, TestSendTimeoutBinary) {
   SendTimeoutTest<TBinaryAsyncChannel>(milliseconds(250)).run();
 }
 
-template<typename ChannelT>
+template <typename ChannelT>
 class SendClosedTest : public SocketPairTest<ChannelT> {
  public:
   explicit SendClosedTest(milliseconds closeTimeout = milliseconds(5))
-    : closeTimeout_(closeTimeout)
-    , start_(false)
-    , msg_(1024*1024) {
-  }
+      : closeTimeout_(closeTimeout), start_(false), msg_(1024 * 1024) {}
 
   void preLoop() override {
     msg_.copyTo(&sendBuf_);
@@ -1184,8 +1120,7 @@ class SendClosedTest : public SocketPairTest<ChannelT> {
 
     // Close the other socket after 25ms
     this->eventBase_.tryRunAfterDelay(
-        std::bind(&TAsyncSocket::close, this->socket1_.get()),
-        closeTimeout_.count());
+        std::bind(&AsyncSocket::close, this->socket1_), closeTimeout_.count());
 
     start_.reset();
   }
