@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -44,30 +44,64 @@ using is_mutable_ref = folly::Conjunction<
     std::is_reference<T>,
     folly::Negation<std::is_const<std::remove_reference_t<T>>>>;
 
+// The type returned by the adapter for the given thrift type.
+template <typename Adapter, typename ThriftT>
+using adapted_t = decltype(Adapter::fromThrift(std::declval<ThriftT&&>()));
+
 // Used to detect if Adapter has the fromThriftField function which takes an
 // additional FieldAdapterContext argument.
+template <typename Adapter, int16_t FieldId, typename ThriftT, typename Struct>
+using FromThriftFieldIdType = decltype(Adapter::fromThriftField(
+    std::declval<ThriftT>(),
+    std::declval<FieldAdapterContext<Struct, FieldId>>()));
 template <typename Adapter, typename ThriftT, typename Struct>
 using FromThriftFieldType = decltype(Adapter::fromThriftField(
     std::declval<ThriftT>(), std::declval<FieldAdapterContext<Struct, 0>>()));
+template <typename Adapter, typename ThriftT, typename Struct>
+constexpr bool is_field_adapter_v =
+    folly::is_detected_v<FromThriftFieldType, Adapter, ThriftT, Struct>;
+template <
+    typename Adapter,
+    typename ThriftT,
+    typename Struct,
+    typename R = adapted_t<Adapter, ThriftT>>
+using if_not_field_adapter =
+    std::enable_if_t<!is_field_adapter_v<Adapter, ThriftT, Struct>, R>;
 
-// Used to detect if Adapter has the construct function.
+// Used to detect if Adapter has a construct function override.
 template <typename Adapter, typename AdaptedT, typename Context>
 using ConstructType = decltype(Adapter::construct(
     std::declval<AdaptedT&>(), std::declval<Context>()));
+template <typename Adapter, typename AdaptedT, typename Context>
+constexpr bool is_ctor_adapter_v =
+    folly::is_detected_v<ConstructType, Adapter, AdaptedT, Context>;
+template <typename Adapter, typename AdaptedT, typename Context>
+using if_ctor_adapter =
+    std::enable_if_t<is_ctor_adapter_v<Adapter, AdaptedT, Context>>;
+template <typename Adapter, typename AdaptedT, typename Context>
+using if_not_ctor_adapter =
+    std::enable_if_t<!is_ctor_adapter_v<Adapter, AdaptedT, Context>>;
+
+// Used to detect if Adapter has a clear function override.
+template <typename Adapter, typename AdaptedT>
+using ClearType = decltype(Adapter::clear(std::declval<AdaptedT&>()));
+template <typename Adapter, typename AdaptedT>
+constexpr bool is_clear_adapter_v =
+    folly::is_detected_v<ClearType, Adapter, AdaptedT>;
+template <typename Adapter, typename AdaptedT, typename R = void>
+using if_clear_adapter =
+    std::enable_if_t<is_clear_adapter_v<Adapter, AdaptedT>, R>;
+template <typename Adapter, typename AdaptedT, typename R = void>
+using if_not_clear_adapter =
+    std::enable_if_t<!is_clear_adapter_v<Adapter, AdaptedT>, R>;
 
 // Converts a Thrift field value into an adapted type via Adapter.
 // This overload passes additional context containing the reference to the
 // Thrift object containing the field and the field ID as a second argument
 // to Adapter::fromThriftField.
-template <
-    typename Adapter,
-    int16_t FieldId,
-    typename ThriftT,
-    typename Struct,
-    std::enable_if_t<
-        folly::is_detected_v<FromThriftFieldType, Adapter, ThriftT, Struct>,
-        int> = 0>
-constexpr decltype(auto) fromThriftField(ThriftT&& value, Struct& object) {
+template <typename Adapter, int16_t FieldId, typename ThriftT, typename Struct>
+constexpr FromThriftFieldIdType<Adapter, FieldId, ThriftT, Struct>
+fromThriftField(ThriftT&& value, Struct& object) {
   return Adapter::fromThriftField(
       std::forward<ThriftT>(value),
       FieldAdapterContext<Struct, FieldId>{object});
@@ -76,22 +110,12 @@ constexpr decltype(auto) fromThriftField(ThriftT&& value, Struct& object) {
 // Converts a Thrift field value into an adapted type via Adapter.
 // This overloads does the conversion via Adapter::fromThrift and is used when
 // Adapter::fromThriftField is unavailable.
-template <
-    typename Adapter,
-    int16_t FieldId,
-    typename ThriftT,
-    typename Struct,
-    std::enable_if_t<
-        !folly::is_detected_v<FromThriftFieldType, Adapter, ThriftT, Struct>,
-        int> = 0>
-constexpr decltype(auto) fromThriftField(ThriftT&& value, Struct&) {
+template <typename Adapter, int16_t FieldId, typename ThriftT, typename Struct>
+constexpr if_not_field_adapter<Adapter, ThriftT, Struct> fromThriftField(
+    ThriftT&& value, Struct&) {
   return Adapter::fromThrift(std::forward<ThriftT>(value));
 }
 
-// The type returned by the adapter for the given thrift type.
-template <typename Adapter, typename ThriftT>
-using adapted_t = decltype(Adapter::fromThrift(std::declval<ThriftT&&>()));
-
 // The type returned by the adapter for the given thrift type of a struct field.
 template <typename Adapter, int16_t FieldId, typename ThriftT, typename Struct>
 using adapted_field_t = decltype(fromThriftField<Adapter, FieldId>(
@@ -112,38 +136,38 @@ void fromThrift(AdaptedT& adapted, ThriftT&& value) {
   adapted = Adapter::fromThrift(std::forward<ThriftT>(value));
 }
 
-// Called during the construction of a Thrift object to perform any additonal
+// Called during the construction of a Thrift object to perform any additional
 // initialization of an adapted type. This overload passes a context containing
 // the reference to the Thrift object containing the field and the field ID as
 // a second argument to Adapter::construct.
-template <
-    typename Adapter,
-    int16_t FieldId,
-    typename AdaptedT,
-    typename Struct,
-    std::enable_if_t<
-        folly::is_detected_v<
-            ConstructType,
-            Adapter,
-            AdaptedT,
-            FieldAdapterContext<Struct, FieldId>>,
-        int> = 0>
-constexpr void construct(AdaptedT& field, Struct& object) {
+template <typename Adapter, int16_t FieldId, typename AdaptedT, typename Struct>
+constexpr if_ctor_adapter<
+    Adapter,
+    AdaptedT,
+    FieldAdapterContext<Struct, FieldId>>
+construct(AdaptedT& field, Struct& object) {
   Adapter::construct(field, FieldAdapterContext<Struct, FieldId>{object});
 }
-template <
-    typename Adapter,
-    int16_t FieldId,
-    typename AdaptedT,
-    typename Struct,
-    std::enable_if_t<
-        !folly::is_detected_v<
-            ConstructType,
-            Adapter,
-            AdaptedT,
-            FieldAdapterContext<Struct, FieldId>>,
-        int> = 0>
-constexpr void construct(AdaptedT&, Struct&) {}
+template <typename Adapter, int16_t FieldId, typename AdaptedT, typename Struct>
+constexpr if_not_ctor_adapter<
+    Adapter,
+    AdaptedT,
+    FieldAdapterContext<Struct, FieldId>>
+construct(AdaptedT&, Struct&) {}
+
+// Clear op based on the adapter, with a fallback to calling the default
+// constructor and Adapter::construct for context population.
+template <typename Adapter, int16_t FieldId, typename AdaptedT, typename Struct>
+constexpr if_clear_adapter<Adapter, AdaptedT> clear(AdaptedT& field, Struct&) {
+  Adapter::clear(field);
+}
+
+template <typename Adapter, int16_t FieldId, typename AdaptedT, typename Struct>
+constexpr if_not_clear_adapter<Adapter, AdaptedT> clear(
+    AdaptedT& field, Struct& object) {
+  field = AdaptedT();
+  construct<Adapter, FieldId>(field, object);
+}
 
 // Equal op based on the thrift types.
 template <typename Adapter, typename AdaptedT>
@@ -289,6 +313,67 @@ constexpr size_t hash(const AdaptedT& value) {
   return adapter_hash<Adapter, AdaptedT>()(value);
 }
 
+// Helpers replace less, hash, equal_to functions
+// for a set, with the appropriate adapted versions.
+template <
+    typename Adapter,
+    template <typename, typename, typename>
+    class SetT,
+    typename Key,
+    typename Less,
+    typename Allocator>
+SetT<Key, adapt_detail::adapted_less<Adapter, Key>, Allocator>
+resolveSetForAdapated(const SetT<Key, Less, Allocator>&);
+template <
+    typename Adapter,
+    template <typename, typename, typename, typename>
+    class SetT,
+    typename Key,
+    typename Hash,
+    typename KeyEqual,
+    typename Allocator>
+SetT<
+    Key,
+    adapt_detail::adapted_hash<Adapter, Key>,
+    adapt_detail::adapted_equal<Adapter, Key>,
+    Allocator>
+resolveSetForAdapated(const SetT<Key, Hash, KeyEqual, Allocator>&);
+template <typename KeyAdapter, typename StandardSet>
+using adapt_set_key_t =
+    decltype(resolveSetForAdapated<KeyAdapter>(std::declval<StandardSet>()));
+
+// Helpers to set the appropriate less, hash, equal_to functions
+// for a map with an adapted key type.
+template <
+    typename Adapter,
+    template <typename, typename, typename, typename>
+    class MapT,
+    typename Key,
+    typename Value,
+    typename Less,
+    typename Allocator>
+MapT<Key, Value, adapt_detail::adapted_less<Adapter, Key>, Allocator>
+resolveMapForAdapated(const MapT<Key, Value, Less, Allocator>&);
+template <
+    typename Adapter,
+    template <typename, typename, typename, typename, typename>
+    class MapT,
+    typename Key,
+    typename Value,
+    typename Hash,
+    typename KeyEqual,
+    typename Allocator>
+MapT<
+    Key,
+    Value,
+    adapt_detail::adapted_hash<Adapter, Key>,
+    adapt_detail::adapted_equal<Adapter, Key>,
+    Allocator>
+resolveMapForAdapated(const MapT<Key, Value, Hash, KeyEqual, Allocator>&);
+template <typename KeyAdapter, typename StandardMap>
+using adapt_map_key_t =
+    decltype(resolveMapForAdapated<KeyAdapter>(std::declval<StandardMap>()));
+
 // Validates an adapter.
 // Checking decltype(equal<Adapter>(...)) is not sufficient for validation.
 template <typename Adapter, typename AdaptedT>
@@ -343,5 +428,18 @@ struct StaticCastAdapter {
   }
 };
 
+// An adapter for types that know how to adapt themselves.
+template <typename T>
+struct InlineAdapter {
+  template <typename U>
+  static decltype(auto) toThrift(U&& obj) {
+    return std::forward<U>(obj).toThrift();
+  }
+  template <typename U>
+  static T fromThrift(U&& val) {
+    return T{std::forward<U>(val)};
+  }
+};
+
 } // namespace thrift
 } // namespace apache
