@@ -37,6 +37,7 @@
 #include <folly/Traits.h>
 #include <folly/synchronization/AtomicUtil.h>
 #include <thrift/lib/cpp2/BoxedValuePtr.h>
+#include <thrift/lib/cpp2/FieldRefTraits.h>
 #include <thrift/lib/cpp2/Thrift.h>
 
 namespace apache {
@@ -48,23 +49,50 @@ using is_set_t =
     std::conditional_t<std::is_const<T>::value, const uint8_t, uint8_t>;
 
 [[noreturn]] void throw_on_bad_field_access();
+[[noreturn]] void throw_on_nullptr_dereferencing();
 
 struct ensure_isset_unsafe_fn;
 struct unset_unsafe_fn;
 struct alias_isset_fn;
 
+// IntWrapper is a wrapper of integer that's always copy/move assignable
+// even if integer is atomic
+template <typename T>
+struct IntWrapper {
+  IntWrapper() = default;
+  explicit IntWrapper(T t) : value(t) {}
+
+  T value{0};
+};
+
+template <typename U>
+struct IntWrapper<std::atomic<U>> {
+  IntWrapper() = default;
+
+  IntWrapper(const IntWrapper& other) noexcept : value(other.value.load()) {}
+  IntWrapper(IntWrapper&& other) noexcept : value(other.value.load()) {}
+
+  IntWrapper& operator=(const IntWrapper& other) noexcept {
+    value = other.value.load();
+    return *this;
+  }
+
+  IntWrapper& operator=(IntWrapper&& other) noexcept {
+    value = other.value.load();
+    return *this;
+  }
+
+  std::atomic<U> value{0};
+};
+
 template <typename T>
 class BitSet {
-  template <typename U>
-  friend class BitSet;
-
  public:
   BitSet() = default;
 
-  explicit BitSet(T value) : value_(value) {}
+  explicit BitSet(T value) : int_(value) {}
 
-  template <typename U>
-  explicit BitSet(const BitSet<U>& other) noexcept : value_(other.value_) {}
+  BitSet(const BitSet&) = default;
 
   class reference {
    public:
@@ -88,44 +116,116 @@ class BitSet {
     const uint8_t bit_;
   };
 
-  bool operator[](const uint8_t bit) const { return get(bit); }
+  bool operator[](const uint8_t bit) const {
+    assert(bit < NUM_BITS);
+    return get(bit);
+  }
 
   reference operator[](const uint8_t bit) {
     assert(bit < NUM_BITS);
     return reference(*this, bit);
   }
 
-  T& value() { return value_; }
+  T& value() { return int_.value; }
 
-  const T& value() const { return value_; }
+  const T& value() const { return int_.value; }
 
  private:
-  void set(const uint8_t bit, std::false_type) { value_ |= (1 << bit); }
-  void set(const uint8_t bit, std::true_type) {
-    folly::atomic_fetch_set(value_, bit);
-  }
-  void set(const uint8_t bit) {
-    set(bit, folly::detail::is_instantiation_of<std::atomic, T>{});
+  template <class U>
+  static bool get(U u, std::size_t bit) {
+    return u & (U(1) << bit);
   }
 
-  void reset(const uint8_t bit, std::false_type) { value_ &= ~(1 << bit); }
-  void reset(const uint8_t bit, std::true_type) {
-    folly::atomic_fetch_reset(value_, bit);
+  template <class U>
+  static void set(U& u, std::size_t bit) {
+    u |= (U(1) << bit);
   }
 
-  void reset(const uint8_t bit) {
-    reset(bit, folly::detail::is_instantiation_of<std::atomic, T>{});
+  template <class U>
+  static void reset(U& u, std::size_t bit) {
+    u &= ~(U(1) << bit);
   }
 
-  bool get(const uint8_t bit) const {
-    assert(bit < NUM_BITS);
-    return (value_ >> bit) & 1;
+  template <class U>
+  static bool get(const std::atomic<U>& u, std::size_t bit) {
+    return u.load(std::memory_order_relaxed) & (U(1) << bit);
   }
 
-  T value_{0};
+  template <class U>
+  static void set(std::atomic<U>& u, std::size_t bit) {
+    folly::atomic_fetch_set(u, bit, std::memory_order_relaxed);
+  }
+
+  template <class U>
+  static void reset(std::atomic<U>& u, std::size_t bit) {
+    folly::atomic_fetch_reset(u, bit, std::memory_order_relaxed);
+  }
+
+  bool get(std::size_t bit) const { return get(int_.value, bit); }
+  void set(std::size_t bit) { set(int_.value, bit); }
+  void reset(std::size_t bit) { reset(int_.value, bit); }
+
+  IntWrapper<T> int_;
 
   static constexpr int NUM_BITS = sizeof(T) * CHAR_BIT;
 };
+
+template <bool kIsConst>
+class BitRef {
+  template <bool B>
+  friend class BitRef;
+
+ public:
+  using Isset = std::conditional_t<kIsConst, const uint8_t, uint8_t>;
+  using AtomicIsset = std::
+      conditional_t<kIsConst, const std::atomic<uint8_t>, std::atomic<uint8_t>>;
+
+  FOLLY_ERASE BitRef(Isset& isset, uint8_t bit_index)
+      : value_(isset), bit_index_(bit_index) {}
+
+  FOLLY_ERASE BitRef(AtomicIsset& isset, uint8_t bit_index)
+      : value_(isset), bit_index_(bit_index), is_atomic_(true) {}
+
+  template <bool B>
+  explicit BitRef(const BitRef<B>& other)
+      : value_(
+            other.is_atomic_ ? IssetBitSet(other.value_.atomic.value())
+                             : IssetBitSet(other.value_.non_atomic.value())),
+        bit_index_(other.bit_index_),
+        is_atomic_(other.is_atomic_) {}
+
+#if FOLLY_MOBILE
+  // We have this attribute to prevent binary size regression
+  // TODO: Remove special attribute for MOBILE
+  FOLLY_ERASE
+#endif
+  void operator=(bool flag) {
+    if (is_atomic_) {
+      value_.atomic[bit_index_] = flag;
+    } else {
+      value_.non_atomic[bit_index_] = flag;
+    }
+  }
+
+  explicit operator bool() const {
+    if (is_atomic_) {
+      return value_.atomic[bit_index_];
+    } else {
+      return value_.non_atomic[bit_index_];
+    }
+  }
+
+ private:
+  union IssetBitSet {
+    explicit IssetBitSet(Isset& isset) : non_atomic(isset) {}
+    explicit IssetBitSet(AtomicIsset& isset) : atomic(isset) {}
+    apache::thrift::detail::BitSet<Isset&> non_atomic;
+    apache::thrift::detail::BitSet<AtomicIsset&> atomic;
+  } value_;
+
+  const uint8_t bit_index_;
+  const bool is_atomic_ = false;
+};
 } // namespace detail
 
 // A reference to an unqualified field of the possibly const-qualified type
@@ -142,11 +242,22 @@ class field_ref {
   using value_type = std::remove_reference_t<T>;
   using reference_type = T;
 
+ private:
+  using BitRef =
+      apache::thrift::detail::BitRef<std::is_const<value_type>::value>;
+
+ public:
   FOLLY_ERASE field_ref(
       reference_type value,
-      apache::thrift::detail::is_set_t<value_type>& is_set,
-      const uint8_t bit = 0) noexcept
-      : value_(value), is_set_(is_set), bit_(bit) {}
+      typename BitRef::Isset& is_set,
+      const uint8_t bit_index = 0) noexcept
+      : value_(value), bitref_(is_set, bit_index) {}
+
+  FOLLY_ERASE field_ref(
+      reference_type value,
+      typename BitRef::AtomicIsset& is_set,
+      const uint8_t bit_index = 0) noexcept
+      : value_(value), bitref_(is_set, bit_index) {}
 
   template <
       typename U,
@@ -157,7 +268,7 @@ class field_ref {
               !(std::is_rvalue_reference<T>{} && std::is_lvalue_reference<U>{}),
           int> = 0>
   FOLLY_ERASE /* implicit */ field_ref(const field_ref<U>& other) noexcept
-      : value_(other.value_), is_set_(other.is_set_), bit_(other.bit_) {}
+      : value_(other.value_), bitref_(other.bitref_) {}
 
   template <typename U = value_type>
   FOLLY_ERASE
@@ -165,15 +276,15 @@ class field_ref {
       operator=(U&& value) noexcept(
           std::is_nothrow_assignable<value_type&, U&&>::value) {
     value_ = static_cast<U&&>(value);
-    is_set_[bit_] = true;
+    bitref_ = true;
     return *this;
   }
 
   // Workaround for https://bugs.llvm.org/show_bug.cgi?id=49442
   FOLLY_ERASE field_ref& operator=(value_type&& value) noexcept(
-      std::is_nothrow_assignable<value_type&, value_type&&>::value) {
+      std::is_nothrow_move_assignable<value_type>::value) {
     value_ = static_cast<value_type&&>(value);
-    is_set_[bit_] = true;
+    bitref_ = true;
     return *this;
     value.~value_type(); // Force emit destructor...
   }
@@ -185,18 +296,18 @@ class field_ref {
   FOLLY_ERASE void copy_from(field_ref<U> other) noexcept(
       std::is_nothrow_assignable<value_type&, U>::value) {
     value_ = other.value();
-    is_set_[bit_] = other.is_set();
+    bitref_ = other.is_set();
   }
 
   [[deprecated("Use is_set() method instead")]] FOLLY_ERASE bool has_value()
       const noexcept {
-    return is_set_[bit_];
+    return bool(bitref_);
   }
 
   // Returns true iff the field is set. field_ref doesn't provide conversion to
   // bool to avoid confusion between checking if the field is set and getting
   // the field's value, particularly for bool fields.
-  FOLLY_ERASE bool is_set() const noexcept { return is_set_[bit_]; }
+  FOLLY_ERASE bool is_set() const noexcept { return bool(bitref_); }
 
   // Returns a reference to the value.
   FOLLY_ERASE reference_type value() const noexcept {
@@ -210,7 +321,7 @@ class field_ref {
   FOLLY_ERASE value_type* operator->() const noexcept { return &value_; }
 
   FOLLY_ERASE reference_type ensure() noexcept {
-    is_set_[bit_] = true;
+    bitref_ = true;
     return static_cast<reference_type>(value_);
   }
 
@@ -221,10 +332,10 @@ class field_ref {
 
   template <typename... Args>
   FOLLY_ERASE value_type& emplace(Args&&... args) {
-    is_set_[bit_] = false; // C++ Standard requires *this to be empty if
-                           // `std::optional::emplace(...)` throws
+    bitref_ = false; // C++ Standard requires *this to be empty if
+                     // `std::optional::emplace(...)` throws
     value_ = value_type(static_cast<Args&&>(args)...);
-    is_set_[bit_] = true;
+    bitref_ = true;
     return value_;
   }
 
@@ -234,17 +345,15 @@ class field_ref {
           value,
       value_type&>
   emplace(std::initializer_list<U> ilist, Args&&... args) {
-    is_set_[bit_] = false;
+    bitref_ = false;
     value_ = value_type(ilist, static_cast<Args&&>(args)...);
-    is_set_[bit_] = true;
+    bitref_ = true;
     return value_;
   }
 
  private:
   value_type& value_;
-  apache::thrift::detail::BitSet<apache::thrift::detail::is_set_t<value_type>&>
-      is_set_;
-  const uint8_t bit_;
+  BitRef bitref_;
 };
 
 template <typename T, typename U>
@@ -353,11 +462,26 @@ class optional_field_ref {
   using value_type = std::remove_reference_t<T>;
   using reference_type = T;
 
+ private:
+  using BitRef =
+      apache::thrift::detail::BitRef<std::is_const<value_type>::value>;
+
+  // for alias_isset_fn
+  FOLLY_ERASE optional_field_ref(reference_type value, BitRef bitref)
+      : value_(value), bitref_(bitref) {}
+
+ public:
   FOLLY_ERASE optional_field_ref(
       reference_type value,
-      apache::thrift::detail::is_set_t<value_type>& is_set,
-      const uint8_t bit = 0) noexcept
-      : value_(value), is_set_(is_set), bit_(bit) {}
+      typename BitRef::Isset& is_set,
+      const uint8_t bit_index = 0) noexcept
+      : value_(value), bitref_(is_set, bit_index) {}
+
+  FOLLY_ERASE optional_field_ref(
+      reference_type value,
+      typename BitRef::AtomicIsset& is_set,
+      const uint8_t bit_index = 0) noexcept
+      : value_(value), bitref_(is_set, bit_index) {}
 
   template <
       typename U,
@@ -369,7 +493,7 @@ class optional_field_ref {
           int> = 0>
   FOLLY_ERASE /* implicit */ optional_field_ref(
       const optional_field_ref<U>& other) noexcept
-      : value_(other.value_), is_set_(other.is_set_), bit_(other.bit_) {}
+      : value_(other.value_), bitref_(other.bitref_) {}
 
   template <
       typename U,
@@ -378,7 +502,7 @@ class optional_field_ref {
           int> = 0>
   FOLLY_ERASE explicit optional_field_ref(
       const optional_field_ref<U&>& other) noexcept
-      : value_(other.value_), is_set_(other.is_set_), bit_(other.bit_) {}
+      : value_(other.value_), bitref_(other.bitref_) {}
 
   template <typename U = value_type>
   FOLLY_ERASE std::enable_if_t<
@@ -387,15 +511,15 @@ class optional_field_ref {
   operator=(U&& value) noexcept(
       std::is_nothrow_assignable<value_type&, U&&>::value) {
     value_ = static_cast<U&&>(value);
-    is_set_[bit_] = true;
+    bitref_ = true;
     return *this;
   }
 
   // Workaround for https://bugs.llvm.org/show_bug.cgi?id=49442
   FOLLY_ERASE optional_field_ref& operator=(value_type&& value) noexcept(
-      std::is_nothrow_assignable<value_type&, value_type&&>::value) {
+      std::is_nothrow_move_assignable<value_type>::value) {
     value_ = static_cast<value_type&&>(value);
-    is_set_[bit_] = true;
+    bitref_ = true;
     return *this;
     value.~value_type(); // Force emit destructor...
   }
@@ -410,7 +534,7 @@ class optional_field_ref {
   FOLLY_ERASE void copy_from(const optional_field_ref<U>& other) noexcept(
       std::is_nothrow_assignable<value_type&, U>::value) {
     value_ = other.value_unchecked();
-    is_set_[bit_] = other.has_value() ? true : false;
+    bitref_ = other.has_value();
   }
 
   template <typename U>
@@ -418,7 +542,7 @@ class optional_field_ref {
       std::is_nothrow_assignable<value_type&, std::remove_reference_t<U>&&>::
           value) {
     value_ = static_cast<std::remove_reference_t<U>&&>(other.value_);
-    is_set_[bit_] = other.is_set_[other.bit_];
+    bitref_ = other.has_value();
   }
 
 #ifdef THRIFT_HAS_OPTIONAL
@@ -432,7 +556,7 @@ class optional_field_ref {
     } else {
       value_ = {};
     }
-    is_set_[bit_] = other.has_value() ? true : false;
+    bitref_ = other.has_value();
   }
 
   // Moves the value from std::optional. As std::optional's move constructor,
@@ -447,23 +571,24 @@ class optional_field_ref {
     } else {
       value_ = {};
     }
-    is_set_[bit_] = other.has_value() ? true : false;
+    bitref_ = other.has_value();
   }
 
   FOLLY_ERASE std::optional<std::remove_const_t<value_type>> to_optional()
       const {
     using type = std::optional<std::remove_const_t<value_type>>;
-    return is_set_[bit_] ? type(value_) : type();
+    return bitref_ ? type(value_) : type();
   }
 #endif
 
-  FOLLY_ERASE bool has_value() const noexcept { return is_set_[bit_]; }
+  FOLLY_ERASE bool has_value() const noexcept { return bool(bitref_); }
 
-  FOLLY_ERASE explicit operator bool() const noexcept { return is_set_[bit_]; }
+  FOLLY_ERASE explicit operator bool() const noexcept { return bool(bitref_); }
 
-  FOLLY_ERASE void reset() noexcept {
+  FOLLY_ERASE void reset() noexcept(
+      std::is_nothrow_move_assignable<value_type>::value) {
     value_ = value_type();
-    is_set_[bit_] = false;
+    bitref_ = false;
   }
 
   // Returns a reference to the value if this optional_field_ref has one; throws
@@ -477,8 +602,8 @@ class optional_field_ref {
   FOLLY_ERASE std::remove_const_t<value_type> value_or(
       U&& default_value) const {
     using type = std::remove_const_t<value_type>;
-    return is_set_[bit_] ? type(static_cast<reference_type>(value_))
-                         : type(static_cast<U&&>(default_value));
+    return bitref_ ? type(static_cast<reference_type>(value_))
+                   : type(static_cast<U&&>(default_value));
   }
 
   // Returns a reference to the value without checking whether it is available.
@@ -493,9 +618,11 @@ class optional_field_ref {
     return &value_;
   }
 
-  FOLLY_ERASE reference_type ensure() noexcept {
-    if (!is_set_[bit_]) {
-      emplace();
+  FOLLY_ERASE reference_type
+  ensure() noexcept(std::is_nothrow_move_assignable<value_type>::value) {
+    if (!bitref_) {
+      value_ = value_type();
+      bitref_ = true;
     }
     return static_cast<reference_type>(value_);
   }
@@ -505,7 +632,7 @@ class optional_field_ref {
     reset(); // C++ Standard requires *this to be empty if
              // `std::optional::emplace(...)` throws
     value_ = value_type(static_cast<Args&&>(args)...);
-    is_set_[bit_] = true;
+    bitref_ = true;
     return value_;
   }
 
@@ -517,21 +644,19 @@ class optional_field_ref {
   emplace(std::initializer_list<U> ilist, Args&&... args) {
     reset();
     value_ = value_type(ilist, static_cast<Args&&>(args)...);
-    is_set_[bit_] = true;
+    bitref_ = true;
     return value_;
   }
 
  private:
   FOLLY_ERASE void throw_if_unset() const {
-    if (!is_set_[bit_]) {
+    if (!bitref_) {
       apache::thrift::detail::throw_on_bad_field_access();
     }
   }
 
   value_type& value_;
-  apache::thrift::detail::BitSet<apache::thrift::detail::is_set_t<value_type>&>
-      is_set_;
-  const uint8_t bit_;
+  BitRef bitref_;
 };
 
 template <typename T1, typename T2>
@@ -684,7 +809,8 @@ class optional_boxed_field_ref {
   using value_type = detail::copy_const_t<T, element_type>;
   using reference_type = detail::copy_reference_t<T, value_type>;
 
-  FOLLY_ERASE optional_boxed_field_ref(T value) noexcept : value_(value) {}
+  FOLLY_ERASE explicit optional_boxed_field_ref(T value) noexcept
+      : value_(value) {}
 
   template <
       typename U,
@@ -970,19 +1096,19 @@ struct can_throw_fn {
 struct ensure_isset_unsafe_fn {
   template <typename T>
   void operator()(optional_field_ref<T> ref) const noexcept {
-    ref.is_set_[ref.bit_] = true;
+    ref.bitref_ = true;
   }
 };
 
 struct unset_unsafe_fn {
   template <typename T>
   void operator()(field_ref<T> ref) const noexcept {
-    ref.is_set_[ref.bit_] = false;
+    ref.bitref_ = false;
   }
 
   template <typename T>
   void operator()(optional_field_ref<T> ref) const noexcept {
-    ref.is_set_[ref.bit_] = false;
+    ref.bitref_ = false;
   }
 };
 
@@ -992,7 +1118,7 @@ struct alias_isset_fn {
       noexcept(noexcept(functor(ref.value_))) {
     auto&& result = functor(ref.value_);
     return optional_field_ref<decltype(result)>(
-        static_cast<decltype(result)>(result), ref.is_set_.value());
+        static_cast<decltype(result)>(result), ref.bitref_);
   }
 };
 
@@ -1078,7 +1204,7 @@ class required_field_ref {
 
   // Workaround for https://bugs.llvm.org/show_bug.cgi?id=49442
   FOLLY_ERASE required_field_ref& operator=(value_type&& value) noexcept(
-      std::is_nothrow_assignable<value_type&, value_type&&>::value) {
+      std::is_nothrow_move_assignable<value_type>::value) {
     value_ = static_cast<value_type&&>(value);
     return *this;
     value.~value_type(); // Force emit destructor...
@@ -1252,19 +1378,6 @@ template <typename T>
 FOLLY_INLINE_VARIABLE constexpr union_field_ref_owner_vtable //
     union_field_ref_owner_vtable_for<T const&>{nullptr};
 
-template <class T, class = void>
-struct element_type {
-  using type = T;
-};
-
-template <class T>
-struct element_type<T, folly::void_t<typename T::element_type>> {
-  using type = typename T::element_type;
-};
-
-template <class T>
-using is_boxed = folly::detail::is_instantiation_of<boxed_value_ptr, T>;
-
 } // namespace detail
 
 // A reference to an union field of the possibly const-qualified type
@@ -1275,9 +1388,18 @@ class union_field_ref {
   template <typename>
   friend class union_field_ref;
 
-  using element_type =
-      typename detail::element_type<folly::remove_cvref_t<T>>::type;
-  using is_boxed = detail::is_boxed<folly::remove_cvref_t<T>>;
+  using is_cpp_ref_or_boxed = folly::Disjunction<
+      detail::is_boxed_value_ptr<folly::remove_cvref_t<T>>,
+      detail::is_shared_or_unique_ptr<folly::remove_cvref_t<T>>>;
+
+  struct element_type_adapter {
+    using element_type = folly::remove_cvref_t<T>;
+  };
+
+  using element_type = typename std::conditional_t<
+      is_cpp_ref_or_boxed::value,
+      folly::remove_cvref_t<T>,
+      element_type_adapter>::element_type;
 
   using storage_reference_type = T;
   using storage_value_type = std::remove_reference_t<T>;
@@ -1287,10 +1409,8 @@ class union_field_ref {
   using reference_type = detail::copy_reference_t<T, value_type>;
 
  private:
-  using int_t =
-      std::conditional_t<std::is_const<value_type>::value, const int, int>;
-  using owner =
-      std::conditional_t<std::is_const<value_type>::value, void const*, void*>;
+  using int_t = detail::copy_const_t<T, int>;
+  using owner = detail::copy_const_t<T, void>*;
   using vtable = apache::thrift::detail::union_field_ref_owner_vtable;
 
  public:
@@ -1315,7 +1435,8 @@ class union_field_ref {
   FOLLY_ERASE union_field_ref& operator=(U&& other) noexcept(
       std::is_nothrow_constructible<value_type, U>::value&&
           std::is_nothrow_assignable<value_type, U>::value) {
-    if (has_value()) {
+    if (has_value() &&
+        !detail::is_shared_or_unique_ptr<folly::remove_cvref_t<T>>{}) {
       get_value() = static_cast<U&&>(other);
     } else {
       emplace(static_cast<U&&>(other));
@@ -1334,6 +1455,14 @@ class union_field_ref {
     return static_cast<reference_type>(get_value());
   }
 
+  template <typename U = std::remove_const_t<value_type>>
+  FOLLY_ERASE std::remove_const_t<value_type> value_or(
+      U&& default_value) const {
+    using type = std::remove_const_t<value_type>;
+    return has_value() ? type(static_cast<reference_type>(get_value()))
+                       : type(static_cast<U&&>(default_value));
+  }
+
   FOLLY_ERASE reference_type operator*() const { return value(); }
 
   FOLLY_ERASE value_type* operator->() const {
@@ -1351,7 +1480,7 @@ class union_field_ref {
   template <typename... Args>
   FOLLY_ERASE value_type& emplace(Args&&... args) {
     vtable_.reset(owner_);
-    ::new (&storage_value_) storage_value_type(static_cast<Args&&>(args)...);
+    emplace_impl(is_cpp_ref_or_boxed{}, static_cast<Args&&>(args)...);
     type_ = field_type_;
     return get_value();
   }
@@ -1363,8 +1492,7 @@ class union_field_ref {
       value_type&>
   emplace(std::initializer_list<U> ilist, Args&&... args) {
     vtable_.reset(owner_);
-    ::new (&storage_value_)
-        storage_value_type(ilist, static_cast<Args&&>(args)...);
+    emplace_impl(is_cpp_ref_or_boxed{}, ilist, static_cast<Args&&>(args)...);
     type_ = field_type_;
     return get_value();
   }
@@ -1376,13 +1504,32 @@ class union_field_ref {
     }
   }
 
-  FOLLY_ERASE value_type& get_value() const { return get_value(is_boxed{}); }
-  FOLLY_ERASE value_type& get_value(std::true_type) const {
-    return *storage_value_;
+  FOLLY_ERASE value_type& get_value() const {
+    return get_value(is_cpp_ref_or_boxed{});
   }
   FOLLY_ERASE value_type& get_value(std::false_type) const {
     return storage_value_;
   }
+  FOLLY_ERASE value_type& get_value(std::true_type) const {
+    if (storage_value_ == nullptr) {
+      // This can only happen if user used setter/getter to clear cpp.ref
+      // pointer. It won't happen if user didn't use setter/getter API at all.
+      apache::thrift::detail::throw_on_nullptr_dereferencing();
+    }
+    return *storage_value_;
+  }
+
+  template <class... Args>
+  FOLLY_ERASE void emplace_impl(std::false_type, Args&&... args) {
+    ::new (&storage_value_) storage_value_type(static_cast<Args&&>(args)...);
+  }
+
+  template <class... Args>
+  FOLLY_ERASE void emplace_impl(std::true_type, Args&&... args) {
+    ::new (&storage_value_) storage_value_type();
+    // TODO: use make_shared to initialize cpp.ref_type = "shared" field
+    storage_value_.reset(new element_type(static_cast<Args&&>(args)...));
+  }
 
   storage_value_type& storage_value_;
   int_t& type_;
@@ -1484,5 +1631,198 @@ bool operator>=(const U& a, union_field_ref<T> b) {
   return b <= a;
 }
 
+// A reference to a terse field of the possibly const-qualified type
+// std::remove_reference_t<T> in a Thrift-generated struct. Note, a terse field
+// does not need isset since we do not need to distinguish if the field is set
+// or unset.
+template <typename T>
+class terse_field_ref {
+  static_assert(std::is_reference<T>::value, "not a reference");
+
+  template <typename U>
+  friend class terse_field_ref;
+
+ public:
+  using value_type = std::remove_reference_t<T>;
+  using reference_type = T;
+
+  FOLLY_ERASE terse_field_ref(reference_type value) noexcept : value_(value) {}
+
+  template <
+      typename U,
+      std::enable_if_t<
+          std::is_same<
+              std::add_const_t<std::remove_reference_t<U>>,
+              value_type>{} &&
+              !(std::is_rvalue_reference<T>{} && std::is_lvalue_reference<U>{}),
+          int> = 0>
+  FOLLY_ERASE /* implicit */ terse_field_ref(
+      const terse_field_ref<U>& other) noexcept
+      : value_(other.value_) {}
+
+  template <
+      typename U,
+      std::enable_if_t<
+          std::is_same<T, U&&>{} || std::is_same<T, const U&&>{},
+          int> = 0>
+  FOLLY_ERASE explicit terse_field_ref(
+      const terse_field_ref<U&>& other) noexcept
+      : value_(other.value_) {}
+
+  template <typename U = value_type>
+  FOLLY_ERASE std::
+      enable_if_t<std::is_assignable<value_type&, U&&>::value, terse_field_ref&>
+      operator=(U&& value) noexcept(
+          std::is_nothrow_assignable<value_type&, U&&>::value) {
+    value_ = static_cast<U&&>(value);
+    return *this;
+  }
+
+  // Workaround for https://bugs.llvm.org/show_bug.cgi?id=49442
+  FOLLY_ERASE terse_field_ref& operator=(value_type&& value) noexcept(
+      std::is_nothrow_move_assignable<value_type>::value) {
+    value_ = static_cast<value_type&&>(value);
+    return *this;
+    value.~value_type(); // Force emit destructor...
+  }
+
+  template <typename U>
+  FOLLY_ERASE void copy_from(const terse_field_ref<U>& other) noexcept(
+      std::is_nothrow_assignable<value_type&, U>::value) {
+    value_ = other.value_;
+  }
+
+  template <typename U>
+  FOLLY_ERASE void move_from(terse_field_ref<U> other) noexcept(
+      std::is_nothrow_assignable<value_type&, std::remove_reference_t<U>&&>::
+          value) {
+    value_ = static_cast<std::remove_reference_t<U>&&>(other.value_);
+  }
+
+  FOLLY_ERASE reference_type value() const noexcept {
+    return static_cast<reference_type>(value_);
+  }
+
+  FOLLY_ERASE reference_type operator*() const noexcept {
+    return static_cast<reference_type>(value_);
+  }
+
+  FOLLY_ERASE value_type* operator->() const noexcept { return &value_; }
+
+  template <typename Index>
+  FOLLY_ERASE auto operator[](const Index& index) const -> decltype(auto) {
+    return value_[index];
+  }
+
+  template <typename... Args>
+  FOLLY_ERASE value_type& emplace(Args&&... args) {
+    value_ = value_type(static_cast<Args&&>(args)...);
+    return value_;
+  }
+
+  template <class U, class... Args>
+  FOLLY_ERASE std::enable_if_t<
+      std::is_constructible<value_type, std::initializer_list<U>, Args&&...>::
+          value,
+      value_type&>
+  emplace(std::initializer_list<U> ilist, Args&&... args) {
+    value_ = value_type(ilist, static_cast<Args&&>(args)...);
+    return value_;
+  }
+
+ private:
+  value_type& value_;
+};
+
+template <typename T, typename U>
+bool operator==(terse_field_ref<T> lhs, terse_field_ref<U> rhs) {
+  return *lhs == *rhs;
+}
+
+template <typename T, typename U>
+bool operator!=(terse_field_ref<T> lhs, terse_field_ref<U> rhs) {
+  return *lhs != *rhs;
+}
+
+template <typename T, typename U>
+bool operator<(terse_field_ref<T> lhs, terse_field_ref<U> rhs) {
+  return *lhs < *rhs;
+}
+
+template <typename T, typename U>
+bool operator>(terse_field_ref<T> lhs, terse_field_ref<U> rhs) {
+  return *lhs > *rhs;
+}
+
+template <typename T, typename U>
+bool operator<=(terse_field_ref<T> lhs, terse_field_ref<U> rhs) {
+  return *lhs <= *rhs;
+}
+
+template <typename T, typename U>
+bool operator>=(terse_field_ref<T> lhs, terse_field_ref<U> rhs) {
+  return *lhs >= *rhs;
+}
+
+template <typename T, typename U>
+bool operator==(terse_field_ref<T> lhs, const U& rhs) {
+  return *lhs == rhs;
+}
+
+template <typename T, typename U>
+bool operator!=(terse_field_ref<T> lhs, const U& rhs) {
+  return *lhs != rhs;
+}
+
+template <typename T, typename U>
+bool operator<(terse_field_ref<T> lhs, const U& rhs) {
+  return *lhs < rhs;
+}
+
+template <typename T, typename U>
+bool operator>(terse_field_ref<T> lhs, const U& rhs) {
+  return *lhs > rhs;
+}
+
+template <typename T, typename U>
+bool operator<=(terse_field_ref<T> lhs, const U& rhs) {
+  return *lhs <= rhs;
+}
+
+template <typename T, typename U>
+bool operator>=(terse_field_ref<T> lhs, const U& rhs) {
+  return *lhs >= rhs;
+}
+
+template <typename T, typename U>
+bool operator==(const T& lhs, terse_field_ref<U> rhs) {
+  return lhs == *rhs;
+}
+
+template <typename T, typename U>
+bool operator!=(const T& lhs, terse_field_ref<U> rhs) {
+  return lhs != *rhs;
+}
+
+template <typename T, typename U>
+bool operator<(const T& lhs, terse_field_ref<U> rhs) {
+  return lhs < *rhs;
+}
+
+template <typename T, typename U>
+bool operator>(const T& lhs, terse_field_ref<U> rhs) {
+  return lhs > *rhs;
+}
+
+template <typename T, typename U>
+bool operator<=(const T& lhs, terse_field_ref<U> rhs) {
+  return lhs <= *rhs;
+}
+
+template <typename T, typename U>
+bool operator>=(const T& lhs, terse_field_ref<U> rhs) {
+  return lhs >= *rhs;
+}
+
 } // namespace thrift
 } // namespace apache
