@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -18,14 +18,13 @@
 #include <folly/Portability.h>
 
 #include <thrift/lib/cpp2/GeneratedCodeHelper.h>
+#include <thrift/lib/cpp2/TrustedServerException.h>
 #include <thrift/lib/cpp2/protocol/BinaryProtocol.h>
 #include <thrift/lib/cpp2/protocol/CompactProtocol.h>
 #include <thrift/lib/cpp2/protocol/Protocol.h>
 
 using namespace std;
 using namespace folly;
-using namespace apache::thrift;
-using namespace apache::thrift::protocol;
 using namespace apache::thrift::transport;
 
 namespace apache {
@@ -207,16 +206,21 @@ MessageBegin deserializeMessageBegin(
 
 namespace detail {
 namespace si {
-std::string formatUnimplementedMethodException(std::string_view methodName) {
-  return fmt::format("Function {} is unimplemented", methodName);
+namespace {
+TrustedServerException createUnimplementedMethodException(
+    std::string_view methodName) {
+  return TrustedServerException::unimplementedMethodError(
+      fmt::format("Function {} is unimplemented", methodName).c_str());
 }
+} // namespace
 
-TApplicationException create_app_exn_unimplemented(const char* name) {
-  return TApplicationException(formatUnimplementedMethodException(name));
+folly::exception_wrapper create_app_exn_unimplemented(const char* name) {
+  return folly::make_exception_wrapper<TrustedServerException>(
+      createUnimplementedMethodException(name));
 }
 
 [[noreturn]] void throw_app_exn_unimplemented(char const* const name) {
-  throw create_app_exn_unimplemented(name);
+  throw createUnimplementedMethodException(name);
 }
 } // namespace si
 } // namespace detail
@@ -252,6 +256,7 @@ namespace util {
 
 void appendExceptionToHeader(
     const folly::exception_wrapper& ew, Cpp2RequestContext& ctx) {
+  ctx.setException();
   auto* ex = ew.get_exception();
   if (const auto* aex = dynamic_cast<const AppBaseError*>(ex)) {
     setUserExceptionHeader(
