@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -803,6 +803,19 @@ GeneratedAsyncProcessor::ProcessFunc<Derived> getProcessFuncFromProtocol(
   return funcs.binary;
 }
 
+template <typename Derived>
+GeneratedAsyncProcessor::ExecuteFunc<Derived> getExecuteFuncFromProtocol(
+    folly::tag_t<CompactProtocolReader> /* unused */,
+    const GeneratedAsyncProcessor::ProcessFuncs<Derived>& funcs) {
+  return funcs.compactExecute;
+}
+template <typename Derived>
+GeneratedAsyncProcessor::ExecuteFunc<Derived> getExecuteFuncFromProtocol(
+    folly::tag_t<BinaryProtocolReader> /* unused */,
+    const GeneratedAsyncProcessor::ProcessFuncs<Derived>& funcs) {
+  return funcs.binaryExecute;
+}
+
 inline void nonRecursiveProcessMissing(
     const std::string& methodName,
     ResponseChannelRequest::UniquePtr req,
@@ -904,7 +917,7 @@ void recursiveProcess(
  * See ServerInterface::GeneratedMethodMetadata.
  */
 template <class ProtocolReader, class Processor>
-void nonRecursiveProcess(
+void nonRecursiveProcessForInteraction(
     Processor* processor,
     ResponseChannelRequest::UniquePtr req,
     apache::thrift::SerializedCompressedRequest&& serializedRequest,
@@ -917,16 +930,103 @@ void nonRecursiveProcess(
   const auto& methodMetadata =
       AsyncProcessorHelper::expectMetadataOfType<Metadata>(
           untypedMethodMetadata);
+
+  DCHECK(
+      untypedMethodMetadata.interactionType ==
+          AsyncProcessor::MethodMetadata::InteractionType::INTERACTION_V1 ||
+      ctx->getInteractionId());
   auto pfn = getProcessFuncFromProtocol(
       folly::tag<ProtocolReader>, methodMetadata.processFuncs);
   (processor->*pfn)(std::move(req), std::move(serializedRequest), ctx, eb, tm);
 }
 
+inline void processViaExecuteRequest(
+    AsyncProcessor* processor,
+    ResponseChannelRequest::UniquePtr req,
+    apache::thrift::SerializedCompressedRequest&& serializedRequest,
+    const apache::thrift::AsyncProcessor::MethodMetadata& untypedMethodMetadata,
+    protocol::PROTOCOL_TYPES protType,
+    Cpp2RequestContext* ctx,
+    concurrency::ThreadManager* tm) {
+  DCHECK(
+      untypedMethodMetadata.executorType !=
+      AsyncProcessor::MethodMetadata::ExecutorType::UNKNOWN);
+  DCHECK(
+      untypedMethodMetadata.interactionType ==
+      AsyncProcessor::MethodMetadata::InteractionType::NONE);
+  DCHECK(untypedMethodMetadata.rpcKind);
+
+  if (!apache::thrift::GeneratedAsyncProcessor::validateRpcKind(
+          req, *untypedMethodMetadata.rpcKind)) {
+    return;
+  }
+
+  folly::Executor::KeepAlive<> executor;
+  if (untypedMethodMetadata.executorType ==
+          AsyncProcessor::MethodMetadata::ExecutorType::ANY &&
+      tm) {
+    executor = tm->getKeepAlive(
+        ctx->getRequestExecutionScope(),
+        concurrency::ThreadManager::Source::INTERNAL);
+  }
+
+  auto task = [serverRequest =
+                   ServerRequest{
+                       std::move(req),
+                       std::move(serializedRequest),
+                       ctx,
+                       protType,
+                       {},
+                       {},
+                       {}},
+               oneway =
+                   (*untypedMethodMetadata.rpcKind ==
+                    RpcKind::SINGLE_REQUEST_NO_RESPONSE),
+               processor = processor,
+               executor = std::move(executor),
+               &untypedMethodMetadata](bool runInline) mutable {
+    if (!runInline) {
+      if (serverRequest.requestContext()
+              ->getTimestamps()
+              .getSamplingStatus()
+              .isEnabled()) {
+        // Since this request was queued, reset the processBegin
+        // time to the actual start time, and not the queue time.
+        serverRequest.requestContext()->getTimestamps().processBegin =
+            std::chrono::steady_clock::now();
+      }
+    }
+    if (!oneway && !serverRequest.request()->getShouldStartProcessing()) {
+      HandlerCallbackBase::releaseRequest(
+          detail::ServerRequestHelper::request(std::move(serverRequest)),
+          detail::ServerRequestHelper::eventBase(serverRequest));
+      return;
+    }
+
+    detail::ServerRequestHelper::setExecutor(
+        serverRequest, std::move(executor));
+
+    processor->executeRequest(std::move(serverRequest), untypedMethodMetadata);
+  };
+
+  if (untypedMethodMetadata.executorType ==
+          AsyncProcessor::MethodMetadata::ExecutorType::ANY &&
+      tm) {
+    tm->getKeepAlive(
+          ctx->getRequestExecutionScope(),
+          concurrency::ThreadManager::Source::UPSTREAM)
+        ->add([task = std::move(task)]() mutable { task(false); });
+  } else {
+    task(true);
+  }
+}
+
 // Generated AsyncProcessor::processSerializedCompressedRequestWithMetadata just
 // calls this
 template <class Processor>
 void process(
     Processor* processor,
+    ServerInterface* si,
     ResponseChannelRequest::UniquePtr req,
     apache::thrift::SerializedCompressedRequest&& serializedRequest,
     const apache::thrift::AsyncProcessor::MethodMetadata& methodMetadata,
@@ -934,9 +1034,33 @@ void process(
     Cpp2RequestContext* ctx,
     folly::EventBase* eb,
     concurrency::ThreadManager* tm) {
+  using Metadata = ServerInterface::GeneratedMethodMetadata<Processor>;
+  static_assert(std::is_final_v<Metadata>);
+  AsyncProcessorHelper::expectMetadataOfType<Metadata>(methodMetadata);
+
+  if (methodMetadata.interactionType !=
+          AsyncProcessor::MethodMetadata::InteractionType::INTERACTION_V1 &&
+      !ctx->getInteractionId()) {
+    if (methodMetadata.executorType ==
+            AsyncProcessor::MethodMetadata::ExecutorType::ANY &&
+        tm) {
+      ctx->setRequestExecutionScope(si->getRequestExecutionScope(
+          ctx, methodMetadata.priority.value_or(concurrency::NORMAL)));
+    }
+
+    return processViaExecuteRequest(
+        processor,
+        std::move(req),
+        std::move(serializedRequest),
+        methodMetadata,
+        protType,
+        ctx,
+        tm);
+  }
+
   switch (protType) {
     case protocol::T_BINARY_PROTOCOL: {
-      return nonRecursiveProcess<BinaryProtocolReader>(
+      return nonRecursiveProcessForInteraction<BinaryProtocolReader>(
           processor,
           std::move(req),
           std::move(serializedRequest),
@@ -946,7 +1070,7 @@ void process(
           tm);
     }
     case protocol::T_COMPACT_PROTOCOL: {
-      return nonRecursiveProcess<CompactProtocolReader>(
+      return nonRecursiveProcessForInteraction<CompactProtocolReader>(
           processor,
           std::move(req),
           std::move(serializedRequest),
@@ -961,7 +1085,8 @@ void process(
   }
 }
 
-// Generated AsyncProcessor::processSerializedCompressedRequest just calls this
+// Generated AsyncProcessor::processSerializedCompressedRequest just calls
+// this
 template <class Processor>
 void process(
     Processor* processor,
@@ -986,6 +1111,39 @@ void process(
   }
 }
 
+template <class Processor>
+void execute(
+    Processor* processor,
+    ServerRequest&& request,
+    protocol::PROTOCOL_TYPES protType,
+    AsyncProcessor::MethodMetadata const& metadata) {
+  using Metadata = ServerInterface::GeneratedMethodMetadata<Processor>;
+  static_assert(std::is_final_v<Metadata>);
+  const bool isWildcard =
+      AsyncProcessorHelper::isWildcardMethodMetadata(metadata);
+
+  if (!isWildcard) {
+    const auto& methodMetadata =
+        AsyncProcessorHelper::expectMetadataOfType<Metadata>(metadata);
+    switch (protType) {
+      case protocol::T_BINARY_PROTOCOL: {
+        auto pfn = getExecuteFuncFromProtocol(
+            folly::tag<BinaryProtocolReader>, methodMetadata.processFuncs);
+        (processor->*pfn)(std::move(request));
+      } break;
+      case protocol::T_COMPACT_PROTOCOL: {
+        auto pfn = getExecuteFuncFromProtocol(
+            folly::tag<CompactProtocolReader>, methodMetadata.processFuncs);
+        (processor->*pfn)(std::move(request));
+      } break;
+      default:
+        LOG(ERROR) << "invalid protType: " << folly::to_underlying(protType);
+        return;
+    }
+  } else {
+    LOG(FATAL) << "Wildcard method metadata not yet supported here";
+  }
+}
 struct MessageBegin : folly::MoveOnly {
   std::string methodName;
   struct Metadata {
@@ -1019,34 +1177,50 @@ std::enable_if_t<
 downcastProcessFuncs(
     const GeneratedAsyncProcessor::ProcessFuncs<BaseProcessor>& processFuncs) {
   return GeneratedAsyncProcessor::ProcessFuncs<DerivedProcessor>{
-      processFuncs.compact, processFuncs.binary};
+      processFuncs.compact,
+      processFuncs.binary,
+      processFuncs.compactExecute,
+      processFuncs.binaryExecute};
 }
 
 template <
     class MostDerivedProcessor,
     class CurrentProcessor = MostDerivedProcessor>
-void populateMethodMetadataMap(AsyncProcessorFactory::MethodMetadataMap& map) {
+void populateMethodMetadataMap(
+    AsyncProcessorFactory::MethodMetadataMap& map,
+    const ServiceRequestInfoMap& requestInfoMap) {
   for (const auto& [methodName, processFuncs] :
        CurrentProcessor::getOwnProcessMap()) {
+    const auto& requestInfo = requestInfoMap.at(methodName);
     map.emplace(
         methodName,
         // Always create GeneratatedMethodMetadata<MostDerivedProcessor> so that
         // all entries in the map are of the same type.
         std::make_shared<
             ServerInterface::GeneratedMethodMetadata<MostDerivedProcessor>>(
-            downcastProcessFuncs<MostDerivedProcessor>(processFuncs)));
+            downcastProcessFuncs<MostDerivedProcessor>(processFuncs),
+            requestInfo.isSync
+                ? AsyncProcessorFactory::MethodMetadata::ExecutorType::EVB
+                : AsyncProcessorFactory::MethodMetadata::ExecutorType::ANY,
+            requestInfo.interactionName
+                ? AsyncProcessorFactory::MethodMetadata::InteractionType::
+                      INTERACTION_V1
+                : AsyncProcessorFactory::MethodMetadata::InteractionType::NONE,
+            requestInfo.rpcKind,
+            requestInfo.priority));
   }
   if constexpr (!is_root_async_processor<CurrentProcessor>) {
     populateMethodMetadataMap<
         MostDerivedProcessor,
-        typename CurrentProcessor::BaseAsyncProcessor>(map);
+        typename CurrentProcessor::BaseAsyncProcessor>(map, requestInfoMap);
   }
 }
 
 template <class Processor>
-AsyncProcessorFactory::MethodMetadataMap createMethodMetadataMap() {
+AsyncProcessorFactory::MethodMetadataMap createMethodMetadataMap(
+    const apache::thrift::ServiceRequestInfoMap& requestInfoMap) {
   AsyncProcessorFactory::MethodMetadataMap result;
-  populateMethodMetadataMap<Processor>(result);
+  populateMethodMetadataMap<Processor>(result, requestInfoMap);
   return result;
 }
 
@@ -1091,18 +1265,18 @@ EncodedStreamError encode_stream_exception(folly::exception_wrapper ew) {
     constexpr size_t kQueueAppenderGrowth = 4096;
     prot.setOutput(&queue, kQueueAppenderGrowth);
     TApplicationException ex(ew.what().toStdString());
-    exceptionMetadataBase.what_utf8_ref() = ex.what();
+    exceptionMetadataBase.what_utf8() = ex.what();
     apache::thrift::detail::serializeExceptionBody(&prot, &ex);
     PayloadAppUnknownExceptionMetdata aue;
-    aue.errorClassification_ref().ensure().blame_ref() = Blame;
+    aue.errorClassification().ensure().blame() = Blame;
     exceptionMetadata.appUnknownException_ref() = std::move(aue);
   }
 
-  exceptionMetadataBase.metadata_ref() = std::move(exceptionMetadata);
+  exceptionMetadataBase.metadata() = std::move(exceptionMetadata);
   StreamPayloadMetadata streamPayloadMetadata;
   PayloadMetadata payloadMetadata;
   payloadMetadata.exceptionMetadata_ref() = std::move(exceptionMetadataBase);
-  streamPayloadMetadata.payloadMetadata_ref() = std::move(payloadMetadata);
+  streamPayloadMetadata.payloadMetadata() = std::move(payloadMetadata);
   return EncodedStreamError(
       StreamPayload(std::move(queue).move(), std::move(streamPayloadMetadata)));
 }
@@ -1119,7 +1293,7 @@ class StreamElementEncoderImpl final
     StreamPayloadMetadata streamPayloadMetadata;
     PayloadMetadata payloadMetadata;
     payloadMetadata.responseMetadata_ref().ensure();
-    streamPayloadMetadata.payloadMetadata_ref() = std::move(payloadMetadata);
+    streamPayloadMetadata.payloadMetadata() = std::move(payloadMetadata);
     return folly::Try<StreamPayload>(
         {encode_stream_payload<Protocol, PResult>(std::move(val)),
          std::move(streamPayloadMetadata)});
@@ -1186,13 +1360,13 @@ folly::exception_wrapper decode_stream_exception(folly::exception_wrapper ew) {
       },
       [&hijacked](apache::thrift::detail::EncodedStreamError& err) {
         auto& payload = err.encoded;
-        DCHECK_EQ(payload.metadata.payloadMetadata_ref().has_value(), true);
+        DCHECK_EQ(payload.metadata.payloadMetadata().has_value(), true);
         DCHECK_EQ(
-            payload.metadata.payloadMetadata_ref()->getType(),
+            payload.metadata.payloadMetadata()->getType(),
             PayloadMetadata::exceptionMetadata);
         auto& exceptionMetadataBase =
-            payload.metadata.payloadMetadata_ref()->get_exceptionMetadata();
-        if (auto exceptionMetadataRef = exceptionMetadataBase.metadata_ref()) {
+            payload.metadata.payloadMetadata()->get_exceptionMetadata();
+        if (auto exceptionMetadataRef = exceptionMetadataBase.metadata()) {
           if (exceptionMetadataRef->getType() ==
               PayloadExceptionMetadata::declaredException) {
             PResult result;
@@ -1215,7 +1389,7 @@ folly::exception_wrapper decode_stream_exception(folly::exception_wrapper ew) {
             }
           } else {
             hijacked = TApplicationException(
-                exceptionMetadataBase.what_utf8_ref().value_or(""));
+                exceptionMetadataBase.what_utf8().value_or(""));
           }
         } else {
           hijacked =
@@ -1229,14 +1403,14 @@ folly::exception_wrapper decode_stream_exception(folly::exception_wrapper ew) {
         streamRpcError.read(&reader);
         TApplicationException::TApplicationExceptionType exType{
             TApplicationException::UNKNOWN};
-        auto code = streamRpcError.code_ref();
+        auto code = streamRpcError.code();
         if (code &&
             (code.value() == StreamRpcErrorCode::CREDIT_TIMEOUT ||
              code.value() == StreamRpcErrorCode::CHUNK_TIMEOUT)) {
           exType = TApplicationException::TIMEOUT;
         }
         hijacked = TApplicationException(
-            exType, streamRpcError.what_utf8_ref().value_or(""));
+            exType, streamRpcError.what_utf8().value_or(""));
       },
       [](...) {});
 
@@ -1373,7 +1547,11 @@ class AsyncTmPrep {
  public:
   AsyncTmPrep(ServerInterface* si, CallbackBase* callback) : si_{si} {
     si->setEventBase(callback->getEventBase());
-    si->setThreadManager(callback->getThreadManager());
+    if (auto threadManager = callback->getThreadManager_deprecated();
+        threadManager != nullptr) {
+      si->setThreadManager(threadManager);
+    }
+    si->setHandlerExecutor(callback->getHandlerExecutor());
     si->setRequestContext(callback->getRequestContext());
   }
 
@@ -1453,8 +1631,7 @@ void async_tm_coro(CallbackPtr<T> callback, folly::coro::Task<T>&& task) {
 }
 #endif
 
-std::string formatUnimplementedMethodException(std::string_view methodName);
-TApplicationException create_app_exn_unimplemented(const char* name);
+folly::exception_wrapper create_app_exn_unimplemented(const char* name);
 [[noreturn]] void throw_app_exn_unimplemented(char const* name);
 
 } // namespace si
@@ -1509,26 +1686,26 @@ std::string serializeExceptionMeta(const folly::exception_wrapper& ew) {
 
   constexpr auto errorKind = apache::thrift::detail::st::struct_private_access::
       __fbthrift_cpp2_gen_exception_kind<T>();
-  errorClassification.kind_ref() = fromExceptionKind(errorKind);
+  errorClassification.kind() = fromExceptionKind(errorKind);
   constexpr auto errorBlame = apache::thrift::detail::st::
       struct_private_access::__fbthrift_cpp2_gen_exception_blame<T>();
-  errorClassification.blame_ref() = fromExceptionBlame(errorBlame);
+  errorClassification.blame() = fromExceptionBlame(errorBlame);
   constexpr auto errorSafety = apache::thrift::detail::st::
       struct_private_access::__fbthrift_cpp2_gen_exception_safety<T>();
-  errorClassification.safety_ref() = fromExceptionSafety(errorSafety);
+  errorClassification.safety() = fromExceptionSafety(errorSafety);
 
-  ew.with_exception([&errorClassification](
-                        const ExceptionMetadataOverrideBase& ex) {
-    if (ex.errorKind() != ExceptionKind::UNSPECIFIED) {
-      errorClassification.kind_ref() = fromExceptionKind(ex.errorKind());
-    }
-    if (ex.errorBlame() != ExceptionBlame::UNSPECIFIED) {
-      errorClassification.blame_ref() = fromExceptionBlame(ex.errorBlame());
-    }
-    if (ex.errorSafety() != ExceptionSafety::UNSPECIFIED) {
-      errorClassification.safety_ref() = fromExceptionSafety(ex.errorSafety());
-    }
-  });
+  ew.with_exception(
+      [&errorClassification](const ExceptionMetadataOverrideBase& ex) {
+        if (ex.errorKind() != ExceptionKind::UNSPECIFIED) {
+          errorClassification.kind() = fromExceptionKind(ex.errorKind());
+        }
+        if (ex.errorBlame() != ExceptionBlame::UNSPECIFIED) {
+          errorClassification.blame() = fromExceptionBlame(ex.errorBlame());
+        }
+        if (ex.errorSafety() != ExceptionSafety::UNSPECIFIED) {
+          errorClassification.safety() = fromExceptionSafety(ex.errorSafety());
+        }
+      });
 
   return apache::thrift::detail::serializeErrorClassification(
       errorClassification);
