@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -59,40 +59,17 @@ namespace apache {
 namespace thrift {
 namespace detail {
 
-class PluggableFunctionSetter;
+template <typename Sig>
+class PluggableFunction;
 
 template <typename Ret, typename... Args>
-class PluggableFunction {
+class PluggableFunction<Ret(Args...)> {
  public:
   using signature = Ret(Args...);
 
   constexpr explicit PluggableFunction(signature& init) noexcept
       : init_{init} {}
 
-  template <typename... A>
-  FOLLY_ERASE auto operator()(A&&... a) const
-      -> decltype(FOLLY_DECLVAL(signature&)(static_cast<A&&>(a)...)) {
-    auto impl = impl_.load();
-    // MSVC 2017 dislikes the terse ternary form
-    if (FOLLY_UNLIKELY(!impl)) {
-      impl = &choose_slow();
-    }
-    return impl(static_cast<A&&>(a)...);
-  }
-
- private:
-  friend class PluggableFunctionSetter;
-
-  FOLLY_NOINLINE signature& choose_slow() const {
-    auto impl = impl_.load();
-    while (!impl) {
-      if (impl_.compare_exchange_weak(impl, &init_)) {
-        return init_;
-      }
-    }
-    return *impl;
-  }
-
   PluggableFunction const& operator=(signature& next) const noexcept {
     if (auto prev = impl_.exchange(&next)) {
       auto msg = prev == &init_
@@ -103,50 +80,50 @@ class PluggableFunction {
     return *this;
   }
 
+  template <typename... A>
+  FOLLY_ERASE auto operator()(A&&... a) const
+      -> decltype(FOLLY_DECLVAL(signature&)(static_cast<A&&>(a)...)) {
+    return choose()(static_cast<A&&>(a)...);
+  }
+
+ private:
+  FOLLY_ERASE signature* choose() const {
+    auto const impl = impl_.load();
+    return FOLLY_LIKELY(!!impl) ? impl : choose_slow();
+  }
+
+  FOLLY_NOINLINE signature* choose_slow() const {
+    auto impl = impl_.load();
+    while (!impl) {
+      if (impl_.compare_exchange_weak(impl, &init_)) {
+        return &init_;
+      }
+    }
+    return impl;
+  }
+
   //  impl_ should be first to avoid extra arithmetic in the fast path
   mutable folly::relaxed_atomic<signature*> impl_{nullptr};
   signature& init_;
 };
 
-class PluggableFunctionSetter {
- public:
-  template <typename R, typename... A>
-  PluggableFunctionSetter(
-      PluggableFunction<R, A...> const& plug, R (&next)(A...)) noexcept {
-    plug = next;
-  }
-};
-
-template <typename>
-struct pluggable_function_type_;
-template <typename Ret, typename... Args>
-struct pluggable_function_type_<Ret (*)(Args...)> {
-  using type = PluggableFunction<Ret, Args...>;
-};
-template <typename F>
-using pluggable_function_type_t = typename pluggable_function_type_<F>::type;
-
 } // namespace detail
 } // namespace thrift
 } // namespace apache
 
-#define THRIFT_PLUGGABLE_FUNC_DECLARE(_ret, _name, ...)             \
-  _ret THRIFT__PLUGGABLE_FUNC_DEFAULT_##_name(__VA_ARGS__);         \
-  using THRIFT__PLUGGABLE_FUNC_TYPE_##_name =                       \
-      decltype(&THRIFT__PLUGGABLE_FUNC_DEFAULT_##_name);            \
-  extern const ::apache::thrift::detail::pluggable_function_type_t< \
-      THRIFT__PLUGGABLE_FUNC_TYPE_##_name>                          \
+#define THRIFT_PLUGGABLE_FUNC_DECLARE(_ret, _name, ...)                       \
+  _ret THRIFT__PLUGGABLE_FUNC_DEFAULT_##_name(__VA_ARGS__);                   \
+  extern const ::apache::thrift::detail::PluggableFunction<_ret(__VA_ARGS__)> \
       _name
 
-#define THRIFT_PLUGGABLE_FUNC_REGISTER(_ret, _name, ...)             \
-  FOLLY_STORAGE_CONSTEXPR const ::apache::thrift::detail::           \
-      pluggable_function_type_t<THRIFT__PLUGGABLE_FUNC_TYPE_##_name> \
-          _name{THRIFT__PLUGGABLE_FUNC_DEFAULT_##_name};             \
+#define THRIFT_PLUGGABLE_FUNC_REGISTER(_ret, _name, ...)                     \
+  FOLLY_STORAGE_CONSTEXPR const ::apache::thrift::detail::PluggableFunction< \
+      _ret(__VA_ARGS__)>                                                     \
+      _name{THRIFT__PLUGGABLE_FUNC_DEFAULT_##_name};                         \
   _ret THRIFT__PLUGGABLE_FUNC_DEFAULT_##_name(__VA_ARGS__)
 
-#define THRIFT_PLUGGABLE_FUNC_SET(_ret, _name, ...)        \
-  _ret THRIFT__PLUGGABLE_FUNC_IMPL_##_name(__VA_ARGS__);   \
-  static ::apache::thrift::detail::PluggableFunctionSetter \
-      THRIFT__PLUGGABLE_FUNC_SETTER_##_name{               \
-          _name, THRIFT__PLUGGABLE_FUNC_IMPL_##_name};     \
+#define THRIFT_PLUGGABLE_FUNC_SET(_ret, _name, ...)      \
+  _ret THRIFT__PLUGGABLE_FUNC_IMPL_##_name(__VA_ARGS__); \
+  static bool THRIFT__PLUGGABLE_FUNC_SETTER_##_name =    \
+      (_name = THRIFT__PLUGGABLE_FUNC_IMPL_##_name, 0);  \
   _ret THRIFT__PLUGGABLE_FUNC_IMPL_##_name(__VA_ARGS__)
