@@ -17,6 +17,7 @@
 #ifndef THRIFT_CPP2_H_
 #define THRIFT_CPP2_H_
 
+#include <thrift/lib/cpp/FieldId.h>
 #include <thrift/lib/cpp/Thrift.h>
 #include <thrift/lib/cpp2/TypeClass.h>
 
@@ -31,6 +32,18 @@
 namespace apache {
 namespace thrift {
 
+enum class Ordinal : uint16_t {};
+
+namespace detail {
+template <typename Tag>
+struct invoke_reffer;
+} // namespace detail
+
+template <typename Tag>
+using access_field_fn = detail::invoke_reffer<Tag>;
+template <typename Tag>
+FOLLY_INLINE_VARIABLE constexpr access_field_fn<Tag> access_field{};
+
 enum FragileConstructor {
   FRAGILE,
 };
@@ -66,6 +79,9 @@ namespace st {
 //
 //  Thrift structures have private members but it may be necessary for the
 //  Thrift support library to access those private members.
+//
+//  TODO(dokwon): Rename struct_private_access to have single `private access`
+//  struct.
 struct struct_private_access {
   //  These should be alias templates but Clang has a bug where it does not
   //  permit member alias templates of a friend struct to access private
@@ -76,16 +92,14 @@ struct struct_private_access {
   __fbthrift_cpp2_gen_json();
 
   template <typename T>
-  static folly::bool_constant<T::__fbthrift_cpp2_gen_nimble> //
-  __fbthrift_cpp2_gen_nimble();
+  static typename T::__fbthrift_fields __fbthrift_fields();
 
   template <typename T>
-  static folly::bool_constant<T::__fbthrift_cpp2_gen_has_thrift_uri> //
-  __fbthrift_cpp2_gen_has_thrift_uri();
+  using fields = decltype(__fbthrift_fields<T>());
 
   template <typename T>
-  static const char* __fbthrift_cpp2_gen_thrift_uri() {
-    return T::__fbthrift_cpp2_gen_thrift_uri();
+  static const char* __fbthrift_thrift_uri() {
+    return T::__fbthrift_thrift_uri();
   }
 
   template <typename T>
@@ -105,8 +119,12 @@ struct struct_private_access {
 
   FOLLY_CREATE_MEMBER_INVOKER(clear_fn, __fbthrift_clear);
   FOLLY_CREATE_MEMBER_INVOKER(empty_fn, __fbthrift_is_empty);
+  FOLLY_CREATE_STATIC_MEMBER_INVOKER(ordinal_fn, __fbthrift_ordinal);
 
-  FOLLY_INLINE_VARIABLE static constexpr empty_fn is_empty{};
+  template <typename T, FieldId Id>
+  static decltype(T::__fbthrift_ident(FieldIdTag<Id>{})) __fbthrift_ident();
+  template <typename T, FieldId Id>
+  using ident_tag = decltype(__fbthrift_ident<T, Id>());
 };
 
 template <typename T, typename = void>
@@ -129,6 +147,17 @@ struct IsThriftUnion<T, folly::void_t<typename T::__fbthrift_cpp2_type>>
 using clear_fn = detail::st::struct_private_access::clear_fn;
 FOLLY_INLINE_VARIABLE constexpr clear_fn clear{};
 
+using empty_fn = detail::st::struct_private_access::empty_fn;
+FOLLY_INLINE_VARIABLE static constexpr empty_fn empty{};
+
+// TODO(dokwon): Add apache::thrift::uri support for generated enum types.
+template <typename T>
+FOLLY_EXPORT const std::string& uri() {
+  static const auto& kUri = *new std::string(
+      detail::st::struct_private_access::__fbthrift_thrift_uri<T>());
+  return kUri;
+}
+
 template <typename T>
 constexpr bool is_thrift_class_v =
     apache::thrift::detail::st::IsThriftClass<T>::value;
