@@ -34,7 +34,7 @@ EventTask::~EventTask() {
 
 void EventTask::expired() {
   // only expire req_ once
-  if (!req_) {
+  if (!req_.request()) {
     return;
   }
   failWith(
@@ -44,7 +44,8 @@ void EventTask::expired() {
 
 void EventTask::failWith(folly::exception_wrapper ex, std::string exCode) {
   auto cleanUp = [oneway = oneway_,
-                  req = std::move(req_),
+                  req = apache::thrift::detail::ServerRequestHelper::request(
+                      std::move(req_)),
                   ex = std::move(ex),
                   exCode = std::move(exCode)]() mutable {
     // if oneway, skip sending back anything
@@ -54,15 +55,21 @@ void EventTask::failWith(folly::exception_wrapper ex, std::string exCode) {
     req->sendErrorWrapped(std::move(ex), std::move(exCode));
   };
 
-  if (eb_->inRunningEventBaseThread()) {
+  auto eb = apache::thrift::detail::ServerRequestHelper::eventBase(req_);
+
+  if (eb->inRunningEventBaseThread()) {
     cleanUp();
   } else {
-    eb_->runInEventBaseThread(std::move(cleanUp));
+    eb->runInEventBaseThread(std::move(cleanUp));
   }
 }
 
 void EventTask::setTile(TilePtr&& tile) {
-  ctx_->setTile(std::move(tile));
+  req_.requestContext()->setTile(std::move(tile));
+}
+
+char const* AsyncProcessor::getServiceName() {
+  return "NoServiceNameSet";
 }
 
 void AsyncProcessor::terminateInteraction(
@@ -104,16 +111,6 @@ void AsyncProcessor::processSerializedCompressedRequestWithMetadata(
          "Therefore, this method must be overridden alongside processSerializedRequest.";
 }
 
-bool AsyncProcessor::useResourcePools(
-    AsyncProcessorFactory::MethodMetadata const*) const {
-  return false;
-}
-
-SelectPoolResult AsyncProcessor::selectPool(
-    ServerRequest const&, const AsyncProcessorFactory::MethodMetadata&) const {
-  LOG(FATAL) << "Unimplemented selectPool called";
-}
-
 void AsyncProcessor::executeRequest(
     ServerRequest&&, const AsyncProcessorFactory::MethodMetadata&) {
   LOG(FATAL) << "Unimplemented executeRequest called";
@@ -126,7 +123,8 @@ bool GeneratedAsyncProcessor::createInteraction(
     Cpp2RequestContext& ctx,
     concurrency::ThreadManager* tm,
     folly::EventBase& eb,
-    ServerInterface* si) {
+    ServerInterface* si,
+    bool isFactoryFunction) {
   eb.dcheckIsInEventBaseThread();
 
   auto nullthrows = [](std::unique_ptr<Tile> tile) {
@@ -138,8 +136,9 @@ bool GeneratedAsyncProcessor::createInteraction(
   };
   auto& conn = *ctx.getConnectionContext();
 
-  // In the eb model we create the interaction inline.
-  if (!tm) {
+  // In the eb model with old-style constructor we create the interaction
+  // inline.
+  if (!tm && !isFactoryFunction) {
     si->setEventBase(&eb);
     si->setRequestContext(&ctx);
     auto tile = folly::makeTryWith(
@@ -155,40 +154,46 @@ bool GeneratedAsyncProcessor::createInteraction(
     return conn.addTile(id, {tile->release(), &eb});
   }
 
-  // In the tm model we use a promise.
-  auto promisePtr = new TilePromise; // freed by RefGuard on next line
+  // Otherwise we use a promise.
+  auto promisePtr =
+      new TilePromise(isFactoryFunction); // freed by RefGuard on next line
   if (!conn.addTile(id, {promisePtr, &eb})) {
     return false;
   }
 
-  tm->add([=, &eb, &ctx, name = std::move(name), &conn] {
-    si->setEventBase(&eb);
-    si->setThreadManager(tm);
-    si->setRequestContext(&ctx);
+  // Old-style constructor + tm : schedule constructor and return
+  if (!isFactoryFunction) {
+    tm->add([=, &eb, &ctx, name = std::move(name), &conn] {
+      si->setEventBase(&eb);
+      si->setThreadManager(tm);
+      si->setRequestContext(&ctx);
 
-    std::exception_ptr ex;
-    try {
-      auto tilePtr = nullthrows(createInteractionImpl(name));
-      eb.add([=, &conn, &eb, t = std::move(tilePtr)]() mutable {
-        TilePtr tile{t.release(), &eb};
-        promisePtr->fulfill(*tile, *tm, eb);
-        conn.tryReplaceTile(id, std::move(tile));
+      std::exception_ptr ex;
+      try {
+        auto tilePtr = nullthrows(createInteractionImpl(name));
+        eb.add([=, &conn, &eb, t = std::move(tilePtr)]() mutable {
+          TilePtr tile{t.release(), &eb};
+          promisePtr->fulfill(*tile, tm, eb);
+          conn.tryReplaceTile(id, std::move(tile));
+        });
+        return;
+      } catch (...) {
+        ex = std::current_exception();
+      }
+      DCHECK(ex);
+      eb.add([promisePtr, ex = std::move(ex)]() {
+        promisePtr->failWith(
+            folly::make_exception_wrapper<TApplicationException>(
+                folly::to<std::string>(
+                    "Interaction constructor failed with ",
+                    folly::exceptionStr(ex))),
+            kInteractionConstructorErrorErrorCode);
       });
-      return;
-    } catch (...) {
-      ex = std::current_exception();
-    }
-    DCHECK(ex);
-    eb.add([=, &conn, ex = std::move(ex)]() {
-      promisePtr->failWith(
-          folly::make_exception_wrapper<TApplicationException>(
-              folly::to<std::string>(
-                  "Interaction constructor failed with ",
-                  folly::exceptionStr(ex))),
-          kInteractionConstructorErrorErrorCode);
-      conn.removeTile(id);
     });
-  });
+    return true;
+  }
+
+  // Factory function: the handler method will fulfill the promise
   return true;
 }
 
@@ -201,7 +206,9 @@ void GeneratedAsyncProcessor::terminateInteraction(
     int64_t id, Cpp2ConnContext& conn, folly::EventBase& eb) noexcept {
   eb.dcheckIsInEventBaseThread();
 
-  conn.removeTile(id);
+  if (auto tile = conn.removeTile(id)) {
+    Tile::__fbthrift_onTermination(std::move(tile), eb);
+  }
 }
 
 void GeneratedAsyncProcessor::destroyAllInteractions(
@@ -218,7 +225,7 @@ void GeneratedAsyncProcessor::destroyAllInteractions(
     ids.push_back(id);
   }
   for (auto id : ids) {
-    terminateInteraction(id, conn, eb);
+    conn.removeTile(id);
   }
 }
 
@@ -267,7 +274,8 @@ bool GeneratedAsyncProcessor::setUpRequestProcessing(
     concurrency::ThreadManager* tm,
     RpcKind kind,
     ServerInterface* si,
-    folly::StringPiece interaction) {
+    folly::StringPiece interaction,
+    bool isInteractionFactoryFunction) {
   if (!validateRpcKind(req, kind)) {
     return false;
   }
@@ -284,7 +292,8 @@ bool GeneratedAsyncProcessor::setUpRequestProcessing(
                      *ctx,
                      tm,
                      *eb,
-                     si)) {
+                     si,
+                     isInteractionFactoryFunction)) {
         // Duplicate id is a contract violation so close the connection.
         // Terminate this interaction first so queued requests can't use it
         // (which could result in UB).
@@ -341,11 +350,18 @@ void ServerInterface::setEventBase(folly::EventBase* eb) {
 }
 
 void ServerInterface::BlockingThreadManager::add(folly::Func f) {
-  std::shared_ptr<concurrency::Runnable> task =
-      concurrency::FunctionRunner::create(std::move(f));
   try {
-    executor_->add(
-        std::move(task), std::chrono::milliseconds(kTimeout).count(), 0, false);
+    if (threadManagerKa_) {
+      std::shared_ptr<concurrency::Runnable> task =
+          concurrency::FunctionRunner::create(std::move(f));
+      threadManagerKa_->add(
+          std::move(task),
+          std::chrono::milliseconds(kTimeout).count() /* deprecated */,
+          0,
+          false);
+    } else {
+      executorKa_->add(std::move(f));
+    }
     return;
   } catch (...) {
     LOG(FATAL) << "Failed to schedule a task within timeout: "
@@ -369,18 +385,23 @@ void ServerInterface::BlockingThreadManager::keepAliveRelease() noexcept {
 }
 
 folly::Executor::KeepAlive<> ServerInterface::getInternalKeepAlive() {
-  return getThreadManager()->getKeepAlive(
-      getRequestContext()->getRequestExecutionScope(),
-      apache::thrift::concurrency::ThreadManager::Source::INTERNAL);
+  if (getThreadManager_deprecated()) {
+    return getThreadManager_deprecated()->getKeepAlive(
+        getRequestContext()->getRequestExecutionScope(),
+        apache::thrift::concurrency::ThreadManager::Source::INTERNAL);
+  } else {
+    return folly::Executor::getKeepAliveToken(getHandlerExecutor());
+  }
 }
 
-folly::Executor::KeepAlive<> HandlerCallbackBase::getInternalKeepAlive() {
-  return getThreadManager()->getKeepAlive(
-      getRequestContext()->getRequestExecutionScope(),
-      apache::thrift::concurrency::ThreadManager::Source::INTERNAL);
+const folly::Executor::KeepAlive<>&
+HandlerCallbackBase::getInternalKeepAlive() {
+  DCHECK(executor_);
+  return executor_;
 }
 
 HandlerCallbackBase::~HandlerCallbackBase() {
+  maybeNotifyComplete();
   // req must be deleted in the eb
   if (req_) {
     if (req_->isActive() && ewp_) {
@@ -389,15 +410,16 @@ HandlerCallbackBase::~HandlerCallbackBase() {
           "apache::thrift::HandlerCallback not completed"));
       return;
     }
-    assert(eb_ != nullptr);
-    releaseRequest(std::move(req_), eb_, std::move(interaction_));
+    if (getEventBase()) {
+      releaseRequest(std::move(req_), getEventBase(), std::move(interaction_));
+    }
   }
 }
 
 void HandlerCallbackBase::releaseRequest(
     ResponseChannelRequest::UniquePtr request,
     folly::EventBase* eb,
-    TilePtr&& interaction) {
+    TilePtr interaction) {
   DCHECK(request);
   DCHECK(eb != nullptr);
   if (!eb->inRunningEventBaseThread()) {
@@ -407,19 +429,31 @@ void HandlerCallbackBase::releaseRequest(
 }
 
 folly::EventBase* HandlerCallbackBase::getEventBase() {
-  assert(eb_ != nullptr);
   return eb_;
 }
 
-concurrency::ThreadManager* HandlerCallbackBase::getThreadManager() {
-  assert(tm_ != nullptr);
-  return tm_;
+concurrency::ThreadManager* HandlerCallbackBase::getThreadManager_deprecated() {
+  if (reqCtx_) {
+    if (auto connCtx = reqCtx_->getConnectionContext()) {
+      if (auto workerCtx = connCtx->getWorkerContext()) {
+        if (auto serverCtx = workerCtx->getServerContext()) {
+          return serverCtx->getThreadManager_deprecated().get();
+        }
+      }
+    }
+  }
+  return nullptr;
 }
 
-void HandlerCallbackBase::forward(const HandlerCallbackBase& other) {
-  eb_ = other.eb_;
-  tm_ = other.tm_;
-  ewp_ = other.ewp_;
+folly::Executor* HandlerCallbackBase::getThreadManager() {
+  return getHandlerExecutor();
+}
+
+folly::Executor* HandlerCallbackBase::getHandlerExecutor() {
+  if (!executor_) {
+    return getThreadManager_deprecated();
+  }
+  return executor_.get();
 }
 
 folly::Optional<uint32_t> HandlerCallbackBase::checksumIfNeeded(
@@ -457,17 +491,6 @@ void HandlerCallbackBase::doExceptionWrapped(folly::exception_wrapper ew) {
   }
 }
 
-void HandlerCallbackBase::doAppOverloadedException(const std::string& message) {
-  if (eb_->inRunningEventBaseThread()) {
-    AppOverloadExceptionInfo::send(std::move(req_), message);
-  } else {
-    putMessageInReplyQueue(
-        std::in_place_type_t<AppOverloadExceptionInfo>(),
-        std::move(req_),
-        message);
-  }
-}
-
 void HandlerCallbackBase::sendReply(SerializedResponse response) {
   folly::Optional<uint32_t> crc32c = checksumIfNeeded(response);
   auto payload = std::move(response).extractPayload(
@@ -477,9 +500,8 @@ void HandlerCallbackBase::sendReply(SerializedResponse response) {
       MessageType::T_REPLY,
       reqCtx_->getMethodName());
   payload = transform(std::move(payload));
-  if (getEventBase()->isInEventBaseThread()) {
-    QueueReplyInfo(
-        std::move(req_), std::move(payload), crc32c)(*getEventBase());
+  if (getEventBase() && getEventBase()->inRunningEventBaseThread()) {
+    QueueReplyInfo(std::move(req_), std::move(payload), crc32c)();
   } else {
     putMessageInReplyQueue(
         std::in_place_type_t<QueueReplyInfo>(),
@@ -505,8 +527,7 @@ void HandlerCallbackBase::sendReply(
   stream.setInteraction(std::move(interaction_));
   if (getEventBase()->isInEventBaseThread()) {
     StreamReplyInfo(
-        std::move(req_), std::move(stream), std::move(payload), crc32c)(
-        *getEventBase());
+        std::move(req_), std::move(stream), std::move(payload), crc32c)();
   } else {
     putMessageInReplyQueue(
         std::in_place_type_t<StreamReplyInfo>(),
@@ -537,8 +558,7 @@ void HandlerCallbackBase::sendReply(
 
   if (getEventBase()->isInEventBaseThread()) {
     SinkConsumerReplyInfo(
-        std::move(req_), std::move(sinkConsumer), std::move(payload), crc32c)(
-        *getEventBase());
+        std::move(req_), std::move(sinkConsumer), std::move(payload), crc32c)();
   } else {
     putMessageInReplyQueue(
         std::in_place_type_t<SinkConsumerReplyInfo>(),
@@ -552,6 +572,44 @@ void HandlerCallbackBase::sendReply(
 #endif
 }
 
+bool HandlerCallbackBase::fulfillTilePromise(std::unique_ptr<Tile> ptr) {
+  if (!ptr) {
+    DLOG(FATAL) << "Nullptr interaction yielded from handler";
+    exception(TApplicationException(
+        TApplicationException::MISSING_RESULT,
+        "Nullptr interaction yielded from handler"));
+    return false;
+  }
+
+  auto fn = [ctx = reqCtx_,
+             interaction = std::move(interaction_),
+             ptr = std::move(ptr),
+             tm = getThreadManager_deprecated(),
+             eb = eb_]() mutable {
+    TilePtr tile{ptr.release(), eb};
+    DCHECK(dynamic_cast<TilePromise*>(interaction.get()));
+    static_cast<TilePromise&>(*interaction).fulfill(*tile, tm, *eb);
+    ctx->getConnectionContext()->tryReplaceTile(
+        ctx->getInteractionId(), std::move(tile));
+  };
+
+  eb_->runImmediatelyOrRunInEventBaseThread(std::move(fn));
+  return true;
+}
+
+void HandlerCallbackBase::breakTilePromise() {
+  auto fn = [interaction = std::move(interaction_)]() mutable {
+    DCHECK(dynamic_cast<TilePromise*>(interaction.get()));
+    static_cast<TilePromise&>(*interaction)
+        .failWith(
+            folly::make_exception_wrapper<TApplicationException>(
+                "Interaction constructor failed"),
+            kInteractionConstructorErrorErrorCode);
+  };
+
+  eb_->runImmediatelyOrRunInEventBaseThread(std::move(fn));
+}
+
 HandlerCallback<void>::HandlerCallback(
     ResponseChannelRequest::UniquePtr req,
     std::unique_ptr<ContextStack> ctx,
@@ -574,7 +632,38 @@ HandlerCallback<void>::HandlerCallback(
   this->protoSeqId_ = protoSeqId;
 }
 
+HandlerCallback<void>::HandlerCallback(
+    ResponseChannelRequest::UniquePtr req,
+    std::unique_ptr<ContextStack> ctx,
+    cob_ptr cp,
+    exnw_ptr ewp,
+    int32_t protoSeqId,
+    folly::EventBase* eb,
+    folly::Executor::KeepAlive<> executor,
+    Cpp2RequestContext* reqCtx,
+    RequestPileInterface* notifyRequestPile,
+    RequestPileInterface::UserData notifyRequestPileUserData,
+    ConcurrencyControllerInterface* notifyConcurrencyController,
+    ConcurrencyControllerInterface::UserData notifyConcurrencyControllerData,
+    TilePtr&& interaction)
+    : HandlerCallbackBase(
+          std::move(req),
+          std::move(ctx),
+          ewp,
+          eb,
+          std::move(executor),
+          reqCtx,
+          notifyRequestPile,
+          notifyRequestPileUserData,
+          notifyConcurrencyController,
+          notifyConcurrencyControllerData,
+          std::move(interaction)),
+      cp_(cp) {
+  this->protoSeqId_ = protoSeqId;
+}
+
 void HandlerCallback<void>::complete(folly::Try<folly::Unit>&& r) {
+  maybeNotifyComplete();
   if (r.hasException()) {
     exception(std::move(r.exception()));
   } else {
