@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,6 +16,10 @@
 
 #pragma once
 
+#include <algorithm>
+#include <array>
+#include <variant>
+
 #include <folly/GLog.h>
 #include <folly/Portability.h>
 #include <folly/experimental/coro/AsyncGenerator.h>
@@ -120,7 +124,7 @@ class ClientBufferedStream {
 #if FOLLY_HAS_COROUTINES
   folly::coro::AsyncGenerator<T&&> toAsyncGenerator() && {
     return bufferOptions_.memSize
-        ? toAsyncGeneratorWithSizeTarget<false>(
+        ? toAsyncGeneratorWithSizeTarget(
               std::move(streamBridge_),
               bufferOptions_.chunkSize,
               decode_,
@@ -131,20 +135,23 @@ class ClientBufferedStream {
   }
 
   struct PayloadAndHeader {
-    std::optional<T> payload; // empty for header packet
+    T payload;
     StreamPayloadMetadata metadata;
   };
-  folly::coro::AsyncGenerator<PayloadAndHeader&&>
+  struct UnorderedHeader {
+    StreamPayloadMetadata metadata;
+  };
+  struct OrderedHeader {
+    StreamPayloadMetadata metadata;
+  };
+  using MessageVariant =
+      std::variant<T, PayloadAndHeader, UnorderedHeader, OrderedHeader>;
+  folly::coro::AsyncGenerator<MessageVariant&&>
   toAsyncGeneratorWithHeader() && {
-    return bufferOptions_.memSize
-        ? toAsyncGeneratorWithSizeTarget<true>(
-              std::move(streamBridge_),
-              bufferOptions_.chunkSize,
-              decode_,
-              bufferOptions_.memSize,
-              bufferOptions_.maxChunkSize)
-        : toAsyncGeneratorImpl<true>(
-              std::move(streamBridge_), bufferOptions_.chunkSize, decode_);
+    CHECK_EQ(bufferOptions_.memSize, 0)
+        << "MemoryBufferSize not supported by toAsyncGeneratorWithHeader()";
+    return toAsyncGeneratorImpl<true>(
+        std::move(streamBridge_), bufferOptions_.chunkSize, decode_);
   }
 #endif // FOLLY_HAS_COROUTINES
 
@@ -172,7 +179,7 @@ class ClientBufferedStream {
 #if FOLLY_HAS_COROUTINES
   template <bool WithHeader>
   static folly::coro::AsyncGenerator<
-      std::conditional_t<WithHeader, PayloadAndHeader, T>&&>
+      std::conditional_t<WithHeader, MessageVariant, T>&&>
   toAsyncGeneratorImpl(
       apache::thrift::detail::ClientStreamBridge::ClientPtr streamBridge,
       int32_t chunkBufferSize,
@@ -185,6 +192,15 @@ class ClientBufferedStream {
     int32_t outstanding = chunkBufferSize;
     size_t payloadDataSize = 0;
 
+    auto updateCredits = [&] {
+      if ((--outstanding <= chunkBufferSize / 2) ||
+          (payloadDataSize >= kRequestCreditPayloadSize)) {
+        streamBridge->requestN(chunkBufferSize - outstanding);
+        outstanding = chunkBufferSize;
+        payloadDataSize = 0;
+      }
+    };
+
     apache::thrift::detail::ClientStreamBridge::ClientQueue queue;
     class ReadyCallback : public apache::thrift::detail::ClientStreamConsumer {
      public:
@@ -219,30 +235,46 @@ class ClientBufferedStream {
         if (!payload.hasValue() && !payload.hasException()) {
           break;
         }
+        const size_t payloadSize = payload.hasValue() && payload->payload
+            ? payload->payload->computeChainDataLength()
+            : 0;
         if (payload.hasValue()) {
-          if (!payload->payload) {
+          if (!payloadSize) {
             if constexpr (!WithHeader) {
               FB_LOG_EVERY_MS(WARNING, 1000)
                   << "Dropping unhandled stream header frame";
+              if (payload->isOrderedHeader) {
+                updateCredits();
+              }
               queue.pop();
               continue;
             }
           } else {
-            payloadDataSize += payload->payload->computeChainDataLength();
+            payloadDataSize += payloadSize;
           }
         }
         if constexpr (WithHeader) {
           if (payload.hasValue()) {
-            PayloadAndHeader ret;
-            bool usedCredit = false;
-            ret.metadata = std::move(payload->metadata);
-            if (payload->payload) {
-              ret.payload = *decode(std::move(payload));
-              usedCredit = true;
-            }
-            queue.pop();
-            co_yield std::move(ret);
-            if (!usedCredit) {
+            if (payloadSize) {
+              if (payload->metadata.otherMetadata().has_value()) {
+                PayloadAndHeader ret;
+                ret.metadata = std::move(payload->metadata);
+                ret.payload = *decode(std::move(payload));
+                queue.pop();
+                co_yield std::move(ret);
+              } else {
+                T ret = *decode(std::move(payload));
+                queue.pop();
+                co_yield std::move(ret);
+              }
+            } else if (payload->isOrderedHeader) {
+              OrderedHeader ret{std::move(payload->metadata)};
+              queue.pop();
+              co_yield std::move(ret);
+            } else {
+              UnorderedHeader ret{std::move(payload->metadata)};
+              queue.pop();
+              co_yield std::move(ret);
               continue;
             }
           } else {
@@ -254,21 +286,12 @@ class ClientBufferedStream {
           queue.pop();
           co_yield folly::coro::co_result(std::move(value));
         }
-
-        if ((--outstanding <= chunkBufferSize / 2) ||
-            (payloadDataSize >= kRequestCreditPayloadSize)) {
-          streamBridge->requestN(chunkBufferSize - outstanding);
-          outstanding = chunkBufferSize;
-          payloadDataSize = 0;
-        }
+        updateCredits();
       }
     }
   }
 
-  template <bool WithHeader>
-  static folly::coro::AsyncGenerator<
-      std::conditional_t<WithHeader, PayloadAndHeader, T>&&>
-  toAsyncGeneratorWithSizeTarget(
+  static folly::coro::AsyncGenerator<T&&> toAsyncGeneratorWithSizeTarget(
       apache::thrift::detail::ClientStreamBridge::ClientPtr streamBridge,
       int32_t chunkBufferSize,
       folly::Try<T> (*decode)(folly::Try<StreamPayload>&&),
@@ -280,7 +303,51 @@ class ClientBufferedStream {
     }
 
     int32_t outstanding = chunkBufferSize;
-    size_t maxPayloadSize = 0, bufferMemSize = 0;
+    size_t bufferMemSize = 0;
+
+    // Use a circular buffer with the latest payload sizes to estimate the
+    // recent average payload size.
+    constexpr size_t kEstimationWindowSize = 128;
+    std::array<size_t, kEstimationWindowSize> payloadSizesWindow;
+    std::fill(payloadSizesWindow.begin(), payloadSizesWindow.end(), 0);
+
+    size_t windowSum = 0;
+    size_t numReceivedPayloads = 0;
+    size_t maxPayloadSize = 0;
+
+    auto updateCredits = [&] {
+      DCHECK_LT(0, outstanding);
+      --outstanding;
+      // If enough payloads have been received estimate the recent average
+      // payload size, otherwise conservatively use the largest received
+      // size.
+      size_t estPayloadSize = std::max<size_t>(
+          numReceivedPayloads < kEstimationWindowSize
+              ? maxPayloadSize
+              : (windowSum + kEstimationWindowSize - 1) / kEstimationWindowSize,
+          1);
+      size_t outstandingSize = bufferMemSize + outstanding * estPayloadSize;
+      size_t spaceAvailable =
+          std::max<ssize_t>(memBufferTarget - outstandingSize, 0);
+
+      // Issue more credits when available space is at least 16kB (to
+      // amortize the request over 16kB worth of received payloads) or half
+      // of the buffer (to ensure there are enough outstanding credits in
+      // the small buffer / small payloads regime).
+      if (spaceAvailable >= kRequestCreditPayloadSize ||
+          spaceAvailable >= memBufferTarget / 2) {
+        // Convert to credits using the size estimate, but cap credits and
+        // outstanding requests to maxChunkBufferSize if this was requested
+        // in BufferOptions.
+        DCHECK_LE(outstanding, maxChunkBufferSize);
+        const int32_t remainingCredits = maxChunkBufferSize - outstanding;
+        int32_t request =
+            (spaceAvailable + estPayloadSize - 1) / estPayloadSize;
+        request = std::min(remainingCredits, request);
+        streamBridge->requestN(request);
+        outstanding += request;
+      }
+    };
 
     apache::thrift::detail::ClientStreamBridge::ClientQueueWithTailPtr queue;
     class ReadyCallback : public apache::thrift::detail::ClientStreamConsumer {
@@ -332,71 +399,31 @@ class ClientBufferedStream {
         if (!payload.hasValue() && !payload.hasException()) {
           break;
         }
+        const size_t payloadSize = payload.hasValue() && payload->payload
+            ? payload->payload->computeChainDataLength()
+            : 0;
         if (payload.hasValue()) {
-          if (!payload->payload) {
-            if constexpr (!WithHeader) {
-              FB_LOG_EVERY_MS(WARNING, 1000)
-                  << "Dropping unhandled stream header frame";
-              queue.pop();
-              continue;
-            }
-          } else {
-            maxPayloadSize = std::max(
-                maxPayloadSize, payload->payload->computeChainDataLength());
-          }
-        }
-        if constexpr (WithHeader) {
-          if (payload.hasValue()) {
-            PayloadAndHeader ret;
-            bool usedCredit = false;
-            ret.metadata = std::move(payload->metadata);
-            if (payload->payload) {
-              bufferMemSize -= payload->payload->computeChainDataLength();
-              ret.payload = *decode(std::move(payload));
-              usedCredit = true;
+          if (!payloadSize) {
+            FB_LOG_EVERY_MS(WARNING, 1000)
+                << "Dropping unhandled stream header frame";
+            if (payload->isOrderedHeader) {
+              updateCredits();
             }
             queue.pop();
-            co_yield std::move(ret);
-            if (!usedCredit) {
-              continue;
-            }
-          } else {
-            co_yield folly::coro::co_error(
-                decode(std::move(payload)).exception());
+            continue;
           }
-        } else {
-          if (payload.hasValue() && payload->payload) {
-            bufferMemSize -= payload->payload->computeChainDataLength();
-          }
-          auto value = decode(std::move(payload));
-          queue.pop();
-          co_yield folly::coro::co_result(std::move(value));
-        }
-
-        DCHECK_LT(0, outstanding);
-        --outstanding;
-        // Assuming all outstanding payloads come back with largest seen size.
-        size_t outstandingSize = bufferMemSize + outstanding * maxPayloadSize;
-        size_t spaceAvailable =
-            std::max<ssize_t>(memBufferTarget - outstandingSize, 0);
-        // Issue more credits when available space is at least half of the
-        // buffer, or 16kB if that's less than the payload size.
-        size_t threshold = (maxPayloadSize <= kRequestCreditPayloadSize)
-            ? std::min(memBufferTarget / 2, kRequestCreditPayloadSize)
-            : memBufferTarget / 2;
-
-        if (spaceAvailable >= threshold) {
-          // Convert to credits, again assuming largest size for new payloads,
-          // but cap credits and outstanding requests to maxChunkBufferSize if
-          // this was requested in BufferOptions
-          DCHECK_LE(outstanding, maxChunkBufferSize);
-          const int32_t remainingCredits = maxChunkBufferSize - outstanding;
-          int32_t request =
-              (spaceAvailable + maxPayloadSize - 1) / maxPayloadSize;
-          request = std::min(remainingCredits, request);
-          streamBridge->requestN(request);
-          outstanding += request;
+          windowSum -= std::exchange(
+              payloadSizesWindow[numReceivedPayloads % kEstimationWindowSize],
+              payloadSize);
+          windowSum += payloadSize;
+          numReceivedPayloads += 1;
+          maxPayloadSize = std::max(maxPayloadSize, payloadSize);
+          bufferMemSize -= payloadSize;
         }
+        auto value = decode(std::move(payload));
+        queue.pop();
+        co_yield folly::coro::co_result(std::move(value));
+        updateCredits();
       }
     }
   }
