@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -25,17 +25,11 @@
 #include <thrift/lib/cpp2/async/HeaderChannelTrait.h>
 #include <thrift/lib/cpp2/protocol/Serializer.h>
 
-using folly::IOBuf;
-using folly::IOBufQueue;
 using std::make_unique;
-using std::pair;
 using std::unique_ptr;
 using namespace apache::thrift::transport;
-using namespace apache::thrift;
-using apache::thrift::TApplicationException;
 using apache::thrift::protocol::PROTOCOL_TYPES;
 using apache::thrift::server::TServerObserver;
-using folly::EventBase;
 
 namespace apache {
 namespace thrift {
