@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -223,10 +223,12 @@ class RequestChannel : virtual public folly::DelayedDestruction {
   uint64_t checksumSamplingRate_{0};
 };
 
-template <bool oneWay>
-class ClientSyncCallback : public RequestClientCallback {
+template <bool oneWay, bool sync>
+class ClientBatonCallback : public RequestClientCallback {
  public:
-  explicit ClientSyncCallback(ClientReceiveState* rs) : rs_(rs) {}
+  explicit ClientBatonCallback(
+      ClientReceiveState* rs, folly::Executor* executor = {})
+      : rs_(rs), executor_(executor) {}
 
   template <typename F>
   void waitUntilDone(folly::EventBase* evb, F&& sendF) {
@@ -268,13 +270,24 @@ class ClientSyncCallback : public RequestClientCallback {
 
   bool isInlineSafe() const override { return true; }
 
-  bool isSync() const override { return true; }
+  bool isSync() const override { return sync; }
+
+  folly::Executor::KeepAlive<> getExecutor() const override {
+    return executor_;
+  }
 
  private:
   ClientReceiveState* rs_;
   folly::fibers::Baton doneBaton_;
+  folly::Executor* executor_{};
 };
 
+template <bool oneWay>
+using ClientSyncCallback = ClientBatonCallback<oneWay, true>;
+
+template <bool oneWay>
+using ClientCoroCallback = ClientBatonCallback<oneWay, false>;
+
 StreamClientCallback* createStreamClientCallback(
     RequestClientCallback::Ptr requestCallback,
     const BufferOptions& bufferOptions);
