@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -22,7 +22,6 @@
 #include <fmt/core.h>
 #include <folly/ExceptionString.h>
 #include <folly/GLog.h>
-#include <folly/Likely.h>
 #include <folly/Memory.h>
 #include <folly/Range.h>
 #include <folly/Try.h>
@@ -36,6 +35,7 @@
 
 #include <thrift/lib/cpp/TApplicationException.h>
 #include <thrift/lib/cpp/protocol/TBase64Utils.h>
+#include <thrift/lib/cpp/protocol/TProtocolTypes.h>
 #include <thrift/lib/cpp/transport/THeader.h>
 #include <thrift/lib/cpp2/Flags.h>
 #include <thrift/lib/cpp2/async/HeaderChannel.h>
@@ -54,14 +54,16 @@
 #include <thrift/lib/cpp2/transport/rocket/client/RocketClient.h>
 #include <thrift/lib/thrift/gen-cpp2/RpcMetadata_constants.h>
 #include <thrift/lib/thrift/gen-cpp2/RpcMetadata_types.h>
+#include <thrift/lib/thrift/gen-cpp2/any_rep_types.h>
 
 namespace {
-const int64_t kRocketClientMaxVersion = 8;
-const int64_t kRocketClientMinVersion = 6;
+const int64_t kRocketClientMaxVersion = 10;
+const int64_t kRocketClientMinVersion = 8;
 } // namespace
 
 THRIFT_FLAG_DEFINE_bool(rocket_client_new_protocol_key, true);
 THRIFT_FLAG_DEFINE_int64(rocket_client_max_version, kRocketClientMaxVersion);
+THRIFT_FLAG_DEFINE_bool(rocket_client_rocket_skip_protocol_key, false);
 
 THRIFT_FLAG_DEFINE_bool(rocket_client_upgrade_zlib_to_zstd_v2, true);
 
@@ -194,6 +196,13 @@ folly::Try<FirstResponsePayload> decodeResponseError(
     case ResponseRpcErrorCode::INTERACTION_CONSTRUCTOR_ERROR:
       exCode = kInteractionConstructorErrorErrorCode;
       break;
+    case ResponseRpcErrorCode::UNIMPLEMENTED_METHOD:
+      exCode = kUnimplementedMethodErrorCode;
+      exType = TApplicationException::UNKNOWN_METHOD;
+      break;
+    case ResponseRpcErrorCode::TENANT_QUOTA_EXCEEDED:
+      exCode = kTenantQuotaExceededErrorCode;
+      exType = TApplicationException::LOADSHEDDING;
     default:
       exCode = kUnknownErrorCode;
       break;
@@ -215,6 +224,7 @@ folly::Try<FirstResponsePayload> decodeResponseError(
 
 template <class Handler>
 FOLLY_NODISCARD folly::exception_wrapper processFirstResponse(
+    uint16_t protocolId,
     ResponseRpcMetadata& metadata,
     std::unique_ptr<folly::IOBuf>& payload,
     Handler& handler) {
@@ -257,7 +267,7 @@ FOLLY_NODISCARD folly::exception_wrapper processFirstResponse(
               }
               payload = handler.handleReply(std::move(payload));
               break;
-            case PayloadExceptionMetadata::proxyException:
+            case PayloadExceptionMetadata::DEPRECATED_proxyException:
               (*otherMetadataRef)
                   [isProxiedResponse ? "servicerouter:sr_error"
                                      : "servicerouter:sr_internal_error"] =
@@ -265,12 +275,44 @@ FOLLY_NODISCARD folly::exception_wrapper processFirstResponse(
               payload = handler.handleException(
                   TApplicationException(exceptionWhatRef.value_or("")));
               break;
+            case PayloadExceptionMetadata::anyException: {
+              type::SemiAnyStruct anyException;
+              try {
+                if (protocolId == protocol::T_COMPACT_PROTOCOL) {
+                  CompactSerializer::deserialize(payload.get(), anyException);
+                } else {
+                  BinarySerializer::deserialize(payload.get(), anyException);
+                }
+              } catch (...) {
+                return TApplicationException(
+                    "anyException deserialization failure: " +
+                    folly::exceptionStr(std::current_exception())
+                        .toStdString());
+              }
+              if (*anyException.protocol_ref() == type::kNoProtocol) {
+                anyException.protocol_ref() =
+                    protocolId == protocol::T_COMPACT_PROTOCOL
+                    ? type::Protocol::get<type::StandardProtocol::Compact>()
+                    : type::Protocol::get<type::StandardProtocol::Binary>();
+              }
+              if (anyException.type_ref() ==
+                      type::Type(
+                          type::exception_c{},
+                          "facebook.com/servicerouter/ServiceRouterError") &&
+                  anyException.protocol_ref() ==
+                      type::Protocol::get<type::StandardProtocol::Compact>()) {
+                (*otherMetadataRef)
+                    [isProxiedResponse ? "servicerouter:sr_error"
+                                       : "servicerouter:sr_internal_error"] =
+                        protocol::base64Encode(
+                            anyException.data_ref()->coalesce());
+              }
+              payload = handler.handleException(
+                  TApplicationException(exceptionWhatRef.value_or("")));
+              break;
+            }
             default:
               switch (metaType) {
-                case PayloadExceptionMetadata::DEPRECATED_appClientException:
-                  (*otherMetadataRef)[isProxiedResponse ? "pex" : "ex"] =
-                      kAppClientErrorCode;
-                  break;
                 case PayloadExceptionMetadata::appUnknownException:
                   if (auto ec = exceptionMetadataRef->appUnknownException_ref()
                                     ->errorClassification_ref()) {
@@ -328,7 +370,10 @@ class FirstRequestProcessorStream : public StreamClientCallback,
     DCHECK_EQ(evb, evb_);
     LegacyResponseSerializationHandler handler(protocolId_, methodName_.view());
     if (auto error = processFirstResponse(
-            firstResponse.metadata, firstResponse.payload, handler)) {
+            protocolId_,
+            firstResponse.metadata,
+            firstResponse.payload,
+            handler)) {
       serverCallback->onStreamCancel();
       clientCallback_->onFirstResponseError(std::move(error));
       return false;
@@ -397,7 +442,10 @@ class FirstRequestProcessorSink : public SinkClientCallback,
     SCOPE_EXIT { delete this; };
     LegacyResponseSerializationHandler handler(protocolId_, methodName_.view());
     if (auto error = processFirstResponse(
-            firstResponse.metadata, firstResponse.payload, handler)) {
+            protocolId_,
+            firstResponse.metadata,
+            firstResponse.payload,
+            handler)) {
       serverCallback->onSinkError(
           folly::make_exception_wrapper<TApplicationException>(
               TApplicationException::INTERRUPTION,
@@ -491,6 +539,8 @@ void RocketClientChannel::setCompression(
 
 class RocketClientChannel::SingleRequestSingleResponseCallback final
     : public rocket::RocketClient::RequestResponseCallback {
+  using clock = std::chrono::steady_clock;
+
  public:
   SingleRequestSingleResponseCallback(
       RequestClientCallback::Ptr cb,
@@ -502,17 +552,20 @@ class RocketClientChannel::SingleRequestSingleResponseCallback final
         protocolId_(protocolId),
         methodName_(std::move(methodName)),
         requestSerializedSize_(requestSerializedSize),
-        requestWireSize_(requestWireSize) {}
+        requestWireSize_(requestWireSize),
+        timeBeginSend_(clock::now()) {}
 
-  void onWriteSuccess() noexcept override {}
+  void onWriteSuccess() noexcept override { timeEndSend_ = clock::now(); }
 
   void onResponsePayload(
       folly::Try<rocket::Payload>&& payload) noexcept override {
     folly::Try<FirstResponsePayload> response;
-    ResponseSerializationHandler handler(protocolId_);
     RpcSizeStats stats;
     stats.requestSerializedSizeBytes = requestSerializedSize_;
     stats.requestWireSizeBytes = requestWireSize_;
+    stats.requestLatency = timeEndSend_ - timeBeginSend_;
+    stats.responseLatency = clock::now() - timeEndSend_;
+    ResponseSerializationHandler handler(protocolId_);
     if (payload.hasException()) {
       if (!payload.exception().with_exception<rocket::RocketException>(
               [&](auto& ex) {
@@ -535,7 +588,7 @@ class RocketClientChannel::SingleRequestSingleResponseCallback final
         return;
       }
       if (auto error = processFirstResponse(
-              response->metadata, response->payload, handler)) {
+              protocolId_, response->metadata, response->payload, handler)) {
         cb_.release()->onResponseError(std::move(error));
         return;
       }
@@ -564,6 +617,8 @@ class RocketClientChannel::SingleRequestSingleResponseCallback final
   ManagedStringView methodName_;
   const size_t requestSerializedSize_;
   const size_t requestWireSize_;
+  const std::chrono::time_point<clock> timeBeginSend_;
+  std::chrono::time_point<clock> timeEndSend_;
 };
 
 class RocketClientChannel::SingleRequestNoResponseCallback final
@@ -611,15 +666,19 @@ rocket::SetupFrame RocketClientChannel::makeSetupFrame(
   folly::IOBufQueue queue;
   queue.append(std::move(buf));
   folly::io::QueueAppender appender(&queue, /* do not grow */ 0);
-  const uint32_t protocolKey = THRIFT_FLAG(rocket_client_new_protocol_key)
-      ? RpcMetadata_constants::kRocketProtocolKey()
-      : 1;
-  appender.writeBE<uint32_t>(protocolKey); // Rocket protocol key
+  if (!THRIFT_FLAG(rocket_client_rocket_skip_protocol_key)) {
+    const uint32_t protocolKey = THRIFT_FLAG(rocket_client_new_protocol_key)
+        ? RpcMetadata_constants::kRocketProtocolKey()
+        : 1;
+
+    appender.writeBE<uint32_t>(protocolKey); // Rocket protocol key
+  }
   // Append serialized setup parameters to setup frame metadata
   appender.insert(paramQueue.move());
 
   return rocket::SetupFrame(
-      rocket::Payload::makeFromMetadataAndData(queue.move(), {}));
+      rocket::Payload::makeFromMetadataAndData(queue.move(), {}),
+      /* rocketMimeTypes = */ true);
 }
 
 RocketClientChannel::RocketClientChannel(
