@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -251,5 +251,28 @@ const std::shared_ptr<void>& RpcOptions::getCallerContext() const {
   return callerContext_;
 }
 
+void RpcOptions::setFaultToInject(
+    const std::string& key, const std::string& value) {
+  if (!faultsToInject_) {
+    faultsToInject_.emplace();
+  }
+  (*faultsToInject_)[key] = value;
+}
+
+const std::optional<std::unordered_map<std::string, std::string>>&
+RpcOptions::getFaultsToInject() const {
+  return faultsToInject_;
+}
+
+RpcOptions& RpcOptions::setDefconPriority(DefconPriority defconPriority) {
+  defconPriority_ = defconPriority;
+  return *this;
+}
+
+const std::optional<RpcOptions::DefconPriority>& RpcOptions::getDefconPriority()
+    const {
+  return defconPriority_;
+}
+
 } // namespace thrift
 } // namespace apache
