@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -20,10 +20,8 @@
 #include <thrift/lib/cpp2/frozen/FrozenUtil.h>
 #include <thrift/lib/cpp2/frozen/test/gen-cpp2/Example_layouts.h>
 
-using namespace apache::thrift;
 using namespace apache::thrift::frozen;
 using namespace apache::thrift::test;
-using namespace apache::thrift::util;
 using namespace testing;
 
 TEST(FrozenUnion, union_contains_int) {
@@ -79,10 +77,10 @@ TEST(FrozenUnion, union_contains_set) {
 TEST(FrozenUnion, union_contains_sturct) {
   TestUnion u;
   Member thatStruct;
-  *thatStruct.adId_ref() = 2010;
-  *thatStruct.name_ref() = "Tiger";
+  *thatStruct.adId() = 2010;
+  *thatStruct.name() = "Tiger";
   std::vector<int64_t> thatCreativeIds{9, 5, 376, 28};
-  thatStruct.creativeIds_ref() = thatCreativeIds;
+  thatStruct.creativeIds() = thatCreativeIds;
 
   u.set_aStruct(thatStruct);
   auto f = freeze(u);
@@ -97,9 +95,9 @@ TEST(FrozenUnion, union_contains_sturct) {
 TEST(FrozenUnion, union_contains_cpp_shared_ref) {
   TestUnion u;
   auto max = std::make_shared<Pet1>();
-  *max->name_ref() = "max";
-  max->age_ref() = 7;
-  max->vegan_ref() = true;
+  *max->name() = "max";
+  max->age() = 7;
+  max->vegan() = true;
 
   u.set_aPet1(*max);
   auto f = freeze(u);
@@ -113,10 +111,10 @@ TEST(FrozenUnion, union_contains_cpp_shared_ref) {
 TEST(FrozenUnion, union_contains_cpp_unique_ref) {
   TestUnion u;
   Tiny tiny;
-  *tiny.a_ref() = "aaa";
-  *tiny.b_ref() = "bbb";
-  *tiny.c_ref() = "ccc";
-  *tiny.d_ref() = "ddd";
+  *tiny.a() = "aaa";
+  *tiny.b() = "bbb";
+  *tiny.c() = "ccc";
+  *tiny.d() = "ddd";
 
   u.set_aTiny(tiny);
   auto f = freeze(u);
@@ -131,8 +129,8 @@ TEST(FrozenUnion, union_contains_cpp_unique_ref) {
 TEST(FrozenUnion, union_contains_cpp_ref_true) {
   TestUnion u;
   Place place;
-  *place.name_ref() = "somewhere";
-  place.popularityByHour_ref()[37] = 21;
+  *place.name() = "somewhere";
+  place.popularityByHour()[37] = 21;
 
   u.set_aPlace(std::move(place));
   auto f = freeze(u);
@@ -145,19 +143,19 @@ TEST(FrozenUnion, union_contains_cpp_ref_true) {
 TEST(FrozenUnion, union_as_member) {
   TestUnion u;
   Member thatStruct;
-  *thatStruct.adId_ref() = 2015;
-  *thatStruct.name_ref() = "Jayden";
+  *thatStruct.adId() = 2015;
+  *thatStruct.name() = "Jayden";
   std::vector<int64_t> thatCreativeIds{9, 5, 376, 28};
-  thatStruct.creativeIds_ref() = thatCreativeIds;
+  thatStruct.creativeIds() = thatCreativeIds;
   u.set_aStruct(thatStruct);
 
   Big big;
-  big.anOptionalString_ref() = "so good!!";
-  *big.anId_ref() = 9527;
+  big.anOptionalString() = "so good!!";
+  *big.anId() = 9527;
   std::vector<int64_t> thatList{19, 15, 1376, 7, 128};
-  big.anOptionalList_ref() = thatList;
-  *big.aTestUnion_ref() = u;
-  *big.aString_ref() =
+  big.anOptionalList() = thatList;
+  *big.aTestUnion() = u;
+  *big.aString() =
       "Run fast then you can catch the snowflake and keep it forever.";
 
   auto f = freeze(big);
