@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -33,6 +33,7 @@
 #include <thrift/lib/cpp/TApplicationException.h>
 #include <thrift/lib/cpp/transport/THeader.h>
 #include <thrift/lib/cpp2/GeneratedCodeHelper.h>
+#include <thrift/lib/cpp2/TrustedServerException.h>
 #include <thrift/lib/cpp2/async/ResponseChannel.h>
 #include <thrift/lib/cpp2/protocol/Protocol.h>
 #include <thrift/lib/cpp2/protocol/detail/protocol_methods.h>
@@ -98,15 +99,15 @@ void process_handle_exn_deserialization(
     Cpp2RequestContext* const ctx,
     folly::EventBase* const eb,
     char const* const method) {
-  if (auto rpe = dynamic_cast<const RequestParsingError*>(ew.get_exception())) {
-    eb->runInEventBaseThread([request = std::move(req),
-                              msg = std::string(rpe->what())]() {
-      request->sendErrorWrapped(
-          folly::make_exception_wrapper<TApplicationException>(
-              TApplicationException::TApplicationExceptionType::PROTOCOL_ERROR,
-              msg),
-          kRequestParsingErrorCode);
-    });
+  if (auto trustedServerEx =
+          dynamic_cast<const TrustedServerException*>(ew.get_exception())) {
+    eb->runInEventBaseThread(
+        [request = std::move(req), ex = *trustedServerEx]() {
+          request->sendErrorWrapped(
+              folly::make_exception_wrapper<TApplicationException>(
+                  ex.toApplicationException()),
+              std::string(ex.errorCode()));
+        });
     return;
   }
   ::apache::thrift::util::appendExceptionToHeader(ew, *ctx);
@@ -128,7 +129,15 @@ void process_throw_wrapped_handler_error(
     Cpp2RequestContext* const ctx,
     ContextStack* const stack,
     char const* const method) {
-  LOG(ERROR) << ew << " in function " << method;
+  if (auto trustedServerEx =
+          dynamic_cast<const TrustedServerException*>(ew.get_exception())) {
+    req->sendErrorWrapped(
+        folly::make_exception_wrapper<TApplicationException>(
+            trustedServerEx->toApplicationException()),
+        std::string(trustedServerEx->errorCode()));
+    return;
+  }
+  LOG(ERROR) << "uncaught exception in function " << method << ": " << ew;
   if (stack) {
     stack->userExceptionWrapped(false, ew);
     stack->handlerErrorWrapped(ew);
