@@ -21,7 +21,6 @@
 #include <thrift/lib/cpp2/op/Testing.h>
 #include <thrift/lib/cpp2/type/Name.h>
 #include <thrift/lib/cpp2/type/Traits.h>
-#include <thrift/lib/thrift/gen-cpp2/type_fatal_all.h>
 #include <thrift/lib/thrift/gen-cpp2/type_types.h>
 #include <thrift/test/testset/gen-cpp2/testset_fatal_all.h>
 #include <thrift/test/testset/gen-cpp2/testset_types.h>
@@ -30,15 +29,20 @@ namespace apache::thrift::op {
 namespace {
 using namespace apache::thrift::test;
 
+// TODO(dokwon): Remove get_underlying_tag after supporting field_t for all
+// type/op library.
 template <typename Tag>
-void testClear(
+void testClearImpl(
     const type::native_type<Tag>& expected,
     type::native_type<Tag> unexpected,
-    bool emptiable = true) {
-  SCOPED_TRACE(type::getName<Tag>());
+    bool emptiable) {
+  // TODO(afuller): Remove dep on fatal.
+  // SCOPED_TRACE(type::getName<Tag>());
 
   EXPECT_EQ(isEmpty<Tag>(expected), emptiable);
-  EXPECT_THAT(getIntrinsicDefault<Tag>(), IsIdenticalTo<Tag>(expected));
+  EXPECT_THAT(
+      getIntrinsicDefault<get_underlying_tag_t<Tag>>(),
+      IsIdenticalTo<Tag>(expected));
 
   EXPECT_FALSE(isEmpty<Tag>(unexpected));
   EXPECT_THAT(unexpected, ::testing::Not(IsIdenticalTo<Tag>(expected)));
@@ -48,14 +52,24 @@ void testClear(
   EXPECT_THAT(unexpected, IsIdenticalTo<Tag>(expected));
 }
 
+template <typename Tag>
+void testClear(
+    const type::native_type<Tag>& expected,
+    type::native_type<Tag> unexpected,
+    bool emptiable = true) {
+  testClearImpl<Tag>(expected, unexpected, emptiable);
+  testClearImpl<type::field_t<FieldId{1}, Tag>>(
+      expected, unexpected, emptiable);
+}
+
 TEST(ClearTest, Integral) {
   testClear<type::bool_t>(false, true);
   testClear<type::byte_t>(0, 1);
   testClear<type::i16_t>(0, 1);
   testClear<type::i32_t>(0, 1);
   testClear<type::i64_t>(0, 1);
-  testClear<type::enum_t<type::ThriftBaseType>>(
-      type::ThriftBaseType::Void, type::ThriftBaseType::Bool);
+  testClear<type::enum_t<type::BaseTypeEnum>>(
+      type::BaseTypeEnum::Void, type::BaseTypeEnum::Bool);
 }
 
 TEST(ClearTest, FloatingPoint) {
@@ -68,6 +82,12 @@ TEST(ClearTest, String) {
   testClear<type::binary_t>("", "bye");
 }
 
+TEST(ClearTest, Container) {
+  testClear<type::list<type::string_t>>({}, {"hi"});
+  testClear<type::set<type::string_t>>({}, {"hi"});
+  testClear<type::map<type::string_t, type::string_t>>({}, {{"hi", "bye"}});
+}
+
 TEST(ClearTest, Structured) {
   testClear<type::struct_t<testset::struct_i64>>(
       {}, {apache::thrift::FragileConstructor(), 1}, false);
