@@ -18,7 +18,9 @@
 
 #include <thrift/lib/cpp2/op/detail/Hash.h>
 
-namespace apache::thrift::op {
+namespace apache {
+namespace thrift {
+namespace op {
 
 template <typename Tag>
 using Hash = detail::Hash<Tag>;
@@ -31,6 +33,8 @@ using Hash = detail::Hash<Tag>;
 //   hash<i32_t>(myInt) // returns hash of myInt.
 //   hash<set<i32_t>>(myIntSet) // returns hash of myIntSet
 template <typename Tag>
-constexpr Hash<Tag> hash;
+constexpr Hash<Tag> hash{};
 
-} // namespace apache::thrift::op
+} // namespace op
+} // namespace thrift
+} // namespace apache
