@@ -18,13 +18,16 @@
 
 #include <cmath>
 
+#include <folly/Overload.h>
 #include <thrift/lib/cpp2/Thrift.h>
 #include <thrift/lib/cpp2/op/Compare.h>
 #include <thrift/lib/cpp2/type/Tag.h>
 #include <thrift/lib/cpp2/type/ThriftType.h>
-#include <thrift/lib/cpp2/type/Traits.h>
 
-namespace apache::thrift::op::detail {
+namespace apache {
+namespace thrift {
+namespace op {
+namespace detail {
 
 // C++'s intrinsic default for the underlying native type, is the intrisitic
 // default for for all unstructured types.
@@ -53,18 +56,17 @@ FOLLY_EXPORT const T& getIntrinsicDefault(type::structured_c) noexcept {
 
 template <typename Tag>
 struct Clear {
-  static_assert(type::is_concrete_v<Tag>);
+  static_assert(type::is_concrete_v<Tag>, "");
   template <typename T>
   void operator()(T& value) const {
-    if constexpr (type::is_a_v<Tag, type::structured_c>) {
-      apache::thrift::clear(value);
-    } else if constexpr (type::is_a_v<Tag, type::container_c>) {
-      value.clear();
-    } else {
-      // All unstructured types can be cleared by assigning to the intrinsic
-      // default.
-      value = getIntrinsicDefault<T>(Tag{});
-    }
+    folly::overload(
+        [](auto& v, type::structured_c) { apache::thrift::clear(v); },
+        [](auto& v, type::container_c) { v.clear(); },
+        [](auto& v, type::all_c) {
+          // All unstructured types can be cleared by assigning to the intrinsic
+          // default.
+          v = getIntrinsicDefault<T>(Tag{});
+        })(value, Tag{});
   }
 };
 
@@ -73,24 +75,27 @@ struct Clear<type::adapted<Adapter, Tag>> {
   // TODO(afuller): implement.
 };
 
+template <typename Tag, FieldId Id>
+struct Clear<type::field_t<Id, Tag>> : Clear<Tag> {};
+
 template <typename Tag>
 struct Empty {
-  static_assert(type::is_concrete_v<Tag>);
+  static_assert(type::is_concrete_v<Tag>, "");
   template <typename T = type::native_type<Tag>>
   constexpr bool operator()(const T& value) const {
-    if constexpr (type::is_a_v<Tag, type::string_c>) {
-      return StringTraits<T>::isEmpty(value);
-    } else if constexpr (type::is_a_v<Tag, type::container_c>) {
-      return value.empty();
-    } else if constexpr (type::is_a_v<Tag, type::union_c>) {
-      return value.getType() == T::__EMPTY__;
-    } else if constexpr (type::is_a_v<Tag, type::struct_except_c>) {
-      return apache::thrift::detail::st::struct_private_access::is_empty(value);
-    }
-
-    // All unstructured values are 'empty' if they are identical to their
-    // intrinsic default.
-    return op::identical<Tag>(value, getIntrinsicDefault<T>(Tag{}));
+    return folly::overload(
+        [](const auto& v, type::string_c) {
+          return StringTraits<T>::isEmpty(v);
+        },
+        [](const auto& v, type::container_c) { return v.empty(); },
+        [](const auto& v, type::structured_c) {
+          return apache::thrift::empty(v);
+        },
+        [](const auto& v, type::all_c) {
+          // All unstructured values are 'empty' if they are identical to their
+          // intrinsic default.
+          return op::identical<Tag>(v, getIntrinsicDefault<T>(Tag{}));
+        })(value, Tag{});
   }
 };
 
@@ -99,4 +104,10 @@ struct Empty<type::adapted<Adapter, Tag>> {
   // TODO(afuller): implement.
 };
 
-} // namespace apache::thrift::op::detail
+template <typename Tag, FieldId Id>
+struct Empty<type::field_t<Id, Tag>> : Empty<Tag> {};
+
+} // namespace detail
+} // namespace op
+} // namespace thrift
+} // namespace apache
