@@ -20,24 +20,38 @@
 #include <unordered_map>
 
 #include <folly/CPortability.h>
+#include <folly/Overload.h>
 #include <thrift/lib/cpp2/op/Hash.h>
 #include <thrift/lib/cpp2/protocol/Protocol.h>
 #include <thrift/lib/cpp2/type/ThriftType.h>
-#include <thrift/lib/cpp2/type/Traits.h>
 
-namespace apache::thrift::op::detail {
+namespace apache {
+namespace thrift {
+namespace op {
+namespace detail {
 
 template <typename Tag>
 struct EqualTo {
-  static_assert(type::is_concrete_v<Tag>);
+  static_assert(type::is_concrete_v<Tag>, "");
   template <typename T1 = type::native_type<Tag>, typename T2 = T1>
   constexpr bool operator()(const T1& lhs, const T2& rhs) const {
-    if constexpr (type::is_a_v<Tag, type::string_c>) {
-      return StringTraits<T1>::isEqual(lhs, rhs);
-    } else {
-      // Use the native c++ operator by default.
-      return lhs == rhs;
-    }
+    return folly::overload(
+        [](const auto& v1, const auto& v2, type::string_c) {
+          return StringTraits<T1>::isEqual(v1, v2);
+        },
+        [](const auto& v1, const auto& v2, type::all_c) { return v1 == v2; })(
+        lhs, rhs, Tag{});
+  }
+
+ private:
+  template <typename T1 = type::native_type<Tag>, typename T2 = T1>
+  constexpr bool equalTo(const T1& lhs, const T2& rhs, type::string_c) const {
+    return StringTraits<T1>::isEqual(lhs, rhs);
+  }
+
+  template <typename T1 = type::native_type<Tag>, typename T2 = T1>
+  constexpr bool equalTo(const T1& lhs, const T2& rhs, type::all_c) const {
+    return lhs == rhs;
   }
 };
 
@@ -49,12 +63,13 @@ struct EqualTo<type::adapted<Adapter, Tag>> {
 template <typename Tag>
 struct IdenticalTo : EqualTo<Tag> {
   // Identical is the same as equal for integral, enum and string types.
+  // TODO(afuller): Implement proper specializations for all structured
+  // types.
   static_assert(
       type::is_a_v<Tag, type::integral_c> || type::is_a_v<Tag, type::enum_c> ||
-      type::is_a_v<Tag, type::string_c> ||
-      // TODO(afuller): Implement proper specializations for all structured
-      // types.
-      type::structured_types::contains<Tag>());
+          type::is_a_v<Tag, type::string_c> ||
+          type::is_a_v<Tag, type::structured_c>,
+      "");
 };
 
 template <typename Adapter, typename Tag>
@@ -62,6 +77,9 @@ struct IdenticalTo<type::adapted<Adapter, Tag>> {
   // TODO(afuller): Implement.
 };
 
+template <typename Tag, FieldId Id>
+struct IdenticalTo<type::field_t<Id, Tag>> : IdenticalTo<Tag> {};
+
 template <typename F, typename I>
 struct FloatIdenticalTo {
   bool operator()(F lhs, F rhs) const {
@@ -76,9 +94,9 @@ struct IdenticalTo<type::float_t> : FloatIdenticalTo<float, int32_t> {};
 template <>
 struct IdenticalTo<type::double_t> : FloatIdenticalTo<double, int64_t> {};
 
-template <typename ValTag, template <typename...> typename ListT>
-struct IdenticalTo<type::list<ValTag, ListT>> {
-  template <typename T = type::native_type<type::list<ValTag, ListT>>>
+template <typename ValTag>
+struct IdenticalTo<type::list<ValTag>> {
+  template <typename T = type::native_type<type::list<ValTag>>>
   bool operator()(const T& lhs, const T& rhs) const {
     if (lhs.size() != rhs.size()) {
       return false;
@@ -88,8 +106,8 @@ struct IdenticalTo<type::list<ValTag, ListT>> {
   }
 };
 
-template <typename KeyTag, template <typename...> typename SetT>
-struct IdenticalTo<type::set<KeyTag, SetT>> {
+template <typename KeyTag>
+struct IdenticalTo<type::set<KeyTag>> {
   // Create a multimap from hash(key)->&key
   template <typename C>
   static auto createHashMap(const C& set) {
@@ -110,7 +128,7 @@ struct IdenticalTo<type::set<KeyTag, SetT>> {
     return false;
   }
 
-  template <typename T = type::native_type<type::set<KeyTag, SetT>>>
+  template <typename T = type::native_type<type::set<KeyTag>>>
   bool operator()(const T& lhs, const T& rhs) const {
     if (lhs.size() != rhs.size()) {
       return false;
@@ -127,12 +145,8 @@ struct IdenticalTo<type::set<KeyTag, SetT>> {
   }
 };
 
-template <
-    typename KeyTag,
-    typename ValTag,
-    template <typename...>
-    typename MapT>
-struct IdenticalTo<type::map<KeyTag, ValTag, MapT>> {
+template <typename KeyTag, typename ValTag>
+struct IdenticalTo<type::map<KeyTag, ValTag>> {
   // Create a multimap from hash(key)->&pair(key, value)
   template <typename C>
   static auto createHashMap(const C& map) {
@@ -154,7 +168,7 @@ struct IdenticalTo<type::map<KeyTag, ValTag, MapT>> {
     return false;
   }
 
-  template <typename T = type::native_type<type::map<KeyTag, ValTag, MapT>>>
+  template <typename T = type::native_type<type::map<KeyTag, ValTag>>>
   bool operator()(const T& lhs, const T& rhs) const {
     if (lhs.size() != rhs.size()) {
       return false;
@@ -171,4 +185,7 @@ struct IdenticalTo<type::map<KeyTag, ValTag, MapT>> {
   }
 };
 
-} // namespace apache::thrift::op::detail
+} // namespace detail
+} // namespace op
+} // namespace thrift
+} // namespace apache
