@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -52,12 +52,11 @@ FOLLY_ERASE constexpr T identity(T t) {
 }
 
 template <typename T, typename U = void>
-using enable_if_smart_ptr_t =
-    std::enable_if_t<is_shared_or_unique_ptr<T>::value, U>;
+using enable_if_smart_ptr_t = std::enable_if_t<is_shared_or_unique_ptr_v<T>, U>;
 
 template <typename T, typename U = void>
 using enable_if_not_smart_ptr_t =
-    std::enable_if_t<!is_shared_or_unique_ptr<T>::value, U>;
+    std::enable_if_t<!is_shared_or_unique_ptr_v<T>, U>;
 
 template <typename T, typename Enable = void>
 struct maybe_get_element_type {
@@ -80,13 +79,22 @@ enum class StringFieldType {
   IOBufObj,
 };
 
+// A required field is handled as an unqualified field, because the required
+// qualifier is deprecated and has same serialization semantic with the
+// unqualified qualifier.
+enum class FieldQualifier {
+  Unqualified,
+  Optional,
+  Terse,
+};
+
 struct FieldInfo {
   // Field id in thrift definition.
   FieldID id;
 
   // Unqualified fields need to be differentiated from optional fields to always
   // write unqualified fields despite the value of __isset.
-  bool isUnqualified;
+  FieldQualifier qualifier;
 
   const char* name;
 
@@ -283,7 +291,7 @@ enable_if_not_smart_ptr_t<ObjectType, void*> set(void* object) {
 }
 
 template <typename PtrType>
-std::enable_if_t<is_shared_ptr<PtrType>::value, void*> set(void* object) {
+std::enable_if_t<is_shared_ptr_v<PtrType>, void*> set(void* object) {
   using Element = typename PtrType::element_type;
   auto& ptr = *static_cast<PtrType*>(object);
   ptr = std::make_shared<Element>();
@@ -291,7 +299,7 @@ std::enable_if_t<is_shared_ptr<PtrType>::value, void*> set(void* object) {
 }
 
 template <typename PtrType>
-std::enable_if_t<is_unique_ptr<PtrType>::value, void*> set(void* object) {
+std::enable_if_t<is_unique_ptr_v<PtrType>, void*> set(void* object) {
   using Element = typename PtrType::element_type;
   auto& ptr = *static_cast<PtrType*>(object);
   ptr = std::make_unique<Element>();
@@ -304,7 +312,7 @@ enable_if_not_smart_ptr_t<ObjectType> set(void* object, PrimitiveType val) {
 }
 
 template <typename PtrType, typename PrimitiveType>
-std::enable_if_t<is_unique_ptr<PtrType>::value> set(
+std::enable_if_t<is_unique_ptr_v<PtrType>> set(
     void* object, PrimitiveType val) {
   using Element = typename PtrType::element_type;
   *static_cast<PtrType*>(object) =
@@ -312,7 +320,7 @@ std::enable_if_t<is_unique_ptr<PtrType>::value> set(
 }
 
 template <typename PtrType, typename PrimitiveType>
-std::enable_if_t<is_shared_ptr<PtrType>::value> set(
+std::enable_if_t<is_shared_ptr_v<PtrType>> set(
     void* object, PrimitiveType val) {
   using Element = typename PtrType::element_type;
   *static_cast<PtrType*>(object) =
