@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -26,17 +26,20 @@
 #include <folly/Conv.h>
 #include <folly/GLog.h>
 #include <folly/Memory.h>
+#include <folly/Portability.h>
 #include <folly/Random.h>
 #include <folly/ScopeGuard.h>
 #include <folly/portability/Sockets.h>
 
-THRIFT_FLAG_DEFINE_int64(server_default_socket_queue_timeout_ms, 0);
-THRIFT_FLAG_DEFINE_int64(server_default_queue_timeout_ms, 0);
+THRIFT_FLAG_DEFINE_int64(server_default_socket_queue_timeout_ms, 100);
+THRIFT_FLAG_DEFINE_int64(server_default_queue_timeout_ms, 100);
 
 THRIFT_FLAG_DEFINE_int64(server_polled_service_health_liveness_ms, 100);
 THRIFT_FLAG_DEFINE_int64(
     server_ingress_memory_limit_enforcement_payload_size_min_bytes, 1024);
 
+THRIFT_FLAG_DEFINE_bool(server_reject_header_connections, false);
+
 namespace apache {
 namespace thrift {
 
@@ -51,11 +54,7 @@ THRIFT_PLUGGABLE_FUNC_REGISTER(
 
 } // namespace detail
 
-using namespace apache::thrift::protocol;
-using namespace apache::thrift::server;
-using namespace apache::thrift::transport;
 using namespace std;
-using std::shared_ptr;
 
 const size_t BaseThriftServer::T_ASYNC_DEFAULT_WORKER_THREADS =
     std::thread::hardware_concurrency();
@@ -163,5 +162,19 @@ std::string BaseThriftServer::getLoadInfo(int64_t load) const {
   return stream.str();
 }
 
+std::string BaseThriftServer::RuntimeServerActions::explain() const {
+  std::string result;
+  result = std::string(userSuppliedThreadManager ? "setThreadManager, " : "") +
+      (interactionInService ? "interactionInService, " : "") +
+      (wildcardMethods ? "wildcardMethods, " : "") +
+      (noServiceRequestInfo ? "noServiceRequestInfo, " : "") +
+      (activeRequestTrackingDisabled ? "activeRequestTrackingDisabled, " : "") +
+      (setPreprocess ? "setPreprocess, " : "") +
+      (setIsOverloaded ? "setIsOverloaded, " : "") +
+      (codelEnabled ? "codelEnabled, " : "") +
+      (!resourcePoolFlagSet ? "flagsNotSet, " : "");
+  return result;
+}
+
 } // namespace thrift
 } // namespace apache
