@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -26,8 +26,10 @@
 #include <vector>
 
 #include <folly/Memory.h>
+#include <folly/Portability.h>
 #include <folly/SocketAddress.h>
 #include <folly/Synchronized.h>
+#include <folly/io/SocketOptionMap.h>
 #include <folly/io/async/AsyncTransport.h>
 #include <folly/io/async/EventBase.h>
 
@@ -37,15 +39,17 @@
 #include <thrift/lib/cpp/server/TServerObserver.h>
 #include <thrift/lib/cpp/transport/THeader.h>
 #include <thrift/lib/cpp2/Flags.h>
-#include <thrift/lib/cpp2/PluggableFunction.h>
 #include <thrift/lib/cpp2/Thrift.h>
 #include <thrift/lib/cpp2/async/AsyncProcessor.h>
 #include <thrift/lib/cpp2/server/AdaptiveConcurrency.h>
 #include <thrift/lib/cpp2/server/ControlServerInterface.h>
 #include <thrift/lib/cpp2/server/MonitoringServerInterface.h>
+#include <thrift/lib/cpp2/server/ResourcePool.h>
 #include <thrift/lib/cpp2/server/ServerAttribute.h>
 #include <thrift/lib/cpp2/server/ServerConfigs.h>
+#include <thrift/lib/cpp2/server/ServerFlags.h>
 #include <thrift/lib/cpp2/server/StatusServerInterface.h>
+#include <thrift/lib/cpp2/server/ThreadManagerLoggingWrapper.h>
 
 THRIFT_FLAG_DECLARE_int64(server_default_socket_queue_timeout_ms);
 THRIFT_FLAG_DECLARE_int64(server_default_queue_timeout_ms);
@@ -54,6 +58,8 @@ THRIFT_FLAG_DECLARE_int64(server_polled_service_health_liveness_ms);
 THRIFT_FLAG_DECLARE_int64(
     server_ingress_memory_limit_enforcement_payload_size_min_bytes);
 
+THRIFT_FLAG_DECLARE_bool(server_reject_header_connections);
+
 namespace wangle {
 class ConnectionManager;
 }
@@ -78,14 +84,6 @@ using IsOverloadedFunc = folly::Function<bool(
 using PreprocessFunc =
     folly::Function<PreprocessResult(const server::PreprocessParams&) const>;
 
-namespace detail {
-
-THRIFT_PLUGGABLE_FUNC_DECLARE(
-    folly::observer::Observer<AdaptiveConcurrencyController::Config>,
-    makeAdaptiveConcurrencyConfig);
-
-} // namespace detail
-
 template <typename T>
 class ThriftServerAsyncProcessorFactory : public AsyncProcessorFactory {
  public:
@@ -98,7 +96,7 @@ class ThriftServerAsyncProcessorFactory : public AsyncProcessorFactory {
         new typename T::ProcessorType(svIf_.get()));
   }
 
-  std::vector<ServiceHandler*> getServiceHandlers() override {
+  std::vector<ServiceHandlerBase*> getServiceHandlers() override {
     return {svIf_.get()};
   }
 
@@ -171,6 +169,38 @@ class BaseThriftServer : public apache::thrift::concurrency::Runnable,
    */
   struct DynamicAttributeTag {};
 
+  /**
+   * The type of thread manager to create for the server.
+   */
+  enum class ThreadManagerType : int {
+    PRIORITY = 0, //! Use a PriorityThreadManager
+    SIMPLE = 1, //! Use a SimpleThreadManager
+    PRIORITY_QUEUE = 2, //! Use a PriorityQueueThreadManager
+  };
+
+  struct RuntimeServerActions {
+    bool userSuppliedThreadManager{false};
+    bool interactionInService{false};
+    bool wildcardMethods{false};
+    bool noServiceRequestInfo{false};
+    bool activeRequestTrackingDisabled{false};
+    bool setPreprocess{false};
+    bool setIsOverloaded{false};
+    bool resourcePoolEnabled{false};
+    bool resourcePoolFlagSet{false};
+    bool codelEnabled{false};
+    std::string executorToThreadManagerUnexpectedFunctionName{};
+
+    std::string explain() const;
+  };
+
+  /**
+   * Get the flags used to support migrations and rollouts.
+   */
+  RuntimeServerActions& getRuntimeServerActions() const {
+    return runtimeServerActions_;
+  }
+
  private:
   //! Default number of worker threads (should be # of processor cores).
   static const size_t T_ASYNC_DEFAULT_WORKER_THREADS;
@@ -186,9 +216,6 @@ class BaseThriftServer : public apache::thrift::concurrency::Runnable,
   static constexpr std::chrono::milliseconds DEFAULT_STREAM_EXPIRE_TIME =
       std::chrono::milliseconds(60000);
 
-  static constexpr std::chrono::milliseconds DEFAULT_QUEUE_TIMEOUT =
-      std::chrono::milliseconds(0);
-
   static constexpr std::chrono::milliseconds DEFAULT_SOCKET_WRITE_TIMEOUT =
       std::chrono::milliseconds(60000);
 
@@ -204,6 +231,8 @@ class BaseThriftServer : public apache::thrift::concurrency::Runnable,
   // Cpp2 ProcessorFactory.
   std::shared_ptr<apache::thrift::AsyncProcessorFactory> cpp2Pfac_;
 
+  ServerInterface* applicationServerInterface_{};
+
   // Explicitly set monitoring service interface handler
   std::shared_ptr<MonitoringServerInterface> monitoringServiceHandler_;
 
@@ -248,8 +277,13 @@ class BaseThriftServer : public apache::thrift::concurrency::Runnable,
    */
   ServerAttributeDynamic<std::chrono::milliseconds> queueTimeout_{
       folly::observer::makeValueObserver(
-          [o = THRIFT_FLAG_OBSERVE(server_default_queue_timeout_ms)]() {
-            return std::chrono::milliseconds(**o);
+          [timeoutMs = THRIFT_FLAG_OBSERVE(server_default_queue_timeout_ms)]() {
+            // Timeouts should be disabled for debug builds since this can
+            // generate false negatives in unit tests.
+            if (folly::kIsDebug) {
+              return std::chrono::milliseconds(0);
+            }
+            return std::chrono::milliseconds(**timeoutMs);
           })};
   /**
    * The time we'll allow a new connection socket to wait on the queue before
@@ -260,6 +294,11 @@ class BaseThriftServer : public apache::thrift::concurrency::Runnable,
           [timeoutMs =
                THRIFT_FLAG_OBSERVE(server_default_socket_queue_timeout_ms)]()
               -> std::chrono::nanoseconds {
+            // Disable timeout for debug builds and unit tests by default - this
+            // is a production overload protection feature.
+            if (folly::kIsDebug) {
+              return std::chrono::milliseconds::zero();
+            }
             return std::chrono::milliseconds(**timeoutMs);
           })};
 
@@ -387,6 +426,20 @@ class BaseThriftServer : public apache::thrift::concurrency::Runnable,
             return std::chrono::milliseconds(**livenessMs);
           })};
 
+  /**
+   * Enable to reject all header-backed connections
+   */
+  ServerAttributeDynamic<bool> disableHeaderTransport_{
+      THRIFT_FLAG_OBSERVE(server_reject_header_connections)};
+
+  /**
+   * Socket options that will be applied to every connection to clients.
+   * If the socket does not support the specific option, it is silently ignored.
+   * Refer to setsockopt() for more details.
+   */
+  ServerAttributeDynamic<folly::SocketOptionMap> perConnectionSocketOptions_{
+      folly::emptySocketOptionMap};
+
   std::shared_ptr<server::TServerEventHandler> eventHandler_;
   std::vector<std::shared_ptr<server::TServerEventHandler>> eventHandlers_;
   AdaptiveConcurrencyController adaptiveConcurrencyController_;
@@ -398,11 +451,30 @@ class BaseThriftServer : public apache::thrift::concurrency::Runnable,
   //! The server's listening port
   uint16_t port_ = 0;
 
+  //! The type of thread manager to create.
+  ThreadManagerType threadManagerType_{ThreadManagerType::PRIORITY};
+
+  //! The thread pool sizes for priority thread manager.
+  std::array<size_t, concurrency::N_PRIORITIES> threadManagerPoolSizes_{
+      {0, 0, 0, 0, 0}};
+
+  //! The ResourcePoolsSet used by this ThriftServer (if in ResourcePools
+  //! are enabled).
+  ResourcePoolSet resourcePoolSet_;
+
+  //! Flags used to track certain actions of thrift servers to help support
+  //! migrations and rollouts.
+  mutable RuntimeServerActions runtimeServerActions_;
+
   /**
    * The thread manager used for sync calls.
    */
   mutable std::mutex threadManagerMutex_;
   std::shared_ptr<apache::thrift::concurrency::ThreadManager> threadManager_;
+  // we need to make the wrapper stick to the server because the users calling
+  // getThreadManager are relying on the server to maintatin the tm lifetime
+  std::shared_ptr<apache::thrift::ThreadManagerLoggingWrapper>
+      tmLoggingWrapper_;
 
   // If set, the thread factory that should be used to create worker threads.
   std::shared_ptr<concurrency::ThreadFactory> threadFactory_;
@@ -446,6 +518,21 @@ class BaseThriftServer : public apache::thrift::concurrency::Runnable,
     return failureInjection_.test();
   }
 
+  // This is meant to be used internally
+  // We separate setThreadManager and configureThreadManager
+  // so that we can have proper logging for the former
+  // These APIs will be deprecated eventually when ResourcePool
+  // migration is done.
+  void setThreadManagerInternal(
+      std::shared_ptr<apache::thrift::concurrency::ThreadManager>
+          threadManager) {
+    CHECK(configMutable());
+    std::lock_guard<std::mutex> lock(threadManagerMutex_);
+    threadManager_ = threadManager;
+    tmLoggingWrapper_ =
+        std::make_shared<ThreadManagerLoggingWrapper>(threadManager_, this);
+  }
+
   getHandlerFunc getHandler_;
   GetHeaderHandlerFunc getHeaderHandler_;
 
@@ -475,7 +562,7 @@ class BaseThriftServer : public apache::thrift::concurrency::Runnable,
     allocIOBufFn_ = std::move(fn);
   }
 
-  std::shared_ptr<server::TServerEventHandler> getEventHandler() {
+  std::shared_ptr<server::TServerEventHandler> getEventHandler() const {
     return eventHandler_;
   }
 
@@ -489,7 +576,7 @@ class BaseThriftServer : public apache::thrift::concurrency::Runnable,
    * getEventHandlersUnsafe.
    */
   const std::vector<std::shared_ptr<server::TServerEventHandler>>&
-  getEventHandlersUnsafe() {
+  getEventHandlersUnsafe() const {
     return eventHandlers_;
   }
 
@@ -530,10 +617,32 @@ class BaseThriftServer : public apache::thrift::concurrency::Runnable,
   void setThreadFactory(
       std::shared_ptr<concurrency::ThreadFactory> threadFactory) {
     CHECK(configMutable());
+    std::lock_guard<std::mutex> lock(threadManagerMutex_);
     CHECK(!threadManager_);
     threadFactory_ = std::move(threadFactory);
   }
 
+  /**
+   * Set the type of ThreadManager to use for this server.
+   */
+  void setThreadManagerType(ThreadManagerType threadManagerType) {
+    CHECK(configMutable());
+    std::lock_guard<std::mutex> lock(threadManagerMutex_);
+    CHECK(!threadManager_);
+    threadManagerType_ = threadManagerType;
+  }
+
+  /**
+   * Set the size of thread pools when using ThreadManagerType::PRIORITY
+   */
+  void setThreadManagerPoolSizes(
+      const std::array<size_t, concurrency::N_PRIORITIES>& poolSizes) {
+    CHECK(configMutable());
+    std::lock_guard<std::mutex> lock(threadManagerMutex_);
+    CHECK(!threadManager_);
+    threadManagerPoolSizes_ = poolSizes;
+  }
+
   /**
    * Get the prefix for naming the CPU (pool) threads.
    *
@@ -559,15 +668,16 @@ class BaseThriftServer : public apache::thrift::concurrency::Runnable,
   /**
    * Set Thread Manager (for queuing mode).
    * If not set, defaults to the number of worker threads.
+   * This is meant to be used as an external API
    *
    * @param threadManager a shared pointer to the thread manager
    */
   void setThreadManager(
       std::shared_ptr<apache::thrift::concurrency::ThreadManager>
           threadManager) {
-    CHECK(configMutable());
-    std::lock_guard<std::mutex> lock(threadManagerMutex_);
-    threadManager_ = threadManager;
+    setThreadManagerInternal(threadManager);
+    runtimeDisableResourcePools();
+    runtimeServerActions_.userSuppliedThreadManager = true;
   }
 
   /**
@@ -577,12 +687,12 @@ class BaseThriftServer : public apache::thrift::concurrency::Runnable,
    */
   void setThreadManagerFromExecutor(
       folly::Executor* executor, std::string name = "") {
-    CHECK(configMutable());
-    std::lock_guard<std::mutex> lock(threadManagerMutex_);
     concurrency::ThreadManagerExecutorAdapter::Options opts(std::move(name));
-    threadManager_ =
+    setThreadManagerInternal(
         std::make_shared<concurrency::ThreadManagerExecutorAdapter>(
-            folly::getKeepAliveToken(executor), std::move(opts));
+            folly::getKeepAliveToken(executor), std::move(opts)));
+    runtimeDisableResourcePools();
+    runtimeServerActions_.userSuppliedThreadManager = true;
   }
 
   /**
@@ -590,11 +700,34 @@ class BaseThriftServer : public apache::thrift::concurrency::Runnable,
    *
    * @return a shared pointer to the thread manager
    */
-  std::shared_ptr<concurrency::ThreadManager> getThreadManager() const {
+  std::shared_ptr<concurrency::ThreadManager> getThreadManager_deprecated()
+      const override {
+    std::lock_guard<std::mutex> lock(threadManagerMutex_);
+    return tmLoggingWrapper_;
+  }
+
+  std::shared_ptr<folly::Executor> getThreadManager() const override {
     std::lock_guard<std::mutex> lock(threadManagerMutex_);
     return threadManager_;
   }
 
+  /**
+   * Get the executor for the general pool of async CPUWorkerThreads.
+   *
+   * @return a pointer to the executor
+   */
+
+  folly::Executor* getExecutor() const {
+    if (!resourcePoolSet().empty()) {
+      return resourcePoolSet()
+          .resourcePool(ResourcePoolHandle::defaultAsync())
+          .executor()
+          .value();
+    }
+    std::lock_guard<std::mutex> lock(threadManagerMutex_);
+    return threadManager_.get();
+  }
+
   /**
    * Get the maximum # of connections allowed before overload.
    *
@@ -654,6 +787,19 @@ class BaseThriftServer : public apache::thrift::concurrency::Runnable,
       AttributeSource source = AttributeSource::OVERRIDE,
       DynamicAttributeTag = DynamicAttributeTag{}) {
     maxRequests_.set(maxRequests, source);
+    // Eventually we'll remove the simple setMaxRequests but for now ensure it
+    // updates the concurrency controller for the default async pool.
+    if (!resourcePoolSet().empty()) {
+      if (resourcePoolSet_.hasResourcePool(
+              ResourcePoolHandle::defaultAsync())) {
+        resourcePoolSet_.resourcePool(ResourcePoolHandle::defaultAsync())
+            .concurrencyController()
+            .value()
+            ->setExecutionLimitRequests(
+                maxRequests != 0 ? maxRequests
+                                 : std::numeric_limits<uint32_t>::max());
+      }
+    }
   }
 
   uint64_t getMaxResponseSize() const final { return maxResponseSize_.get(); }
@@ -861,36 +1007,37 @@ class BaseThriftServer : public apache::thrift::concurrency::Runnable,
    * Codel queuing timeout - limit queueing time before overload
    * http://en.wikipedia.org/wiki/CoDel
    */
-  void setEnableCodel(
+  virtual void setEnableCodel(
       bool enableCodel,
       AttributeSource source = AttributeSource::OVERRIDE,
       DynamicAttributeTag = DynamicAttributeTag{}) {
     enableCodel_.set(enableCodel, source);
   }
 
-  bool getEnableCodel() { return enableCodel_.get(); }
+  bool getEnableCodel() const { return enableCodel_.get(); }
 
   /**
-   * Set the processor factory as the one built into the
-   * ServerInterface.
-   *
-   * setInterface() can take both unique_ptr and shared_ptr to handler
-   * interface.
-   *
-   * @param handler interface shared_ptr
+   * Sets the main server interface that exposes user-defined methods.
    */
-  void setInterface(std::shared_ptr<ServerInterface> iface) {
+  void setInterface(std::shared_ptr<AsyncProcessorFactory> iface) {
     setProcessorFactory(std::move(iface));
   }
 
   /**
-   * Sets an explicit AsyncProcessorFactory
-   *
+   * DEPRECATED! Use setInterface instead.
    */
   virtual void setProcessorFactory(
       std::shared_ptr<AsyncProcessorFactory> pFac) {
     CHECK(configMutable());
     cpp2Pfac_ = pFac;
+    applicationServerInterface_ = nullptr;
+    for (auto* serviceHandler : cpp2Pfac_->getServiceHandlers()) {
+      if (auto serverInterface =
+              dynamic_cast<ServerInterface*>(serviceHandler)) {
+        applicationServerInterface_ = serverInterface;
+        break;
+      }
+    }
   }
 
   const std::shared_ptr<apache::thrift::AsyncProcessorFactory>&
@@ -898,6 +1045,15 @@ class BaseThriftServer : public apache::thrift::concurrency::Runnable,
     return cpp2Pfac_;
   }
 
+  concurrency::ThreadManager::ExecutionScope getRequestExecutionScope(
+      Cpp2RequestContext* ctx, concurrency::PRIORITY defaultPriority) override {
+    if (applicationServerInterface_) {
+      return applicationServerInterface_->getRequestExecutionScope(
+          ctx, defaultPriority);
+    }
+    return ServerConfigs::getRequestExecutionScope(ctx, defaultPriority);
+  }
+
   /**
    * Sets the interface that will be used for monitoring connections only.
    */
@@ -907,7 +1063,8 @@ class BaseThriftServer : public apache::thrift::concurrency::Runnable,
     monitoringServiceHandler_ = std::move(iface);
   }
 
-  const std::shared_ptr<MonitoringServerInterface>& getMonitoringInterface() {
+  const std::shared_ptr<MonitoringServerInterface>& getMonitoringInterface()
+      const {
     return monitoringServiceHandler_;
   }
 
@@ -931,7 +1088,7 @@ class BaseThriftServer : public apache::thrift::concurrency::Runnable,
     controlServiceHandler_ = std::move(iface);
   }
 
-  const std::shared_ptr<ControlServerInterface>& getControlInterface() {
+  const std::shared_ptr<ControlServerInterface>& getControlInterface() const {
     return controlServiceHandler_;
   }
 
@@ -979,10 +1136,8 @@ class BaseThriftServer : public apache::thrift::concurrency::Runnable,
    * Note, queuing is an indication that your server cannot keep
    * up with load, and realtime systems should not queue. Only
    * override this if you do heavily batched requests.
-   *
-   * @return queue timeout
    */
-  void setQueueTimeout(
+  virtual void setQueueTimeout(
       std::chrono::milliseconds timeout,
       AttributeSource source = AttributeSource::OVERRIDE,
       DynamicAttributeTag = DynamicAttributeTag{}) {
@@ -1038,7 +1193,7 @@ class BaseThriftServer : public apache::thrift::concurrency::Runnable,
     socketWriteTimeout_.set(timeout, source);
   }
 
-  std::chrono::milliseconds getSocketWriteTimeout() {
+  std::chrono::milliseconds getSocketWriteTimeout() const {
     return socketWriteTimeout_.get();
   }
 
@@ -1101,13 +1256,23 @@ class BaseThriftServer : public apache::thrift::concurrency::Runnable,
    */
   int getListenBacklog() const { return listenBacklog_.get(); }
 
-  [[deprecated("Use setPreprocess instead")]] void setIsOverloaded(
+  // Do not try to access ThreadManager in this function as
+  // ThreadManagers are being deprecated from thrift server
+  // e.g. don't call getThreadManager() inside this
+  [[deprecated("Use setPreprocess instead")]] virtual void setIsOverloaded(
       IsOverloadedFunc isOverloaded) {
     isOverloaded_ = std::move(isOverloaded);
+    runtimeServerActions_.setIsOverloaded = true;
+    LOG(INFO) << "thrift server: isOverloaded() set.";
   }
 
-  void setPreprocess(PreprocessFunc preprocess) {
+  // Do not try to access ThreadManager in this function as
+  // ThreadManagers are being deprecated from thrift server
+  // e.g. don't call getThreadManager() inside this
+  virtual void setPreprocess(PreprocessFunc preprocess) {
     preprocess_ = std::move(preprocess);
+    runtimeServerActions_.setPreprocess = true;
+    LOG(INFO) << "thrift server: preprocess() set.";
   }
 
   void setMethodsBypassMaxRequestsLimit(
@@ -1129,7 +1294,9 @@ class BaseThriftServer : public apache::thrift::concurrency::Runnable,
     getLoad_ = getLoad;
   }
 
-  std::function<int64_t(const std::string&)> getGetLoad() { return getLoad_; }
+  std::function<int64_t(const std::string&)> getGetLoad() const {
+    return getLoad_;
+  }
 
   /**
    * Set failure injection parameters.
@@ -1175,7 +1342,7 @@ class BaseThriftServer : public apache::thrift::concurrency::Runnable,
   /**
    * Return the maximum memory usage by each debug payload.
    */
-  uint64_t getMaxDebugPayloadMemoryPerRequest() {
+  uint64_t getMaxDebugPayloadMemoryPerRequest() const {
     return maxDebugPayloadMemoryPerRequest_.get();
   }
 
@@ -1194,7 +1361,7 @@ class BaseThriftServer : public apache::thrift::concurrency::Runnable,
    * Return the maximum memory usage by each worker to keep track of debug
    * payloads.
    */
-  uint64_t getMaxDebugPayloadMemoryPerWorker() {
+  uint64_t getMaxDebugPayloadMemoryPerWorker() const {
     return maxDebugPayloadMemoryPerWorker_.get();
   }
 
@@ -1214,7 +1381,7 @@ class BaseThriftServer : public apache::thrift::concurrency::Runnable,
    * Return the maximum memory usage by each worker to keep track of debug
    * payloads.
    */
-  uint16_t getMaxFinishedDebugPayloadsPerWorker() {
+  uint16_t getMaxFinishedDebugPayloadsPerWorker() const {
     return maxFinishedDebugPayloadsPerWorker_.get();
   }
 
@@ -1294,7 +1461,7 @@ class BaseThriftServer : public apache::thrift::concurrency::Runnable,
 
   size_t getIngressMemoryLimit() const { return ingressMemoryLimit_.get(); }
 
-  folly::observer::Observer<size_t> getIngressMemoryLimitObserver() {
+  folly::observer::Observer<size_t> getIngressMemoryLimitObserver() const {
     return ingressMemoryLimit_.getObserver();
   }
 
@@ -1313,7 +1480,7 @@ class BaseThriftServer : public apache::thrift::concurrency::Runnable,
 
   size_t getEgressMemoryLimit() const { return egressMemoryLimit_.get(); }
 
-  folly::observer::Observer<size_t> getEgressMemoryLimitObserver() {
+  folly::observer::Observer<size_t> getEgressMemoryLimitObserver() const {
     return egressMemoryLimit_.getObserver();
   }
 
@@ -1334,7 +1501,7 @@ class BaseThriftServer : public apache::thrift::concurrency::Runnable,
   }
 
   folly::observer::Observer<size_t>
-  getMinPayloadSizeToEnforceIngressMemoryLimitObserver() {
+  getMinPayloadSizeToEnforceIngressMemoryLimitObserver() const {
     return minPayloadSizeToEnforceIngressMemoryLimit_.getObserver();
   }
 
@@ -1385,6 +1552,39 @@ class BaseThriftServer : public apache::thrift::concurrency::Runnable,
   const auto& adaptiveConcurrencyController() const {
     return adaptiveConcurrencyController_;
   }
+
+  // Rejects all header-backed connections to this server
+  void disableLegacyTransports(
+      bool value = true,
+      AttributeSource source = AttributeSource::OVERRIDE,
+      DynamicAttributeTag = DynamicAttributeTag{}) {
+    disableHeaderTransport_.set(value, source);
+  }
+  bool isHeaderDisabled() const { return disableHeaderTransport_.get(); }
+
+  const folly::SocketOptionMap& getPerConnectionSocketOptions() const {
+    return perConnectionSocketOptions_.get();
+  }
+  void setPerConnectionSocketOptions(
+      folly::SocketOptionMap options,
+      AttributeSource source = AttributeSource::OVERRIDE,
+      DynamicAttributeTag = DynamicAttributeTag{}) {
+    perConnectionSocketOptions_.set(std::move(options), source);
+  }
+
+  /**
+   * Get the ResourcePoolSet used by this ThriftServer. There is always one, but
+   * it may be empty if ResourcePools are not in use.
+   */
+  const ResourcePoolSet& resourcePoolSet() const override {
+    return resourcePoolSet_;
+  }
+
+  /**
+   * Get the ResourcePoolSet used by this ThriftServer. There is always one, but
+   * it may be empty if ResourcePools are not in use.
+   */
+  ResourcePoolSet& resourcePoolSet() override { return resourcePoolSet_; }
 };
 } // namespace thrift
 } // namespace apache
