@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -52,6 +52,7 @@ using ClientIdentityHook = std::function<std::unique_ptr<void, void (*)(void*)>(
 class RequestChannel;
 class TClientBase;
 class Cpp2Worker;
+class IOWorkerContext;
 
 class ClientMetadataRef {
  public:
@@ -65,6 +66,51 @@ class ClientMetadataRef {
 };
 
 class Cpp2ConnContext : public apache::thrift::server::TConnectionContext {
+  Cpp2ConnContext(
+      const folly::SocketAddress* address,
+      const folly::AsyncTransport* transport,
+      folly::EventBaseManager* manager,
+      const std::shared_ptr<RequestChannel>& duplexChannel,
+      std::shared_ptr<X509> peerCert,
+      apache::thrift::ClientIdentityHook clientIdentityHook,
+      const Cpp2Worker* worker,
+      const IOWorkerContext* workerContext)
+      : transport_(transport),
+        manager_(manager),
+        duplexChannel_(duplexChannel),
+        worker_(worker),
+        workerContext_(workerContext) {
+    if (address) {
+      transportInfo_.peerAddress = *address;
+    }
+    if (transport) {
+      // require worker to be passed when wrapping a real connection
+      DCHECK(worker != nullptr);
+      DCHECK(workerContext != nullptr);
+      transport->getLocalAddress(&transportInfo_.localAddress);
+      if (auto osslCert = folly::OpenSSLTransportCertificate::tryExtractX509(
+              transport->getPeerCertificate())) {
+        peerCert = std::move(osslCert);
+      }
+      transportInfo_.securityProtocol = transport->getSecurityProtocol();
+
+      if (transportInfo_.localAddress.getFamily() == AF_UNIX) {
+        auto wrapper = transport->getUnderlyingTransport<folly::AsyncSocket>();
+        if (wrapper) {
+          peerCred_ = PeerCred::queryFromSocket(wrapper->getNetworkSocket());
+        }
+      }
+    }
+    transportInfo_.peerCertificate = std::move(peerCert);
+
+    if (clientIdentityHook) {
+      transportInfo_.peerIdentities = clientIdentityHook(
+          transport,
+          transportInfo_.peerCertificate.get(),
+          transportInfo_.peerAddress);
+    }
+  }
+
  public:
   enum class TransportType {
     HEADER,
@@ -77,60 +123,57 @@ class Cpp2ConnContext : public apache::thrift::server::TConnectionContext {
       const folly::AsyncTransport* transport = nullptr,
       folly::EventBaseManager* manager = nullptr,
       const std::shared_ptr<RequestChannel>& duplexChannel = nullptr,
-      const std::shared_ptr<X509> peerCert = nullptr /*overridden from socket*/,
-      apache::thrift::ClientIdentityHook clientIdentityHook = nullptr,
-      const Cpp2Worker* worker = nullptr)
-      : manager_(manager),
-        duplexChannel_(duplexChannel),
-        transport_(transport),
-        worker_(worker) {
-    if (address) {
-      peerAddress_ = *address;
-    }
-    X509* x509 = peerCert.get();
-    if (transport) {
-      // require worker to be passed when wrapping a real connection
-      DCHECK(worker != nullptr);
-      transport->getLocalAddress(&localAddress_);
-      auto cert = transport->getPeerCertificate();
-      if (cert) {
-        auto osslCert =
-            dynamic_cast<const folly::OpenSSLTransportCertificate*>(cert);
-        x509 = osslCert ? osslCert->getX509().get() : nullptr;
-      }
-      securityProtocol_ = transport->getSecurityProtocol();
+      std::shared_ptr<X509> peerCert = nullptr /*overridden from socket*/,
+      apache::thrift::ClientIdentityHook clientIdentityHook = nullptr)
+      : Cpp2ConnContext(
+            address,
+            transport,
+            manager,
+            duplexChannel,
+            std::move(peerCert),
+            std::move(clientIdentityHook),
+            nullptr,
+            nullptr) {}
 
-      if (localAddress_.getFamily() == AF_UNIX) {
-        auto wrapper = transport->getUnderlyingTransport<folly::AsyncSocket>();
-        if (wrapper) {
-          peerCred_ = PeerCred::queryFromSocket(wrapper->getNetworkSocket());
-        }
-      }
-    }
-
-    if (clientIdentityHook) {
-      peerIdentities_ = clientIdentityHook(transport, x509, peerAddress_);
-    }
-  }
+  template <typename WorkerT>
+  explicit Cpp2ConnContext(
+      const folly::SocketAddress* address,
+      const folly::AsyncTransport* transport,
+      folly::EventBaseManager* manager,
+      const std::shared_ptr<RequestChannel>& duplexChannel,
+      std::shared_ptr<X509> peerCert /*overridden from socket*/,
+      apache::thrift::ClientIdentityHook clientIdentityHook,
+      const WorkerT* worker)
+      : Cpp2ConnContext(
+            address,
+            transport,
+            manager,
+            duplexChannel,
+            std::move(peerCert),
+            std::move(clientIdentityHook),
+            worker,
+            worker) {}
 
   ~Cpp2ConnContext() override { DCHECK(tiles_.empty()); }
   Cpp2ConnContext(Cpp2ConnContext&&) = default;
   Cpp2ConnContext& operator=(Cpp2ConnContext&&) = default;
 
   void reset() {
-    peerAddress_.reset();
-    localAddress_.reset();
+    transportInfo_.peerAddress.reset();
+    transportInfo_.localAddress.reset();
     userData_.reset();
   }
 
   const folly::SocketAddress* getPeerAddress() const final {
-    return &peerAddress_;
+    return &transportInfo_.peerAddress;
   }
 
-  const folly::SocketAddress* getLocalAddress() const { return &localAddress_; }
+  const folly::SocketAddress* getLocalAddress() const {
+    return &transportInfo_.localAddress;
+  }
 
   void setLocalAddress(const folly::SocketAddress& localAddress) {
-    localAddress_ = localAddress;
+    transportInfo_.localAddress = localAddress;
   }
 
   void setRequestHeader(apache::thrift::transport::THeader* header) {
@@ -163,10 +206,16 @@ class Cpp2ConnContext : public apache::thrift::server::TConnectionContext {
   }
 
   virtual const std::string& getSecurityProtocol() const {
-    return securityProtocol_;
+    return transportInfo_.securityProtocol;
   }
 
-  virtual void* getPeerIdentities() const { return peerIdentities_.get(); }
+  virtual void* getPeerIdentities() const {
+    return transportInfo_.peerIdentities.get();
+  }
+
+  const X509* FOLLY_NULLABLE getPeerCertificate() const {
+    return transportInfo_.peerCertificate.get();
+  }
 
   virtual const folly::AsyncTransport* getTransport() const {
     return transport_;
@@ -244,6 +293,8 @@ class Cpp2ConnContext : public apache::thrift::server::TConnectionContext {
 
   const Cpp2Worker* getWorker() const { return worker_; }
 
+  const IOWorkerContext* getWorkerContext() const { return workerContext_; }
+
   std::optional<TransportType> getTransportType() const {
     return transportType_;
   }
@@ -298,6 +349,7 @@ class Cpp2ConnContext : public apache::thrift::server::TConnectionContext {
   friend class GeneratedAsyncProcessor;
   friend class Tile;
   friend class TilePromise;
+  friend class HandlerCallbackBase;
 
   void setTransportType(TransportType transportType) {
     transportType_ = transportType;
@@ -391,19 +443,26 @@ class Cpp2ConnContext : public apache::thrift::server::TConnectionContext {
   };
 
   folly::erased_unique_ptr userData_{folly::empty_erased_unique_ptr()};
-  folly::SocketAddress peerAddress_;
-  folly::SocketAddress localAddress_;
+
+  // Snapshot of info from AsyncTransport since this object may outlive it
+  struct TransportInfo {
+    folly::SocketAddress peerAddress;
+    folly::SocketAddress localAddress;
+    folly::erased_unique_ptr peerIdentities{folly::empty_erased_unique_ptr()};
+    std::string securityProtocol;
+    std::shared_ptr<X509> peerCertificate;
+  } transportInfo_;
+  const folly::AsyncTransport* transport_;
+
   folly::EventBaseManager* manager_;
   std::shared_ptr<RequestChannel> duplexChannel_;
   std::shared_ptr<TClientBase> duplexClient_;
-  folly::erased_unique_ptr peerIdentities_{folly::empty_erased_unique_ptr()};
-  std::string securityProtocol_;
-  const folly::AsyncTransport* transport_;
   PeerCred peerCred_;
   // A CancellationSource that will be signaled when the connection is closed.
   folly::CancellationSource cancellationSource_;
   folly::F14FastMap<int64_t, TilePtr> tiles_;
   const Cpp2Worker* worker_;
+  const IOWorkerContext* workerContext_;
   InterfaceKind interfaceKind_{InterfaceKind::USER};
   std::optional<TransportType> transportType_;
   std::optional<CLIENT_TYPE> clientType_;
@@ -464,6 +523,10 @@ class Cpp2RequestContext : public apache::thrift::server::TConnectionContext {
     return ctx_->getEventBaseManager();
   }
 
+  void setException() { isException_ = true; }
+
+  bool isException() const { return isException_; }
+
   void* getUserData() const override { return ctx_->getUserData(); }
 
   void* setUserData(folly::erased_unique_ptr data) override {
@@ -545,6 +608,7 @@ class Cpp2RequestContext : public apache::thrift::server::TConnectionContext {
   std::string methodName_;
   int32_t protoSeqId_{0};
   int64_t interactionId_{0};
+  bool isException_{false};
   folly::Optional<InteractionCreate> interactionCreate_;
   TilePtr tile_;
   concurrency::ThreadManager::ExecutionScope executionScope_{
