@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -22,6 +22,7 @@
 #include <thrift/lib/cpp2/Flags.h>
 #include <thrift/lib/cpp2/GeneratedCodeHelper.h>
 #include <thrift/lib/cpp2/async/AsyncProcessorHelper.h>
+#include <thrift/lib/cpp2/async/ResponseChannel.h>
 #include <thrift/lib/cpp2/protocol/BinaryProtocol.h>
 #include <thrift/lib/cpp2/protocol/CompactProtocol.h>
 #include <thrift/lib/cpp2/server/Cpp2Worker.h>
@@ -32,21 +33,15 @@
 
 THRIFT_FLAG_DEFINE_bool(server_rocket_upgrade_enabled, false);
 THRIFT_FLAG_DEFINE_bool(server_header_reject_http, true);
-THRIFT_FLAG_DEFINE_bool(server_header_reject_framed, false);
-THRIFT_FLAG_DEFINE_bool(server_header_reject_unframed, false);
+THRIFT_FLAG_DEFINE_bool(server_header_reject_framed, true);
+THRIFT_FLAG_DEFINE_bool(server_header_reject_unframed, true);
 
 THRIFT_FLAG_DEFINE_int64(monitoring_over_header_logging_sample_rate, 1'000'000);
 
 namespace apache {
 namespace thrift {
 
-using namespace apache::thrift::protocol;
-using namespace apache::thrift::server;
-using namespace apache::thrift::transport;
-using namespace apache::thrift::concurrency;
-using namespace apache::thrift::async;
 using namespace std;
-using apache::thrift::TApplicationException;
 
 namespace {
 // This is a SendCallback used for transport upgrade from header to rocket
@@ -115,6 +110,15 @@ class TransportUpgradeSendCallback : public MessageChannel::SendCallback {
   Cpp2Connection* cpp2Conn_;
   HeaderServerChannel* headerChannel_;
 };
+
+std::shared_ptr<apache::thrift::AsyncProcessorFactory>
+getProcessorFactoryOverride(folly::AsyncTransport& transport) {
+  if (auto newConnectionContext =
+          ThriftServer::extractNewConnectionContext(transport)) {
+    return newConnectionContext->processorFactory;
+  }
+  return {};
+}
 } // namespace
 
 Cpp2Connection::Cpp2Connection(
@@ -122,8 +126,14 @@ Cpp2Connection::Cpp2Connection(
     const folly::SocketAddress* address,
     std::shared_ptr<Cpp2Worker> worker,
     const std::shared_ptr<HeaderServerChannel>& serverChannel)
-    : processorFactory_(worker->getServer()->getDecoratedProcessorFactory()),
-      serviceMetadata_(worker->getMetadataForService(processorFactory_)),
+    : processorFactoryOverride_(
+          transport ? getProcessorFactoryOverride(*transport) : nullptr),
+      processorFactory_(
+          processorFactoryOverride_
+              ? *processorFactoryOverride_
+              : worker->getServer()->getDecoratedProcessorFactory()),
+      serviceMetadata_(worker->getMetadataForService(
+          processorFactory_, processorFactoryOverride_)),
       processor_(processorFactory_.getProcessor()),
       duplexChannel_(
           worker->getServer()->isDuplex()
@@ -147,7 +157,10 @@ Cpp2Connection::Cpp2Connection(
           worker_->getServer()->getClientIdentityHook(),
           worker_.get()),
       transport_(transport),
-      threadManager_(worker_->getServer()->getThreadManager()) {
+      executor_(worker_->getServer()->getExecutor()) {
+  if (worker_->getServer()->resourcePoolSet().empty()) {
+    threadManager_ = worker_->getServer()->getThreadManager_deprecated();
+  }
   context_.setTransportType(Cpp2ConnContext::TransportType::HEADER);
 
   if (auto* observer = worker_->getServer()->getObserver()) {
@@ -164,8 +177,10 @@ Cpp2Connection::~Cpp2Connection() {
     handler->connectionDestroyed(&context_);
   }
 
-  if (auto* observer = worker_->getServer()->getObserver()) {
-    observer->connClosed();
+  if (connectionAdded_) {
+    if (auto* observer = worker_->getServer()->getObserver()) {
+      observer->connClosed();
+    }
   }
 
   channel_.reset();
@@ -428,6 +443,11 @@ void Cpp2Connection::requestReceived(
     }
   }
 
+  if (worker_->getServer()->isHeaderDisabled()) {
+    disconnect("Rejecting Header connection");
+    return;
+  }
+
   using PerServiceMetadata = Cpp2Worker::PerServiceMetadata;
   const PerServiceMetadata::FindMethodResult methodMetadataResult =
       serviceMetadata_.findMethod(methodName);
@@ -569,7 +589,9 @@ void Cpp2Connection::requestReceived(
     timestamps.readEnd = readEnd;
     timestamps.processBegin = std::chrono::steady_clock::now();
     if (samplingStatus.isEnabledByServer() && observer) {
-      observer->queuedRequests(threadManager_->pendingUpstreamTaskCount());
+      if (threadManager_) {
+        observer->queuedRequests(threadManager_->pendingUpstreamTaskCount());
+      }
       observer->activeRequests(server->getActiveRequests());
     }
   }
@@ -610,48 +632,26 @@ void Cpp2Connection::requestReceived(
     }
   }
 
-  try {
-    ResponseChannelRequest::UniquePtr req = std::move(t2r);
-    if (!apache::thrift::detail::ap::setupRequestContextWithMessageBegin(
-            meta, protoId, req, reqContext, worker_->getEventBase())) {
-      return;
-    }
-
-    folly::variant_match(
-        methodMetadataResult,
-        [&](PerServiceMetadata::MetadataNotImplemented) {
-          logSetupConnectionEventsOnce(setupLoggingFlag_, context_);
-
-          // The AsyncProcessorFactory does not implement createMethodMetadata
-          // so we need to fallback to processSerializedCompressedRequest.
-          processor_->processSerializedCompressedRequest(
-              std::move(req),
-              SerializedCompressedRequest(std::move(serializedRequest)),
-              protoId,
-              reqContext,
-              worker_->getEventBase(),
-              threadManager_.get());
-        },
-        [&](PerServiceMetadata::MetadataNotFound) {
-          AsyncProcessorHelper::sendUnknownMethodError(
-              std::move(req), reqContext->getMethodName());
-        },
-        [&](const PerServiceMetadata::MetadataFound& found) {
-          logSetupConnectionEventsOnce(setupLoggingFlag_, context_);
-
-          processor_->processSerializedCompressedRequestWithMetadata(
-              std::move(req),
-              SerializedCompressedRequest(std::move(serializedRequest)),
-              found.metadata,
-              protoId,
-              reqContext,
-              worker_->getEventBase(),
-              threadManager_.get());
-        });
-  } catch (...) {
-    LOG(DFATAL) << "AsyncProcessor::process exception: "
-                << folly::exceptionStr(std::current_exception());
+  ResponseChannelRequest::UniquePtr req = std::move(t2r);
+  if (!apache::thrift::detail::ap::setupRequestContextWithMessageBegin(
+          meta, protoId, req, reqContext, worker_->getEventBase())) {
+    return;
   }
+
+  if (!std::get_if<PerServiceMetadata::MetadataNotFound>(
+          &methodMetadataResult)) {
+    logSetupConnectionEventsOnce(setupLoggingFlag_, context_);
+  }
+
+  Cpp2Worker::dispatchRequest(
+      processor_.get(),
+      std::move(req),
+      SerializedCompressedRequest(std::move(serializedRequest)),
+      methodMetadataResult,
+      protoId,
+      reqContext,
+      threadManager_.get(),
+      worker_->getServer());
 }
 
 void Cpp2Connection::channelClosed(folly::exception_wrapper&& ex) {
