@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -41,8 +41,9 @@ void logNonTLSEvent(const ConnectionLoggingContext& context) {
   DCHECK(context.getWorker() && context.getWorker()->getServer());
   auto server = context.getWorker()->getServer();
   // There's actually no point in logging these at all as wangle will not set
-  // up tls if theres no config.
-  if (!server->getSSLConfig() || server->isSSLPolicySet() ||
+  // up tls if there's no config.
+  if (!server->getSSLConfig() ||
+      server->getSSLPolicy() != SSLPolicy::REQUIRED ||
       (server->isPlaintextAllowedOnLoopback() &&
        context.getPeerAddress()->isLoopbackAddress())) {
     THRIFT_CONNECTION_EVENT(non_tls.manual_policy).log(context);
