@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -21,9 +21,11 @@
 #include <string>
 #include <variant>
 
+#include <folly/Executor.h>
 #include <folly/lang/Assume.h>
 
 #include <thrift/lib/cpp/TApplicationException.h>
+#include <thrift/lib/cpp/concurrency/ThreadManager.h>
 #include <thrift/lib/cpp/server/TServerObserver.h>
 #include <thrift/lib/cpp/transport/THeader.h>
 #include <thrift/lib/thrift/gen-cpp2/RpcMetadata_types.h>
@@ -38,7 +40,13 @@ using PreprocessResult = std::variant<
     AppOverloadedException>;
 
 class Cpp2ConnContext;
+class Cpp2RequestContext;
 class AdaptiveConcurrencyController;
+class ResourcePoolSet;
+
+namespace concurrency {
+class ThreadManager;
+}
 
 namespace server {
 
@@ -100,6 +108,16 @@ class ServerConfigs {
   // @see ThriftServer::getListenerTos function.
   virtual uint32_t getListenerTos() const = 0;
 
+  // @see BaseThriftServer::resourcePoolSet function.
+  virtual const ResourcePoolSet& resourcePoolSet() const {
+    LOG(FATAL) << "Unimplemented resourcePoolSet const";
+  }
+
+  // @see BaseThriftServer::resourcePoolSet function.
+  virtual ResourcePoolSet& resourcePoolSet() {
+    LOG(FATAL) << "Unimplemented resourcePoolSet";
+  }
+
   /**
    * Disables tracking of number of active requests in the server.
    *
@@ -173,6 +191,17 @@ class ServerConfigs {
     rejectRequestsUntilStarted_ = rejectRequestsUntilStarted;
   }
 
+  virtual std::shared_ptr<folly::Executor> getThreadManager() const = 0;
+
+  [[deprecated("If possible use getThreadManager() instead")]] virtual std::
+      shared_ptr<concurrency::ThreadManager>
+      getThreadManager_deprecated() const = 0;
+
+  virtual concurrency::ThreadManager::ExecutionScope getRequestExecutionScope(
+      Cpp2RequestContext*, concurrency::PRIORITY defaultPriority) {
+    return concurrency::ThreadManager::ExecutionScope(defaultPriority);
+  }
+
  private:
   folly::relaxed_atomic<int32_t> activeRequests_{0};
 
