@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -28,11 +28,14 @@
 #include <folly/ScopeGuard.h>
 #include <folly/executors/CPUThreadPoolExecutor.h>
 #include <folly/executors/IOThreadPoolDeadlockDetectorObserver.h>
+#include <folly/executors/thread_factory/NamedThreadFactory.h>
+#include <folly/executors/thread_factory/PriorityThreadFactory.h>
 #include <folly/experimental/coro/BlockingWait.h>
 #include <folly/experimental/coro/CurrentExecutor.h>
 #include <folly/experimental/coro/Invoke.h>
 #include <folly/io/GlobalShutdownSocketSet.h>
 #include <folly/portability/Sockets.h>
+#include <folly/system/Pid.h>
 #include <thrift/lib/cpp/concurrency/PosixThreadFactory.h>
 #include <thrift/lib/cpp/concurrency/Thread.h>
 #include <thrift/lib/cpp/concurrency/ThreadManager.h>
@@ -41,7 +44,11 @@
 #include <thrift/lib/cpp2/async/MultiplexAsyncProcessor.h>
 #include <thrift/lib/cpp2/server/Cpp2Connection.h>
 #include <thrift/lib/cpp2/server/Cpp2Worker.h>
+#include <thrift/lib/cpp2/server/ExecutorToThreadManagerAdaptor.h>
 #include <thrift/lib/cpp2/server/LoggingEvent.h>
+#include <thrift/lib/cpp2/server/ParallelConcurrencyController.h>
+#include <thrift/lib/cpp2/server/RoundRobinRequestPile.h>
+#include <thrift/lib/cpp2/server/ServerFlags.h>
 #include <thrift/lib/cpp2/server/ServerInstrumentation.h>
 #include <thrift/lib/cpp2/server/ThriftProcessor.h>
 #include <thrift/lib/cpp2/transport/core/ManagedConnectionIf.h>
@@ -67,13 +74,16 @@ DEFINE_string(
 
 THRIFT_FLAG_DEFINE_bool(server_alpn_prefer_rocket, true);
 THRIFT_FLAG_DEFINE_bool(server_enable_stoptls, false);
-THRIFT_FLAG_DEFINE_bool(ssl_policy_default_required, true);
 THRIFT_FLAG_DEFINE_bool(alpn_allow_mismatch, true);
 
 THRIFT_FLAG_DEFINE_bool(dump_snapshot_on_long_shutdown, true);
 
 THRIFT_FLAG_DEFINE_bool(server_check_unimplemented_extra_interfaces, true);
 
+THRIFT_FLAG_DEFINE_bool(enable_on_stop_serving, true);
+
+THRIFT_FLAG_DEFINE_bool(enable_io_queue_lag_detection, true);
+
 namespace apache::thrift::detail {
 THRIFT_PLUGGABLE_FUNC_REGISTER(
     apache::thrift::ThriftServer::DumpSnapshotOnLongShutdownResult,
@@ -94,6 +104,7 @@ THRIFT_PLUGGABLE_FUNC_REGISTER(
     AsyncProcessorFactory& /* service */) {
   return ThriftServer::UnimplementedExtraInterfacesResult::UNRECOGNIZED;
 }
+
 } // namespace apache::thrift::detail
 
 namespace {
@@ -110,20 +121,11 @@ namespace {
 namespace apache {
 namespace thrift {
 
-using namespace apache::thrift::protocol;
 using namespace apache::thrift::server;
-using namespace apache::thrift::transport;
-using namespace apache::thrift::async;
 using namespace std;
-using apache::thrift::concurrency::PosixThreadFactory;
 using apache::thrift::concurrency::PriorityThreadManager;
 using apache::thrift::concurrency::Runnable;
-using apache::thrift::concurrency::ThreadFactory;
 using apache::thrift::concurrency::ThreadManager;
-using folly::IOThreadPoolExecutor;
-using folly::NamedThreadFactory;
-using RequestSnapshot = ThriftServer::RequestSnapshot;
-using std::shared_ptr;
 using wangle::TLSCredProcessor;
 
 namespace {
@@ -172,22 +174,22 @@ std::unique_ptr<AsyncProcessorFactory> createDecoratedProcessorFactory(
 // instead of AsyncProcessor.h
 
 #if FOLLY_HAS_COROUTINES
-folly::coro::CancellableAsyncScope* ServiceHandler::getAsyncScope() {
+folly::coro::CancellableAsyncScope* ServiceHandlerBase::getAsyncScope() {
   return server_->getAsyncScope();
 }
 #endif
 
-void ServiceHandler::attachServer(ThriftServer& server) {
+void ServiceHandlerBase::attachServer(ThriftServer& server) {
   server_ = &server;
   serverStopController_.lock()->emplace(server.getStopController());
 }
 
-void ServiceHandler::detachServer() {
+void ServiceHandlerBase::detachServer() {
   server_ = nullptr;
   serverStopController_.lock()->reset();
 }
 
-void ServiceHandler::shutdownServer() {
+void ServiceHandlerBase::shutdownServer() {
   // shutdownServer should be idempotent -- this means that it can race with
   // detachServer. Thus we should sychronize access to it.
   serverStopController_.withLock([](auto& stopController) {
@@ -223,6 +225,12 @@ ThriftServer::ThriftServer()
   setMonitoringInterface(std::move(extraInterfaces.monitoring));
   setStatusInterface(std::move(extraInterfaces.status));
   setControlInterface(std::move(extraInterfaces.control));
+  getAdaptiveConcurrencyController().setConfigUpdateCallback(
+      [this](auto snapshot) {
+        if (snapshot->isEnabled()) {
+          THRIFT_SERVER_EVENT(ACC_enabled).log(*this);
+        }
+      });
 }
 
 ThriftServer::ThriftServer(
@@ -260,19 +268,6 @@ ThriftServer::~ThriftServer() {
   stopWorkers();
 }
 
-SSLPolicy ThriftServer::getSSLPolicy() const {
-  // If it's explicitly set in constructor through gflags or through the
-  // setSSLPolicy setter, then use that value.
-  if (sslPolicy_.has_value()) {
-    return *sslPolicy_;
-  }
-  // Otherwise, fallback to default (currently defined through a ThriftFlag).
-  // REQUIRED is the new default we're migrating to. We use ThriftFlags to
-  // opt-out services that still need to use PERMITTED.
-  return THRIFT_FLAG(ssl_policy_default_required) ? SSLPolicy::REQUIRED
-                                                  : SSLPolicy::PERMITTED;
-}
-
 void ThriftServer::setProcessorFactory(
     std::shared_ptr<AsyncProcessorFactory> pFac) {
   CHECK(configMutable());
@@ -421,24 +416,68 @@ void ThriftServer::setup() {
       setObserver(server::observerFactory_->getObserver());
     }
 
-    // We always need a threadmanager for cpp2.
-    setupThreadManager();
-    threadManager_->setExpireCallback([&](std::shared_ptr<Runnable> r) {
-      EventTask* task = dynamic_cast<EventTask*>(r.get());
-      if (task) {
-        task->expired();
-      }
-    });
-    threadManager_->setCodelCallback([&](std::shared_ptr<Runnable>) {
-      auto observer = getObserver();
-      if (observer) {
-        if (getEnableCodel()) {
-          observer->queueTimeout();
-        } else {
-          observer->shadowQueueTimeout();
+    runtimeResourcePoolsChecks();
+
+    // Past this point no modification to the enablement of
+    // ResourcePool should be made in the same server
+    runtimeServerActions_.resourcePoolFlagSet =
+        THRIFT_FLAG(experimental_use_resource_pools);
+    if (!useResourcePools()) {
+      // We always need a threadmanager for cpp2.
+      LOG(INFO) << "Using thread manager (resource pools not enabled) "
+                << runtimeServerActions_.explain();
+      setupThreadManager();
+      threadManager_->setExpireCallback([&](std::shared_ptr<Runnable> r) {
+        EventTask* task = dynamic_cast<EventTask*>(r.get());
+        if (task) {
+          task->expired();
         }
+      });
+      threadManager_->setCodelCallback([&](std::shared_ptr<Runnable>) {
+        auto observer = getObserver();
+        if (observer) {
+          if (getEnableCodel()) {
+            observer->queueTimeout();
+          } else {
+            observer->shadowQueueTimeout();
+          }
+        }
+      });
+
+      // Log the case when we tried to use resource pools and disabled it
+      // because of run-time holdouts.
+      if (runtimeServerActions_.resourcePoolFlagSet) {
+        THRIFT_SERVER_EVENT(resourcepoolsruntimedisallowed).log(*this);
       }
-    });
+    } else {
+      LOG(INFO) << "Using resource pools";
+      runtimeServerActions_.resourcePoolEnabled = true;
+      DCHECK(!threadManager_);
+      ensureResourcePools();
+      // Keep concurrency controller in sync with max requests for now.
+      auto maxRequests = getMaxRequests();
+      resourcePoolSet()
+          .resourcePool(ResourcePoolHandle::defaultAsync())
+          .concurrencyController()
+          .value()
+          ->setExecutionLimitRequests(
+              maxRequests != 0 ? maxRequests
+                               : std::numeric_limits<uint32_t>::max());
+
+      if (resourcePoolSet().hasResourcePool(
+              ResourcePoolHandle::defaultAsync())) {
+        auto exPtr = resourcePoolSet()
+                         .resourcePool(ResourcePoolHandle::defaultAsync())
+                         .executor()
+                         .value();
+        auto extm = std::make_shared<ExecutorToThreadManagerAdaptor>(*exPtr);
+        setThreadManagerInternal(extm);
+      }
+
+      // During resource pools roll out we want to track services that get
+      // enrolled in the roll out.
+      THRIFT_SERVER_EVENT(resourcepoolsenabled).log(*this);
+    }
 
     if (!serverChannel_) {
       ServerBootstrap::socketConfig.acceptBacklog = getListenBacklog();
@@ -457,7 +496,8 @@ void ThriftServer::setup() {
               ioThreadPool_->getName()));
       ioObserverFactories.withRLock([this](auto& factories) {
         for (auto& f : factories) {
-          ioThreadPool_->addObserver(f(ioThreadPool_->getName()));
+          ioThreadPool_->addObserver(f(
+              ioThreadPool_->getName(), ioThreadPool_->getThreadIdCollector()));
         }
       });
 
@@ -566,7 +606,7 @@ void ThriftServer::setup() {
                 cachedServiceHealth_.store(*value, std::memory_order_relaxed);
               }
             });
-        asyncScope_->add(std::move(loop).scheduleOn(threadManager_.get()));
+        asyncScope_->add(std::move(loop).scheduleOn(getExecutor()));
       }
     }
 #endif
@@ -594,9 +634,30 @@ void ThriftServer::setup() {
 
 void ThriftServer::setupThreadManager() {
   if (!threadManager_) {
-    std::shared_ptr<apache::thrift::concurrency::ThreadManager> threadManager(
-        PriorityThreadManager::newPriorityThreadManager(
-            getNumCPUWorkerThreads()));
+    std::shared_ptr<apache::thrift::concurrency::ThreadManager> threadManager;
+    switch (threadManagerType_) {
+      case ThreadManagerType::PRIORITY:
+        if (std::any_of(
+                std::begin(threadManagerPoolSizes_),
+                std::end(threadManagerPoolSizes_),
+                [](std::size_t c) { return c != 0; })) {
+          // The priorities were specified using setThreadManagerPoolSizes
+          threadManager = PriorityThreadManager::newPriorityThreadManager(
+              threadManagerPoolSizes_);
+        } else {
+          threadManager = PriorityThreadManager::newPriorityThreadManager(
+              getNumCPUWorkerThreads());
+        }
+        break;
+      case ThreadManagerType::SIMPLE:
+        threadManager =
+            ThreadManager::newSimpleThreadManager(getNumCPUWorkerThreads());
+        break;
+      case ThreadManagerType::PRIORITY_QUEUE:
+        threadManager = ThreadManager::newPriorityQueueThreadManager(
+            getNumCPUWorkerThreads());
+        break;
+    }
     threadManager->enableCodel(getEnableCodel());
     // If a thread factory has been specified, use it.
     if (threadFactory_) {
@@ -607,10 +668,183 @@ void ThriftServer::setupThreadManager() {
       threadManager->setNamePrefix(poolThreadName);
     }
     threadManager->start();
-    setThreadManager(threadManager);
+    setThreadManagerInternal(threadManager);
   }
 }
 
+void ThriftServer::runtimeResourcePoolsChecks() {
+  // Check whether there are any wildcard services.
+  auto methodMetadata = getDecoratedProcessorFactory().createMethodMetadata();
+  auto* methodMetadataMap =
+      std::get_if<AsyncProcessorFactory::MethodMetadataMap>(&methodMetadata);
+  if (!methodMetadataMap) {
+    // Only accept services with full method metadata
+    LOG(INFO) << "Resource pools disabled. Wildcard methods";
+    runtimeServerActions_.wildcardMethods = true;
+    runtimeDisableResourcePools();
+  } else {
+    for (const auto& methodToMetadataPtr : *methodMetadataMap) {
+      const auto& metadata = *methodToMetadataPtr.second;
+      if (metadata.executorType ==
+              AsyncProcessorFactory::MethodMetadata::ExecutorType::UNKNOWN ||
+          metadata.interactionType ==
+              AsyncProcessorFactory::MethodMetadata::InteractionType::UNKNOWN ||
+          !metadata.rpcKind || !metadata.priority) {
+        // Disable resource pools if there is no service request info
+        LOG(INFO) << "Resource pools disabled. Incomplete metadata";
+        runtimeServerActions_.noServiceRequestInfo = true;
+        runtimeDisableResourcePools();
+      }
+      if (metadata.interactionType ==
+          AsyncProcessorFactory::MethodMetadata::InteractionType::
+              INTERACTION_V1) {
+        // We've found an interaction in this service. Mark it is incompatible
+        // with resource pools
+        LOG(INFO) << "Resource pools disabled. Interaction on request "
+                  << methodToMetadataPtr.first;
+        runtimeServerActions_.interactionInService = true;
+        runtimeDisableResourcePools();
+      }
+    }
+  }
+
+  if (isActiveRequestsTrackingDisabled()) {
+    LOG(INFO) << "Resource pools disabled. Active request tracking disabled";
+    runtimeServerActions_.activeRequestTrackingDisabled = true;
+    runtimeDisableResourcePools();
+  }
+
+  if (getEnableCodel() || FLAGS_codel_enabled ||
+      (threadManager_ && threadManager_->codelEnabled())) {
+    LOG(INFO) << "Resource pools disabled. Codel enabled";
+    runtimeServerActions_.codelEnabled = true;
+    runtimeDisableResourcePools();
+  }
+}
+
+void ThriftServer::ensureResourcePools() {
+  auto resourcePoolSupplied = !resourcePoolSet().empty();
+
+  if (!resourcePoolSet().hasResourcePool(ResourcePoolHandle::defaultSync())) {
+    // Ensure there is a sync resource pool.
+    resourcePoolSet().setResourcePool(
+        ResourcePoolHandle::defaultSync(),
+        /*requestPile=*/nullptr,
+        /*executor=*/nullptr,
+        /*concurrencyController=*/nullptr);
+  }
+
+  // The user provided the resource pool. Use it as is.
+  if (resourcePoolSupplied) {
+    resourcePoolSet().lock();
+    return;
+  }
+
+  struct Pool {
+    std::string_view name;
+    std::string_view suffix;
+    int nicePriority;
+    size_t numThreads;
+    std::optional<ResourcePoolHandle> handle;
+    concurrency::PRIORITY priority;
+  };
+
+  std::vector<Pool> pools;
+
+  switch (threadManagerType_) {
+    case ThreadManagerType::PRIORITY: {
+      // TODO: T111371879 [thrift][resourcepools] Figure out priorities for
+      // default setup in ensureResourcePool including non-linux These priority
+      // numbers are what thrift currently derives for a nice range of 19 to
+      // -20.
+      Pool priorityPools[] = {
+          {"HIGH_IMPORTANT",
+           "HI",
+           -13,
+           2,
+           std::nullopt,
+           concurrency::HIGH_IMPORTANT},
+          {"HIGH", "H", -7, 2, std::nullopt, concurrency::HIGH},
+          {"IMPORTANT", "I", -7, 2, std::nullopt, concurrency::IMPORTANT},
+          {"NORMAL",
+           "N",
+           0,
+           getNumCPUWorkerThreads(),
+           ResourcePoolHandle::defaultAsync(),
+           concurrency::NORMAL},
+          {"BEST_EFFORT", "BE", 6, 2, std::nullopt, concurrency::BEST_EFFORT}};
+      if (std::any_of(
+              std::begin(threadManagerPoolSizes_),
+              std::end(threadManagerPoolSizes_),
+              [](std::size_t c) { return c != 0; })) {
+        // The priorities were specified using setThreadManagerPoolSizes
+        for (std::size_t i = 0; i < std::size(priorityPools); ++i) {
+          priorityPools[i].numThreads = threadManagerPoolSizes_.at(i);
+        }
+      }
+      std::copy(
+          std::begin(priorityPools),
+          std::end(priorityPools),
+          std::back_inserter(pools));
+      break;
+    }
+    case ThreadManagerType::SIMPLE: {
+      pools.push_back(Pool{
+          "NORMAL",
+          "N",
+          0,
+          getNumCPUWorkerThreads(),
+          ResourcePoolHandle::defaultAsync(),
+          concurrency::NORMAL});
+      break;
+    }
+    case ThreadManagerType::PRIORITY_QUEUE: {
+      pools.push_back(Pool{
+          "NORMAL",
+          "N",
+          0,
+          getNumCPUWorkerThreads(),
+          ResourcePoolHandle::defaultAsync(),
+          concurrency::NORMAL});
+      break;
+    }
+  }
+  for (auto const& pool : pools) {
+    std::string name =
+        fmt::format("{}.{}", getCPUWorkerThreadName(), pool.suffix);
+    auto factory = std::make_shared<folly::PriorityThreadFactory>(
+        std::make_shared<folly::NamedThreadFactory>(name), pool.nicePriority);
+    auto executor = std::make_shared<folly::CPUThreadPoolExecutor>(
+        pool.numThreads, std::move(factory));
+    apache::thrift::RoundRobinRequestPile::Options options;
+    if (threadManagerType_ == ThreadManagerType::PRIORITY_QUEUE) {
+      options.setNumPriorities(concurrency::N_PRIORITIES);
+      options.setPileSelectionFunction(options.getDefaultPileSelectionFunc());
+    }
+    auto requestPile = std::make_unique<apache::thrift::RoundRobinRequestPile>(
+        std::move(options));
+    auto concurrencyController =
+        std::make_unique<apache::thrift::ParallelConcurrencyController>(
+            *requestPile.get(), *executor.get());
+    if (pool.handle) {
+      resourcePoolSet().setResourcePool(
+          ResourcePoolHandle::defaultAsync(),
+          std::move(requestPile),
+          executor,
+          std::move(concurrencyController),
+          pool.priority);
+    } else {
+      resourcePoolSet().addResourcePool(
+          pool.name,
+          std::move(requestPile),
+          executor,
+          std::move(concurrencyController),
+          pool.priority);
+    }
+  }
+  resourcePoolSet().lock();
+}
+
 /**
  * Preferably use this method in order to start ThriftServer created for
  * DuplexChannel instead of the serve() method.
@@ -788,7 +1022,7 @@ void ThriftServer::stopAcceptingAndJoinOutstandingRequests() {
     }
   }
 
-  callOnStopServing();
+  callOnStopRequested();
 
   internalStatus_.store(
       ServerStatus::DRAINING_UNTIL_STOPPED, std::memory_order_release);
@@ -849,7 +1083,7 @@ void ThriftServer::stopAcceptingAndJoinOutstandingRequests() {
           }
         }
 
-        auto msgTemplate =
+        constexpr auto msgTemplate =
             "Could not drain active requests within allotted deadline. "
             "Deadline value: {} secs. {} because undefined behavior is possible. "
             "Underlying reasons could be either requests that have never "
@@ -908,21 +1142,27 @@ void ThriftServer::callOnStartServing() {
           return handler->semifuture_onStartServing();
         }));
   }
-  folly::collectAll(futures).via(getThreadManager().get()).get();
+  folly::collectAll(futures).via(getExecutor()).get();
 }
 
-void ThriftServer::callOnStopServing() {
+void ThriftServer::callOnStopRequested() {
   auto handlerList = collectServiceHandlers();
   std::vector<folly::SemiFuture<folly::Unit>> futures;
   futures.reserve(handlerList.size());
   for (auto handler : handlerList) {
-    futures.emplace_back(handler->semifuture_onStopServing());
+    futures.emplace_back(
+        THRIFT_FLAG(enable_on_stop_serving)
+            ? folly::makeSemiFuture().deferValue([handler](folly::Unit) {
+                return handler->semifuture_onStopRequested();
+              })
+            : handler->semifuture_onStopRequested());
   }
-  auto result =
-      folly::collectAll(futures).via(getThreadManager().get()).getTry();
-  if (result.hasException()) {
-    LOG(FATAL) << "Exception thrown by onStopServing(): "
-               << folly::exceptionStr(result.exception());
+  auto results = folly::collectAll(futures).via(getExecutor()).get();
+  for (auto& result : results) {
+    if (result.hasException()) {
+      LOG(FATAL) << "Exception thrown by onStopRequested(): "
+                 << folly::exceptionStr(result.exception());
+    }
   }
 }
 
@@ -948,7 +1188,12 @@ void ThriftServer::stopCPUWorkers() {
   // finish, then stop the task queue workers. Have to do this now, so
   // there aren't tasks completing and trying to write to i/o thread
   // workers after we've stopped the i/o workers.
-  threadManager_->join();
+  if (useResourcePools()) {
+    resourcePoolSet().stopAndJoin();
+  } else {
+    CHECK(threadManager_);
+    threadManager_->join();
+  }
 #if FOLLY_HAS_COROUTINES
   // Wait for tasks running on AsyncScope to join
   folly::coro::blockingWait(asyncScope_->joinAsync());
@@ -1097,6 +1342,39 @@ void ThriftServer::replaceShutdownSocketSet(
   wShutdownSocketSet_ = newSSS;
 }
 
+folly::SemiFuture<ThriftServer::ServerIOMemory>
+ThriftServer::getUsedIOMemory() {
+  // WorkerIOMemory looks the same as the server, except they are unaggregated
+  using WorkerIOMemory = ServerIOMemory;
+  std::vector<folly::SemiFuture<WorkerIOMemory>> tasks;
+
+  forEachWorker([&tasks](wangle::Acceptor* acceptor) {
+    auto worker = dynamic_cast<Cpp2Worker*>(acceptor);
+    if (!worker) {
+      return;
+    }
+    auto fut = folly::via(worker->getEventBase(), [worker]() {
+      auto& ingressMemTracker = worker->getIngressMemoryTracker();
+      auto& egressMemTracker = worker->getEgressMemoryTracker();
+      return WorkerIOMemory{
+          ingressMemTracker.getUsage(), egressMemTracker.getUsage()};
+    });
+    tasks.emplace_back(std::move(fut));
+  });
+
+  return folly::collect(tasks.begin(), tasks.end())
+      .deferValue(
+          [](std::vector<WorkerIOMemory> workerIOMems) -> ServerIOMemory {
+            ServerIOMemory ret{0, 0};
+            // Sum all ingress and egress usages
+            for (const auto& workerIOMem : workerIOMems) {
+              ret.ingress += workerIOMem.ingress;
+              ret.egress += workerIOMem.egress;
+            }
+            return ret;
+          });
+}
+
 folly::SemiFuture<ThriftServer::ServerSnapshot> ThriftServer::getServerSnapshot(
     const SnapshotOptions& options) {
   // WorkerSnapshots look the same as the server, except they are unaggregated
@@ -1148,10 +1426,16 @@ folly::SemiFuture<ThriftServer::ServerSnapshot> ThriftServer::getServerSnapshot(
               }
             });
           }
+
+          ServerIOMemory serverIOMemory;
+          serverIOMemory.ingress = worker->getIngressMemoryTracker().getUsage();
+          serverIOMemory.egress = worker->getEgressMemoryTracker().getUsage();
+
           return WorkerSnapshot{
               worker->getRequestsRegistry()->getRequestCounter().get(),
               std::move(requestSnapshots),
-              std::move(connectionSnapshots)};
+              std::move(connectionSnapshots),
+              std::move(serverIOMemory)};
         });
     tasks.emplace_back(std::move(fut));
   });
@@ -1161,7 +1445,7 @@ folly::SemiFuture<ThriftServer::ServerSnapshot> ThriftServer::getServerSnapshot(
           [](std::vector<WorkerSnapshot> workerSnapshots) -> ServerSnapshot {
             ServerSnapshot ret{};
 
-            // Sum all request and connection counts
+            // Sum all request and connection counts and memory usages
             size_t numRequests = 0;
             size_t numConnections = 0;
             for (const auto& workerSnapshot : workerSnapshots) {
@@ -1173,10 +1457,12 @@ folly::SemiFuture<ThriftServer::ServerSnapshot> ThriftServer::getServerSnapshot(
               }
               numRequests += workerSnapshot.requests.size();
               numConnections += workerSnapshot.connections.size();
+              ret.memory.ingress += workerSnapshot.memory.ingress;
+              ret.memory.egress += workerSnapshot.memory.egress;
             }
 
-            // Move all RequestSnapshots and ConnectionSnapshots to
-            // ServerSnapshot
+            // Move all RequestSnapshots, ServerIOMemory and ConnectionSnapshots
+            // to ServerSnapshot
             ret.requests.reserve(numRequests);
             ret.connections.reserve(numConnections);
             for (auto& workerSnapshot : workerSnapshots) {
@@ -1234,7 +1520,23 @@ folly::observer::Observer<bool> ThriftServer::enableStopTLS() {
 }
 
 folly::observer::CallbackHandle ThriftServer::getSSLCallbackHandle() {
-  return sslContextObserver_->addCallback([&](auto ssl) {
+  auto originalPid = folly::get_cached_pid();
+
+  return sslContextObserver_->addCallback([&, originalPid](auto ssl) {
+    // Because we are posting to an EventBase, we need to ensure that this
+    // observer callback is not executing on a fork()'d child.
+    //
+    // The scenario this can happen is if:
+    //  1) The FlagsBackend observer implementation persists in the child. (e.g.
+    //     a custom atfork handler reinitializes and resubscribes to updates)
+    //  2) A thrift handler fork()s (e.g. Python thrift server which
+    //     uses concurrent.futures.ProcessPoolExecutor)
+    //  3) A flag is changed that causes an update to sslContextObserver
+    if (folly::get_cached_pid() != originalPid) {
+      LOG(WARNING)
+          << "Ignoring SSLContext update triggered by observer in forked process.";
+      return;
+    }
     if (sharedSSLContextManager_) {
       sharedSSLContextManager_->updateSSLConfigAndReloadContexts(*ssl);
     } else {
@@ -1266,5 +1568,58 @@ void ThriftServer::addIOThreadPoolObserver(
   ioObserverFactories.wlock()->push_back(std::move(factory));
 }
 
+/* static */ std::shared_ptr<folly::IOThreadPoolExecutor>
+ThriftServer::createIOThreadPool() {
+  return std::make_shared<folly::IOThreadPoolExecutor>(
+      0,
+      std::make_shared<folly::NamedThreadFactory>("ThriftIO"),
+      folly::EventBaseManager::get(),
+      folly::IOThreadPoolExecutor::Options().setEnableThreadIdCollection(
+          THRIFT_FLAG(enable_io_queue_lag_detection)));
+}
+
+namespace {
+struct NewConnectionContextHolder
+    : public folly::AsyncTransport::LifecycleObserver {
+  explicit NewConnectionContextHolder(ThriftServer::NewConnectionContext c)
+      : ctx(std::move(c)) {}
+
+  void observerAttach(folly::AsyncTransport*) noexcept override {}
+  void observerDetach(folly::AsyncTransport*) noexcept override { delete this; }
+  void destroy(folly::AsyncTransport*) noexcept override { delete this; }
+  void close(folly::AsyncTransport*) noexcept override {}
+
+  ThriftServer::NewConnectionContext ctx;
+};
+} // namespace
+
+void ThriftServer::acceptConnection(
+    folly::NetworkSocket fd,
+    const folly::SocketAddress& clientAddr,
+    folly::AsyncServerSocket::AcceptCallback::AcceptInfo info,
+    std::shared_ptr<AsyncProcessorFactory> processor) {
+  this->acceptConnection(
+      fd,
+      clientAddr,
+      info,
+      new NewConnectionContextHolder(
+          ThriftServer::NewConnectionContext{processor}));
+}
+
+folly::Optional<ThriftServer::NewConnectionContext>
+ThriftServer::extractNewConnectionContext(folly::AsyncTransport& transport) {
+  if (auto sock = transport.getUnderlyingTransport<folly::AsyncSocket>()) {
+    for (auto observer : sock->getLifecycleObservers()) {
+      if (auto ctxHolder =
+              dynamic_cast<NewConnectionContextHolder*>(observer)) {
+        auto ctx = std::move(ctxHolder->ctx);
+        sock->removeLifecycleObserver(observer);
+        return ctx;
+      }
+    }
+  }
+  return folly::none;
+}
+
 } // namespace thrift
 } // namespace apache
