@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -31,6 +31,7 @@
 #include <folly/Memory.h>
 #include <folly/Singleton.h>
 #include <folly/SocketAddress.h>
+#include <folly/dynamic.h>
 #include <folly/executors/IOThreadPoolExecutor.h>
 #include <folly/experimental/PrimaryPtr.h>
 #include <folly/experimental/coro/AsyncScope.h>
@@ -50,6 +51,7 @@
 #include <thrift/lib/cpp2/async/AsyncProcessor.h>
 #include <thrift/lib/cpp2/async/HeaderServerChannel.h>
 #include <thrift/lib/cpp2/server/BaseThriftServer.h>
+#include <thrift/lib/cpp2/server/LoggingEvent.h>
 #include <thrift/lib/cpp2/server/PolledServiceHealth.h>
 #include <thrift/lib/cpp2/server/PreprocessParams.h>
 #include <thrift/lib/cpp2/server/RequestDebugLog.h>
@@ -72,6 +74,7 @@ DECLARE_string(service_identity);
 THRIFT_FLAG_DECLARE_bool(dump_snapshot_on_long_shutdown);
 THRIFT_FLAG_DECLARE_bool(alpn_allow_mismatch);
 THRIFT_FLAG_DECLARE_bool(server_check_unimplemented_extra_interfaces);
+THRIFT_FLAG_DECLARE_bool(enable_io_queue_lag_detection);
 
 namespace apache {
 namespace thrift {
@@ -151,7 +154,6 @@ class ThriftServer : public apache::thrift::BaseThriftServer,
   std::optional<folly::observer::Observer<wangle::SSLContextConfig>>
       sslContextObserver_;
   std::optional<wangle::TLSTicketKeySeeds> ticketSeeds_;
-  folly::observer::CallbackHandle getSSLCallbackHandle();
 
   std::optional<bool> reusePort_;
   std::optional<bool> enableTFO_;
@@ -162,7 +164,7 @@ class ThriftServer : public apache::thrift::BaseThriftServer,
   ThriftTlsConfig thriftConfig_;
 
   // Security negotiation settings
-  std::optional<SSLPolicy> sslPolicy_;
+  SSLPolicy sslPolicy_{SSLPolicy::REQUIRED};
   bool strictSSL_ = false;
   // whether we allow plaintext connections from loopback in REQUIRED mode
   bool allowPlaintextOnLoopback_ = false;
@@ -205,10 +207,12 @@ class ThriftServer : public apache::thrift::BaseThriftServer,
   //! Manager of per-thread EventBase objects.
   folly::EventBaseManager* eventBaseManager_ = folly::EventBaseManager::get();
 
+  // Creates the default ThriftIO IOThreadPoolExecutor
+  static std::shared_ptr<folly::IOThreadPoolExecutor> createIOThreadPool();
+
   //! IO thread pool. Drives Cpp2Workers.
   std::shared_ptr<folly::IOThreadPoolExecutor> ioThreadPool_ =
-      std::make_shared<folly::IOThreadPoolExecutor>(
-          0, std::make_shared<folly::NamedThreadFactory>("ThriftIO"));
+      createIOThreadPool();
 
   /**
    * The speed for adjusting connection accept rate.
@@ -255,7 +259,7 @@ class ThriftServer : public apache::thrift::BaseThriftServer,
   void stopAcceptingAndJoinOutstandingRequests();
 
   void callOnStartServing();
-  void callOnStopServing();
+  void callOnStopRequested();
 
   void ensureDecoratedProcessorFactoryInitialized();
 
@@ -279,6 +283,9 @@ class ThriftServer : public apache::thrift::BaseThriftServer,
 
   bool quickExitOnShutdownTimeout_ = false;
 
+ protected:
+  folly::observer::CallbackHandle getSSLCallbackHandle();
+
  public:
   /**
    * The goal of this enum is to capture every state the server goes through in
@@ -320,12 +327,12 @@ class ThriftServer : public apache::thrift::BaseThriftServer,
      */
     PRE_STOPPING,
     /**
-     * The server is about to stop and ServiceHandler::semifuture_onStopServing
-     * hooks are still executing.
+     * The server is about to stop and
+     * ServiceHandler::semifuture_onStopRequested hooks are still executing.
      */
     STOPPING,
     /**
-     * ServiceHandler::semifuture_onStopServing hooks have finished executing.
+     * ServiceHandler::semifuture_onStopRequested hooks have finished executing.
      * Outstanding requests are being joined. New requests are rejected.
      */
     DRAINING_UNTIL_STOPPED,
@@ -397,11 +404,11 @@ class ThriftServer : public apache::thrift::BaseThriftServer,
    * Collects service handlers of the current service of a specific type.
    */
   template <
-      typename TServiceHandler = ServiceHandler,
-      typename =
-          std::enable_if_t<std::is_base_of_v<ServiceHandler, TServiceHandler>>>
+      typename TServiceHandler = ServiceHandlerBase,
+      typename = std::enable_if_t<
+          std::is_base_of_v<ServiceHandlerBase, TServiceHandler>>>
   std::vector<TServiceHandler*> collectServiceHandlers() const {
-    if constexpr (std::is_same_v<TServiceHandler, ServiceHandler>) {
+    if constexpr (std::is_same_v<TServiceHandler, ServiceHandlerBase>) {
       return getDecoratedProcessorFactory().getServiceHandlers();
     }
     std::vector<TServiceHandler*> matchedServiceHandlers;
@@ -478,7 +485,7 @@ class ThriftServer : public apache::thrift::BaseThriftServer,
    */
   using IOObserverFactory =
       folly::Function<std::shared_ptr<folly::ThreadPoolExecutor::Observer>(
-          std::string) const>;
+          std::string, folly::WorkerProvider*) const>;
   static void addIOThreadPoolObserver(IOObserverFactory factory);
 
   /**
@@ -717,10 +724,7 @@ class ThriftServer : public apache::thrift::BaseThriftServer,
     return const_cast<ThriftServer*>(this)->getEventBaseManager();
   }
 
-  SSLPolicy getSSLPolicy() const;
-
-  // Convenience method to check if SSLPolicy is explicitly set
-  bool isSSLPolicySet() const { return sslPolicy_.has_value(); }
+  SSLPolicy getSSLPolicy() const { return sslPolicy_; }
 
   void setSSLPolicy(SSLPolicy policy) { sslPolicy_ = policy; }
 
@@ -850,6 +854,19 @@ class ThriftServer : public apache::thrift::BaseThriftServer,
    */
   void setupThreadManager();
 
+  /**
+   * Apply various runtime checks to determine whether we can use resource pools
+   * in this service
+   */
+  void runtimeResourcePoolsChecks();
+
+  /**
+   * Ensure that this Thrift Server has ResourcePools set up. If there is
+   * already a non-empty ResourcePoolSet, nothing will be done. Otherwise, the
+   * default setup of ResourcePools will be created.
+   */
+  void ensureResourcePools();
+
   /**
    * Kill the workers and wait for listeners to quit
    */
@@ -869,6 +886,37 @@ class ThriftServer : public apache::thrift::BaseThriftServer,
    */
   void stopDuplex(std::shared_ptr<ThriftServer> thisServer);
 
+  void setEnableCodel(
+      bool enableCodel,
+      AttributeSource source = AttributeSource::OVERRIDE,
+      DynamicAttributeTag = DynamicAttributeTag{}) override final {
+    THRIFT_SERVER_EVENT(call.setEnableCodel).log(*this, [enableCodel]() {
+      return folly::dynamic::object("enableCodel", enableCodel);
+    });
+    BaseThriftServer::setEnableCodel(enableCodel, source);
+  }
+
+  void setQueueTimeout(
+      std::chrono::milliseconds timeout,
+      AttributeSource source = AttributeSource::OVERRIDE,
+      DynamicAttributeTag = DynamicAttributeTag{}) override final {
+    THRIFT_SERVER_EVENT(call.setQueueTimeout).log(*this, [timeout]() {
+      return folly::dynamic::object("timeout_ms", timeout.count());
+    });
+    BaseThriftServer::setQueueTimeout(timeout, source);
+  }
+
+  [[deprecated("Use setPreprocess instead")]] void setIsOverloaded(
+      IsOverloadedFunc isOverloaded) override final {
+    THRIFT_SERVER_EVENT(call.setIsOverloaded).log(*this);
+    BaseThriftServer::setIsOverloaded(std::move(isOverloaded));
+  }
+
+  void setPreprocess(PreprocessFunc preprocess) override final {
+    THRIFT_SERVER_EVENT(call.setPreprocess).log(*this);
+    BaseThriftServer::setPreprocess(std::move(preprocess));
+  }
+
   /**
    * One stop solution:
    *
@@ -1066,11 +1114,11 @@ class ThriftServer : public apache::thrift::BaseThriftServer,
       auto requestPayload = rocket::unpack<RequestPayload>(stub.clonePayload());
       payload_ = std::move(*requestPayload->payload);
       auto& metadata = requestPayload->metadata;
-      if (metadata.otherMetadata_ref()) {
-        headers_ = std::move(*requestPayload->metadata.otherMetadata_ref());
+      if (metadata.otherMetadata()) {
+        headers_ = std::move(*requestPayload->metadata.otherMetadata());
       }
-      clientId_ = metadata.clientId_ref().to_optional();
-      serviceTraceMeta_ = metadata.serviceTraceMeta_ref().to_optional();
+      clientId_ = metadata.clientId().to_optional();
+      serviceTraceMeta_ = metadata.serviceTraceMeta().to_optional();
       auto req = stub.getRequest();
       DCHECK(
           req != nullptr || finishedTimestamp_.time_since_epoch().count() != 0);
@@ -1134,6 +1182,16 @@ class ThriftServer : public apache::thrift::BaseThriftServer,
     bool startedProcessing_;
   };
 
+  struct ServerIOMemory {
+    size_t ingress;
+    size_t egress;
+  };
+  /**
+   * Returns structure highlighting the ingress and egress memory usage in
+   * thrift server
+   */
+  folly::SemiFuture<ServerIOMemory> getUsedIOMemory();
+
   struct ConnectionSnapshot {
     size_t numActiveRequests{0};
     size_t numPendingWrites{0};
@@ -1146,6 +1204,7 @@ class ThriftServer : public apache::thrift::BaseThriftServer,
     RecentRequestCounter::Values recentCounters;
     RequestSnapshots requests;
     ConnectionSnapshots connections;
+    ServerIOMemory memory;
   };
   struct SnapshotOptions {
     std::chrono::microseconds connectionsAgeMax;
@@ -1184,6 +1243,22 @@ class ThriftServer : public apache::thrift::BaseThriftServer,
      */
     UNIMPLEMENTED,
   };
+
+  struct NewConnectionContext {
+    std::shared_ptr<AsyncProcessorFactory> processorFactory;
+  };
+
+  static folly::Optional<NewConnectionContext> extractNewConnectionContext(
+      folly::AsyncTransport& sock);
+
+  void acceptConnection(
+      folly::NetworkSocket fd,
+      const folly::SocketAddress& clientAddr,
+      folly::AsyncServerSocket::AcceptCallback::AcceptInfo info,
+      std::shared_ptr<AsyncProcessorFactory> processor);
+
+ private:
+  using ServerBootstrap::acceptConnection;
 };
 
 template <typename AcceptorClass, typename SharedSSLContextManagerClass>
