@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -21,6 +21,7 @@
 #include <folly/executors/GlobalExecutor.h>
 #include <folly/portability/GMock.h>
 #include <folly/portability/GTest.h>
+#include <folly/test/TestUtils.h>
 
 #include <thrift/lib/cpp2/GeneratedCodeHelper.h>
 #include <thrift/lib/cpp2/PluggableFunction.h>
@@ -32,6 +33,7 @@
 #include <thrift/lib/cpp2/server/Cpp2ConnContext.h>
 #include <thrift/lib/cpp2/server/Cpp2Worker.h>
 #include <thrift/lib/cpp2/server/MonitoringServerInterface.h>
+#include <thrift/lib/cpp2/server/ServerFlags.h>
 #include <thrift/lib/cpp2/server/ThriftServer.h>
 #include <thrift/lib/cpp2/transport/http2/common/HTTP2RoutingHandler.h>
 #include <thrift/lib/cpp2/transport/rocket/server/RocketRoutingHandler.h>
@@ -49,16 +51,13 @@ namespace apache::thrift::test {
 
 using MethodMetadata = AsyncProcessorFactory::MethodMetadata;
 using MethodMetadataMap = AsyncProcessorFactory::MethodMetadataMap;
-using WildcardMethodMetadata = AsyncProcessorFactory::WildcardMethodMetadata;
-using WildcardMethodMetadataMap =
-    AsyncProcessorFactory::WildcardMethodMetadataMap;
 using CreateMethodMetadataResult =
     AsyncProcessorFactory::CreateMethodMetadataResult;
 
 using TransportType = Cpp2ConnContext::TransportType;
 
 namespace {
-class Child : public ChildSvIf {
+class ChildHandler : public ChildSvIf {
   MOCK_METHOD(std::unique_ptr<InteractionIf>, createInteraction, ());
 
   int parentMethod1() override { return 42; }
@@ -98,7 +97,7 @@ TEST(AsyncProcessorMetadataTest, ParentMetadata) {
 }
 
 TEST(AsyncProcessorMetadataTest, ChildMetadata) {
-  Child service;
+  ChildHandler service;
   auto createMethodMetadataResult = service.createMethodMetadata();
   auto& metadataMap = expectMethodMetadataMap(createMethodMetadataResult);
 
@@ -113,17 +112,17 @@ TEST(AsyncProcessorMetadataTest, ChildMetadata) {
 }
 
 namespace {
-class ChildWithMetadata : public Child {
+class ChildHandlerWithMetadata : public ChildHandler {
  public:
   using MetadataFactory = std::function<CreateMethodMetadataResult(
       CreateMethodMetadataResult defaultResult)>;
 
-  explicit ChildWithMetadata(MetadataFactory metadataFactory)
+  explicit ChildHandlerWithMetadata(MetadataFactory metadataFactory)
       : metadataFactory_(std::move(metadataFactory)) {}
 
  private:
   CreateMethodMetadataResult createMethodMetadata() override {
-    return metadataFactory_(Child::createMethodMetadata());
+    return metadataFactory_(ChildHandler::createMethodMetadata());
   }
 
   MetadataFactory metadataFactory_;
@@ -156,8 +155,12 @@ class AsyncProcessorMethodResolutionTestP
               return HeaderClientChannel::newChannel(std::move(socket));
             case TransportType::ROCKET:
               return RocketClientChannel::newChannel(std::move(socket));
-            case TransportType::HTTP2:
-              return HTTPClientChannel::newHTTP2Channel(std::move(socket));
+            case TransportType::HTTP2: {
+              auto channel =
+                  HTTPClientChannel::newHTTP2Channel(std::move(socket));
+              channel->setProtocolId(protocol::T_COMPACT_PROTOCOL);
+              return channel;
+            }
             default:
               throw std::logic_error{"Unreachable!"};
           }
@@ -167,7 +170,7 @@ class AsyncProcessorMethodResolutionTestP
 } // namespace
 
 TEST_P(AsyncProcessorMethodResolutionTestP, CreateMethodMetadataNotSupported) {
-  auto service = std::make_shared<ChildWithMetadata>(
+  auto service = std::make_shared<ChildHandlerWithMetadata>(
       [](auto&&) -> CreateMethodMetadataResult { return {}; });
   auto runner = makeServer(service);
   auto client = makeClientFor<ChildAsyncClient>(*runner);
@@ -178,7 +181,7 @@ TEST_P(AsyncProcessorMethodResolutionTestP, CreateMethodMetadataNotSupported) {
 }
 
 TEST_P(AsyncProcessorMethodResolutionTestP, EmptyMap) {
-  auto service = std::make_shared<ChildWithMetadata>(
+  auto service = std::make_shared<ChildHandlerWithMetadata>(
       [](auto&&) -> MethodMetadataMap { return {}; });
   auto runner = makeServer(service);
   auto client = makeClientFor<ChildAsyncClient>(*runner);
@@ -192,16 +195,19 @@ TEST_P(AsyncProcessorMethodResolutionTestP, MistypedMetadataDeathTest) {
     // Opt-mode causes UB instead of dying
     return;
   }
+  folly::SingletonVault::singleton()->destroyInstances();
+  folly::SingletonVault::singleton()->reenableInstances();
   auto runTest = [&](auto&& callback) {
-    auto service = std::make_shared<ChildWithMetadata>([](auto defaultResult) {
-      MethodMetadataMap result;
-      const auto& defaultMap = expectMethodMetadataMap(defaultResult);
-      for (auto& [name, _] : defaultMap) {
-        class DummyMethodMetadata : public MethodMetadata {};
-        result.emplace(name, std::make_shared<DummyMethodMetadata>());
-      }
-      return result;
-    });
+    auto service =
+        std::make_shared<ChildHandlerWithMetadata>([](auto defaultResult) {
+          MethodMetadataMap result;
+          const auto& defaultMap = expectMethodMetadataMap(defaultResult);
+          for (auto& [name, _] : defaultMap) {
+            class DummyMethodMetadata : public MethodMetadata {};
+            result.emplace(name, std::make_shared<DummyMethodMetadata>());
+          }
+          return result;
+        });
     auto runner = makeServer(service);
     callback(makeClientFor<ChildAsyncClient>(*runner));
   };
@@ -223,7 +229,7 @@ TEST_P(AsyncProcessorMethodResolutionTestP, ParentMapDeathTest) {
   // might contain all the function pointers we need.
   EXPECT_DEATH(
       [&] {
-        auto service = std::make_shared<ChildWithMetadata>(
+        auto service = std::make_shared<ChildHandlerWithMetadata>(
             [](auto&&) { return ParentSvIf{}.createMethodMetadata(); });
         auto runner = makeServer(service);
         auto client = makeClientFor<ChildAsyncClient>(*runner);
@@ -234,9 +240,9 @@ TEST_P(AsyncProcessorMethodResolutionTestP, ParentMapDeathTest) {
 
 TEST_P(AsyncProcessorMethodResolutionTestP, Wildcard) {
   auto runTest = [&](auto&& callback) {
-    class ChildImpl : public Child {
+    class ChildImpl : public ChildHandler {
       CreateMethodMetadataResult createMethodMetadata() override {
-        auto defaultResult = Child::createMethodMetadata();
+        auto defaultResult = ChildHandler::createMethodMetadata();
         auto& defaultMap = expectMethodMetadataMap(defaultResult);
         MethodMetadataMap knownMethods;
         // swap out for another method to make sure we are using this map
@@ -276,6 +282,28 @@ TEST_P(AsyncProcessorMethodResolutionTestP, Wildcard) {
                 eb,
                 tm);
           }
+
+          void executeRequest(
+              ServerRequest&& request,
+              const AsyncProcessorFactory::MethodMetadata& methodMetadata)
+              override {
+            if (dynamic_cast<const WildcardMethodMetadata*>(&methodMetadata)) {
+              // Instead of crashing when receiving a wildcard method metadata,
+              // return an error so we can check that the correct metadata was
+              // propagated.
+              auto eb = detail::ServerRequestHelper::eventBase(request);
+              eb->runInEventBaseThread(
+                  [request = std::move(request)]() mutable {
+                    request.request()->sendErrorWrapped(
+                        folly::make_exception_wrapper<TApplicationException>(
+                            TApplicationException::UNKNOWN_METHOD, ""),
+                        "");
+                  });
+              return;
+            }
+            ChildAsyncProcessor::executeRequest(
+                std::move(request), methodMetadata);
+          }
         };
         return std::make_unique<ProcessorImpl>(this);
       }
@@ -333,12 +361,13 @@ TEST_P(
   class Control : public DummyControlSvIf, public ControlServerInterface {
     std::int64_t getOption() override { return 42; }
   };
-  auto service = std::make_shared<ChildWithMetadata>([](auto defaultResult) {
-    MethodMetadataMap result;
-    auto& defaultMap = expectMethodMetadataMap(defaultResult);
-    result.emplace("parentMethod1", std::move(defaultMap["parentMethod1"]));
-    return result;
-  });
+  auto service =
+      std::make_shared<ChildHandlerWithMetadata>([](auto defaultResult) {
+        MethodMetadataMap result;
+        auto& defaultMap = expectMethodMetadataMap(defaultResult);
+        result.emplace("parentMethod1", std::move(defaultMap["parentMethod1"]));
+        return result;
+      });
   auto runner = makeServer(service, [&](ThriftServer& server) {
     server.setStatusInterface(std::make_shared<Status>());
     server.setMonitoringInterface(std::make_shared<Monitor>());
@@ -368,13 +397,13 @@ TEST_P(
 TEST_P(
     AsyncProcessorMethodResolutionTestP,
     MonitoringMethodMultiplexingCollision) {
-  class ChildMonitor : public Child, public MonitoringServerInterface {
+  class ChildMonitor : public ChildHandler, public MonitoringServerInterface {
     void childMethod2(std::string& result) override {
       result = "hello from Monitor";
     }
   };
   auto runner =
-      makeServer(std::make_shared<Child>(), [&](ThriftServer& server) {
+      makeServer(std::make_shared<ChildHandler>(), [&](ThriftServer& server) {
         server.setMonitoringInterface(std::make_shared<ChildMonitor>());
       });
 
@@ -399,10 +428,10 @@ TEST_P(
   class Control : public DummyControlSvIf, public ControlServerInterface {
     std::int64_t getOption() override { return 42; }
   };
-  class ChildWithWildcard : public ChildWithMetadata {
+  class ChildHandlerWithWildcard : public ChildHandlerWithMetadata {
    public:
-    ChildWithWildcard()
-        : ChildWithMetadata([](auto defaultResult) {
+    ChildHandlerWithWildcard()
+        : ChildHandlerWithMetadata([](auto defaultResult) {
             auto& defaultMap = expectMethodMetadataMap(defaultResult);
             return WildcardMethodMetadataMap{std::move(defaultMap)};
           }) {}
@@ -444,6 +473,22 @@ TEST_P(
               tm);
         }
 
+        void executeRequest(
+            ServerRequest&& request,
+            const AsyncProcessorFactory::MethodMetadata& methodMetadata)
+            override {
+          if (AsyncProcessorHelper::isWildcardMethodMetadata(methodMetadata)) {
+            auto eb = detail::ServerRequestHelper::eventBase(request);
+            eb->runInEventBaseThread([request = std::move(request)]() mutable {
+              AsyncProcessorHelper::sendUnknownMethodError(
+                  detail::ServerRequestHelper::request(std::move(request)),
+                  "someUnknownMethod");
+            });
+            return;
+          }
+          processor_->executeRequest(std::move(request), methodMetadata);
+        }
+
         explicit Processor(std::unique_ptr<AsyncProcessor> processor)
             : processor_(std::move(processor)) {}
 
@@ -451,10 +496,11 @@ TEST_P(
         std::unique_ptr<AsyncProcessor> processor_;
       };
 
-      return std::make_unique<Processor>(ChildWithMetadata::getProcessor());
+      return std::make_unique<Processor>(
+          ChildHandlerWithMetadata::getProcessor());
     }
   };
-  auto service = std::make_shared<ChildWithWildcard>();
+  auto service = std::make_shared<ChildHandlerWithWildcard>();
   auto runner = makeServer(service, [&](ThriftServer& server) {
     server.setStatusInterface(std::make_shared<Status>());
     server.setMonitoringInterface(std::make_shared<Monitor>());
@@ -532,7 +578,7 @@ TEST(AsyncProcessorMethodResolutionTest, MultipleService) {
     }
   };
   ScopedServerInterfaceThread runner{
-      std::make_shared<Child>(), [&](ThriftServer& server) {
+      std::make_shared<ChildHandler>(), [&](ThriftServer& server) {
         server.setMonitoringInterface(std::make_shared<Monitor>());
       }};
 
