@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -35,10 +35,7 @@
 
 using namespace apache::thrift;
 using namespace apache::thrift::concurrency;
-using namespace apache::thrift::protocol;
 using namespace apache::thrift::test;
-using namespace apache::thrift::transport;
-using namespace apache::thrift::util;
 using std::string;
 
 class TestServiceHandler : public TestServiceSvIf {
@@ -64,14 +61,12 @@ std::unique_ptr<HTTP2RoutingHandler> createHTTP2RoutingHandler(
 
 std::shared_ptr<BaseThriftServer> createHttpServer() {
   auto handler = std::make_shared<TestServiceHandler>();
-  auto tm = ThreadManager::newSimpleThreadManager(1);
-  tm->threadFactory(std::make_shared<PosixThreadFactory>());
-  tm->start();
   auto server = std::make_shared<ThriftServer>();
   server->setAddress({"::1", 0});
   server->setInterface(handler);
   server->setNumIOWorkerThreads(1);
-  server->setThreadManager(tm);
+  server->setThreadFactory(std::make_shared<PosixThreadFactory>());
+  server->setThreadManagerType(BaseThriftServer::ThreadManagerType::SIMPLE);
   server->addRoutingHandler(createHTTP2RoutingHandler(*server));
   return server;
 }
