@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -20,6 +20,7 @@
 #include <thread>
 
 #include <folly/ThreadLocal.h>
+#include <folly/executors/CPUThreadPoolExecutor.h>
 #include <folly/experimental/coro/BlockingWait.h>
 #include <folly/io/async/AsyncSocket.h>
 #include <folly/io/async/EventBase.h>
@@ -36,7 +37,10 @@
 #include <thrift/lib/cpp2/async/RequestChannel.h>
 #include <thrift/lib/cpp2/async/RocketClientChannel.h>
 #include <thrift/lib/cpp2/server/Cpp2Worker.h>
+#include <thrift/lib/cpp2/server/ParallelConcurrencyController.h>
 #include <thrift/lib/cpp2/server/RequestDebugLog.h>
+#include <thrift/lib/cpp2/server/RoundRobinRequestPile.h>
+#include <thrift/lib/cpp2/server/ServerFlags.h>
 #include <thrift/lib/cpp2/server/ServerInstrumentation.h>
 #include <thrift/lib/cpp2/server/ThriftServer.h>
 #include <thrift/lib/cpp2/test/gen-cpp2/DebugTestService.h>
@@ -49,7 +53,6 @@ using namespace apache::thrift;
 using namespace apache::thrift::concurrency;
 using namespace apache::thrift::test;
 using namespace std::literals::chrono_literals;
-using namespace ::testing;
 
 class InstrumentationRequestPayload : public folly::RequestData {
  public:
@@ -73,31 +76,18 @@ class InstrumentationTestProcessor
   explicit InstrumentationTestProcessor(InstrumentationTestServiceSvIf* iface)
       : InstrumentationTestServiceAsyncProcessor(iface) {}
 
-  /**
-   * Intercepts and clones incoming payload buffers, then passes them down to
-   * method handlers.
-   */
-  void processSerializedCompressedRequestWithMetadata(
-      ResponseChannelRequest::UniquePtr req,
-      apache::thrift::SerializedCompressedRequest&& serializedRequest,
-      const MethodMetadata& methodMetadata,
-      apache::thrift::protocol::PROTOCOL_TYPES protType,
-      apache::thrift::Cpp2RequestContext* context,
-      folly::EventBase* eb,
-      apache::thrift::concurrency::ThreadManager* tm) override {
+  void executeRequest(
+      ServerRequest&& request, const MethodMetadata& methodMetadata) override {
     folly::RequestContext::get()->setContextData(
         InstrumentationRequestPayload::getRequestToken(),
         std::make_unique<InstrumentationRequestPayload>(
-            serializedRequest.clone().uncompress().buffer));
-    InstrumentationTestServiceAsyncProcessor::
-        processSerializedCompressedRequestWithMetadata(
-            std::move(req),
-            std::move(serializedRequest),
-            methodMetadata,
-            protType,
-            context,
-            eb,
-            tm);
+            apache::thrift::detail::ServerRequestHelper::compressedRequest(
+                request)
+                .clone()
+                .uncompress()
+                .buffer));
+    InstrumentationTestServiceAsyncProcessor::executeRequest(
+        std::move(request), methodMetadata);
   }
 };
 
@@ -219,8 +209,30 @@ class DebuggingFrameHandler : public rocket::SetupFrameHandler {
       const RequestSetupMetadata& meta) override {
     if (meta.interfaceKind_ref().has_value() &&
         meta.interfaceKind_ref().value() == InterfaceKind::DEBUGGING) {
-      return rocket::ProcessorInfo{
-          debug_, tm_, origServer_, reqRegistry_.get()};
+      if (apache::thrift::useResourcePools()) {
+        auto asyncHandle = apache::thrift::ResourcePoolHandle::defaultAsync();
+        // Ensure there is an async handler set up in the resource pool.
+        if (!resourcePoolSet_.hasResourcePool(asyncHandle)) {
+          auto executor = std::make_shared<folly::CPUThreadPoolExecutor>(
+              1, std::make_shared<folly::NamedThreadFactory>("DebugInterface"));
+          apache::thrift::RoundRobinRequestPile::Options options;
+          auto requestPile =
+              std::make_unique<apache::thrift::RoundRobinRequestPile>(options);
+          auto concurrencyController =
+              std::make_unique<apache::thrift::ParallelConcurrencyController>(
+                  *requestPile.get(), *executor.get());
+          resourcePoolSet_.setResourcePool(
+              asyncHandle,
+              std::move(requestPile),
+              executor,
+              std::move(concurrencyController));
+        }
+        return rocket::ProcessorInfo(
+            debug_, nullptr, origServer_, reqRegistry_.get());
+      } else {
+        return rocket::ProcessorInfo(
+            debug_, tm_, origServer_, reqRegistry_.get());
+      }
     }
     return std::nullopt;
   }
@@ -230,6 +242,7 @@ class DebuggingFrameHandler : public rocket::SetupFrameHandler {
   DebugInterface debug_;
   std::shared_ptr<ThreadManager> tm_;
   folly::ThreadLocal<RequestsRegistry> reqRegistry_;
+  apache::thrift::ResourcePoolSet resourcePoolSet_;
 };
 
 namespace {
@@ -424,6 +437,8 @@ TEST_F(RequestInstrumentationTest, threadSnapshot) {
 }
 
 TEST_F(RequestInstrumentationTest, PendingTaskCount) {
+  THRIFT_OMIT_TEST_WITH_RESOURCE_POOLS(
+      /* pendingTaskCounts in thread manager don't apply to resource pools */);
   auto client = makeRocketClient();
   folly::Baton baton;
   folly::Baton baton2;
@@ -431,24 +446,37 @@ TEST_F(RequestInstrumentationTest, PendingTaskCount) {
     ti->getThreadManager()->add([] {});
     ti->getThreadManager()->add([] {});
     ti->getThreadManager()->add([] {});
+    auto& baton2_ = baton2; // avoid data-race on lambda storage in wait below
     baton.post();
-    baton2.wait();
+    baton2_.wait();
   });
   auto firstReq = client->semifuture_runCallback();
   baton.wait();
-  EXPECT_EQ(3, thriftServer()->getThreadManager()->pendingTaskCount());
-  EXPECT_EQ(0, thriftServer()->getThreadManager()->pendingUpstreamTaskCount());
+  EXPECT_EQ(
+      3, thriftServer()->getThreadManager_deprecated()->pendingTaskCount());
+  EXPECT_EQ(
+      0,
+      thriftServer()
+          ->getThreadManager_deprecated()
+          ->pendingUpstreamTaskCount());
 
   handler()->setCallback([&](TestInterface*) {});
   auto secondReq = client->semifuture_runCallback();
 
   auto expire = std::chrono::steady_clock::now() + 5s;
-  while (thriftServer()->getThreadManager()->pendingUpstreamTaskCount() < 1 &&
+  while (thriftServer()
+                 ->getThreadManager_deprecated()
+                 ->pendingUpstreamTaskCount() < 1 &&
          std::chrono::steady_clock::now() < expire) {
     std::this_thread::yield();
   }
-  EXPECT_EQ(4, thriftServer()->getThreadManager()->pendingTaskCount());
-  EXPECT_EQ(1, thriftServer()->getThreadManager()->pendingUpstreamTaskCount());
+  EXPECT_EQ(
+      4, thriftServer()->getThreadManager_deprecated()->pendingTaskCount());
+  EXPECT_EQ(
+      1,
+      thriftServer()
+          ->getThreadManager_deprecated()
+          ->pendingUpstreamTaskCount());
 
   baton2.post();
   std::move(firstReq).get();
@@ -487,8 +515,11 @@ TEST_F(RequestInstrumentationTest, debugInterfaceTest) {
     client->semifuture_sendRequest();
   }
 
-  auto echoed = debugClient->semifuture_echo("echome").get();
-  EXPECT_TRUE(folly::StringPiece(echoed).startsWith("echome:DebugInterface-"));
+  if (!useResourcePools()) {
+    auto echoed = debugClient->semifuture_echo("echome").get();
+    EXPECT_TRUE(
+        folly::StringPiece(echoed).startsWith("echome:DebugInterface-"));
+  }
 
   for (auto& reqSnapshot : getRequestSnapshots(2 * reqNum)) {
     auto methodName = reqSnapshot.getMethodName();
@@ -753,8 +784,9 @@ TEST_F(
 
   // "started processing" request
   handler()->setCallback([&](TestInterface*) {
+    auto& baton3_ = baton3; // avoid data-race on lambda storage in wait below
     baton2.post();
-    baton3.wait();
+    baton3_.wait();
   });
   auto req2 = client->semifuture_runCallback();
   baton2.wait();
@@ -834,11 +866,11 @@ TEST(ThriftServerDeathTest, getSnapshotOnServerShutdown) {
           // connection and one without any connections
           server.setNumIOWorkerThreads(2);
           // We need at least 2 cpu threads for the test
-          auto tm = ThreadManager::newSimpleThreadManager(2);
-          tm->threadFactory(std::make_shared<PosixThreadFactory>(
+          server.setNumCPUWorkerThreads(2);
+          server.setThreadManagerType(
+              apache::thrift::BaseThriftServer::ThreadManagerType::SIMPLE);
+          server.setThreadFactory(std::make_shared<PosixThreadFactory>(
               PosixThreadFactory::ATTACHED));
-          tm->start();
-          server.setThreadManager(tm);
           server.setWorkersJoinTimeout(1s);
         });
 
@@ -851,8 +883,8 @@ TEST(ThriftServerDeathTest, getSnapshotOnServerShutdown) {
                         .getServerSnapshot()
                         .get();
                 ASSERT_EQ(snapshot.requests.size(), 1);
-                // We exit here with a specific exit code to test that this code
-                // is reached
+                // We exit here with a specific exit code to test that this
+                // code is reached
                 std::quick_exit(kExitCode);
               }),
               2s};
@@ -1066,6 +1098,7 @@ class MaxRequestsTest : public RequestInstrumentationTest,
 };
 
 TEST_P(MaxRequestsTest, Bypass) {
+  THRIFT_OMIT_TEST_WITH_RESOURCE_POOLS(/* max requests bypass not supported */);
   auto client = rocket ? makeRocketClient() : makeHeaderClient();
 
   client->semifuture_sendRequest();
@@ -1177,7 +1210,8 @@ TEST_P(TimestampsTest, QueueTimeout) {
   } catch (const TApplicationException& ex) {
     ASSERT_EQ(ex.getType(), TApplicationException::TIMEOUT);
   }
-  // callCompleted should not be called for requests which timed out the queue.
+  // callCompleted should not be called for requests which timed out the
+  // queue.
   EXPECT_FALSE(timestamps.has_value());
 }
 
