@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -18,6 +18,8 @@
 
 #include <folly/experimental/coro/BlockingWait.h>
 #include <folly/experimental/coro/Collect.h>
+#include <folly/experimental/coro/FutureUtil.h>
+#include <folly/experimental/coro/GtestHelpers.h>
 #include <folly/experimental/coro/Sleep.h>
 #include <folly/experimental/coro/Task.h>
 #include <folly/portability/GMock.h>
@@ -57,17 +59,44 @@ struct SemiCalculatorHandler : CalculatorSvIf {
   std::unique_ptr<AdditionIf> createAddition() override {
     return std::make_unique<SemiAdditionHandler>();
   }
-  std::unique_ptr<AdditionFastIf> createAdditionFast() override {
-    std::terminate();
-  }
-  std::unique_ptr<SerialAdditionIf> createSerialAddition() override {
-    std::terminate();
-  }
 
   folly::SemiFuture<int32_t> semifuture_addPrimitive(
       int32_t a, int32_t b) override {
     return a + b;
   }
+
+  TileAndResponse<AdditionIf, int> initializedAddition(int x) override {
+    auto handler = std::make_unique<SemiAdditionHandler>();
+    handler->acc_ = x;
+    return {std::move(handler), x};
+  }
+
+  struct FastAdditionHandler : CalculatorSvIf::AdditionFastIf {
+    int acc_{0};
+    void async_eb_accumulatePrimitive(
+        std::unique_ptr<HandlerCallback<void>> cb, int32_t a) override {
+      cb->getEventBase()->checkIsInEventBaseThread();
+      acc_ += a;
+      cb->exception(std::runtime_error("Not Implemented Yet"));
+    }
+    void async_eb_getPrimitive(
+        std::unique_ptr<HandlerCallback<int32_t>> cb) override {
+      cb->getEventBase()->checkIsInEventBaseThread();
+      cb->result(acc_);
+    }
+  };
+
+  TileAndResponse<AdditionFastIf, void> fastAddition() override {
+    CHECK(!getEventBase()->isInEventBaseThread());
+    return {std::make_unique<FastAdditionHandler>()};
+  }
+
+  void async_eb_veryFastAddition(
+      std::unique_ptr<HandlerCallback<TileAndResponse<AdditionFastIf, void>>>
+          cb) override {
+    cb->getEventBase()->checkIsInEventBaseThread();
+    cb->result({std::make_unique<FastAdditionHandler>()});
+  }
 };
 
 TEST(InteractionTest, TerminateUsed) {
@@ -139,6 +168,17 @@ TEST(InteractionTest, TerminateWithoutSetupPRC) {
   auto adder = client->createAddition();
 }
 
+TEST(InteractionTest, TerminateStressPRC) {
+  ScopedServerInterfaceThread runner{std::make_shared<SemiCalculatorHandler>()};
+  auto client = runner.newClient<CalculatorAsyncClient>(
+      nullptr, RocketClientChannel::newChannel);
+
+  for (int i = 0; i < 100; i++) {
+    auto adder = client->createAddition();
+    adder.semifuture_noop();
+  }
+}
+
 TEST(InteractionTest, IsDetachable) {
   ScopedServerInterfaceThread runner{std::make_shared<SemiCalculatorHandler>()};
   folly::EventBase eb;
@@ -177,12 +217,6 @@ TEST(InteractionTest, QueueTimeout) {
     std::unique_ptr<AdditionIf> createAddition() override {
       return std::make_unique<SemiAdditionHandler>();
     }
-    std::unique_ptr<AdditionFastIf> createAdditionFast() override {
-      std::terminate();
-    }
-    std::unique_ptr<SerialAdditionIf> createSerialAddition() override {
-      std::terminate();
-    }
   };
 
   ScopedServerInterfaceThread runner{std::make_shared<SlowCalculatorHandler>()};
@@ -197,6 +231,173 @@ TEST(InteractionTest, QueueTimeout) {
   EXPECT_TRUE(std::move(f2).getTry().hasException());
 }
 
+TEST(InteractionTest, OnTermination) {
+  struct TerminationHandler : CalculatorSvIf {
+    folly::coro::Baton create;
+    folly::coro::Baton terminate;
+    folly::coro::Baton terminated;
+    folly::coro::Baton destroyed;
+    struct AdditionHandler : CalculatorSvIf::AdditionIf {
+      TerminationHandler& handler;
+
+      explicit AdditionHandler(TerminationHandler& parent) : handler(parent) {}
+      ~AdditionHandler() override { handler.destroyed.post(); }
+
+      folly::coro::Task<void> co_onTermination() override {
+        co_await handler.terminate;
+        handler.terminated.post();
+      }
+
+      folly::coro::Task<void> co_accumulatePrimitive(int32_t a) override {
+        co_return;
+      }
+
+      folly::coro::Task<int32_t> co_getPrimitive() override {
+        co_await handler.terminated;
+        co_return 42;
+      }
+    };
+
+    struct AdditionFastHandler : CalculatorSvIf::AdditionFastIf {
+      TerminationHandler& handler;
+
+      explicit AdditionFastHandler(TerminationHandler& parent)
+          : handler(parent) {}
+      ~AdditionFastHandler() override { handler.destroyed.post(); }
+
+      folly::coro::Task<void> co_onTermination() override {
+        co_await handler.terminate;
+        handler.terminated.post();
+      }
+
+      void async_eb_accumulatePrimitive(
+          std::unique_ptr<HandlerCallback<void>> cb, int32_t) override {
+        cb->done();
+      }
+
+      void async_eb_getPrimitive(
+          std::unique_ptr<HandlerCallback<int32_t>> cb) override {
+        folly::coro::toSemiFuture(std::ref(handler.terminated))
+            .toUnsafeFuture()
+            .thenValue([cb = std::move(cb)](auto&&) { cb->result(42); });
+      }
+    };
+
+    std::unique_ptr<AdditionIf> createAddition() override {
+      return std::make_unique<AdditionHandler>(*this);
+    }
+    std::unique_ptr<AdditionFastIf> createAdditionFast() override {
+      return std::make_unique<AdditionFastHandler>(*this);
+    }
+
+    folly::SemiFuture<
+        apache::thrift::TileAndResponse<CalculatorSvIf::AdditionIf, void>>
+    semifuture_newAddition() override {
+      return folly::coro::toSemiFuture(std::ref(create))
+          .deferValue([&](auto&&) {
+            return apache::thrift::
+                TileAndResponse<CalculatorSvIf::AdditionIf, void>{
+                    std::make_unique<AdditionHandler>(*this)};
+          });
+    }
+
+    folly::SemiFuture<int32_t> semifuture_addPrimitive(
+        int32_t a, int32_t b) override {
+      return a + b;
+    }
+  };
+
+  auto handler = std::make_shared<TerminationHandler>();
+  auto client = makeTestClient<CalculatorAsyncClient>(handler);
+  handler->terminate.post();
+  auto checkAndReset = [](auto& baton) {
+    EXPECT_TRUE(baton.ready());
+    baton.reset();
+  };
+
+  // No active request
+  {
+    auto adder = client->createAddition();
+    adder.sync_accumulatePrimitive(0);
+  }
+  client->sync_addPrimitive(0, 0);
+  checkAndReset(handler->terminated);
+  checkAndReset(handler->destroyed);
+
+  // Active request
+  auto fut = folly::SemiFuture<int>::makeEmpty();
+  {
+    auto adder = client->createAddition();
+    fut = adder.semifuture_getPrimitive();
+    client->sync_addPrimitive(0, 0);
+  }
+  client->sync_addPrimitive(0, 0);
+  checkAndReset(handler->terminated);
+  checkAndReset(handler->destroyed);
+  std::move(fut).get();
+
+  // Client crash
+  {
+    ScopedServerInterfaceThread runner{handler};
+    folly::EventBase eb;
+    folly::AsyncSocket* sock = new folly::AsyncSocket(&eb, runner.getAddress());
+    CalculatorAsyncClient localClient(
+        RocketClientChannel::newChannel(folly::AsyncSocket::UniquePtr(sock)));
+
+    auto adder = localClient.createAddition();
+    adder.sync_accumulatePrimitive(0);
+    sock->closeNow();
+    folly::coro::blockingWait(handler->destroyed);
+    checkAndReset(handler->destroyed);
+    EXPECT_FALSE(handler->terminated.ready());
+  }
+
+  // Slow onTermination
+  {
+    handler->terminate.reset();
+    auto adder = client->createAddition();
+    adder.sync_accumulatePrimitive(0);
+  }
+  for (int i = 0; i < 10; i++) {
+    client->sync_addPrimitive(0, 0);
+    EXPECT_FALSE(handler->destroyed.ready());
+  }
+  handler->terminate.post();
+  folly::coro::blockingWait(handler->destroyed);
+  checkAndReset(handler->terminated);
+  checkAndReset(handler->destroyed);
+
+  // Slow factory
+  RpcOptions opts;
+  std::ignore = client->eager_semifuture_newAddition(opts);
+  client->sync_addPrimitive(0, 0);
+  EXPECT_FALSE(handler->terminated.ready());
+  handler->create.post();
+  folly::coro::blockingWait(handler->destroyed);
+  checkAndReset(handler->terminated);
+  checkAndReset(handler->destroyed);
+
+  // No active request, eb
+  {
+    auto adder = client->createAdditionFast();
+    adder.sync_accumulatePrimitive(0);
+  }
+  client->sync_addPrimitive(0, 0);
+  checkAndReset(handler->terminated);
+  checkAndReset(handler->destroyed);
+
+  // Active request, eb
+  {
+    auto adder = client->createAdditionFast();
+    fut = adder.semifuture_getPrimitive();
+    client->sync_addPrimitive(0, 0);
+  }
+  client->sync_addPrimitive(0, 0);
+  checkAndReset(handler->terminated);
+  checkAndReset(handler->destroyed);
+  std::move(fut).get();
+}
+
 struct CalculatorHandler : CalculatorSvIf {
   struct AdditionHandler : CalculatorSvIf::AdditionIf {
     int acc_{0};
@@ -225,12 +426,6 @@ struct CalculatorHandler : CalculatorSvIf {
   std::unique_ptr<AdditionIf> createAddition() override {
     return std::make_unique<AdditionHandler>();
   }
-  std::unique_ptr<AdditionFastIf> createAdditionFast() override {
-    std::terminate();
-  }
-  std::unique_ptr<SerialAdditionIf> createSerialAddition() override {
-    std::terminate();
-  }
 
   folly::SemiFuture<int32_t> semifuture_addPrimitive(
       int32_t a, int32_t b) override {
@@ -353,12 +548,6 @@ TEST(InteractionCodegenTest, Error) {
     std::unique_ptr<AdditionIf> createAddition() override {
       throw std::runtime_error("Plus key is broken");
     }
-    std::unique_ptr<AdditionFastIf> createAdditionFast() override {
-      std::terminate();
-    }
-    std::unique_ptr<SerialAdditionIf> createSerialAddition() override {
-      std::terminate();
-    }
   };
   ScopedServerInterfaceThread runner{
       std::make_shared<BrokenCalculatorHandler>()};
@@ -399,12 +588,6 @@ TEST(InteractionCodegenTest, MethodException) {
     std::unique_ptr<AdditionIf> createAddition() override {
       return std::make_unique<AdditionHandler>();
     }
-    std::unique_ptr<AdditionFastIf> createAdditionFast() override {
-      std::terminate();
-    }
-    std::unique_ptr<SerialAdditionIf> createSerialAddition() override {
-      std::terminate();
-    }
   };
 
   ScopedServerInterfaceThread runner{
@@ -430,12 +613,6 @@ TEST(InteractionCodegenTest, SlowConstructor) {
       b.wait();
       return std::make_unique<AdditionHandler>();
     }
-    std::unique_ptr<AdditionFastIf> createAdditionFast() override {
-      std::terminate();
-    }
-    std::unique_ptr<SerialAdditionIf> createSerialAddition() override {
-      std::terminate();
-    }
 
     folly::Baton<> b;
   };
@@ -479,12 +656,6 @@ TEST(InteractionCodegenTest, FastTermination) {
     std::unique_ptr<AdditionIf> createAddition() override {
       return std::make_unique<SlowAdditionHandler>(baton, destroyed);
     }
-    std::unique_ptr<AdditionFastIf> createAdditionFast() override {
-      std::terminate();
-    }
-    std::unique_ptr<SerialAdditionIf> createSerialAddition() override {
-      std::terminate();
-    }
 
     folly::coro::Baton baton, destroyed;
   };
@@ -525,12 +696,6 @@ TEST(InteractionCodegenTest, ClientCrashDuringInteraction) {
     std::unique_ptr<AdditionIf> createAddition() override {
       return std::make_unique<SlowAdditionHandler>(baton, destroyed);
     }
-    std::unique_ptr<AdditionFastIf> createAdditionFast() override {
-      std::terminate();
-    }
-    std::unique_ptr<SerialAdditionIf> createSerialAddition() override {
-      std::terminate();
-    }
 
     folly::coro::Baton baton, destroyed;
   };
@@ -574,19 +739,13 @@ TEST(InteractionCodegenTest, ClientCrashDuringInteractionConstructor) {
       folly::coro::blockingWait(baton);
       return std::make_unique<SlowAdditionHandler>(baton, destroyed, executed);
     }
-    std::unique_ptr<AdditionFastIf> createAdditionFast() override {
-      std::terminate();
-    }
-    std::unique_ptr<SerialAdditionIf> createSerialAddition() override {
-      std::terminate();
-    }
 
     folly::coro::Baton baton, destroyed;
     bool executed = false;
   };
   auto handler = std::make_shared<SlowCalculatorHandler>();
   ScopedServerInterfaceThread runner{handler};
-  runner.getThriftServer().getThreadManager()->addWorker();
+  runner.getThriftServer().getThreadManager_deprecated()->addWorker();
   folly::EventBase eb;
   folly::AsyncSocket* sock = new folly::AsyncSocket(&eb, runner.getAddress());
   CalculatorAsyncClient client(
@@ -613,12 +772,6 @@ TEST(InteractionCodegenTest, ReuseIdDuringConstructor) {
       }
       return std::make_unique<AdditionHandler>();
     }
-    std::unique_ptr<AdditionFastIf> createAdditionFast() override {
-      std::terminate();
-    }
-    std::unique_ptr<SerialAdditionIf> createSerialAddition() override {
-      std::terminate();
-    }
 
     folly::Baton<> b1, b2;
     bool first = true;
@@ -654,12 +807,6 @@ TEST(InteractionCodegenTest, ConstructorExceptionPropagated) {
       b.wait();
       throw std::runtime_error("Custom exception");
     }
-    std::unique_ptr<AdditionFastIf> createAdditionFast() override {
-      std::terminate();
-    }
-    std::unique_ptr<SerialAdditionIf> createSerialAddition() override {
-      std::terminate();
-    }
 
     folly::Baton<> b;
   };
@@ -718,10 +865,6 @@ TEST(InteractionCodegenTest, SerialInteraction) {
       folly::coro::blockingWait(baton1);
       return std::make_unique<SerialAdditionHandler>(baton2, baton3);
     }
-    std::unique_ptr<AdditionFastIf> createAdditionFast() override {
-      std::terminate();
-    }
-    std::unique_ptr<AdditionIf> createAddition() override { std::terminate(); }
 
     folly::coro::Baton baton1, baton2, baton3;
   };
@@ -846,7 +989,7 @@ TEST(InteractionCodegenTest, StreamExtendsInteractionLifetime) {
 
   auto handler = std::make_shared<StreamingHandler>();
   ScopedServerInterfaceThread runner{handler};
-  runner.getThriftServer().getThreadManager()->addWorker();
+  runner.getThriftServer().getThreadManager_deprecated()->addWorker();
   auto client = runner.newClient<StreamerAsyncClient>(nullptr, [](auto socket) {
     return RocketClientChannel::newChannel(std::move(socket));
   });
@@ -962,10 +1105,6 @@ TEST(InteractionCodegenTest, BasicEB) {
     std::unique_ptr<AdditionFastIf> createAdditionFast() override {
       return std::make_unique<AdditionHandler>();
     }
-    std::unique_ptr<AdditionIf> createAddition() override { std::terminate(); }
-    std::unique_ptr<SerialAdditionIf> createSerialAddition() override {
-      std::terminate();
-    }
   };
 
   ScopedServerInterfaceThread runner{
@@ -989,10 +1128,6 @@ TEST(InteractionCodegenTest, ErrorEB) {
     std::unique_ptr<AdditionFastIf> createAdditionFast() override {
       throw std::runtime_error("Unimplemented");
     }
-    std::unique_ptr<AdditionIf> createAddition() override { std::terminate(); }
-    std::unique_ptr<SerialAdditionIf> createSerialAddition() override {
-      std::terminate();
-    }
   };
 
   ScopedServerInterfaceThread runner{
@@ -1010,3 +1145,273 @@ TEST(InteractionCodegenTest, ErrorEB) {
   }());
 #endif
 }
+
+CO_TEST(InteractionCodegenTest, Factory) {
+  auto client = makeTestClient<CalculatorAsyncClient>(
+      std::make_shared<SemiCalculatorHandler>());
+
+  {
+    auto [adder, ret] = client->sync_initializedAddition(42);
+    EXPECT_EQ(ret, 42);
+    adder.sync_accumulatePrimitive(1);
+    EXPECT_EQ(adder.sync_getPrimitive(), 43);
+  }
+
+  {
+    auto [adder, ret] = co_await client->semifuture_initializedAddition(42);
+    EXPECT_EQ(ret, 42);
+    co_await adder.semifuture_accumulatePrimitive(1);
+    EXPECT_EQ(co_await adder.semifuture_getPrimitive(), 43);
+  }
+
+  {
+    auto [adder, ret] = co_await client->co_initializedAddition(42);
+    EXPECT_EQ(ret, 42);
+    co_await adder.co_accumulatePrimitive(1);
+    EXPECT_EQ(co_await adder.co_getPrimitive(), 43);
+  }
+}
+
+CO_TEST(InteractionCodegenTest, FactoryError) {
+  auto client = makeTestClient<CalculatorAsyncClient>(
+      std::make_shared<SemiCalculatorHandler>());
+
+  // unimplemented in handler
+  EXPECT_THROW(client->sync_newAddition(), TApplicationException);
+  EXPECT_THROW(
+      co_await client->semifuture_newAddition(), TApplicationException);
+  EXPECT_THROW(co_await client->co_newAddition(), TApplicationException);
+}
+
+CO_TEST(InteractionCodegenTest, FactoryEager) {
+  auto client = makeTestClient<CalculatorAsyncClient>(
+      std::make_shared<SemiCalculatorHandler>());
+
+  {
+    RpcOptions opts;
+    auto [adder, sf1] = client->eager_semifuture_initializedAddition(opts, 42);
+    auto sf2 = adder.semifuture_accumulatePrimitive(1);
+    EXPECT_FALSE(sf1.isReady());
+    auto [ret1, ret2] =
+        co_await folly::coro::collectAll(std::move(sf1), std::move(sf2));
+    EXPECT_EQ(ret1, 42);
+    EXPECT_EQ(co_await adder.co_getPrimitive(), 43);
+  }
+}
+
+TEST(InteractionCodegenTest, FactoryEb) {
+  auto client = makeTestClient<CalculatorAsyncClient>(
+      std::make_shared<SemiCalculatorHandler>());
+
+  {
+    auto adder = client->sync_fastAddition();
+    EXPECT_THROW(adder.sync_accumulatePrimitive(1), TApplicationException);
+    EXPECT_EQ(adder.sync_getPrimitive(), 1);
+  }
+
+  {
+    auto adder = client->sync_veryFastAddition();
+    EXPECT_THROW(adder.sync_accumulatePrimitive(1), TApplicationException);
+    EXPECT_EQ(adder.sync_getPrimitive(), 1);
+  }
+
+  {
+    RpcOptions opts;
+    auto [adder, sf1] = client->eager_semifuture_fastAddition(opts);
+    auto sf2 = adder.semifuture_accumulatePrimitive(1);
+    auto sf3 = adder.semifuture_getPrimitive();
+    auto [res1, res2, res3] =
+        folly::collectAll(std::move(sf1), std::move(sf2), std::move(sf3)).get();
+    EXPECT_TRUE(sf1.hasValue());
+    EXPECT_FALSE(sf2.hasValue());
+    EXPECT_TRUE(sf3.hasValue());
+  }
+
+  {
+    RpcOptions opts;
+    auto [adder, sf1] = client->eager_semifuture_veryFastAddition(opts);
+    auto sf2 = adder.semifuture_accumulatePrimitive(1);
+    auto sf3 = adder.semifuture_getPrimitive();
+    auto [res1, res2, res3] =
+        folly::collectAll(std::move(sf1), std::move(sf2), std::move(sf3)).get();
+    EXPECT_TRUE(sf1.hasValue());
+    EXPECT_FALSE(sf2.hasValue());
+    EXPECT_TRUE(sf3.hasValue());
+  }
+}
+
+TEST(InteractionCodegenTest, FactoryHandlerCallback) {
+  struct HandlerResult : CalculatorSvIf {
+    void async_tm_newAddition(
+        std::unique_ptr<
+            apache::thrift::HandlerCallback<TileAndResponse<AdditionIf, void>>>
+            cb) override {
+      auto handler =
+          std::make_unique<SemiCalculatorHandler::SemiAdditionHandler>();
+      cb->result({std::move(handler)});
+    }
+
+    void async_tm_initializedAddition(
+        std::unique_ptr<apache::thrift::HandlerCallback<
+            TileAndResponse<AdditionIf, int>>> cb,
+        int x) override {
+      auto handler =
+          std::make_unique<SemiCalculatorHandler::SemiAdditionHandler>();
+      handler->acc_ = x;
+      cb->result({std::move(handler), x});
+    }
+
+    void async_tm_stringifiedAddition(
+        std::unique_ptr<apache::thrift::HandlerCallback<
+            TileAndResponse<AdditionIf, std::unique_ptr<std::string>>>> cb,
+        int x) override {
+      auto handler =
+          std::make_unique<SemiCalculatorHandler::SemiAdditionHandler>();
+      handler->acc_ = x;
+      cb->result(
+          {std::move(handler), folly::copy_to_unique_ptr(std::to_string(x))});
+    }
+  };
+
+  struct HandlerComplete : CalculatorSvIf {
+    void async_tm_newAddition(
+        std::unique_ptr<
+            apache::thrift::HandlerCallback<TileAndResponse<AdditionIf, void>>>
+            cb) override {
+      auto handler =
+          std::make_unique<SemiCalculatorHandler::SemiAdditionHandler>();
+      cb->complete(
+          folly::Try<TileAndResponse<AdditionIf, void>>{{std::move(handler)}});
+    }
+
+    void async_tm_initializedAddition(
+        std::unique_ptr<apache::thrift::HandlerCallback<
+            TileAndResponse<AdditionIf, int>>> cb,
+        int x) override {
+      auto handler =
+          std::make_unique<SemiCalculatorHandler::SemiAdditionHandler>();
+      handler->acc_ = x;
+      cb->complete(folly::Try<TileAndResponse<AdditionIf, int>>{
+          {std::move(handler), x}});
+    }
+
+    void async_tm_stringifiedAddition(
+        std::unique_ptr<apache::thrift::HandlerCallback<
+            TileAndResponse<AdditionIf, std::unique_ptr<std::string>>>> cb,
+        int x) override {
+      auto handler =
+          std::make_unique<SemiCalculatorHandler::SemiAdditionHandler>();
+      handler->acc_ = x;
+      cb->complete(
+          folly::Try<TileAndResponse<AdditionIf, std::unique_ptr<std::string>>>{
+              {std::move(handler),
+               folly::copy_to_unique_ptr(std::to_string(x))}});
+    }
+  };
+
+  struct HandlerException : CalculatorSvIf {
+    void async_tm_newAddition(
+        std::unique_ptr<
+            apache::thrift::HandlerCallback<TileAndResponse<AdditionIf, void>>>
+            cb) override {
+      cb->exception(std::runtime_error("foo"));
+    }
+
+    void async_tm_initializedAddition(
+        std::unique_ptr<apache::thrift::HandlerCallback<
+            TileAndResponse<AdditionIf, int>>> cb,
+        int) override {
+      cb->exception(std::runtime_error("foo"));
+    }
+  };
+
+  struct HandlerDrop : CalculatorSvIf {
+    void async_tm_newAddition(
+        std::unique_ptr<
+            apache::thrift::HandlerCallback<TileAndResponse<AdditionIf, void>>>
+            cb) override {
+      (void)cb; // oops!
+    }
+
+    void async_tm_initializedAddition(
+        std::unique_ptr<apache::thrift::HandlerCallback<
+            TileAndResponse<AdditionIf, int>>> cb,
+        int) override {
+      (void)cb; // oops!
+    }
+  };
+
+  // Result
+  {
+    auto client = makeTestClient<CalculatorAsyncClient>(
+        std::make_shared<HandlerResult>());
+    auto adder = client->sync_newAddition();
+    adder.sync_accumulatePrimitive(1);
+    EXPECT_EQ(adder.sync_getPrimitive(), 1);
+  }
+  {
+    auto client = makeTestClient<CalculatorAsyncClient>(
+        std::make_shared<HandlerResult>());
+    auto [adder, ret] = client->sync_initializedAddition(42);
+    EXPECT_EQ(ret, 42);
+    adder.sync_accumulatePrimitive(1);
+    EXPECT_EQ(adder.sync_getPrimitive(), 43);
+  }
+  {
+    auto client = makeTestClient<CalculatorAsyncClient>(
+        std::make_shared<HandlerResult>());
+    auto [adder, ret] = client->sync_stringifiedAddition(42);
+    EXPECT_EQ(ret, "42");
+    adder.sync_accumulatePrimitive(1);
+    EXPECT_EQ(adder.sync_getPrimitive(), 43);
+  }
+
+  // Complete
+  {
+    auto client = makeTestClient<CalculatorAsyncClient>(
+        std::make_shared<HandlerComplete>());
+    auto adder = client->sync_newAddition();
+    adder.sync_accumulatePrimitive(1);
+    EXPECT_EQ(adder.sync_getPrimitive(), 1);
+  }
+  {
+    auto client = makeTestClient<CalculatorAsyncClient>(
+        std::make_shared<HandlerComplete>());
+    auto [adder, ret] = client->sync_initializedAddition(42);
+    EXPECT_EQ(ret, 42);
+    adder.sync_accumulatePrimitive(1);
+    EXPECT_EQ(adder.sync_getPrimitive(), 43);
+  }
+  {
+    auto client = makeTestClient<CalculatorAsyncClient>(
+        std::make_shared<HandlerComplete>());
+    auto [adder, ret] = client->sync_stringifiedAddition(42);
+    EXPECT_EQ(ret, "42");
+    adder.sync_accumulatePrimitive(1);
+    EXPECT_EQ(adder.sync_getPrimitive(), 43);
+  }
+
+  // Exception
+  {
+    auto client = makeTestClient<CalculatorAsyncClient>(
+        std::make_shared<HandlerException>());
+    EXPECT_THROW(client->sync_newAddition(), TApplicationException);
+  }
+  {
+    auto client = makeTestClient<CalculatorAsyncClient>(
+        std::make_shared<HandlerException>());
+    EXPECT_THROW(client->sync_initializedAddition(42), TApplicationException);
+  }
+
+  // Drop
+  {
+    auto client =
+        makeTestClient<CalculatorAsyncClient>(std::make_shared<HandlerDrop>());
+    EXPECT_THROW(client->sync_newAddition(), TApplicationException);
+  }
+  {
+    auto client =
+        makeTestClient<CalculatorAsyncClient>(std::make_shared<HandlerDrop>());
+    EXPECT_THROW(client->sync_initializedAddition(42), TApplicationException);
+  }
+}
