@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,6 +16,7 @@
 
 #include <folly/portability/GMock.h>
 #include <folly/portability/GTest.h>
+#include <folly/test/TestUtils.h>
 
 #include <folly/Conv.h>
 #include <folly/Overload.h>
@@ -23,12 +24,14 @@
 #include <thrift/lib/cpp2/async/AsyncProcessorHelper.h>
 #include <thrift/lib/cpp2/async/MultiplexAsyncProcessor.h>
 #include <thrift/lib/cpp2/protocol/DebugProtocol.h>
+#include <thrift/lib/cpp2/server/ServerFlags.h>
 #include <thrift/lib/cpp2/server/ThriftServer.h>
 #include <thrift/lib/cpp2/util/ScopedServerInterfaceThread.h>
 
 #include <thrift/lib/cpp2/test/gen-cpp2/Conflicts.h>
 #include <thrift/lib/cpp2/test/gen-cpp2/ConflictsInteraction1.h>
 #include <thrift/lib/cpp2/test/gen-cpp2/First.h>
+#include <thrift/lib/cpp2/test/gen-cpp2/FirstAsyncClient.h>
 #include <thrift/lib/cpp2/test/gen-cpp2/Interaction1.h>
 #include <thrift/lib/cpp2/test/gen-cpp2/Interaction2.h>
 #include <thrift/lib/cpp2/test/gen-cpp2/Second.h>
@@ -43,40 +46,57 @@ using namespace ::testing;
 using MethodMetadata = AsyncProcessorFactory::MethodMetadata;
 using MethodMetadataMap = AsyncProcessorFactory::MethodMetadataMap;
 using MetadataNotImplemented = AsyncProcessorFactory::MetadataNotImplemented;
-using WildcardMethodMetadata = AsyncProcessorFactory::WildcardMethodMetadata;
 using WildcardMethodMetadataMap =
     AsyncProcessorFactory::WildcardMethodMetadataMap;
 using CreateMethodMetadataResult =
     AsyncProcessorFactory::CreateMethodMetadataResult;
 
 namespace {
-class First : public FirstSvIf {
+class FirstHandler : public FirstSvIf {
   int one() override { return 1; }
   int two() override { return 2; }
 };
 
-class Second : public SecondSvIf {
+class SecondHandler : public SecondSvIf {
   int three() override { return 3; }
   int four() override { return 4; }
 };
 
-class Third : public ThirdSvIf {
+class ThirdHandler : public ThirdSvIf {
   int five() override { return 5; }
   int six() override { return 6; }
 };
 
-class Conflicts : public ConflictsSvIf {
+class ConflictsHandler : public ConflictsSvIf {
   int four() override { return 444; }
   int five() override { return 555; }
 };
+
+class MultiplexAsyncProcessorTest : public Test {
+ public:
+  std::shared_ptr<AsyncProcessorFactory> multiplex(
+      std::vector<std::shared_ptr<AsyncProcessorFactory>> services) {
+    return std::make_shared<MultiplexAsyncProcessorFactory>(
+        std::move(services));
+  }
+};
+
+class MultiplexAsyncProcessorServerTest : public MultiplexAsyncProcessorTest {
+ public:
+  std::unique_ptr<ScopedServerInterfaceThread> runMultiplexedServices(
+      std::vector<std::shared_ptr<AsyncProcessorFactory>> services) {
+    return std::make_unique<ScopedServerInterfaceThread>(
+        multiplex(std::move(services)));
+  }
+};
 } // namespace
 
-TEST(MultiplexAsyncProcessorTest, getServiceHandlers) {
+TEST_F(MultiplexAsyncProcessorTest, getServiceHandlers) {
   std::vector<std::shared_ptr<AsyncProcessorFactory>> services = {
-      std::make_shared<First>(),
-      std::make_shared<Second>(),
-      std::make_shared<Third>(),
-      std::make_shared<Conflicts>(),
+      std::make_shared<FirstHandler>(),
+      std::make_shared<SecondHandler>(),
+      std::make_shared<ThirdHandler>(),
+      std::make_shared<ConflictsHandler>(),
   };
   auto processorFactory =
       std::make_shared<MultiplexAsyncProcessorFactory>(std::move(services));
@@ -84,18 +104,33 @@ TEST(MultiplexAsyncProcessorTest, getServiceHandlers) {
   EXPECT_EQ(processorFactory->getServiceHandlers().size(), 4);
 }
 
-TEST(MultiplexAsyncProcessorTest, getServiceMetadata) {
+TEST_F(MultiplexAsyncProcessorTest, getServiceHandlers_Nested) {
+  std::vector<std::shared_ptr<AsyncProcessorFactory>> services2 = {
+      std::make_shared<FirstHandler>(),
+      multiplex({
+          std::make_shared<SecondHandler>(),
+          std::make_shared<ThirdHandler>(),
+      }),
+      std::make_shared<ConflictsHandler>(),
+  };
+  auto processorFactory =
+      std::make_shared<MultiplexAsyncProcessorFactory>(std::move(services2));
+  // Generated service handlers are one per service
+  EXPECT_EQ(processorFactory->getServiceHandlers().size(), 4);
+}
+
+TEST_F(MultiplexAsyncProcessorTest, getServiceMetadata) {
   auto getMetadataFromService = [](AsyncProcessorFactory& service) {
     metadata::ThriftServiceMetadataResponse response;
     service.getProcessor()->getServiceMetadata(response);
     return response;
   };
   std::vector<std::shared_ptr<AsyncProcessorFactory>> servicesToMultiplex = {
-      std::make_shared<First>(),
-      std::make_shared<Second>(),
+      std::make_shared<FirstHandler>(),
+      std::make_shared<SecondHandler>(),
       std::make_shared<SomeServiceSvIf>(),
-      std::make_shared<Conflicts>(),
-      std::make_shared<Third>(),
+      std::make_shared<ConflictsHandler>(),
+      std::make_shared<ThirdHandler>(),
   };
   auto processorFactory = std::make_shared<MultiplexAsyncProcessorFactory>(
       std::move(servicesToMultiplex));
@@ -128,26 +163,56 @@ TEST(MultiplexAsyncProcessorTest, getServiceMetadata) {
   EXPECT_EQ(metadata.services_ref()->size(), 5);
 }
 
-namespace {
-class MultiplexAsyncProcessorServerTest : public Test {
- public:
-  std::shared_ptr<AsyncProcessorFactory> multiplex(
-      std::vector<std::shared_ptr<AsyncProcessorFactory>> services) {
-    return std::make_shared<MultiplexAsyncProcessorFactory>(
-        std::move(services));
-  }
+TEST_F(MultiplexAsyncProcessorTest, getServiceMetadata_Nested) {
+  auto getMetadataFromService = [](AsyncProcessorFactory& service) {
+    metadata::ThriftServiceMetadataResponse response;
+    service.getProcessor()->getServiceMetadata(response);
+    return response;
+  };
 
-  std::unique_ptr<ScopedServerInterfaceThread> runMultiplexedServices(
-      std::vector<std::shared_ptr<AsyncProcessorFactory>> services) {
-    return std::make_unique<ScopedServerInterfaceThread>(
-        multiplex(std::move(services)));
-  }
-};
-} // namespace
+  std::vector<std::shared_ptr<AsyncProcessorFactory>> servicesToMultiplex = {
+      std::make_shared<FirstHandler>(),
+      std::make_shared<SecondHandler>(),
+      multiplex({
+          std::make_shared<SomeServiceSvIf>(),
+          std::make_shared<ConflictsHandler>(),
+      }),
+      std::make_shared<ThirdHandler>(),
+  };
+  auto processorFactory = std::make_shared<MultiplexAsyncProcessorFactory>(
+      std::move(servicesToMultiplex));
+  auto response = getMetadataFromService(*processorFactory);
+
+  LOG(INFO) << "ServiceMetadata: " << debugString(response);
+
+  EXPECT_EQ(
+      *response.context_ref()->service_info_ref()->name_ref(),
+      "MultiplexAsyncProcessor.First");
+
+  auto& services = *response.services_ref();
+  EXPECT_EQ(services.size(), 6);
+  EXPECT_EQ(*services[0].service_name_ref(), "MultiplexAsyncProcessor.First");
+  EXPECT_EQ(*services[1].service_name_ref(), "MultiplexAsyncProcessor.Second");
+  EXPECT_EQ(
+      *services[2].service_name_ref(), "MultiplexAsyncProcessor.SomeService");
+  // Base service of SomeService
+  EXPECT_EQ(*services[3].service_name_ref(), "MultiplexAsyncProcessor.Third");
+  EXPECT_EQ(
+      *services[4].service_name_ref(), "MultiplexAsyncProcessor.Conflicts");
+  EXPECT_EQ(*services[5].service_name_ref(), "MultiplexAsyncProcessor.Third");
+
+  const auto& metadata = *response.metadata_ref();
+  EXPECT_EQ(metadata.structs_ref()->size(), 1);
+  EXPECT_EQ(
+      metadata.structs_ref()->begin()->first,
+      "MultiplexAsyncProcessor.SomeStruct");
+  // All composed services are referred to
+  EXPECT_EQ(metadata.services_ref()->size(), 5);
+}
 
 TEST_F(MultiplexAsyncProcessorServerTest, Basic) {
   auto runner = runMultiplexedServices(
-      {std::make_shared<First>(), std::make_shared<Second>()});
+      {std::make_shared<FirstHandler>(), std::make_shared<SecondHandler>()});
 
   auto client1 = runner->newClient<FirstAsyncClient>();
   auto client2 = runner->newClient<SecondAsyncClient>();
@@ -160,9 +225,9 @@ TEST_F(MultiplexAsyncProcessorServerTest, Basic) {
 
 TEST_F(MultiplexAsyncProcessorServerTest, ConflictPrecedence) {
   auto runner = runMultiplexedServices(
-      {std::make_shared<Second>(),
-       std::make_shared<Conflicts>(),
-       std::make_shared<Third>()});
+      {std::make_shared<SecondHandler>(),
+       std::make_shared<ConflictsHandler>(),
+       std::make_shared<ThirdHandler>()});
 
   auto client2 = runner->newClient<SecondAsyncClient>();
   auto client3 = runner->newClient<ThirdAsyncClient>();
@@ -175,10 +240,12 @@ TEST_F(MultiplexAsyncProcessorServerTest, ConflictPrecedence) {
   EXPECT_EQ(client3->semifuture_six().get(), 6);
 }
 
-TEST_F(MultiplexAsyncProcessorServerTest, Nested) {
+TEST_F(MultiplexAsyncProcessorServerTest, Nested_1) {
   auto runner = runMultiplexedServices(
-      {multiplex({std::make_shared<Second>(), std::make_shared<Conflicts>()}),
-       std::make_shared<Third>()});
+      {multiplex(
+           {std::make_shared<SecondHandler>(),
+            std::make_shared<ConflictsHandler>()}),
+       std::make_shared<ThirdHandler>()});
 
   auto client2 = runner->newClient<SecondAsyncClient>();
   auto client3 = runner->newClient<ThirdAsyncClient>();
@@ -191,6 +258,27 @@ TEST_F(MultiplexAsyncProcessorServerTest, Nested) {
   EXPECT_EQ(client3->semifuture_six().get(), 6);
 }
 
+TEST_F(MultiplexAsyncProcessorServerTest, Nested_2) {
+  auto runner = runMultiplexedServices(
+      {std::make_shared<ThirdHandler>(),
+       multiplex(
+           {std::make_shared<FirstHandler>(),
+            std::make_shared<ConflictsHandler>()}),
+       std::make_shared<SecondHandler>()});
+
+  auto client1 = runner->newClient<FirstAsyncClient>();
+  auto client2 = runner->newClient<SecondAsyncClient>();
+  auto client3 = runner->newClient<ThirdAsyncClient>();
+
+  EXPECT_EQ(client1->semifuture_one().get(), 1);
+  EXPECT_EQ(client2->semifuture_three().get(), 3);
+  // Conflict takes precedence
+  EXPECT_EQ(client2->semifuture_four().get(), 444);
+  // Third takes precedence
+  EXPECT_EQ(client3->semifuture_five().get(), 5);
+  EXPECT_EQ(client3->semifuture_six().get(), 6);
+}
+
 namespace {
 
 class ContextData : public folly::RequestData {
@@ -297,6 +385,30 @@ class WildcardThrowsInternalError : public TProcessorFactory {
             tm);
       }
 
+      void executeRequest(
+          ServerRequest&& request,
+          const AsyncProcessorFactory::MethodMetadata& methodMetadata)
+          override {
+        if (AsyncProcessorHelper::isWildcardMethodMetadata(methodMetadata)) {
+          std::string message = folly::variant_match(
+              message_,
+              [](const std::string& m) { return m; },
+              [](FromCurrentContextData) {
+                return folly::to<std::string>(ContextData::readFromCurrent());
+              });
+          auto eb = detail::ServerRequestHelper::eventBase(request);
+          eb->runInEventBaseThread([request = std::move(request),
+                                    message = std::move(message)]() mutable {
+            request.request()->sendErrorWrapped(
+                folly::make_exception_wrapper<TApplicationException>(
+                    TApplicationException::INTERNAL_ERROR, std::move(message)),
+                "" /* errorCode */);
+          });
+          return;
+        }
+        inner_->executeRequest(std::move(request), methodMetadata);
+      }
+
       void terminateInteraction(
           int64_t id,
           Cpp2ConnContext& ctx,
@@ -334,8 +446,9 @@ class WithUnimplementedMethodMetadata : public TProcessorFactory {
 
 TEST_F(MultiplexAsyncProcessorServerTest, BasicWildcard) {
   auto runner = runMultiplexedServices(
-      {std::make_shared<First>(),
-       std::make_shared<WildcardThrowsInternalError<Second>>("BasicWildcard")});
+      {std::make_shared<FirstHandler>(),
+       std::make_shared<WildcardThrowsInternalError<SecondHandler>>(
+           "BasicWildcard")});
 
   auto client1 = runner->newClient<FirstAsyncClient>();
   auto client2 = runner->newClient<SecondAsyncClient>();
@@ -353,9 +466,11 @@ TEST_F(MultiplexAsyncProcessorServerTest, BasicWildcard) {
 
 TEST_F(MultiplexAsyncProcessorServerTest, WildcardSwallows) {
   auto runner = runMultiplexedServices(
-      {std::make_shared<WildcardThrowsInternalError<First>>("WildcardSwallows"),
-       std::make_shared<Second>(),
-       std::make_shared<WildcardThrowsInternalError<Third>>("NeverReached")});
+      {std::make_shared<WildcardThrowsInternalError<FirstHandler>>(
+           "WildcardSwallows"),
+       std::make_shared<SecondHandler>(),
+       std::make_shared<WildcardThrowsInternalError<ThirdHandler>>(
+           "NeverReached")});
 
   auto client1 = runner->newClient<FirstAsyncClient>();
   auto client2 = runner->newClient<SecondAsyncClient>();
@@ -370,8 +485,8 @@ TEST_F(MultiplexAsyncProcessorServerTest, WildcardSwallows) {
 
 TEST_F(MultiplexAsyncProcessorServerTest, WildcardConflicts) {
   auto runner = runMultiplexedServices(
-      {std::make_shared<Second>(),
-       std::make_shared<WildcardThrowsInternalError<Conflicts>>(
+      {std::make_shared<SecondHandler>(),
+       std::make_shared<WildcardThrowsInternalError<ConflictsHandler>>(
            "WildcardConflicts")});
 
   auto client2 = runner->newClient<SecondAsyncClient>();
@@ -388,8 +503,8 @@ TEST_F(MultiplexAsyncProcessorServerTest, WildcardConflicts) {
 
 TEST_F(MultiplexAsyncProcessorServerTest, UnimplementedMetadataActsAsWildcard) {
   auto runner = runMultiplexedServices(
-      {std::make_shared<WithUnimplementedMethodMetadata<First>>(),
-       std::make_shared<Second>()});
+      {std::make_shared<WithUnimplementedMethodMetadata<FirstHandler>>(),
+       std::make_shared<SecondHandler>()});
 
   auto client1 = runner->newClient<FirstAsyncClient>();
   auto client2 = runner->newClient<SecondAsyncClient>();
@@ -549,6 +664,12 @@ TEST_F(MultiplexAsyncProcessorServerTest, Interaction) {
           tm);
     }
 
+    void executeRequest(
+        ServerRequest&& request,
+        const AsyncProcessorFactory::MethodMetadata& methodMetadata) override {
+      delegate_->executeRequest(std::move(request), methodMetadata);
+    }
+
     virtual void terminateInteraction(
         int64_t id,
         Cpp2ConnContext& ctx,
@@ -670,7 +791,7 @@ TEST_F(MultiplexAsyncProcessorServerTest, InteractionConflict) {
   auto runner = runMultiplexedServices(
       {std::make_shared<Interaction1>(),
        std::make_shared<ConflictsInteraction1>(),
-       std::make_shared<WildcardThrowsInternalError<Second>>(
+       std::make_shared<WildcardThrowsInternalError<SecondHandler>>(
            "ConflictsInteraction1")});
 
   auto client1 = runner->newClient<Interaction1AsyncClient>(
