@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -22,11 +22,9 @@
 #include <folly/portability/GTest.h>
 
 using namespace std;
-using namespace std::chrono;
 using namespace folly;
 using namespace apache::thrift;
 using namespace apache::thrift::test;
-using namespace apache::thrift::transport;
 
 THRIFT_FLAG_DECLARE_bool(server_header_reject_http);
 
