@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -33,8 +33,6 @@ namespace {
 using apache::thrift::metadata::ThriftConstStruct;
 using apache::thrift::metadata::ThriftConstValue;
 using apache::thrift::metadata::ThriftConstValuePair;
-using apache::thrift::metadata::ThriftEnum;
-using apache::thrift::metadata::ThriftField;
 using apache::thrift::metadata::ThriftMetadata;
 using apache::thrift::metadata::ThriftPrimitiveType;
 using apache::thrift::metadata::ThriftServiceMetadataResponse;
