@@ -1,11 +1,11 @@
 /*
- * Copyright 2018-present Facebook, Inc.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -28,17 +28,11 @@ class FakeClock {
   typedef std::chrono::duration<rep, period> duration;
   typedef std::chrono::time_point<FakeClock> time_point;
 
-  static void advance(duration const& d) noexcept {
-    now_us_ += d;
-  }
+  static void advance(duration const& d) noexcept { now_us_ += d; }
 
-  static void reset_to_epoch() noexcept {
-    now_us_ -= (now_us_ - time_point());
-  }
+  static void reset_to_epoch() noexcept { now_us_ -= (now_us_ - time_point()); }
 
-  static time_point now() noexcept {
-    return now_us_;
-  }
+  static time_point now() noexcept { return now_us_; }
 
   static constexpr bool is_steady = true;
 
