@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -233,7 +233,7 @@ TEST(ScopedServerInterfaceThread, joinRequestsSinkSlowFinalResponse) {
 TEST(ScopedServerInterfaceThread, TransportMemLimit) {
   auto ts = make_shared<ThriftServer>();
   auto serviceImpl = std::make_shared<SimpleServiceImpl>();
-  ts->setProcessorFactory(serviceImpl);
+  ts->setInterface(serviceImpl);
   ts->setAddress({"::1", 0});
   auto request = folly::IOBuf::create(257 * 1024);
   request->append(257 * 1024);
@@ -295,8 +295,7 @@ TEST(ScopedServerInterfaceThread, faultInjection) {
 }
 
 TEST(ScopedServerInterfaceThread, makeTestClient) {
-  auto cli = makeTestClient<SimpleServiceAsyncClient>(
-      make_shared<SimpleServiceImpl>());
+  auto cli = makeTestClient(make_shared<SimpleServiceImpl>());
   EXPECT_EQ(6, cli->sync_add(-3, 9));
 }
 
@@ -524,7 +523,7 @@ TYPED_TEST(ScopedServerInterfaceThreadTest, joinRequestsRestartServer) {
     ts->setThreadManager(tm);
     ts->setAcceptExecutor({});
     auto serviceImpl = this->newService();
-    ts->setProcessorFactory(serviceImpl);
+    ts->setInterface(serviceImpl);
 
     folly::Optional<ScopedServerInterfaceThread> ssit(folly::in_place, ts);
 
@@ -908,6 +907,7 @@ TYPED_TEST(ScopedServerInterfaceThreadTest, SetMaxRequestsJoinWrites) {
   if (this->isHeaderTransport() || this->isH2Transport()) {
     return; // Joining writes is not implemented for Header/H2 transport.
   }
+  THRIFT_OMIT_TEST_WITH_RESOURCE_POOLS(/* maxRequests limit with resourcePools does not include response IO time */);
 
   std::string message(10000000, 'a');
 
