@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -34,11 +34,10 @@ struct TestThriftServerFactory : public TestServerFactory {
     if (useSimpleThreadManager_) {
       auto threadFactory =
           std::make_shared<apache::thrift::concurrency::PosixThreadFactory>();
-      auto threadManager =
-          apache::thrift::concurrency::ThreadManager::newSimpleThreadManager(1);
-      threadManager->threadFactory(threadFactory);
-      threadManager->start();
-      server->setThreadManager(threadManager);
+      server->setThreadManagerType(
+          apache::thrift::BaseThriftServer::ThreadManagerType::SIMPLE);
+      server->setNumCPUWorkerThreads(1);
+      server->setThreadFactory(threadFactory);
     } else if (exe_) {
       server->setThreadManager(exe_);
     }
