@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -84,6 +84,15 @@ class ServerConfigsMock : public ServerConfigs {
 
   uint32_t getListenerTos() const override { return 0; }
 
+  std::shared_ptr<concurrency::ThreadManager> getThreadManager_deprecated()
+      const override {
+    return {};
+  }
+
+  std::shared_ptr<folly::Executor> getThreadManager() const override {
+    return {};
+  }
+
  public:
   uint64_t maxResponseSize_{0};
   std::chrono::milliseconds queueTimeout_{std::chrono::milliseconds(500)};
