@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -33,6 +33,7 @@
 #include <thrift/lib/cpp2/async/PooledRequestChannel.h>
 #include <thrift/lib/cpp2/async/RocketClientChannel.h>
 #include <thrift/lib/cpp2/server/Cpp2Worker.h>
+#include <thrift/lib/cpp2/server/ServerFlags.h>
 #include <thrift/lib/cpp2/transport/core/ThriftClient.h>
 #include <thrift/lib/cpp2/transport/core/ThriftClientCallback.h>
 #include <thrift/lib/cpp2/transport/core/testutil/MockCallback.h>
@@ -59,7 +60,6 @@ namespace thrift {
 
 using namespace async;
 using namespace testing;
-using namespace apache::thrift;
 using namespace apache::thrift::transport;
 using namespace testutil::testservice;
 
@@ -133,16 +133,22 @@ template <typename Service>
 void SampleServer<Service>::setupServer() {
   DCHECK(!server_) << "First close the server with stopServer()";
 
-  auto cpp2PFac =
-      std::make_shared<ThriftServerAsyncProcessorFactory<Service>>(handler_);
-
   server_ = std::make_unique<ThriftServer>();
   observer_ = std::make_shared<FakeServerObserver>();
   server_->setObserver(observer_);
   server_->setPort(0);
   server_->setNumIOWorkerThreads(numIOThreads_);
   server_->setNumCPUWorkerThreads(numWorkerThreads_);
-  server_->setProcessorFactory(cpp2PFac);
+  if (useResourcePoolsFlagsSet()) {
+    server_->ensureResourcePools();
+    auto& resourcePool = server_->resourcePoolSet().resourcePool(
+        ResourcePoolHandle::defaultAsync());
+    dynamic_cast<folly::ThreadPoolExecutor*>(resourcePool.executor().value())
+        ->setNumThreads(numWorkerThreads_);
+    resourcePool.concurrencyController().value()->setExecutionLimitRequests(
+        numWorkerThreads_);
+  }
+  server_->setInterface(handler_);
   server_->setGetLoad(
       [](const std::string& metric) { return metric.empty() ? 123 : -1; });
 }
@@ -257,7 +263,9 @@ void SampleServer<Service>::connectToServer(
             sslSocket->sslConn(nullptr);
             socket.reset(sslSocket);
           }
-          return HTTPClientChannel::newHTTP2Channel(std::move(socket));
+          auto channel = HTTPClientChannel::newHTTP2Channel(std::move(socket));
+          channel->setProtocolId(protocol::T_COMPACT_PROTOCOL);
+          return channel;
         });
     callMe(std::move(channel), nullptr);
   } else {
@@ -1282,29 +1290,27 @@ void TransportCompatibilityTest::TestCustomAsyncProcessor() {
       : public apache::thrift::AsyncProcessorFactory {
    public:
     explicit TestAsyncProcessorFactory(
-        std::shared_ptr<ThriftServerAsyncProcessorFactory<
-            testutil::testservice::TestServiceMock>> fac)
+        std::shared_ptr<testutil::testservice::TestServiceMock> fac)
         : underlyingFac_(std::move(fac)) {}
     std::unique_ptr<apache::thrift::AsyncProcessor> getProcessor() override {
       return std::make_unique<TestAsyncProcessor>(
           underlyingFac_->getProcessor());
     }
 
-    std::vector<apache::thrift::ServiceHandler*> getServiceHandlers() override {
+    std::vector<apache::thrift::ServiceHandlerBase*> getServiceHandlers()
+        override {
       return underlyingFac_->getServiceHandlers();
     }
 
+    CreateMethodMetadataResult createMethodMetadata() override {
+      return underlyingFac_->createMethodMetadata();
+    }
+
    private:
-    std::shared_ptr<ThriftServerAsyncProcessorFactory<
-        testutil::testservice::TestServiceMock>>
-        underlyingFac_;
+    std::shared_ptr<testutil::testservice::TestServiceMock> underlyingFac_;
   };
 
-  auto cpp2PFac = std::make_shared<ThriftServerAsyncProcessorFactory<
-      testutil::testservice::TestServiceMock>>(handler_);
-
-  server_->getServer()->setProcessorFactory(
-      std::make_shared<TestAsyncProcessorFactory>(std::move(cpp2PFac)));
+  server_->getServer()->setInterface(handler_);
   startServer();
   TestRequestResponse_Simple();
 }
