@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -40,18 +40,15 @@
 namespace apache {
 namespace thrift {
 
-using apache::thrift::protocol::TProtocolException;
 using apache::thrift::transport::TTransportException;
 using folly::EventBase;
 using folly::EventBaseManager;
-using folly::IOBuf;
 using proxygen::HTTPHeaderCode;
 using proxygen::HTTPMessage;
 using proxygen::HTTPMethod;
 using proxygen::HTTPTransaction;
 using proxygen::IOBufPrinter;
 using proxygen::ProxygenError;
-using std::map;
 using std::string;
 
 static constexpr folly::StringPiece kThriftContentType = "application/x-thrift";
