@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -121,7 +121,7 @@ inline T unpackPayload(rocket::Payload&& payload) {
   }
   if constexpr (uncompressPayload) {
     auto data = std::move(payload).data();
-    if (auto compression = t.metadata.compression_ref()) {
+    if (auto compression = t.metadata.compression()) {
       data = uncompressBuffer(std::move(data), *compression);
     }
     unpackCompact(t.payload, std::move(data));
