@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -75,7 +75,6 @@ class RocketClient : public virtual folly::DelayedDestruction,
   using WriteSuccessCallback = RequestContext::WriteSuccessCallback;
   class RequestResponseCallback : public WriteSuccessCallback {
    public:
-    virtual ~RequestResponseCallback() = default;
     virtual void onResponsePayload(folly::Try<Payload>&& response) noexcept = 0;
   };
 
@@ -117,7 +116,8 @@ class RocketClient : public virtual folly::DelayedDestruction,
 
   FOLLY_NODISCARD bool sendRequestN(StreamId streamId, int32_t n);
   void cancelStream(StreamId streamId);
-  void sendPayload(StreamId streamId, StreamPayload&& payload, Flags flags);
+  FOLLY_NODISCARD bool sendPayload(
+      StreamId streamId, StreamPayload&& payload, Flags flags);
   void sendError(StreamId streamId, RocketException&& rex);
   void sendComplete(StreamId streamId, bool closeStream);
   void sendExtAlignedPage(
