@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -158,7 +158,7 @@ bool RocketSinkServerCallback::onSinkNext(StreamPayload&& payload) {
             payload.metadata,
             payload.payload->computeChainDataLength());
       }
-      client_.sendPayload(
+      std::ignore = client_.sendPayload(
           streamId_, std::move(payload), rocket::Flags().next(true));
       break;
     }
