@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -45,9 +45,9 @@ enum class FrameType : uint8_t {
 
 enum class ExtFrameType : uint32_t {
   UNKNOWN = 0x00, // Never transmitted over the wire.
-  HEADERS_PUSH = 0xFB0,
+  // HEADERS_PUSH = 0xFB0,
   ALIGNED_PAGE = 0xFB1,
-  INTERACTION_TERMINATE = 0xFB2,
+  // INTERACTION_TERMINATE = 0xFB2,
   CUSTOM_ALLOC = 0xFB3,
 };
 
