@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -39,6 +39,7 @@
 #include <thrift/lib/cpp2/transport/rocket/server/RocketStreamClientCallback.h>
 #include <thrift/lib/thrift/gen-cpp2/RpcMetadata_constants.h>
 #include <thrift/lib/thrift/gen-cpp2/RpcMetadata_types.h>
+#include <thrift/lib/thrift/gen-cpp2/any_rep_types.h>
 
 namespace apache {
 namespace thrift {
@@ -61,6 +62,7 @@ RocketException makeResponseRpcError(
       case ResponseRpcErrorCode::UNKNOWN_METHOD:
       case ResponseRpcErrorCode::CHECKSUM_MISMATCH:
       case ResponseRpcErrorCode::UNKNOWN_INTERACTION_ID:
+      case ResponseRpcErrorCode::UNIMPLEMENTED_METHOD:
         return ResponseRpcErrorCategory::INVALID_REQUEST;
       case ResponseRpcErrorCode::OVERLOAD:
       case ResponseRpcErrorCode::QUEUE_OVERLOADED:
@@ -96,8 +98,7 @@ RocketException makeResponseRpcError(
 
 void preprocessProxiedExceptionHeaders(
     ResponseRpcMetadata& metadata, int32_t version) {
-  DCHECK_GE(version, 4);
-
+  DCHECK_GE(version, 8);
   auto otherMetadataRef = metadata.otherMetadata_ref();
   if (!otherMetadataRef) {
     return;
@@ -132,7 +133,7 @@ void preprocessProxiedExceptionHeaders(
   }
 }
 
-template <typename ProtocolReader>
+template <typename Serializer>
 FOLLY_NODISCARD folly::exception_wrapper processFirstResponseHelper(
     ResponseRpcMetadata& metadata,
     std::unique_ptr<folly::IOBuf>& payload,
@@ -141,7 +142,7 @@ FOLLY_NODISCARD folly::exception_wrapper processFirstResponseHelper(
     std::string methodNameIgnore;
     MessageType mtype;
     int32_t seqIdIgnore;
-    ProtocolReader reader;
+    typename Serializer::ProtocolReader reader;
     reader.setInput(payload.get());
     reader.readMessageBegin(methodNameIgnore, mtype, seqIdIgnore);
 
@@ -208,8 +209,6 @@ FOLLY_NODISCARD folly::exception_wrapper processFirstResponseHelper(
         break;
       }
       case MessageType::T_EXCEPTION: {
-        DCHECK_GE(version, 2);
-
         preprocessProxiedExceptionHeaders(metadata, version);
 
         TApplicationException ex;
@@ -226,18 +225,38 @@ FOLLY_NODISCARD folly::exception_wrapper processFirstResponseHelper(
                 ? folly::get_ptr(
                       *otherMetadataRef, "servicerouter:sr_internal_error")
                 : nullptr) {
-          exceptionMetadataBase.name_utf8_ref() = "ProxyException";
-          PayloadExceptionMetadata exceptionMetadata;
-          exceptionMetadata.set_proxyException(PayloadProxyExceptionMetadata());
-          exceptionMetadataBase.metadata_ref() = std::move(exceptionMetadata);
+          if (version < 10) {
+            exceptionMetadataBase.name_utf8_ref() = "ProxyException";
+            PayloadExceptionMetadata exceptionMetadata;
+            exceptionMetadata.set_DEPRECATED_proxyException(
+                PayloadProxyExceptionMetadata());
+            exceptionMetadataBase.metadata_ref() = std::move(exceptionMetadata);
 
-          payload = protocol::base64Decode(*proxyErrorPtr);
+            payload = protocol::base64Decode(*proxyErrorPtr);
+          } else {
+            exceptionMetadataBase.name_utf8_ref() = "ServiceRouterError";
+            exceptionMetadataBase.metadata_ref()
+                .ensure()
+                .anyException_ref()
+                .ensure();
+
+            type::SemiAnyStruct anyException;
+            anyException.type_ref() = type::Type(
+                type::exception_c{},
+                "facebook.com/servicerouter/ServiceRouterError");
+            if (!std::is_same_v<Serializer, CompactSerializer>) {
+              anyException.protocol_ref() = type::StandardProtocol::Compact;
+            }
+            anyException.data_ref() =
+                std::move(*protocol::base64Decode(*proxyErrorPtr));
+            folly::IOBufQueue payloadQueue;
+            Serializer::serialize(anyException, &payloadQueue);
+            payload = payloadQueue.move();
+          }
 
           otherMetadataRef->erase("servicerouter:sr_internal_error");
           otherMetadataRef->erase("ex");
         } else {
-          DCHECK_GE(version, 3);
-
           auto exPtr = otherMetadataRef
               ? folly::get_ptr(*otherMetadataRef, "ex")
               : nullptr;
@@ -279,7 +298,11 @@ FOLLY_NODISCARD folly::exception_wrapper processFirstResponseHelper(
                        {kRequestParsingErrorCode,
                         ResponseRpcErrorCode::REQUEST_PARSING_FAILURE},
                        {kChecksumMismatchErrorCode,
-                        ResponseRpcErrorCode::CHECKSUM_MISMATCH}});
+                        ResponseRpcErrorCode::CHECKSUM_MISMATCH},
+                       {kUnimplementedMethodErrorCode,
+                        ResponseRpcErrorCode::UNIMPLEMENTED_METHOD},
+                       {kTenantQuotaExceededErrorCode,
+                        ResponseRpcErrorCode::TENANT_QUOTA_EXCEEDED}});
                   if (auto errorCode = folly::get_ptr(errorCodeMap, *exPtr)) {
                     return *errorCode;
                   }
@@ -293,29 +316,16 @@ FOLLY_NODISCARD folly::exception_wrapper processFirstResponseHelper(
             exceptionMetadataBase.name_utf8_ref() = *uexPtr;
             otherMetadataRef->erase("uex");
           }
-          PayloadExceptionMetadata exceptionMetadata;
-          if (exPtr && *exPtr == kAppClientErrorCode) {
-            if (version < 8) {
-              exceptionMetadata.set_DEPRECATED_appClientException(
-                  PayloadAppClientExceptionMetadata());
-            } else {
-              PayloadAppUnknownExceptionMetdata aue;
-              aue.errorClassification_ref().ensure().blame_ref() =
-                  ErrorBlame::CLIENT;
-              exceptionMetadata.set_appUnknownException(std::move(aue));
-            }
-          } else {
-            if (version < 8) {
-              exceptionMetadata.set_DEPRECATED_appServerException(
-                  PayloadAppServerExceptionMetadata());
-            } else {
-              PayloadAppUnknownExceptionMetdata aue;
-              aue.errorClassification_ref().ensure().blame_ref() =
-                  ErrorBlame::SERVER;
-              exceptionMetadata.set_appUnknownException(std::move(aue));
-            }
-          }
-          exceptionMetadataBase.metadata_ref() = std::move(exceptionMetadata);
+
+          const auto isClientError = exPtr && *exPtr == kAppClientErrorCode;
+          exceptionMetadataBase.metadata_ref()
+              .ensure()
+              .appUnknownException_ref()
+              .ensure()
+              .errorClassification_ref()
+              .ensure()
+              .blame_ref() =
+              isClientError ? ErrorBlame::CLIENT : ErrorBlame::SERVER;
 
           payload->clear();
 
@@ -332,12 +342,10 @@ FOLLY_NODISCARD folly::exception_wrapper processFirstResponseHelper(
         break;
       }
       default:
-        DCHECK_GE(version, 3);
         return makeResponseRpcError(
             ResponseRpcErrorCode::UNKNOWN, "Invalid message type", metadata);
     }
   } catch (...) {
-    DCHECK_GE(version, 3);
     return makeResponseRpcError(
         ResponseRpcErrorCode::UNKNOWN,
         fmt::format(
@@ -381,16 +389,14 @@ FOLLY_NODISCARD folly::exception_wrapper processFirstResponse(
         *compressionConfig, metadata, payload->computeChainDataLength());
   }
 
-  DCHECK_GE(version, 1);
   switch (protType) {
     case protocol::T_BINARY_PROTOCOL:
-      return processFirstResponseHelper<BinaryProtocolReader>(
+      return processFirstResponseHelper<BinarySerializer>(
           metadata, payload, version);
     case protocol::T_COMPACT_PROTOCOL:
-      return processFirstResponseHelper<CompactProtocolReader>(
+      return processFirstResponseHelper<CompactSerializer>(
           metadata, payload, version);
     default: {
-      DCHECK_GE(version, 3);
       return makeResponseRpcError(
           ResponseRpcErrorCode::UNKNOWN,
           "Invalid response payload protocol id",
@@ -398,6 +404,20 @@ FOLLY_NODISCARD folly::exception_wrapper processFirstResponse(
     }
   }
 }
+
+template <typename Error, typename Callback, typename ResponseChannel>
+void handleStreamError(
+    Error&& error, Callback& callback, ResponseChannel* channel) {
+  error.handle(
+      [&](RocketException& ex) {
+        std::exchange(callback, nullptr)->onFirstResponseError(std::move(ex));
+      },
+      [&](...) {
+        channel->sendErrorWrapped(
+            std::forward<Error>(error), kUnknownErrorCode);
+      });
+}
+
 } // namespace
 
 ThriftServerRequestResponse::ThriftServerRequestResponse(
@@ -575,12 +595,7 @@ bool ThriftServerRequestStream::sendStreamThriftResponse(
   }
   if (auto error = processFirstResponse(
           metadata, data, getProtoId(), version_, getCompressionConfig())) {
-    error.handle(
-        [&](RocketException& ex) {
-          std::exchange(clientCallback_, nullptr)
-              ->onFirstResponseError(std::move(ex));
-        },
-        [&](...) { sendErrorWrapped(std::move(error), kUnknownErrorCode); });
+    handleStreamError(std::move(error), clientCallback_, this);
     return false;
   }
   context_.unsetMarkRequestComplete();
@@ -602,12 +617,7 @@ void ThriftServerRequestStream::sendStreamThriftResponse(
   }
   if (auto error = processFirstResponse(
           metadata, data, getProtoId(), version_, getCompressionConfig())) {
-    error.handle(
-        [&](RocketException& ex) {
-          std::exchange(clientCallback_, nullptr)
-              ->onFirstResponseError(std::move(ex));
-        },
-        [&](...) { sendErrorWrapped(std::move(error), kUnknownErrorCode); });
+    handleStreamError(std::move(error), clientCallback_, this);
     return;
   }
   context_.unsetMarkRequestComplete();
@@ -624,12 +634,7 @@ void ThriftServerRequestStream::sendSerializedError(
     std::unique_ptr<folly::IOBuf> exbuf) noexcept {
   if (auto error = processFirstResponse(
           metadata, exbuf, getProtoId(), version_, getCompressionConfig())) {
-    error.handle(
-        [&](RocketException& ex) {
-          std::exchange(clientCallback_, nullptr)
-              ->onFirstResponseError(std::move(ex));
-        },
-        [&](...) { sendErrorWrapped(std::move(error), kUnknownErrorCode); });
+    handleStreamError(std::move(error), clientCallback_, this);
     return;
   }
   std::exchange(clientCallback_, nullptr)
@@ -695,12 +700,7 @@ void ThriftServerRequestSink::sendSerializedError(
     std::unique_ptr<folly::IOBuf> exbuf) noexcept {
   if (auto error = processFirstResponse(
           metadata, exbuf, getProtoId(), version_, getCompressionConfig())) {
-    error.handle(
-        [&](RocketException& ex) {
-          std::exchange(clientCallback_, nullptr)
-              ->onFirstResponseError(std::move(ex));
-        },
-        [&](...) { sendErrorWrapped(std::move(error), kUnknownErrorCode); });
+    handleStreamError(std::move(error), clientCallback_, this);
     return;
   }
   std::exchange(clientCallback_, nullptr)
@@ -720,12 +720,7 @@ void ThriftServerRequestSink::sendSinkThriftResponse(
   }
   if (auto error = processFirstResponse(
           metadata, data, getProtoId(), version_, getCompressionConfig())) {
-    error.handle(
-        [&](RocketException& ex) {
-          std::exchange(clientCallback_, nullptr)
-              ->onFirstResponseError(std::move(ex));
-        },
-        [&](...) { sendErrorWrapped(std::move(error), kUnknownErrorCode); });
+    handleStreamError(std::move(error), clientCallback_, this);
     return;
   }
   context_.unsetMarkRequestComplete();
@@ -756,12 +751,7 @@ bool ThriftServerRequestSink::sendSinkThriftResponse(
   }
   if (auto error = processFirstResponse(
           metadata, data, getProtoId(), version_, getCompressionConfig())) {
-    error.handle(
-        [&](RocketException& ex) {
-          std::exchange(clientCallback_, nullptr)
-              ->onFirstResponseError(std::move(ex));
-        },
-        [&](...) { sendErrorWrapped(std::move(error), kUnknownErrorCode); });
+    handleStreamError(std::move(error), clientCallback_, this);
     return false;
   }
   context_.unsetMarkRequestComplete();
