@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -50,8 +50,8 @@
 #include <thrift/lib/thrift/gen-cpp2/RpcMetadata_constants.h>
 
 namespace {
-const int64_t kRocketServerMaxVersion = 8;
-const int64_t kRocketServerMinVersion = 6;
+const int64_t kRocketServerMaxVersion = 10;
+const int64_t kRocketServerMinVersion = 8;
 } // namespace
 
 THRIFT_FLAG_DEFINE_bool(rocket_server_legacy_protocol_key, true);
@@ -133,8 +133,14 @@ void ThriftRocketServerHandler::handleSetupFrame(
       ((!THRIFT_FLAG(rocket_server_legacy_protocol_key) ||
         protocolKey != kLegacyRocketProtocolKey) &&
        protocolKey != RpcMetadata_constants::kRocketProtocolKey())) {
-    return connection.close(folly::make_exception_wrapper<RocketException>(
-        ErrorCode::INVALID_SETUP, "Incompatible Thrift version"));
+    if (!frame.rocketMimeTypes()) {
+      return connection.close(folly::make_exception_wrapper<RocketException>(
+          ErrorCode::INVALID_SETUP, "Incompatible Thrift version"));
+    }
+    // If Rocket MIME types are used the protocol key is optional.
+    if (success) {
+      cursor.retreat(4);
+    }
   }
 
   try {
@@ -168,6 +174,12 @@ void ThriftRocketServerHandler::handleSetupFrame(
           ErrorCode::INVALID_SETUP, "Incompatible Rocket version"));
     }
     version_ = std::min(version_, maxVersion);
+
+    if (version_ >= 9 && !frame.rocketMimeTypes()) {
+      return connection.close(folly::make_exception_wrapper<RocketException>(
+          ErrorCode::INVALID_SETUP, "Unsupported MIME types"));
+    }
+
     eventBase_ = connContext_.getTransport()->getEventBase();
     for (const auto& h : setupFrameHandlers_) {
       auto processorInfo = h->tryHandle(meta);
@@ -177,7 +189,10 @@ void ThriftRocketServerHandler::handleSetupFrame(
         serviceMetadata_ =
             std::addressof(worker_->getMetadataForService(*processorFactory_));
         valid &= !!(processor_ = processorFactory_->getProcessor());
-        valid &= !!(threadManager_ = std::move(processorInfo->threadManager_));
+        // Allow no thread manager if resource pools in use
+        valid &=
+            (!!(threadManager_ = std::move(processorInfo->threadManager_)) ||
+             !worker_->getServer()->resourcePoolSet().empty());
         valid &= !!(serverConfigs_ = &processorInfo->serverConfigs_);
         requestsRegistry_ = processorInfo->requestsRegistry_ != nullptr
             ? processorInfo->requestsRegistry_
@@ -194,10 +209,17 @@ void ThriftRocketServerHandler::handleSetupFrame(
     // no custom frame handler was found, do the default
     processorFactory_ =
         std::addressof(worker_->getServer()->getDecoratedProcessorFactory());
-    serviceMetadata_ =
-        std::addressof(worker_->getMetadataForService(*processorFactory_));
+    if (auto newConnectionContext = ThriftServer::extractNewConnectionContext(
+            const_cast<folly::AsyncTransport&>(*connContext_.getTransport()))) {
+      processorFactoryStorage_ = newConnectionContext->processorFactory;
+      processorFactory_ = std::addressof(*processorFactoryStorage_);
+    }
+    serviceMetadata_ = std::addressof(worker_->getMetadataForService(
+        *processorFactory_, processorFactoryStorage_));
     processor_ = processorFactory_->getProcessor();
-    threadManager_ = worker_->getServer()->getThreadManager();
+    if (worker_->getServer()->resourcePoolSet().empty()) {
+      threadManager_ = worker_->getServer()->getThreadManager_deprecated();
+    }
     serverConfigs_ = worker_->getServer();
     requestsRegistry_ = worker_->getRequestsRegistry();
 
@@ -212,7 +234,6 @@ void ThriftRocketServerHandler::handleSetupFrame(
       connection.applyDscpAndMarkToSocket(meta);
     }
 
-    DCHECK_GE(version_, 5);
     ServerPushMetadata serverMeta;
     serverMeta.set_setupResponse();
     serverMeta.setupResponse_ref()->version_ref() = version_;
@@ -496,6 +517,7 @@ void ThriftRocketServerHandler::handleRequestCommon(
   // check if server is overloaded
   const auto& headers = request->getTHeader().getHeaders();
   const auto& name = request->getMethodName();
+
   auto errorCode = serverConfigs_->checkOverload(&headers, &name);
   serverConfigs_->incActiveRequests();
   if (UNLIKELY(errorCode.has_value())) {
@@ -561,7 +583,9 @@ void ThriftRocketServerHandler::handleRequestCommon(
       observer->admittedRequest(&request->getMethodName());
       // Expensive operations; happens only when sampling is enabled
       if (samplingStatus.isEnabledByServer()) {
-        observer->queuedRequests(threadManager_->pendingUpstreamTaskCount());
+        if (threadManager_) {
+          observer->queuedRequests(threadManager_->pendingUpstreamTaskCount());
+        }
         observer->activeRequests(serverConfigs_->getActiveRequests());
       }
     }
@@ -582,42 +606,15 @@ void ThriftRocketServerHandler::handleRequestCommon(
           ? CompressionAlgorithm::NONE
           : metadata.compression_ref().value_or(CompressionAlgorithm::NONE));
 
-  try {
-    if (auto* found = std::get_if<PerServiceMetadata::MetadataFound>(
-            &methodMetadataResult);
-        LIKELY(found != nullptr)) {
-      processor_->processSerializedCompressedRequestWithMetadata(
-          std::move(request),
-          std::move(serializedCompressedRequest),
-          found->metadata,
-          protocolId,
-          cpp2ReqCtx,
-          eventBase_,
-          threadManager_.get());
-    } else if (std::holds_alternative<
-                   PerServiceMetadata::MetadataNotImplemented>(
-                   methodMetadataResult)) {
-      // The AsyncProcessorFactory does not implement createMethodMetadata
-      // so we need to fallback to processSerializedCompressedRequest.
-      processor_->processSerializedCompressedRequest(
-          std::move(request),
-          std::move(serializedCompressedRequest),
-          protocolId,
-          cpp2ReqCtx,
-          eventBase_,
-          threadManager_.get());
-    } else if (std::holds_alternative<PerServiceMetadata::MetadataNotFound>(
-                   methodMetadataResult)) {
-      std::string_view methodName = request->getMethodName();
-      AsyncProcessorHelper::sendUnknownMethodError(
-          std::move(request), methodName);
-    } else {
-      LOG(FATAL) << "Invalid PerServiceMetadata from Cpp2Worker";
-    }
-  } catch (...) {
-    LOG(DFATAL) << "AsyncProcessor::process exception: "
-                << folly::exceptionStr(std::current_exception());
-  }
+  Cpp2Worker::dispatchRequest(
+      processor_.get(),
+      std::move(request),
+      std::move(serializedCompressedRequest),
+      methodMetadataResult,
+      protocolId,
+      cpp2ReqCtx,
+      threadManager_.get(),
+      serverConfigs_);
 }
 
 void ThriftRocketServerHandler::handleRequestWithBadMetadata(
@@ -671,6 +668,9 @@ void ThriftRocketServerHandler::handleAppError(
     const std::string& name,
     const std::string& message,
     bool isClientError) {
+  if (auto* observer = serverConfigs_->getObserver()) {
+    observer->taskKilled();
+  }
   static const std::string headerEx = "uex";
   static const std::string headerExWhat = "uexw";
   auto header = request->getRequestContext()->getHeader();
