@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -106,7 +106,8 @@ class FakeProcessorFactory final
     return std::make_unique<FakeProcessor>();
   }
 
-  std::vector<apache::thrift::ServiceHandler*> getServiceHandlers() override {
+  std::vector<apache::thrift::ServiceHandlerBase*> getServiceHandlers()
+      override {
     return {};
   }
 };
@@ -117,7 +118,7 @@ void testServerOneInput(const uint8_t* Data, size_t Size) {
       new apache::thrift::rocket::test::FakeTransport(&evb));
   auto* const sockPtr = sock.get();
   apache::thrift::ThriftServer server;
-  server.setProcessorFactory(std::make_shared<FakeProcessorFactory>());
+  server.setInterface(std::make_shared<FakeProcessorFactory>());
   auto worker = apache::thrift::Cpp2Worker::create(&server, nullptr, &evb);
   std::vector<std::unique_ptr<apache::thrift::rocket::SetupFrameHandler>> v;
   folly::SocketAddress address;
