@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -18,11 +18,15 @@
 
 #include <cstdint>
 
-namespace apache::thrift::type {
+namespace apache {
+namespace thrift {
+namespace type {
 enum class UniversalHashAlgorithm {
-  Sha2_256 = 1,
+  Sha2_256 = 2,
 };
 
 constexpr int8_t const kMinHashBytes = 8;
 constexpr int8_t const kDefaultHashBytes = 16;
-} // namespace apache::thrift::type
+} // namespace type
+} // namespace thrift
+} // namespace apache
