@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -17,7 +17,7 @@
 #include <thrift/lib/cpp2/type/UniversalHashAlgorithm.h>
 
 #include <folly/portability/GTest.h>
-#include <thrift/lib/thrift/gen-cpp2/type_constants.h>
+#include <thrift/lib/thrift/gen-cpp2/standard_constants.h>
 #include <thrift/lib/thrift/gen-cpp2/type_types.h>
 
 namespace apache::thrift::type {
@@ -25,14 +25,14 @@ namespace {
 
 TEST(UniversalHashAlgorithmTest, Conversions) {
   EXPECT_EQ(
-      static_cast<UniversalHashAlgorithm>(
-          ThriftUniversalHashAlgorithm::Sha2_256),
+      static_cast<UniversalHashAlgorithm>(UniversalHashAlgorithmEnum::Sha2_256),
       UniversalHashAlgorithm::Sha2_256);
 }
 
 TEST(UniversalHashAlgorithmTest, ConstConversions) {
-  static_assert(kDefaultHashBytes == type_constants::defaultTypeHashBytes());
-  static_assert(kMinHashBytes == type_constants::minTypeHashBytes());
+  static_assert(
+      kDefaultHashBytes == standard_constants::defaultTypeHashBytes());
+  static_assert(kMinHashBytes == standard_constants::minTypeHashBytes());
 }
 
 } // namespace
