@@ -1,23 +1,19 @@
 /*
- * Copyright 2019-present Facebook, Inc.
+ * Copyright (c) Facebook, Inc. and its affiliates.
  *
- * Licensed to the Apache Software Foundation (ASF) under one
- * or more contributor license agreements. See the NOTICE file
- * distributed with this work for additional information
- * regarding copyright ownership. The ASF licenses this file
- * to you under the Apache License, Version 2.0 (the
- * "License"); you may not use this file except in compliance
- * with the License. You may obtain a copy of the License at
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
- * Unless required by applicable law or agreed to in writing,
- * software distributed under the License is distributed on an
- * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
- * KIND, either express or implied. See the License for the
- * specific language governing permissions and limitations
- * under the License.
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
+
 package thrift
 
 import (
@@ -31,12 +27,14 @@ import (
 
 // Header keys
 const (
-	IdentityHeader      string = "identity"
-	IDVersionHeader     string = "id_version"
-	IDVersion           string = "1"
-	PriorityHeader      string = "thrift_priority"
-	ClientTimeoutHeader string = "client_timeout"
-	QueueTimeoutHeader  string = "queue_timeout"
+	IdentityHeader       string = "identity"
+	IDVersionHeader      string = "id_version"
+	IDVersion            string = "1"
+	PriorityHeader       string = "thrift_priority"
+	ClientTimeoutHeader  string = "client_timeout"
+	QueueTimeoutHeader   string = "queue_timeout"
+	ClientMetadataHeader string = "client_metadata"
+	ClientMetadata       string = "{\"agent\":\"headertransport.go\"}"
 	// Header Magicks
 	// 0 and 16th bits must be 0 to differentiate from framed & unframed
 	HeaderMagic         uint32 = 0x0FFF0000
@@ -91,7 +89,7 @@ func (c ClientType) String() string {
 	}
 }
 
-type HeaderFlags uint32
+type HeaderFlags uint16
 
 const (
 	HeaderFlagSupportOutOfOrder HeaderFlags = 0x01
