@@ -1,27 +1,22 @@
 /*
- * Copyright 2019-present Facebook, Inc.
+ * Copyright (c) Facebook, Inc. and its affiliates.
  *
- * Licensed to the Apache Software Foundation (ASF) under one
- * or more contributor license agreements. See the NOTICE file
- * distributed with this work for additional information
- * regarding copyright ownership. The ASF licenses this file
- * to you under the Apache License, Version 2.0 (the
- * "License"); you may not use this file except in compliance
- * with the License. You may obtain a copy of the License at
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
- * Unless required by applicable law or agreed to in writing,
- * software distributed under the License is distributed on an
- * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
- * KIND, either express or implied. See the License for the
- * specific language governing permissions and limitations
- * under the License.
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
+
 package thrift
 
 import (
-	"bytes"
 	"io"
 	"io/ioutil"
 	"math"
@@ -176,8 +171,6 @@ func ReadWriteProtocolParallelTest(t *testing.T, protocolFactory ProtocolFactory
 	rConn, wConn := tcpStreamSetupForTest(t)
 	rdr, writer := io.Pipe()
 	transports := []TransportFactory{
-		NewStreamTransportFactory(rdr, writer, false),                             // use a pipe
-		NewStreamTransportFactory(rConn, wConn, false),                            // use tcp over network
 		NewFramedTransportFactory(NewStreamTransportFactory(rdr, writer, false)),  // framed over pipe
 		NewFramedTransportFactory(NewStreamTransportFactory(rConn, wConn, false)), // framed over tcp
 	}
@@ -241,12 +234,10 @@ func ReadWriteProtocolParallelTest(t *testing.T, protocolFactory ProtocolFactory
 }
 
 func ReadWriteProtocolTest(t *testing.T, protocolFactory ProtocolFactory) {
-	buf := bytes.NewBuffer(make([]byte, 0, 1024))
 	l := HTTPClientSetupForTest(t)
 	defer l.Close()
 	transports := []TransportFactory{
 		NewMemoryBufferTransportFactory(1024),
-		NewStreamTransportFactory(buf, buf, true),
 		NewFramedTransportFactory(NewMemoryBufferTransportFactory(1024)),
 		NewHTTPPostClientTransportFactory("http://" + l.Addr().String()),
 	}
