@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -464,7 +464,8 @@ class PythonAsyncProcessorFactory : public AsyncProcessorFactory {
 
   // TODO(T89004867): Call onStartServing() and onStopServing() hooks for
   // non-C++ thrift servers
-  std::vector<apache::thrift::ServiceHandler*> getServiceHandlers() override {
+  std::vector<apache::thrift::ServiceHandlerBase*> getServiceHandlers()
+      override {
     return {};
   }
 
@@ -482,7 +483,7 @@ class CppServerWrapper : public ThriftServer {
     // We use a shared_ptr to manage the adapter so the processor
     // factory handing won't ever try to manipulate python reference
     // counts without the GIL.
-    setProcessorFactory(std::make_unique<PythonAsyncProcessorFactory>(
+    setInterface(std::make_unique<PythonAsyncProcessorFactory>(
         std::make_shared<object>(adapter)));
   }
 
@@ -501,6 +502,11 @@ class CppServerWrapper : public ThriftServer {
     // Thrift main loop.  This will run indefinitely, until stop() is
     // called.
 
+    if (auto sslContextObserver = ThriftServer::getSSLConfig()) {
+      // This mirrors what we do in ThriftServer::serve to handle listening for
+      // cert/key changes and reloading SSLContextConfigs.
+      ThriftServer::getSSLCallbackHandle();
+    }
     getServeEventBase()->loopForever();
   }
 
@@ -674,7 +680,7 @@ class CppServerWrapper : public ThriftServer {
   // the magic is in the custom deleter which takes and releases a refcount on
   // the std::shared_ptr, instead of doing any local deletion.
   boost::shared_ptr<ThreadManager> getThreadManagerHelper() {
-    auto ptr = this->getThreadManager();
+    auto ptr = this->getThreadManager_deprecated();
     return boost::shared_ptr<ThreadManager>(ptr.get(), [ptr](void*) {});
   }
 
@@ -702,10 +708,6 @@ class CppServerWrapper : public ThriftServer {
         numCPUWorkerThreads, AttributeSource::OVERRIDE);
   }
 
-  void setEnableCodel(bool enableCodel) {
-    BaseThriftServer::setEnableCodel(enableCodel, AttributeSource::OVERRIDE);
-  }
-
   void setWrapperName(object wrapperName) {
     BaseThriftServer::metadata().wrapper =
         extract<std::string>(str(wrapperName));
