@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,8 +15,8 @@
 
 import unittest
 
-from b.lite_types import TypedefB
-from c.lite_types import StructC
+from b.thrift_types import TypedefB
+from c.thrift_types import StructC
 
 
 class ExtraNamespaceTests(unittest.TestCase):
