@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,7 +16,7 @@
 import itertools
 import unittest
 
-from testing.types import I32List, StringList, StrList2D, Uint32List, easy, int_list
+from testing.types import easy, I32List, int_list, StringList, StrList2D, Uint32List
 from thrift.py3.types import Container
 
 
