@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,7 +19,7 @@ import testing.metadata
 from apache.thrift.metadata.types import ThriftPrimitiveType
 from testing.clients import TestingService, TestingServiceChild
 from testing.services import TestingServiceInterface
-from testing.types import hard, HardError, Perm, mixed
+from testing.types import hard, HardError, mixed, Perm
 from thrift.py3.metadata import gen_metadata, ThriftKind
 
 
@@ -214,7 +214,7 @@ class MetadataTests(unittest.TestCase):
 
     def test_metadata_structured_annotations(self) -> None:
         annotations = gen_metadata(TestingService).structuredAnnotations
-        self.assertEqual(len(annotations), 1)
+        self.assertEqual(len(annotations), 2)
 
         annotation = annotations[0]
         self.assertEqual(annotation.name, "testing.StructuredAnnotation")
