@@ -1,4 +1,4 @@
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,7 +14,7 @@
 
 import unittest
 
-from testing.a.types import Mixin3, Union, Foo
+from testing.a.types import Foo, Mixin3, Union
 from testing.b.types import Mixin1, Mixin2
 
 
