@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,22 +16,27 @@
 import asyncio
 import pickle
 import unittest
-from typing import Mapping, Union, Hashable
+from typing import Hashable, Mapping, Union
 
+from apache.thrift.test.terse_write.terse_write.types import (
+    EmptyStruct,
+    FieldLevelTerseStruct,
+    MyEnum,
+    MyStruct,
+)
 from testing.types import (
     Digits,
+    easy,
+    hard,
     I32List,
     Integers,
     SetI32,
     StringBucket,
     StrStrMap,
-    easy,
-    hard,
 )
 from thrift.py3.common import Protocol
 from thrift.py3.exceptions import Error
 from thrift.py3.serializer import (
-    Transform,
     deserialize,
     deserialize_from_header,
     deserialize_with_length,
@@ -39,6 +44,7 @@ from thrift.py3.serializer import (
     serialize_iobuf,
     serialize_with_header,
     serialize_with_header_iobuf,
+    Transform,
 )
 from thrift.py3.types import Struct
 
@@ -199,7 +205,7 @@ class SerializerTests(unittest.TestCase):
         self.thrift_serialization_round_robin(control, fixtures)
 
     def test_pickle_Integers_union(self) -> None:
-        control = Integers(large=2 ** 32)
+        control = Integers(large=2**32)
         self.pickle_round_robin(control)
 
     def test_pickle_sequence(self) -> None:
@@ -240,3 +246,52 @@ class SerializerTests(unittest.TestCase):
         with self.assertRaises(UnicodeDecodeError):
             # Accessing the property is when the string is decoded as UTF-8.
             sb.one
+
+    def test_field_level_terse_write(self) -> None:
+        obj = FieldLevelTerseStruct(
+            bool_field=True,
+            byte_field=1,
+            short_field=2,
+            int_field=3,
+            long_field=4,
+            float_field=5,
+            double_field=6,
+            string_field="7",
+            binary_field=b"8",
+            enum_field=MyEnum.ME1,
+            list_field=[1],
+            set_field={1},
+            map_field={1: 1},
+            struct_field=MyStruct(field1=1),
+        )
+        empty = EmptyStruct()
+        for proto in Protocol:
+            encoded = serialize(obj, protocol=proto)
+            decoded, length = deserialize_with_length(
+                type(obj), encoded, protocol=proto
+            )
+            self.assertIsInstance(decoded, type(obj))
+            self.assertEqual(decoded, obj)
+            self.assertEqual(length, len(encoded))
+
+        # Set fields to their intrinsic default.
+        obj = FieldLevelTerseStruct(
+            bool_field=False,
+            byte_field=0,
+            short_field=0,
+            int_field=0,
+            long_field=0,
+            float_field=0,
+            double_field=0,
+            string_field="",
+            binary_field=b"",
+            enum_field=MyEnum.ME0,
+            list_field=[],
+            set_field=set(),
+            map_field={},
+            struct_field=MyStruct(field1=0),
+        )
+        for proto in Protocol:
+            encoded = serialize(obj, protocol=proto)
+            encoded_empty = serialize(empty, protocol=proto)
+            self.assertEqual(encoded, encoded_empty)
