@@ -1,4 +1,4 @@
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,22 +18,22 @@ import unittest
 
 from testing.types import (
     Color,
+    easy,
     File,
+    hard,
     Integers,
     Kind,
     Nested1,
     Nested2,
     Nested3,
     NonCopyable,
+    numerical,
     Optionals,
+    PrivateCppRefField,
     Reserved,
     Runtime,
     SlowCompare,
     UnusedError,
-    easy,
-    hard,
-    numerical,
-    PrivateCppRefField,
 )
 from thrift.py3.common import Protocol
 from thrift.py3.serializer import deserialize
@@ -169,6 +169,8 @@ class StructTests(unittest.TestCase):
     def test_runtime_checks(self) -> None:
         x = Runtime()
         with self.assertRaises(TypeError):
+            # pyre-fixme[6]: For 1st param expected `Union[None, bool, __NotSet]`
+            #  but got `int`.
             x(bool_val=5)
 
         with self.assertRaises(TypeError):
@@ -176,6 +178,8 @@ class StructTests(unittest.TestCase):
             Runtime(bool_val=5)
 
         with self.assertRaises(TypeError):
+            # pyre-fixme[6]: For 1st param expected `Union[None, Color, __NotSet]`
+            #  but got `int`.
             x(enum_val=2)
 
         with self.assertRaises(TypeError):
@@ -183,6 +187,8 @@ class StructTests(unittest.TestCase):
             Runtime(enum_val=2)
 
         with self.assertRaises(TypeError):
+            # pyre-fixme[6]: For 1st param expected `Union[None, Sequence[int],
+            #  __NotSet]` but got `List[str]`.
             x(int_list_val=["foo", "bar", "baz"])
 
         with self.assertRaises(TypeError):
@@ -262,6 +268,12 @@ class StructTests(unittest.TestCase):
             ],
         )
 
+    def test_dir(self) -> None:
+        self.assertEqual(
+            ["an_int", "name", "val", "val_list"],
+            dir(easy()),
+        )
+
     def test_update_nested_fields(self) -> None:
         n = Nested1(a=Nested2(b=Nested3(c=easy(val=42, name="foo"))))
         n = Struct.update_nested_field(n, {"a.b.c": easy(val=128)})
@@ -305,10 +317,10 @@ class StructTests(unittest.TestCase):
 class NumericalConversionsTests(unittest.TestCase):
     def test_overflow(self) -> None:
         with self.assertRaises(OverflowError):
-            numerical(float_val=5, int_val=2 ** 63 - 1)
+            numerical(float_val=5, int_val=2**63 - 1)
 
         with self.assertRaises(OverflowError):
-            numerical(float_val=5, int_val=2, int_list=[5, 2 ** 32])
+            numerical(float_val=5, int_val=2, int_list=[5, 2**32])
 
     def test_int_to_float(self) -> None:
         x = numerical(int_val=5, float_val=5, float_list=[1, 5, 6])
@@ -317,9 +329,9 @@ class NumericalConversionsTests(unittest.TestCase):
         x(float_list=[6, 7, 8])
 
     def test_int_to_i64(self) -> None:
-        large = 2 ** 63 - 1
+        large = 2**63 - 1
         numerical(int_val=5, float_val=5, i64_val=int(large))
-        too_large = 2 ** 65 - 1
+        too_large = 2**65 - 1
         with self.assertRaises(OverflowError):
             numerical(int_val=5, float_val=5, i64_val=int(too_large))
 
