@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,12 +14,14 @@
  * limitations under the License.
  */
 
+use crate::proptest::gen_main_struct;
 use anyhow::Result;
 use fbthrift::simplejson_protocol::{deserialize, serialize};
 use fbthrift::{simplejson_protocol::SimpleJsonProtocolDeserializer, Deserialize};
 use fbthrift_test_if::{
     Basic, Containers, En, MainStruct, MainStructNoBinary, Small, SubStruct, Un, UnOne,
 };
+use proptest::prelude::*;
 use std::collections::BTreeMap;
 use std::io::Cursor;
 
@@ -312,7 +314,7 @@ fn infinite_spaces() -> Result<()> {
     let r = MainStruct {
         foo: "foo".to_string(),
         m,
-        bar: "test".to_string(),
+        bar: " test ".to_string(),
         s: sub,
         l: vec![
             Small {
@@ -322,7 +324,6 @@ fn infinite_spaces() -> Result<()> {
             },
             Small {
                 num: 2,
-                two: 3,
                 ..Default::default()
             },
         ],
@@ -336,7 +337,7 @@ fn infinite_spaces() -> Result<()> {
     let input = r#"{
          "foo"  :  "foo" ,
           "m" : { "m1" :  1   , "m2" : 2 }  ,
-        "bar":"test",
+        "bar":" test ",
         "s":{"optDef":  "IAMOPT"  ,"req_def":  "IAMREQ","bin": ""  },
         "l":[{"num":1,"two":2},{"num"  :2 ," two" : 3 } ],
         "u":{"un1":{"one":  1  } },
@@ -511,3 +512,11 @@ fn test_unknown_union() -> Result<()> {
 
     Ok(())
 }
+
+proptest! {
+#[test]
+fn test_prop_serialize_deserialize(s in gen_main_struct()) {
+    let processed = deserialize(serialize(&s)).unwrap();
+    prop_assert_eq!(s, processed);
+}
+}
