@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -26,7 +26,7 @@ use crate::serialize::Serialize;
 use crate::thrift_protocol::{MessageType, ProtocolID};
 use crate::ttype::TType;
 use crate::Result;
-use anyhow::{anyhow, bail};
+use anyhow::{anyhow, bail, Context};
 use bufsize::SizeCounter;
 use bytes::{buf::Writer, Buf, BufMut, Bytes, BytesMut};
 use ghost::phantom;
@@ -391,13 +391,13 @@ impl<B: Buf> SimpleJsonProtocolDeserializer<B> {
             self.advance(1);
         }
     }
-
-    // strip whitespace before and after
-    fn eat(&mut self, val: &[u8]) -> Result<()> {
-        self.strip_whitespace();
-
+    // Validates that next chars is `val`
+    fn eat_only(&mut self, val: &[u8]) -> Result<()> {
         if self.remaining < val.len() {
-            bail!("Expected {:?}, not enough remaining", val)
+            bail!(
+                "Expected the following chars: {:?}, not enough bytes remaining",
+                String::from_utf8_lossy(val)
+            )
         }
 
         for to_check in val {
@@ -408,6 +408,13 @@ impl<B: Buf> SimpleJsonProtocolDeserializer<B> {
             }
             self.advance(1);
         }
+        Ok(())
+    }
+
+    // Validates that next chars is `val`. Strip whitespace before and after
+    fn eat(&mut self, val: &[u8]) -> Result<()> {
+        self.strip_whitespace();
+        self.eat_only(val)?;
         self.strip_whitespace();
         Ok(())
     }
@@ -831,7 +838,8 @@ impl<B: Buf> ProtocolReader for SimpleJsonProtocolDeserializer<B> {
     }
 
     fn read_string(&mut self) -> Result<String> {
-        self.eat(b"\"")?;
+        self.strip_whitespace();
+        self.eat_only(b"\"")?;
         let mut ret = Vec::new();
         ret.push(b'"');
         loop {
@@ -887,7 +895,8 @@ impl<B: Buf> ProtocolReader for SimpleJsonProtocolDeserializer<B> {
                 None => bail!("Expected some char"),
             }
         }
-        let bin = base64::decode_config(&ret, base64::STANDARD_NO_PAD)?;
+        let bin = base64::decode_config(&ret, base64::STANDARD_NO_PAD)
+            .context("the `binary` data in JSON string does not contain valid base64")?;
         Ok(V::from_vec(bin))
     }
 
@@ -915,12 +924,22 @@ where
 {
 }
 
+pub trait Serializable:
+    Serialize<SimpleJsonProtocolSerializer<SizeCounter>>
+    + Serialize<SimpleJsonProtocolSerializer<BytesMut>>
+{
+}
+
+impl<T> Serializable for T where
+    T: Serialize<SimpleJsonProtocolSerializer<SizeCounter>>
+        + Serialize<SimpleJsonProtocolSerializer<BytesMut>>
+{
+}
+
 /// Serialize a Thrift value using the compact protocol.
 pub fn serialize<T>(v: T) -> Bytes
 where
-    T: Serialize<SimpleJsonProtocolSerializer<SizeCounter>>
-        + Serialize<SimpleJsonProtocolSerializer<BytesMut>>
-        + Copy,
+    T: Serializable,
 {
     let mut sizer = SimpleJsonProtocolSerializer {
         buffer: SizeCounter::new().writer(),
