@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -17,7 +17,11 @@
 #include <thrift/test/AdapterTest.h>
 
 #include <chrono>
+#include <cstdint>
 #include <limits>
+#include <map>
+#include <set>
+#include <vector>
 
 #include <folly/portability/GTest.h>
 #include <thrift/lib/cpp2/Adapt.h>
@@ -263,24 +267,24 @@ TEST_F(AdapterTest, StructCodeGen_Terse) {
 
 namespace basic {
 TEST_F(AdapterTest, UnionCodeGen_Empty) {
-  AdaptTestUnion obj0a;
-  EXPECT_EQ(obj0a.getType(), AdaptTestUnion::__EMPTY__);
+  ThriftAdaptTestUnion obj0a;
+  EXPECT_EQ(obj0a.getType(), ThriftAdaptTestUnion::__EMPTY__);
 
   auto data0 = CompactSerializer::serialize<std::string>(obj0a);
-  AdaptTestUnion obj0b;
+  ThriftAdaptTestUnion obj0b;
   CompactSerializer::deserialize(data0, obj0b);
-  EXPECT_EQ(obj0b.getType(), AdaptTestUnion::__EMPTY__);
+  EXPECT_EQ(obj0b.getType(), ThriftAdaptTestUnion::__EMPTY__);
 
   EXPECT_EQ(obj0b, obj0a);
   EXPECT_FALSE(obj0b < obj0a);
 }
 
 TEST_F(AdapterTest, UnionCodeGen_Delay_Default) {
-  AdaptTestUnion obj1a;
+  ThriftAdaptTestUnion obj1a;
   EXPECT_EQ(obj1a.delay_ref().ensure(), std::chrono::milliseconds(0));
 
   auto data1 = CompactSerializer::serialize<std::string>(obj1a);
-  AdaptTestUnion obj1b;
+  ThriftAdaptTestUnion obj1b;
   CompactSerializer::deserialize(data1, obj1b);
   EXPECT_EQ(obj1b.delay_ref().ensure(), std::chrono::milliseconds(0));
 
@@ -289,13 +293,13 @@ TEST_F(AdapterTest, UnionCodeGen_Delay_Default) {
 }
 
 TEST_F(AdapterTest, UnionCodeGen_Delay) {
-  AdaptTestUnion obj1a;
+  ThriftAdaptTestUnion obj1a;
   EXPECT_EQ(obj1a.delay_ref().ensure(), std::chrono::milliseconds(0));
   obj1a.delay_ref() = std::chrono::milliseconds(7);
   EXPECT_EQ(obj1a.delay_ref(), std::chrono::milliseconds(7));
 
   auto data1 = CompactSerializer::serialize<std::string>(obj1a);
-  AdaptTestUnion obj1b;
+  ThriftAdaptTestUnion obj1b;
   CompactSerializer::deserialize(data1, obj1b);
   EXPECT_EQ(obj1b.delay_ref(), std::chrono::milliseconds(7));
 
@@ -311,11 +315,11 @@ TEST_F(AdapterTest, UnionCodeGen_Delay) {
 }
 
 TEST_F(AdapterTest, UnionCodeGen_Custom_Default) {
-  AdaptTestUnion obj2a;
+  ThriftAdaptTestUnion obj2a;
   EXPECT_EQ(obj2a.custom_ref().ensure().val, 13); // Defined in Num.
 
   auto data2 = CompactSerializer::serialize<std::string>(obj2a);
-  AdaptTestUnion obj2b;
+  ThriftAdaptTestUnion obj2b;
   CompactSerializer::deserialize(data2, obj2b);
   EXPECT_EQ(obj2b.custom_ref()->val, 13);
 
@@ -324,12 +328,12 @@ TEST_F(AdapterTest, UnionCodeGen_Custom_Default) {
 }
 
 TEST_F(AdapterTest, UnionCodeGen_Custom_Zero) {
-  AdaptTestUnion obj2a;
+  ThriftAdaptTestUnion obj2a;
   EXPECT_EQ(obj2a.custom_ref().ensure().val, 13); // Defined in Num.
   obj2a.custom_ref()->val = 0;
 
   auto data2 = CompactSerializer::serialize<std::string>(obj2a);
-  AdaptTestUnion obj2b;
+  ThriftAdaptTestUnion obj2b;
   CompactSerializer::deserialize(data2, obj2b);
   EXPECT_EQ(obj2b.custom_ref()->val, 0);
 
@@ -338,13 +342,13 @@ TEST_F(AdapterTest, UnionCodeGen_Custom_Zero) {
 }
 
 TEST_F(AdapterTest, UnionCodeGen_Custom) {
-  AdaptTestUnion obj2a;
+  ThriftAdaptTestUnion obj2a;
   EXPECT_EQ(obj2a.custom_ref().ensure().val, 13); // Defined in Num.
   obj2a.custom_ref() = Num{std::numeric_limits<int64_t>::min()};
   EXPECT_EQ(obj2a.custom_ref()->val, std::numeric_limits<int64_t>::min());
 
   auto data2 = CompactSerializer::serialize<std::string>(obj2a);
-  AdaptTestUnion obj2b;
+  ThriftAdaptTestUnion obj2b;
   CompactSerializer::deserialize(data2, obj2b);
   EXPECT_EQ(obj2b.custom_ref()->val, std::numeric_limits<int64_t>::min());
 
@@ -361,6 +365,7 @@ TEST_F(AdapterTest, UnionCodeGen_Custom) {
 
 TEST_F(AdapterTest, TemplatedTestAdapter_AdaptTemplatedTestStruct) {
   auto obj = AdaptTemplatedTestStruct();
+  auto int_map = std::map<int64_t, int64_t>{{1, 1}};
   EXPECT_EQ(obj.adaptedBoolDefault_ref()->value, true);
   EXPECT_EQ(obj.adaptedByteDefault_ref()->value, 1);
   EXPECT_EQ(obj.adaptedShortDefault_ref()->value, 2);
@@ -368,6 +373,9 @@ TEST_F(AdapterTest, TemplatedTestAdapter_AdaptTemplatedTestStruct) {
   EXPECT_EQ(obj.adaptedLongDefault_ref()->value, 4);
   EXPECT_EQ(obj.adaptedDoubleDefault_ref()->value, 5);
   EXPECT_EQ(obj.adaptedStringDefault_ref()->value, "6");
+  EXPECT_EQ(obj.adaptedListDefault()->value, std::vector<int64_t>{1});
+  EXPECT_EQ(obj.adaptedSetDefault()->value, std::set<int64_t>{1});
+  EXPECT_EQ(obj.adaptedMapDefault()->value, int_map);
 
   obj.adaptedBool_ref() = Wrapper<bool>{true};
   obj.adaptedByte_ref() = Wrapper<int8_t>{1};
@@ -376,6 +384,11 @@ TEST_F(AdapterTest, TemplatedTestAdapter_AdaptTemplatedTestStruct) {
   obj.adaptedLong_ref() = Wrapper<int64_t>{1};
   obj.adaptedDouble_ref() = Wrapper<double>{2};
   obj.adaptedString_ref() = Wrapper<std::string>{"3"};
+  obj.adaptedList() = Wrapper<std::vector<int64_t>>{{1}};
+  obj.adaptedSet() = Wrapper<std::set<int64_t>>{{1}};
+  obj.adaptedMap() = Wrapper<std::map<int64_t, int64_t>>{{{1, 1}}};
+  obj.doubleTypedefBool_ref() = Wrapper<bool>{true};
+
   EXPECT_EQ(obj.adaptedBool_ref()->value, true);
   EXPECT_EQ(obj.adaptedByte_ref()->value, 1);
   EXPECT_EQ(obj.adaptedShort_ref()->value, 2);
@@ -383,6 +396,10 @@ TEST_F(AdapterTest, TemplatedTestAdapter_AdaptTemplatedTestStruct) {
   EXPECT_EQ(obj.adaptedLong_ref()->value, 1);
   EXPECT_EQ(obj.adaptedDouble_ref()->value, 2);
   EXPECT_EQ(obj.adaptedString_ref()->value, "3");
+  EXPECT_EQ(obj.adaptedList()->value, std::vector<int64_t>{1});
+  EXPECT_EQ(obj.adaptedSet()->value, std::set<int64_t>{1});
+  EXPECT_EQ(obj.adaptedMap()->value, int_map);
+  EXPECT_EQ(obj.doubleTypedefBool_ref()->value, true);
 
   auto objs = CompactSerializer::serialize<std::string>(obj);
   AdaptTemplatedTestStruct objd;
@@ -394,6 +411,10 @@ TEST_F(AdapterTest, TemplatedTestAdapter_AdaptTemplatedTestStruct) {
   EXPECT_EQ(objd.adaptedLong_ref()->value, 1);
   EXPECT_EQ(objd.adaptedDouble_ref()->value, 2);
   EXPECT_EQ(objd.adaptedString_ref()->value, "3");
+  EXPECT_EQ(obj.adaptedList()->value, std::vector<int64_t>{1});
+  EXPECT_EQ(obj.adaptedSet()->value, std::set<int64_t>{1});
+  EXPECT_EQ(obj.adaptedMap()->value, int_map);
+  EXPECT_EQ(obj.doubleTypedefBool_ref()->value, true);
   EXPECT_EQ(obj, objd);
 
   // Adapted fields reset to the intrinsic default.
@@ -405,10 +426,14 @@ TEST_F(AdapterTest, TemplatedTestAdapter_AdaptTemplatedTestStruct) {
   EXPECT_EQ(obj.adaptedLongDefault_ref()->value, 0);
   EXPECT_EQ(obj.adaptedDoubleDefault_ref()->value, 0);
   EXPECT_EQ(obj.adaptedStringDefault_ref()->value, "");
+  EXPECT_TRUE(obj.adaptedListDefault()->value.empty());
+  EXPECT_TRUE(obj.adaptedSetDefault()->value.empty());
+  EXPECT_TRUE(obj.adaptedMapDefault()->value.empty());
 }
 
 TEST_F(AdapterTest, TemplatedTestAdapter_AdaptTemplatedNestedTestStruct) {
   auto obj = AdaptTemplatedNestedTestStruct();
+  auto int_map = std::map<int64_t, int64_t>{{1, 1}};
   EXPECT_EQ(obj.adaptedStruct_ref()->adaptedBoolDefault_ref()->value, true);
   EXPECT_EQ(obj.adaptedStruct_ref()->adaptedByteDefault_ref()->value, 1);
   EXPECT_EQ(obj.adaptedStruct_ref()->adaptedShortDefault_ref()->value, 2);
@@ -416,6 +441,12 @@ TEST_F(AdapterTest, TemplatedTestAdapter_AdaptTemplatedNestedTestStruct) {
   EXPECT_EQ(obj.adaptedStruct_ref()->adaptedLongDefault_ref()->value, 4);
   EXPECT_EQ(obj.adaptedStruct_ref()->adaptedDoubleDefault_ref()->value, 5);
   EXPECT_EQ(obj.adaptedStruct_ref()->adaptedStringDefault_ref()->value, "6");
+  EXPECT_EQ(
+      obj.adaptedStruct()->adaptedListDefault()->value,
+      std::vector<int64_t>{1});
+  EXPECT_EQ(
+      obj.adaptedStruct()->adaptedSetDefault()->value, std::set<int64_t>{1});
+  EXPECT_EQ(obj.adaptedStruct()->adaptedMapDefault()->value, int_map);
 
   obj.adaptedStruct_ref()->adaptedBool_ref() = Wrapper<bool>{true};
   obj.adaptedStruct_ref()->adaptedByte_ref() = Wrapper<int8_t>{1};
@@ -424,6 +455,10 @@ TEST_F(AdapterTest, TemplatedTestAdapter_AdaptTemplatedNestedTestStruct) {
   obj.adaptedStruct_ref()->adaptedLong_ref() = Wrapper<int64_t>{1};
   obj.adaptedStruct_ref()->adaptedDouble_ref() = Wrapper<double>{2};
   obj.adaptedStruct_ref()->adaptedString_ref() = Wrapper<std::string>{"3"};
+  obj.adaptedStruct()->adaptedList() = Wrapper<std::vector<int64_t>>{{1}};
+  obj.adaptedStruct()->adaptedSet() = Wrapper<std::set<int64_t>>{{1}};
+  obj.adaptedStruct()->adaptedMap() =
+      Wrapper<std::map<int64_t, int64_t>>{{{1, 1}}};
 
   EXPECT_EQ(obj.adaptedStruct_ref()->adaptedBool_ref()->value, true);
   EXPECT_EQ(obj.adaptedStruct_ref()->adaptedByte_ref()->value, 1);
@@ -432,6 +467,9 @@ TEST_F(AdapterTest, TemplatedTestAdapter_AdaptTemplatedNestedTestStruct) {
   EXPECT_EQ(obj.adaptedStruct_ref()->adaptedLong_ref()->value, 1);
   EXPECT_EQ(obj.adaptedStruct_ref()->adaptedDouble_ref()->value, 2);
   EXPECT_EQ(obj.adaptedStruct_ref()->adaptedString_ref()->value, "3");
+  EXPECT_EQ(obj.adaptedStruct()->adaptedList()->value, std::vector<int64_t>{1});
+  EXPECT_EQ(obj.adaptedStruct()->adaptedSet()->value, std::set<int64_t>{1});
+  EXPECT_EQ(obj.adaptedStruct()->adaptedMap()->value, int_map);
 
   auto objs = CompactSerializer::serialize<std::string>(obj);
   AdaptTemplatedNestedTestStruct objd;
@@ -444,6 +482,12 @@ TEST_F(AdapterTest, TemplatedTestAdapter_AdaptTemplatedNestedTestStruct) {
   EXPECT_EQ(objd.adaptedStruct_ref()->adaptedLongDefault_ref()->value, 4);
   EXPECT_EQ(objd.adaptedStruct_ref()->adaptedDoubleDefault_ref()->value, 5);
   EXPECT_EQ(objd.adaptedStruct_ref()->adaptedStringDefault_ref()->value, "6");
+  EXPECT_EQ(
+      obj.adaptedStruct()->adaptedListDefault()->value,
+      std::vector<int64_t>{1});
+  EXPECT_EQ(
+      obj.adaptedStruct()->adaptedSetDefault()->value, std::set<int64_t>{1});
+  EXPECT_EQ(obj.adaptedStruct()->adaptedMapDefault()->value, int_map);
 
   EXPECT_EQ(objd.adaptedStruct_ref()->adaptedBool_ref()->value, true);
   EXPECT_EQ(objd.adaptedStruct_ref()->adaptedByte_ref()->value, 1);
@@ -452,6 +496,9 @@ TEST_F(AdapterTest, TemplatedTestAdapter_AdaptTemplatedNestedTestStruct) {
   EXPECT_EQ(objd.adaptedStruct_ref()->adaptedLong_ref()->value, 1);
   EXPECT_EQ(objd.adaptedStruct_ref()->adaptedDouble_ref()->value, 2);
   EXPECT_EQ(objd.adaptedStruct_ref()->adaptedString_ref()->value, "3");
+  EXPECT_EQ(obj.adaptedStruct()->adaptedList()->value, std::vector<int64_t>{1});
+  EXPECT_EQ(obj.adaptedStruct()->adaptedSet()->value, std::set<int64_t>{1});
+  EXPECT_EQ(obj.adaptedStruct()->adaptedMap()->value, int_map);
   EXPECT_EQ(obj, objd);
 
   // Adapted fields reset to the intrinsic default.
@@ -463,11 +510,14 @@ TEST_F(AdapterTest, TemplatedTestAdapter_AdaptTemplatedNestedTestStruct) {
   EXPECT_EQ(obj.adaptedStruct_ref()->adaptedLongDefault_ref()->value, 0);
   EXPECT_EQ(obj.adaptedStruct_ref()->adaptedDoubleDefault_ref()->value, 0);
   EXPECT_EQ(obj.adaptedStruct_ref()->adaptedStringDefault_ref()->value, "");
+  EXPECT_TRUE(obj.adaptedStruct()->adaptedListDefault()->value.empty());
+  EXPECT_TRUE(obj.adaptedStruct()->adaptedSetDefault()->value.empty());
+  EXPECT_TRUE(obj.adaptedStruct()->adaptedMapDefault()->value.empty());
 }
 
 TEST_F(AdapterTest, StructFieldAdaptedStruct) {
   StructFieldAdaptedStruct obj;
-  auto wrapper = Wrapper<AdaptedStruct>();
+  auto wrapper = Wrapper<ThriftAdaptedStruct>();
   wrapper.value.data_ref() = 42;
   obj.adaptedStruct_ref() = wrapper;
   EXPECT_EQ(obj.adaptedStruct_ref()->value.data_ref(), 42);
@@ -609,25 +659,63 @@ TEST(AdaptTest, AdapterWithContext) {
                 AdapterWithContext,
                 int64_t,
                 basic::AdaptTestStruct>);
+  static_assert(folly::is_detected_v<
+                adapt_detail::ConstructType,
+                AdapterWithContext,
+                AdaptedWithContext<int64_t, basic::AdaptTestStruct, 0>,
+                FieldAdapterContext<basic::AdaptTestStruct, 0>>);
 
   auto obj = basic::AdaptTestStruct();
-  EXPECT_EQ(obj.data_ref()->meta, &*obj.meta_ref());
+  EXPECT_EQ(obj.data()->meta, &*obj.meta());
+  EXPECT_EQ(obj.string_data()->meta, &*obj.meta());
 
   auto copy = basic::AdaptTestStruct(obj);
-  EXPECT_EQ(copy.data_ref()->meta, &*copy.meta_ref());
+  EXPECT_EQ(copy.data()->meta, &*copy.meta());
+  EXPECT_EQ(copy.string_data()->meta, &*copy.meta());
 
   auto move = basic::AdaptTestStruct(std::move(copy));
-  EXPECT_EQ(move.data_ref()->meta, &*move.meta_ref());
+  EXPECT_EQ(move.data()->meta, &*move.meta());
+  EXPECT_EQ(move.string_data()->meta, &*move.meta());
+
+  obj.data()->value = 42;
+  obj.meta() = "foo";
+  obj.string_data()->value = "42";
+
+  EXPECT_EQ(obj.data()->value, 42);
+  EXPECT_EQ(obj.data()->fieldId, 4);
+  EXPECT_EQ(*obj.data()->meta, "foo");
+  EXPECT_EQ(obj.string_data()->value, "42");
+  EXPECT_EQ(obj.string_data()->fieldId, 7);
+  EXPECT_EQ(*obj.string_data()->meta, "foo");
 
-  obj.data_ref() = {};
-  obj.data_ref()->value = 42;
-  obj.meta_ref() = "foo";
   auto serialized = CompactSerializer::serialize<std::string>(obj);
   auto obj2 = basic::AdaptTestStruct();
   CompactSerializer::deserialize(serialized, obj2);
-  EXPECT_EQ(obj2.data_ref()->value, 42);
-  EXPECT_EQ(obj2.data_ref()->fieldId, 4);
-  EXPECT_EQ(*obj2.data_ref()->meta, "foo");
+
+  EXPECT_EQ(obj2.data()->value, 42);
+  EXPECT_EQ(obj2.data()->fieldId, 4);
+  EXPECT_EQ(*obj2.data()->meta, "foo");
+  EXPECT_EQ(obj2.string_data()->value, "42");
+  EXPECT_EQ(obj2.string_data()->fieldId, 7);
+  EXPECT_EQ(*obj2.string_data()->meta, "foo");
+}
+
+TEST(AdaptTest, ComposedAdapter) {
+  auto obj = basic::AdaptTestStruct();
+
+  obj.double_wrapped_bool() = {Wrapper<bool>{true}};
+  obj.double_wrapped_integer()->value = Wrapper<int32_t>{42};
+  obj.meta() = "foo";
+
+  EXPECT_EQ(*obj.double_wrapped_integer()->meta, "foo");
+
+  auto serialized = CompactSerializer::serialize<std::string>(obj);
+  auto obj2 = basic::AdaptTestStruct();
+  CompactSerializer::deserialize(serialized, obj2);
+
+  EXPECT_EQ(obj2.double_wrapped_bool()->value.value, true);
+  EXPECT_EQ(obj2.double_wrapped_integer()->value.value, 42);
+  EXPECT_EQ(*obj2.double_wrapped_integer()->meta, "foo");
 }
 
 } // namespace apache::thrift::test
