@@ -30,6 +30,7 @@ using apache::thrift::bad_field_access;
 using apache::thrift::field_ref;
 using apache::thrift::optional_boxed_field_ref;
 using apache::thrift::optional_field_ref;
+using apache::thrift::terse_field_ref;
 using apache::thrift::detail::boxed_value_ptr;
 
 // A struct which is assignable but not constructible from int or other types
@@ -84,6 +85,16 @@ class TestStruct {
     return {std::move(name_), __isset.at(folly::index_constant<0>())};
   }
 
+  terse_field_ref<std::string&> terse_name() & { return {name_}; }
+
+  terse_field_ref<std::string&&> terse_name() && { return {std::move(name_)}; }
+
+  terse_field_ref<const std::string&> terse_name() const& { return {name_}; }
+
+  terse_field_ref<const std::string&&> terse_name() const&& {
+    return {static_cast<const std::string&&>(name_)};
+  }
+
   field_ref<IntAssignable&> int_assign() {
     return {int_assign_, __isset.at(folly::index_constant<1>())};
   }
@@ -92,6 +103,8 @@ class TestStruct {
     return {int_assign_, __isset.at(folly::index_constant<1>())};
   }
 
+  terse_field_ref<IntAssignable&> terse_int_assign() { return {int_assign_}; }
+
   optional_field_ref<std::shared_ptr<int>&> ptr_ref() {
     return {ptr_, __isset.at(folly::index_constant<2>())};
   }
@@ -124,10 +137,22 @@ class TestStruct {
     return {vec_, __isset.at(folly::index_constant<5>())};
   }
 
+  terse_field_ref<std::vector<bool>&> terse_vec() & { return {vec_}; }
+
   optional_field_ref<Nested&> opt_nested() & {
     return {nested_, __isset.at(folly::index_constant<5>())};
   }
 
+  terse_field_ref<int&> terse_int_val() & { return {int_val_}; }
+
+  terse_field_ref<int&&> terse_int_val() && { return {std::move(int_val_)}; }
+
+  terse_field_ref<const int&> terse_int_val() const& { return {int_val_}; }
+
+  terse_field_ref<const int&&> terse_int_val() const&& {
+    return {static_cast<const int&&>(int_val_)};
+  }
+
  private:
   std::string name_ = "default";
   IntAssignable int_assign_;
@@ -142,48 +167,50 @@ class TestStruct {
 
 class TestStructBoxedValuePtr {
  public:
-  optional_boxed_field_ref<boxed_value_ptr<std::string>&> opt_name() & {
-    return {name_};
+  auto opt_name() & {
+    return optional_boxed_field_ref<boxed_value_ptr<std::string>&>{name_};
   }
 
-  optional_boxed_field_ref<boxed_value_ptr<std::string>&&> opt_name() && {
-    return {std::move(name_)};
+  auto opt_name() && {
+    return optional_boxed_field_ref<boxed_value_ptr<std::string>&&>{
+        std::move(name_)};
   }
 
-  optional_boxed_field_ref<const boxed_value_ptr<std::string>&> opt_name()
-      const& {
-    return {name_};
+  auto opt_name() const& {
+    return optional_boxed_field_ref<const boxed_value_ptr<std::string>&>{name_};
   }
 
-  optional_boxed_field_ref<const boxed_value_ptr<std::string>&&> opt_name()
-      const&& {
-    return {std::move(name_)};
+  auto opt_name() const&& {
+    return optional_boxed_field_ref<const boxed_value_ptr<std::string>&&>{
+        std::move(name_)};
   }
 
-  optional_boxed_field_ref<boxed_value_ptr<IntAssignable>&> opt_int_assign() {
-    return {int_assign_};
+  auto opt_int_assign() {
+    return optional_boxed_field_ref<boxed_value_ptr<IntAssignable>&>{
+        int_assign_};
   }
 
-  optional_boxed_field_ref<boxed_value_ptr<std::shared_ptr<int>>&> ptr_ref() {
-    return {ptr_};
+  auto ptr_ref() {
+    return optional_boxed_field_ref<boxed_value_ptr<std::shared_ptr<int>>&>{
+        ptr_};
   }
 
-  optional_boxed_field_ref<boxed_value_ptr<int>&> opt_int_val() {
-    return {int_val_};
+  auto opt_int_val() {
+    return optional_boxed_field_ref<boxed_value_ptr<int>&>{int_val_};
   }
 
-  optional_boxed_field_ref<boxed_value_ptr<std::unique_ptr<int>>&>
-  opt_uptr() & {
-    return {uptr_};
+  auto opt_uptr() & {
+    return optional_boxed_field_ref<boxed_value_ptr<std::unique_ptr<int>>&>{
+        uptr_};
   }
 
-  optional_boxed_field_ref<boxed_value_ptr<std::unique_ptr<int>>&&>
-  opt_uptr() && {
-    return {std::move(uptr_)};
+  auto opt_uptr() && {
+    return optional_boxed_field_ref<boxed_value_ptr<std::unique_ptr<int>>&&>{
+        std::move(uptr_)};
   }
 
-  optional_boxed_field_ref<boxed_value_ptr<Nested>&> opt_nested() & {
-    return {nested_};
+  auto opt_nested() & {
+    return optional_boxed_field_ref<boxed_value_ptr<Nested>&>{nested_};
   }
 
  private:
@@ -733,3 +760,78 @@ TYPED_TEST(optional_field_ref_typed_test, rvalue_ref_method) {
   EXPECT_EQ(*p, 10);
   EXPECT_FALSE(*ref);
 }
+
+TEST(terse_field_ref_test, access_default_value) {
+  auto s = TestStruct();
+  EXPECT_EQ(s.terse_name(), "default");
+  EXPECT_EQ(s.terse_int_val(), 0);
+}
+
+TEST(terse_field_ref_test, is_assignable) {
+  check_is_assignable<terse_field_ref>();
+}
+
+TEST(terse_field_ref_test, assign) {
+  auto s = TestStruct();
+  EXPECT_EQ(*s.terse_name(), "default");
+  s.terse_name() = "foo";
+  EXPECT_EQ(*s.terse_name(), "foo");
+}
+
+TEST(terse_field_ref_test, copy_from) {
+  auto s = TestStruct();
+  auto s2 = TestStruct();
+  s2.terse_name() = "foo";
+  s.terse_name().copy_from(s2.terse_name());
+  EXPECT_EQ(*s.terse_name(), "foo");
+}
+
+TEST(terse_field_ref_test, move_from) {
+  auto s = TestStruct();
+  auto s2 = TestStruct();
+  s2.terse_name() = "foo";
+  s.terse_name().move_from(s2.terse_name());
+  EXPECT_EQ(*s.terse_name(), "foo");
+}
+
+TEST(terse_field_ref_test, access) {
+  auto s = TestStruct();
+  EXPECT_EQ(*s.terse_name(), "default");
+  EXPECT_EQ(s.terse_name().value(), "default");
+}
+
+TEST(terse_field_ref_test, subscript) {
+  TestStruct s;
+  s.terse_vec() = {false};
+  EXPECT_FALSE(s.terse_vec()[0]);
+  s.terse_vec()[0] = true;
+  EXPECT_TRUE(s.terse_vec()[0]);
+}
+
+TEST(terse_field_ref_test, assign_forwards) {
+  auto s = TestStruct();
+  s.terse_int_assign() = 42;
+  EXPECT_EQ(s.terse_int_assign()->value, 42);
+}
+
+TEST(terse_field_ref_test, emplace) {
+  TestStruct s;
+  s.terse_name().emplace({'f', 'o', 'o'});
+  EXPECT_EQ(s.terse_name(), "foo");
+  s.terse_name().emplace({'b', 'a', 'r'});
+  EXPECT_EQ(s.terse_name(), "bar");
+  s.terse_name().emplace({'b', 'a', 'z'}, std::allocator<char>());
+  EXPECT_EQ(s.terse_name(), "baz");
+}
+
+template <typename From, typename To>
+struct TerseFieldRefConversionChecker {
+  static_assert(
+      std::is_convertible<From, To>() ==
+          std::is_convertible<terse_field_ref<From>, terse_field_ref<To>>(),
+      "inconsistent implicit conversion");
+};
+
+TEST(terse_field_ref_test, conversions) {
+  test_conversions<TerseFieldRefConversionChecker>();
+}
