@@ -38,7 +38,7 @@ TEST_F(StructTest, serialization_terse_writes_refs_shared) {
   using apache::thrift::CompactSerializer;
 
   BasicRefsSharedTerseWrites a;
-  EXPECT_FALSE(struct_private_access::is_empty(a));
+  EXPECT_FALSE(apache::thrift::empty(a));
 
   a.shared_field_ref() = std::make_shared<HasInt>();
   a.shared_field_ref()->field_ref() = 3;
@@ -678,58 +678,58 @@ TEST_F(StructTest, CppDataMethod) {
 
 TEST_F(StructTest, EmptiableOptionalFieldsStruct) {
   EmptiableOptionalFieldsStruct obj;
-  EXPECT_TRUE(struct_private_access::is_empty(obj));
+  EXPECT_TRUE(apache::thrift::empty(obj));
 
   obj.int_field() = 1;
-  EXPECT_FALSE(struct_private_access::is_empty(obj));
+  EXPECT_FALSE(apache::thrift::empty(obj));
 
   apache::thrift::clear(obj);
-  EXPECT_TRUE(struct_private_access::is_empty(obj));
+  EXPECT_TRUE(apache::thrift::empty(obj));
 }
 
 TEST_F(StructTest, NotEmptiableStruct) {
   NotEmptiableStruct obj;
-  EXPECT_FALSE(struct_private_access::is_empty(obj));
+  EXPECT_FALSE(apache::thrift::empty(obj));
 
   NotEmptiableTerseFieldsStruct obj2;
-  EXPECT_FALSE(struct_private_access::is_empty(obj2));
+  EXPECT_FALSE(apache::thrift::empty(obj2));
 }
 
 TEST_F(StructTest, EmptyTerseStruct) {
   EmptyTerseStruct obj;
-  EXPECT_TRUE(struct_private_access::is_empty(obj));
+  EXPECT_TRUE(apache::thrift::empty(obj));
 }
 
 TEST_F(StructTest, EmptiableTerseFieldsStruct) {
   EmptiableTerseFieldsStruct obj;
-  EXPECT_TRUE(struct_private_access::is_empty(obj));
+  EXPECT_TRUE(apache::thrift::empty(obj));
 
   obj.bool_field() = true;
-  EXPECT_FALSE(struct_private_access::is_empty(obj));
+  EXPECT_FALSE(apache::thrift::empty(obj));
 
   apache::thrift::clear(obj);
-  EXPECT_TRUE(struct_private_access::is_empty(obj));
+  EXPECT_TRUE(apache::thrift::empty(obj));
 }
 
 TEST_F(StructTest, OptionalFieldsStruct) {
   OptionalFieldsStruct obj;
-  EXPECT_TRUE(struct_private_access::is_empty(obj));
+  EXPECT_TRUE(apache::thrift::empty(obj));
 
   obj.boxed_field_ref() = HasInt();
-  EXPECT_FALSE(struct_private_access::is_empty(obj));
+  EXPECT_FALSE(apache::thrift::empty(obj));
 
   apache::thrift::clear(obj);
-  EXPECT_TRUE(struct_private_access::is_empty(obj));
+  EXPECT_TRUE(apache::thrift::empty(obj));
 }
 
 TEST_F(StructTest, OptionalFieldsTerseStruct) {
   OptionalFieldsTerseStruct obj;
-  EXPECT_TRUE(struct_private_access::is_empty(obj));
+  EXPECT_TRUE(apache::thrift::empty(obj));
 
   obj.boxed_field_ref() = HasInt();
-  EXPECT_FALSE(struct_private_access::is_empty(obj));
+  EXPECT_FALSE(apache::thrift::empty(obj));
 
   apache::thrift::clear(obj);
-  EXPECT_TRUE(struct_private_access::is_empty(obj));
+  EXPECT_TRUE(apache::thrift::empty(obj));
 }
 } // namespace
