@@ -1,11 +1,11 @@
 /*
- * Copyright 2018-present Facebook, Inc.
+ * Copyright (c) Facebook, Inc. and its affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -28,12 +28,10 @@ class BackendServiceHandler : virtual public BackendServiceSvIf {
  public:
   BackendServiceHandler();
 
-  void doWork(BackendResponse& response, std::unique_ptr<BackendRequest> req)
-      override;
+  void doWork(
+      BackendResponse& response, std::unique_ptr<BackendRequest> req) override;
 
-  u_int64_t getAndResetRequestCount() {
-    return requestCount_.exchange(0);
-  }
+  u_int64_t getAndResetRequestCount() { return requestCount_.exchange(0); }
 
  private:
   std::atomic<u_int64_t> requestCount_{0};
