@@ -13,10 +13,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-from __future__ import unicode_literals
+from __future__ import absolute_import, division, print_function, unicode_literals
+
 import unittest
 
 from thrift.transport.TTransport import TMemoryBuffer
@@ -43,5 +41,6 @@ class TestTMemoryBuffer(unittest.TestCase):
         self.assertRaises(RuntimeError, buf.read, 5)
         self.assertRaises(RuntimeError, buf.write, b"world")
 
-if __name__ == '__main__':
+
+if __name__ == "__main__":
     unittest.main()
