@@ -1,5 +1,5 @@
 /*
- * Copyright (c) Facebook, Inc. and its affiliates.
+ * Copyright (c) Meta Platforms, Inc. and affiliates.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,7 +14,9 @@
  * limitations under the License.
  */
 
-#include <thrift/test/reflection/gen-cpp2/reflection_fatal_struct.h>
+#include <thrift/lib/cpp2/TypeClass.h>
+#include <thrift/test/AdapterTest.h>
+#include <thrift/test/reflection/gen-cpp2/reflection_fatal_types.h>
 
 #include <thrift/lib/cpp2/reflection/internal/test_helpers.h>
 
@@ -319,6 +321,11 @@ FATAL_S(structB_annotation2v, "this is its value");
 FATAL_S(structB_annotation3k, "some.other.annotation");
 FATAL_S(structB_annotation3v, "this is its other value");
 
+FATAL_S(struct3_annotation1k, "thrift.uri");
+FATAL_S(
+    struct3_annotation1v,
+    "facebook.com/thrift/test/reflection/reflection/struct3");
+
 TEST(fatal_struct, annotations) {
   EXPECT_SAME<
       fatal::list<>,
@@ -329,7 +336,8 @@ TEST(fatal_struct, annotations) {
       apache::thrift::reflect_struct<struct2>::annotations::map>();
 
   EXPECT_SAME<
-      fatal::list<>,
+      fatal::list<apache::thrift::
+                      annotation<struct3_annotation1k, struct3_annotation1v>>,
       apache::thrift::reflect_struct<struct3>::annotations::map>();
 
   EXPECT_SAME<
@@ -636,5 +644,58 @@ TEST(fatal_struct, renamed_field) {
   EXPECT_EQ("fancy.idl.name", fname);
 }
 
+TEST(fatal_struct, StructWithAdaptedField) {
+  using info = apache::thrift::reflect_struct<StructWithAdaptedField>;
+  using type_adapted_field = fatal::get<
+      info::members,
+      info::member::typeAdapted::name,
+      fatal::get_type::name>;
+  using field_adapted_field = fatal::get<
+      info::members,
+      info::member::fieldAdapted::name,
+      fatal::get_type::name>;
+  StructWithAdaptedField a;
+
+  using type_adapted_type = apache::thrift::test::Wrapper<IntStruct>;
+  using field_adapted_type = apache::thrift::test::
+      AdaptedWithContext<IntStruct, StructWithAdaptedField, 3>;
+
+  static_assert( //
+      std::is_same_v<
+          info::member::typeAdapted::type_class,
+          apache::thrift::type_class::structure>);
+  static_assert( //
+      std::is_same_v<
+          info::member::typeAdapted::type_class,
+          apache::thrift::type_class::structure>);
+  static_assert( //
+      std::is_same_v< //
+          info::member::fieldAdapted::type,
+          field_adapted_type>);
+  static_assert( //
+      std::is_same_v<
+          info::member::fieldAdapted::type_class,
+          apache::thrift::type_class::structure>);
+
+  type_adapted_field::field_ref_getter get_type_adapted;
+  static_assert( //
+      std::is_same_v<
+          apache::thrift::field_ref<type_adapted_type&>,
+          decltype(get_type_adapted(a))>);
+
+  field_adapted_field::field_ref_getter get_field_adapted;
+  static_assert( //
+      std::is_same_v<
+          apache::thrift::field_ref<field_adapted_type&>,
+          decltype(get_field_adapted(a))>);
+}
+
+TEST(fatal_struct, Terse) {
+  using traits = apache::thrift::reflect_struct<struct2>;
+  EXPECT_SAME<
+      required<apache::thrift::optionality::terse>,
+      fatal::get<traits::members, fieldAs, fatal::get_type::name>::optional>();
+}
+
 } // namespace cpp_reflection
 } // namespace test_cpp2
